using System;
using WebCab.Libraries.Finance.Portfolio;
using WebCab.Libraries.Finance.Portfolio.ADO;


/**
 * This example shows how to make SQL calls to your database and use methods
 * from the AssetParameters class at the same time, using ADO Mediator.
 */
public class Example {
	public static void Main () {
		/*
		 * ADO.NET Driver Settings -- please adjust if necessary.
		 */
		Type driver = typeof (CoreLab.MySql.MySqlConnection);
		string connectionString = "User=user; Password=password; Host=hostname; Port=3306; Database=test";
		
		/*
		 * Create an instance of the ADO Mediator class using
		 * using the given database specific connection settings.
		 */
		ADOMediator ado = new ADOMediator (
			driver,
			connectionString, // input database
			driver,
			connectionString); // output database

		/*
		 * Assign an instance of the AssetParameters class
		 * to the ADO Mediator.
		 */
		AssetParameters assetParameters = new AssetParameters ();
		ado.UnderlyingInstance = assetParameters;
	
		/*
		 * Calling the `ExpectedReturns' method using values selected from
		 * the database. Please adapt the SQL statement.
		 */
		double[] expectedReturnsOfFunds = (double[]) ado.OneSelect (
			"ExpectedReturns", "SELECT ASSET1, ASSET2, ASSET3, ASSET4, ASSET5, ASSET6 FROM HISTORICAL_RETURNS");

		/*
		 * Calling the `CovarianceMatrix' method using values selected
		 * from the database. Please adapt the SQL statement.
		 */
		double[][] covarianceOfFunds = (double[][]) ado.OneSelect (
			"CovarianceMatrix", "SELECT ASSET1, ASSET2, ASSET3, ASSET4, ASSET5, ASSET6 FROM HISTORICAL_RETURNS");

		/*
		 * Making calls directly to the underlying AssetParameters
		 * instance.
		 */
		double riskOf1 = assetParameters.PortfolioRisk (
			new double[] {1.0, 0.0, 0.0, 0.0, 0.0, 0.0}, // weight
			covarianceOfFunds // covarianceMatrix
		);

		double riskOf2 = assetParameters.PortfolioRisk( 
			new double[] {0.0, 1.0, 0.0, 0.0, 0.0, 0.0}, // weight
			covarianceOfFunds // covarianceMatrix
		);

		double riskOf3 = assetParameters.PortfolioRisk( 
			new double[] {0.0, 0.0, 1.0, 0.0, 0.0, 0.0}, // weight
			covarianceOfFunds // covarianceMatrix
		);

		double riskOf4 = assetParameters.PortfolioRisk( 
			new double[] {0.0, 0.0, 0.0, 1.0, 0.0, 0.0}, // weight
			covarianceOfFunds // covarianceMatrix
		);

		double riskOf5 = assetParameters.PortfolioRisk( 
			new double[] {0.0, 0.0, 0.0, 0.0, 1.0, 0.0}, // weight
			covarianceOfFunds // covarianceMatrix
		);

		double riskOf6 = assetParameters.PortfolioRisk( 
			new double[] {0.0, 0.0, 0.0, 0.0, 0.0, 1.0}, // weight
			covarianceOfFunds // covarianceMatrix
		);
		
		/*
		 * Printing out the results.
		 */
		Console.WriteLine ("The risks for every asset are: "
			+ riskOf1 + ", "
			+ riskOf2 + ", "
			+ riskOf3 + ", "
			+ riskOf4 + ", "
			+ riskOf5 + ", "
			+ riskOf6);
	}
}