using System;
using WebCab.Lib.Util.Example;

using WebCab.Libraries.Finance.Portfolio;

public class ASX200OptimalWithConstraintsClient {
   public static void Main( String[] args ) {
      // used components
      AssetParameters instanceOfAssetParameters = new AssetParameters();
      CapitalMarket instanceOfCapitalMarket = new CapitalMarket();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "             #######################################################" );
      System.Console.WriteLine( "             # ASX200OptimalWithConstraints Console Client Example #" );
      System.Console.WriteLine( "             #######################################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Overview of this Example" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    Within this example we construct a  variety  of  Optimal  Portfolios" );
      System.Console.WriteLine( "    with   differing  risk/reward  profiles  for  an  Australian  Equity" );
      System.Console.WriteLine( "    Investment Company in accordance  with  the  Capital  Asset  Pricing" );
      System.Console.WriteLine( "    Model (CAPM). This example has the extra  feature  that  each  asset" );
      System.Console.WriteLine( "    within the equity component (i.e. portfolio) of  investment  company" );
      System.Console.WriteLine( "    assets must be no more the 15 percent of  the  total  value  of  the" );
      System.Console.WriteLine( "    equity portfolio." );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    Australian Equity Portfolio Company" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    With this example we consider a equity portfolio company  which  can" );
      System.Console.WriteLine( "    construct  portfolio  consisting  of  the  following  18  Australian" );
      System.Console.WriteLine( "    listed company stocks. Moreover,  the  company  itself  is  able  to" );
      System.Console.WriteLine( "    leverage or deleverage this portfolio by  either  borrowing  from  a" );
      System.Console.WriteLine( "    local bank at the prevailing base rate, or deleveraging the  company" );
      System.Console.WriteLine( "    by deposited excess capital with a local bank at the base rate.  The" );
      System.Console.WriteLine( "    base rate is known to be: 0.25 percent per month." );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    ASX200 Stock Code for the Asset from which the Equity Portfolio  can" );
      System.Console.WriteLine( "    be constructed" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "       - SGP.ax: STOCKLAND. STAPLE" );
      System.Console.WriteLine( "       - BHP.ax: BHP BLT FPO" );
      System.Console.WriteLine( "       - BLD.ax: BORAL LTD FPO" );
      System.Console.WriteLine( "       - CDO.ax: COLORADO FPO" );
      System.Console.WriteLine( "       - COA.ax: COATES FPO" );
      System.Console.WriteLine( "       - CTX.ax: CALTEX FPO" );
      System.Console.WriteLine( "       - DJS.ax: D JONES 95 FPO" );
      System.Console.WriteLine( "       - DOW.ax: DOWNER EDI FPO" );
      System.Console.WriteLine( "       - FKP.ax: FKP LTD FPO" );
      System.Console.WriteLine( "       - FWD.ax: FLEETWOOD FPO" );
      System.Console.WriteLine( "       - GNS.ax: GUNNS FPO" );
      System.Console.WriteLine( "       - GTP.ax: GS PLANT FPO" );
      System.Console.WriteLine( "       - GUD.ax: G.U.D. FPO" );
      System.Console.WriteLine( "       - HWI.ax: HOUSEWARES FPO" );
      System.Console.WriteLine( "       - MGW.ax: MCGUIGAN FPO" );
      System.Console.WriteLine( "       - PBB.ax: PAC GROUP FPO" );
      System.Console.WriteLine( "       - PMM.ax: PORTMAN FPO" );
      System.Console.WriteLine( "       - PMP.ax: PMP LTD FPO" );
      System.Console.WriteLine( "       " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    Period of Historical Data" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    In the application of portfolio theory  (Markowitz  and  CAPM),  you" );
      System.Console.WriteLine( "    are required  to  use  historical  data  for  the  purposes  of  the" );
      System.Console.WriteLine( "    evaluation of the expected returned and  the  covariance  matrix  of" );
      System.Console.WriteLine( "    the historical returns. Within this example we  assume  we  wish  to" );
      System.Console.WriteLine( "    construct  to  optimal  Portfolio  in  the  immediate  future  after" );
      System.Console.WriteLine( "    1/1/2005, and have chosen to use monthly data from the period:" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "       - Beginning of Period: 1/12/1999" );
      System.Console.WriteLine( "       - End period: 1/1/2005" );
      System.Console.WriteLine( "       " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ We first evaluate the covariance of the asset being considered +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix( double[][] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We first evaluate the covariance matrix of  the  historical  returns" );
      System.Console.WriteLine( "     from which the portfolios can be constructed." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalReturns = {{2.5, 2.6, 2.73, 2.8, 2.82, 2.9, 2.87, 3.05, 3.04, 3.0299, 3.05, 3.13, 3.18, 3.21, 3.25, 3.28, 3.2599, 3.21, 3.27, 3.38, 3.67, 3.62, 3.73, 3.72, 3.64, 3.53, 3.77, 3.76, 3.82, 4.0, 3.82, 3.68, 3.99, 4.12, 4.15, 4.4, 4.35, 4.48, 4.39, 4.48, 4.51, 4.8, 4.5199, 4.5599, 4.5599, 4.29, 4.53, 4.76, 4.87, 4.86, 5.18, 5.46, 4.99, 5.13, 5.0, 5.33, 5.61, 5.69, 5.89, 5.88, 5.99, 6.13}," );
      System.Console.WriteLine( "                         {8.05, 7.54, 6.48, 7.18, 7.42, 7.17, 7.95, 7.34, 7.63, 7.71, 7.8, 8.13, 7.91, 7.87, 8.7899, 8.18, 8.98, 9.39, 8.96, 8.36, 8.03, 7.4, 7.7, 8.66, 9.17, 9.77, 10.3, 9.96, 9.44, 9.44, 9.0, 9.01, 8.6199, 8.56, 9.16, 9.57, 9.72, 8.5399, 8.76, 8.88, 8.66, 8.23, 8.27, 9.51, 10.61, 10.25, 11.38, 10.94, 11.95, 11.07, 12.02, 12.02, 11.25, 11.97, 12.41, 13.05, 13.0, 14.36, 13.85, 15.45, 15.34, 15.2}," );
      System.Console.WriteLine( "                         {1.92, 1.92, 1.92, 1.86, 1.65, 1.56, 1.73, 1.77, 1.76, 1.6299, 1.68, 1.66, 1.79, 1.89, 1.95, 2.0, 2.11, 2.25, 2.49, 2.71, 3.0, 2.54, 2.9, 2.88, 2.92, 3.07, 3.58, 3.52, 3.37, 3.52, 3.42, 3.54, 3.92, 3.89, 3.88, 3.91, 4.05, 4.01, 4.14, 4.39, 4.55, 4.68, 4.83, 5.2699, 5.09, 5.0, 5.25, 4.78, 4.85, 5.2699, 5.86, 6.17, 5.86, 5.79, 6.3099, 7.08, 6.86, 6.89, 6.64, 6.9, 6.87, 6.9}," );
      System.Console.WriteLine( "                         {1.01, 0.99, 0.97, 1.0, 0.91, 0.93, 0.98, 1.16, 1.19, 1.27, 1.22, 1.11, 1.17, 1.1299, 1.26, 1.12, 1.16, 1.37, 1.6299, 1.86, 1.71, 1.59, 1.59, 1.71, 2.06, 2.2599, 2.2599, 2.8, 2.97, 2.92, 2.82, 2.5299, 2.74, 2.64, 2.72, 2.61, 2.45, 2.21, 2.1, 2.4, 2.72, 3.2, 3.04, 3.39, 3.56, 3.2599, 3.62, 3.46, 3.32, 3.41, 3.88, 4.47, 4.85, 4.88, 5.07, 5.29, 5.64, 5.67, 6.01, 6.36, 6.1, 5.87}," );
      System.Console.WriteLine( "                         {1.26, 1.25, 1.22, 1.11, 1.21, 0.95, 0.91, 1.11, 1.03, 0.95, 0.79, 0.72, 0.65, 0.7, 0.64, 0.56, 0.67, 0.64, 0.55, 0.61, 0.61, 0.52, 0.73, 0.91, 1.15, 1.37, 1.68, 1.97, 1.95, 2.02, 2.18, 2.08, 2.1, 2.0099, 1.94, 1.94, 2.04, 1.95, 2.22, 2.27, 2.29, 2.31, 2.18, 2.34, 2.4, 2.54, 2.61, 2.7599, 2.7599, 3.29, 3.0, 3.05, 3.14, 3.24, 3.33, 3.65, 3.88, 4.34, 4.73, 4.54, 4.39, 4.57}," );
      System.Console.WriteLine( "                         {2.37, 2.34, 2.1, 2.2, 2.22, 2.34, 2.8, 2.81, 2.7799, 2.29, 2.38, 2.1, 2.05, 2.21, 2.25, 1.93, 1.96, 1.99, 1.59, 1.32, 1.3799, 0.93, 1.18, 1.4, 1.4, 1.6299, 1.6, 1.94, 1.68, 1.57, 1.5, 1.53, 1.7, 1.61, 1.79, 2.15, 2.09, 2.06, 1.95, 2.2799, 2.72, 2.64, 2.83, 3.37, 3.54, 3.73, 4.26, 4.21, 4.51, 5.57, 6.85, 7.0, 7.79, 8.43, 9.09, 9.01, 9.01, 9.13, 8.93, 9.17, 10.86, 10.67}," );
      System.Console.WriteLine( "                         {1.15, 1.19, 1.05, 1.06, 1.08, 0.96, 1.04, 1.08, 1.1399, 1.07, 1.1, 1.23, 1.15, 1.12, 1.1, 0.93, 0.89, 0.95, 0.94, 0.9, 0.93, 0.96, 0.94, 1.02, 1.02, 1.09, 0.98, 1.1, 1.04, 0.99, 0.92, 0.89, 0.96, 0.91, 0.96, 0.96, 0.96, 0.98, 0.99, 0.97, 0.94, 1.01, 1.21, 1.3799, 1.28, 1.35, 1.32, 1.3, 1.31, 1.24, 1.31, 1.44, 1.48, 1.55, 1.6, 1.83, 1.82, 1.86, 1.89, 2.31, 2.22, 2.25}," );
      System.Console.WriteLine( "                         {1.71, 1.56, 1.71, 2.32, 1.7, 1.75, 2.82, 2.21, 2.21, 2.29, 2.25, 2.07, 2.0299, 2.22, 1.9, 1.61, 2.07, 2.08, 2.11, 2.65, 2.69, 2.34, 3.11, 3.09, 2.89, 3.09, 3.24, 3.43, 3.0099, 2.9, 2.55, 2.2799, 2.32, 2.4, 2.13, 2.07, 2.11, 1.99, 1.91, 2.18, 2.34, 2.81, 3.0099, 2.89, 3.05, 3.4, 3.46, 3.67, 3.81, 3.72, 3.33, 3.12, 3.42, 3.19, 3.19, 3.29, 3.75, 3.98, 4.54, 4.48, 4.73, 4.75}," );
      System.Console.WriteLine( "                         {0.81, 0.78, 0.74, 0.73, 0.72, 0.68, 0.65, 0.72, 0.71, 0.64, 0.65, 0.64, 0.63, 0.6, 0.6, 0.58, 0.53, 0.59, 0.63, 0.73, 0.71, 0.72, 0.72, 0.79, 0.82, 0.87, 0.94, 1.03, 0.98, 0.93, 0.95, 0.91, 1.05, 1.04, 1.09, 1.28, 1.25, 1.3, 1.27, 1.3899, 1.6, 1.77, 2.73, 2.35, 2.33, 2.5, 2.42, 2.19, 2.54, 2.34, 2.72, 2.7, 2.73, 2.56, 2.7, 2.68, 2.85, 3.16, 3.68, 3.6, 3.87, 3.93}," );
      System.Console.WriteLine( "                         {1.6, 1.6299, 1.47, 1.28, 1.19, 1.15, 1.15, 1.15, 1.24, 1.24, 1.22, 1.34, 1.37, 1.35, 1.26, 1.18, 1.15, 1.15, 1.15, 1.2, 1.24, 1.33, 1.36, 1.46, 1.48, 1.58, 2.06, 1.97, 2.31, 2.33, 2.34, 2.25, 2.64, 2.57, 2.7, 2.7599, 2.84, 2.96, 3.17, 3.2, 3.47, 3.52, 3.99, 4.3, 4.47, 4.71, 5.42, 5.45, 6.0599, 5.72, 7.18, 7.22, 7.37, 7.3, 7.55, 8.16, 8.58, 8.24, 8.43, 8.68, 8.26, 8.27}," );
      System.Console.WriteLine( "                         {0.31, 0.29, 0.39, 0.35, 0.38, 0.44, 0.45, 0.43, 0.45, 0.52, 0.55, 0.52, 0.55, 0.59, 0.65, 0.79, 0.93, 0.92, 0.9, 0.84, 0.95, 0.79, 0.86, 0.93, 0.98, 1.07, 1.12, 1.37, 1.47, 1.7, 1.61, 1.52, 1.72, 1.87, 1.85, 1.96, 1.97, 2.13, 2.32, 2.21, 2.38, 2.54, 2.73, 2.86, 2.85, 3.04, 3.08, 2.82, 3.0, 2.72, 2.92, 2.82, 2.98, 3.23, 3.21, 3.32, 3.51, 3.54, 4.13, 4.2, 4.73, 4.71}," );
      System.Console.WriteLine( "                         {2.0099, 1.98, 1.94, 2.1, 1.52, 1.22, 1.1299, 1.17, 1.17, 0.98, 0.94, 0.88, 0.8, 0.74, 0.74, 0.54, 0.6899, 0.63, 0.41, 0.23, 0.28, 0.28, 0.44, 0.5, 0.54, 0.58, 0.56, 0.51, 0.42, 0.36, 0.5699, 0.55, 0.61, 0.54, 0.56, 0.5699, 0.61, 0.62, 0.62, 0.59, 0.68, 0.76, 0.92, 1.1399, 1.52, 1.55, 1.58, 1.47, 1.6299, 1.93, 2.37, 2.59, 2.52, 2.47, 2.82, 3.18, 3.08, 3.12, 3.38, 3.79, 4.21, 4.44}," );
      System.Console.WriteLine( "                         {1.21, 1.34, 1.17, 1.36, 1.34, 1.3899, 1.51, 1.56, 1.51, 1.36, 1.44, 1.42, 1.29, 1.47, 1.52, 1.49, 1.47, 1.43, 1.46, 1.54, 1.49, 1.3, 1.51, 1.6299, 1.72, 1.86, 2.05, 2.14, 2.49, 2.63, 3.04, 3.04, 3.34, 3.51, 3.58, 3.48, 3.53, 3.82, 3.78, 3.9, 3.86, 4.01, 4.5, 5.35, 5.23, 5.73, 6.1, 5.88, 6.57, 7.46, 7.55, 7.86, 7.95, 8.06, 8.73, 9.47, 9.86, 9.78, 10.84, 11.06, 9.3, 9.21}," );
      System.Console.WriteLine( "                         {0.66, 0.6899, 0.71, 0.66, 0.68, 0.68, 0.67, 0.73, 0.76, 0.74, 0.81, 0.85, 0.84, 0.87, 0.84, 0.7, 0.6899, 0.68, 0.71, 0.7, 0.73, 0.61, 0.6899, 0.8, 0.85, 1.12, 1.28, 1.61, 1.78, 1.73, 1.74, 1.69, 1.95, 1.73, 1.6299, 1.75, 1.77, 1.78, 1.61, 1.57, 1.64, 1.78, 1.94, 2.11, 2.25, 2.44, 2.44, 2.48, 2.5, 2.5299, 2.25, 2.15, 2.27, 2.21, 2.12, 2.31, 2.21, 2.05, 2.37, 2.68, 2.66, 2.67}," );
      System.Console.WriteLine( "                         {3.22, 3.21, 3.04, 2.88, 2.7799, 2.61, 2.69, 2.7799, 2.59, 2.56, 2.49, 2.5, 2.49, 2.5, 2.56, 2.48, 2.43, 2.71, 3.09, 3.04, 3.55, 3.48, 3.98, 3.89, 3.76, 3.87, 4.42, 4.49, 3.58, 4.3, 4.21, 3.9, 4.12, 4.25, 4.24, 4.38, 4.38, 4.4, 3.99, 4.0599, 3.82, 3.76, 3.59, 3.94, 4.19, 4.5199, 4.5599, 4.58, 4.64, 4.48, 4.57, 4.49, 4.63, 4.55, 4.55, 4.82, 4.94, 4.92, 4.96, 5.2699, 5.65, 5.65}," );
      System.Console.WriteLine( "                         {4.41, 4.32, 4.1, 4.14, 3.87, 3.66, 3.66, 4.1, 3.65, 3.21, 3.07, 2.66, 2.7599, 2.71, 2.57, 3.07, 3.58, 3.14, 3.1, 3.4, 3.12, 2.62, 2.73, 3.16, 2.95, 2.98, 3.22, 3.47, 3.42, 3.71, 3.44, 3.32, 3.58, 3.8, 3.59, 3.42, 3.61, 3.62, 3.51, 3.16, 3.55, 3.48, 3.5, 3.92, 4.8, 4.83, 4.86, 4.49, 4.11, 3.97, 4.0, 4.12, 4.49, 4.0, 3.96, 4.07, 3.55, 3.62, 3.68, 3.66, 3.17, 3.07}," );
      System.Console.WriteLine( "                         {0.56, 0.56, 0.6, 0.67, 0.5699, 0.58, 0.67, 0.71, 0.82, 0.72, 0.72, 0.8, 0.98, 1.11, 1.08, 0.98, 1.05, 1.09, 1.25, 1.16, 1.1, 0.92, 1.12, 1.11, 1.18, 1.36, 1.6, 1.97, 2.08, 1.75, 1.59, 1.26, 1.1, 1.1299, 1.12, 1.02, 0.87, 0.94, 0.83, 0.94, 0.95, 0.87, 0.99, 1.22, 1.18, 1.3799, 1.54, 1.43, 1.37, 1.28, 1.3799, 1.46, 1.5, 1.66, 1.72, 1.93, 1.93, 2.14, 2.43, 3.02, 2.81, 3.05}," );
      System.Console.WriteLine( "                         {2.1, 2.13, 2.11, 2.04, 1.84, 1.79, 1.75, 2.0099, 1.96, 1.89, 1.83, 1.68, 1.05, 1.12, 1.18, 1.1, 0.9, 0.83, 0.45, 0.47, 0.44, 0.55, 0.61, 0.75, 0.65, 0.58, 0.7, 0.8, 0.8, 0.93, 0.82, 0.79, 0.91, 1.08, 0.9, 0.44, 0.41, 0.43, 0.44, 0.46, 0.65, 0.72, 0.71, 0.96, 0.96, 0.91, 1.1299, 1.2, 1.4, 1.29, 1.3, 1.43, 1.65, 1.66, 1.64, 1.85, 1.69, 1.68, 1.86, 2.02, 2.12, 2.13}}" );
      // Here we evaluate the covariance matrix of the  historical  returns  of
      // the assets from which the portfolios can be constructed.
      double[][] covariance1 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {2.5, 2.6, 2.73, 2.8, 2.82, 2.9, 2.87, 3.05, 3.04, 3.0299, 3.05, 3.13, 3.18, 3.21, 3.25, 3.28, 3.2599, 3.21, 3.27, 3.38, 3.67, 3.62, 3.73, 3.72, 3.64, 3.53, 3.77, 3.76, 3.82, 4.0, 3.82, 3.68, 3.99, 4.12, 4.15, 4.4, 4.35, 4.48, 4.39, 4.48, 4.51, 4.8, 4.5199, 4.5599, 4.5599, 4.29, 4.53, 4.76, 4.87, 4.86, 5.18, 5.46, 4.99, 5.13, 5.0, 5.33, 5.61, 5.69, 5.89, 5.88, 5.99, 6.13},
                         new double[] {8.05, 7.54, 6.48, 7.18, 7.42, 7.17, 7.95, 7.34, 7.63, 7.71, 7.8, 8.13, 7.91, 7.87, 8.7899, 8.18, 8.98, 9.39, 8.96, 8.36, 8.03, 7.4, 7.7, 8.66, 9.17, 9.77, 10.3, 9.96, 9.44, 9.44, 9.0, 9.01, 8.6199, 8.56, 9.16, 9.57, 9.72, 8.5399, 8.76, 8.88, 8.66, 8.23, 8.27, 9.51, 10.61, 10.25, 11.38, 10.94, 11.95, 11.07, 12.02, 12.02, 11.25, 11.97, 12.41, 13.05, 13.0, 14.36, 13.85, 15.45, 15.34, 15.2},
                         new double[] {1.92, 1.92, 1.92, 1.86, 1.65, 1.56, 1.73, 1.77, 1.76, 1.6299, 1.68, 1.66, 1.79, 1.89, 1.95, 2.0, 2.11, 2.25, 2.49, 2.71, 3.0, 2.54, 2.9, 2.88, 2.92, 3.07, 3.58, 3.52, 3.37, 3.52, 3.42, 3.54, 3.92, 3.89, 3.88, 3.91, 4.05, 4.01, 4.14, 4.39, 4.55, 4.68, 4.83, 5.2699, 5.09, 5.0, 5.25, 4.78, 4.85, 5.2699, 5.86, 6.17, 5.86, 5.79, 6.3099, 7.08, 6.86, 6.89, 6.64, 6.9, 6.87, 6.9},
                         new double[] {1.01, 0.99, 0.97, 1.0, 0.91, 0.93, 0.98, 1.16, 1.19, 1.27, 1.22, 1.11, 1.17, 1.1299, 1.26, 1.12, 1.16, 1.37, 1.6299, 1.86, 1.71, 1.59, 1.59, 1.71, 2.06, 2.2599, 2.2599, 2.8, 2.97, 2.92, 2.82, 2.5299, 2.74, 2.64, 2.72, 2.61, 2.45, 2.21, 2.1, 2.4, 2.72, 3.2, 3.04, 3.39, 3.56, 3.2599, 3.62, 3.46, 3.32, 3.41, 3.88, 4.47, 4.85, 4.88, 5.07, 5.29, 5.64, 5.67, 6.01, 6.36, 6.1, 5.87},
                         new double[] {1.26, 1.25, 1.22, 1.11, 1.21, 0.95, 0.91, 1.11, 1.03, 0.95, 0.79, 0.72, 0.65, 0.7, 0.64, 0.56, 0.67, 0.64, 0.55, 0.61, 0.61, 0.52, 0.73, 0.91, 1.15, 1.37, 1.68, 1.97, 1.95, 2.02, 2.18, 2.08, 2.1, 2.0099, 1.94, 1.94, 2.04, 1.95, 2.22, 2.27, 2.29, 2.31, 2.18, 2.34, 2.4, 2.54, 2.61, 2.7599, 2.7599, 3.29, 3.0, 3.05, 3.14, 3.24, 3.33, 3.65, 3.88, 4.34, 4.73, 4.54, 4.39, 4.57},
                         new double[] {2.37, 2.34, 2.1, 2.2, 2.22, 2.34, 2.8, 2.81, 2.7799, 2.29, 2.38, 2.1, 2.05, 2.21, 2.25, 1.93, 1.96, 1.99, 1.59, 1.32, 1.3799, 0.93, 1.18, 1.4, 1.4, 1.6299, 1.6, 1.94, 1.68, 1.57, 1.5, 1.53, 1.7, 1.61, 1.79, 2.15, 2.09, 2.06, 1.95, 2.2799, 2.72, 2.64, 2.83, 3.37, 3.54, 3.73, 4.26, 4.21, 4.51, 5.57, 6.85, 7.0, 7.79, 8.43, 9.09, 9.01, 9.01, 9.13, 8.93, 9.17, 10.86, 10.67},
                         new double[] {1.15, 1.19, 1.05, 1.06, 1.08, 0.96, 1.04, 1.08, 1.1399, 1.07, 1.1, 1.23, 1.15, 1.12, 1.1, 0.93, 0.89, 0.95, 0.94, 0.9, 0.93, 0.96, 0.94, 1.02, 1.02, 1.09, 0.98, 1.1, 1.04, 0.99, 0.92, 0.89, 0.96, 0.91, 0.96, 0.96, 0.96, 0.98, 0.99, 0.97, 0.94, 1.01, 1.21, 1.3799, 1.28, 1.35, 1.32, 1.3, 1.31, 1.24, 1.31, 1.44, 1.48, 1.55, 1.6, 1.83, 1.82, 1.86, 1.89, 2.31, 2.22, 2.25},
                         new double[] {1.71, 1.56, 1.71, 2.32, 1.7, 1.75, 2.82, 2.21, 2.21, 2.29, 2.25, 2.07, 2.0299, 2.22, 1.9, 1.61, 2.07, 2.08, 2.11, 2.65, 2.69, 2.34, 3.11, 3.09, 2.89, 3.09, 3.24, 3.43, 3.0099, 2.9, 2.55, 2.2799, 2.32, 2.4, 2.13, 2.07, 2.11, 1.99, 1.91, 2.18, 2.34, 2.81, 3.0099, 2.89, 3.05, 3.4, 3.46, 3.67, 3.81, 3.72, 3.33, 3.12, 3.42, 3.19, 3.19, 3.29, 3.75, 3.98, 4.54, 4.48, 4.73, 4.75},
                         new double[] {0.81, 0.78, 0.74, 0.73, 0.72, 0.68, 0.65, 0.72, 0.71, 0.64, 0.65, 0.64, 0.63, 0.6, 0.6, 0.58, 0.53, 0.59, 0.63, 0.73, 0.71, 0.72, 0.72, 0.79, 0.82, 0.87, 0.94, 1.03, 0.98, 0.93, 0.95, 0.91, 1.05, 1.04, 1.09, 1.28, 1.25, 1.3, 1.27, 1.3899, 1.6, 1.77, 2.73, 2.35, 2.33, 2.5, 2.42, 2.19, 2.54, 2.34, 2.72, 2.7, 2.73, 2.56, 2.7, 2.68, 2.85, 3.16, 3.68, 3.6, 3.87, 3.93},
                         new double[] {1.6, 1.6299, 1.47, 1.28, 1.19, 1.15, 1.15, 1.15, 1.24, 1.24, 1.22, 1.34, 1.37, 1.35, 1.26, 1.18, 1.15, 1.15, 1.15, 1.2, 1.24, 1.33, 1.36, 1.46, 1.48, 1.58, 2.06, 1.97, 2.31, 2.33, 2.34, 2.25, 2.64, 2.57, 2.7, 2.7599, 2.84, 2.96, 3.17, 3.2, 3.47, 3.52, 3.99, 4.3, 4.47, 4.71, 5.42, 5.45, 6.0599, 5.72, 7.18, 7.22, 7.37, 7.3, 7.55, 8.16, 8.58, 8.24, 8.43, 8.68, 8.26, 8.27},
                         new double[] {0.31, 0.29, 0.39, 0.35, 0.38, 0.44, 0.45, 0.43, 0.45, 0.52, 0.55, 0.52, 0.55, 0.59, 0.65, 0.79, 0.93, 0.92, 0.9, 0.84, 0.95, 0.79, 0.86, 0.93, 0.98, 1.07, 1.12, 1.37, 1.47, 1.7, 1.61, 1.52, 1.72, 1.87, 1.85, 1.96, 1.97, 2.13, 2.32, 2.21, 2.38, 2.54, 2.73, 2.86, 2.85, 3.04, 3.08, 2.82, 3.0, 2.72, 2.92, 2.82, 2.98, 3.23, 3.21, 3.32, 3.51, 3.54, 4.13, 4.2, 4.73, 4.71},
                         new double[] {2.0099, 1.98, 1.94, 2.1, 1.52, 1.22, 1.1299, 1.17, 1.17, 0.98, 0.94, 0.88, 0.8, 0.74, 0.74, 0.54, 0.6899, 0.63, 0.41, 0.23, 0.28, 0.28, 0.44, 0.5, 0.54, 0.58, 0.56, 0.51, 0.42, 0.36, 0.5699, 0.55, 0.61, 0.54, 0.56, 0.5699, 0.61, 0.62, 0.62, 0.59, 0.68, 0.76, 0.92, 1.1399, 1.52, 1.55, 1.58, 1.47, 1.6299, 1.93, 2.37, 2.59, 2.52, 2.47, 2.82, 3.18, 3.08, 3.12, 3.38, 3.79, 4.21, 4.44},
                         new double[] {1.21, 1.34, 1.17, 1.36, 1.34, 1.3899, 1.51, 1.56, 1.51, 1.36, 1.44, 1.42, 1.29, 1.47, 1.52, 1.49, 1.47, 1.43, 1.46, 1.54, 1.49, 1.3, 1.51, 1.6299, 1.72, 1.86, 2.05, 2.14, 2.49, 2.63, 3.04, 3.04, 3.34, 3.51, 3.58, 3.48, 3.53, 3.82, 3.78, 3.9, 3.86, 4.01, 4.5, 5.35, 5.23, 5.73, 6.1, 5.88, 6.57, 7.46, 7.55, 7.86, 7.95, 8.06, 8.73, 9.47, 9.86, 9.78, 10.84, 11.06, 9.3, 9.21},
                         new double[] {0.66, 0.6899, 0.71, 0.66, 0.68, 0.68, 0.67, 0.73, 0.76, 0.74, 0.81, 0.85, 0.84, 0.87, 0.84, 0.7, 0.6899, 0.68, 0.71, 0.7, 0.73, 0.61, 0.6899, 0.8, 0.85, 1.12, 1.28, 1.61, 1.78, 1.73, 1.74, 1.69, 1.95, 1.73, 1.6299, 1.75, 1.77, 1.78, 1.61, 1.57, 1.64, 1.78, 1.94, 2.11, 2.25, 2.44, 2.44, 2.48, 2.5, 2.5299, 2.25, 2.15, 2.27, 2.21, 2.12, 2.31, 2.21, 2.05, 2.37, 2.68, 2.66, 2.67},
                         new double[] {3.22, 3.21, 3.04, 2.88, 2.7799, 2.61, 2.69, 2.7799, 2.59, 2.56, 2.49, 2.5, 2.49, 2.5, 2.56, 2.48, 2.43, 2.71, 3.09, 3.04, 3.55, 3.48, 3.98, 3.89, 3.76, 3.87, 4.42, 4.49, 3.58, 4.3, 4.21, 3.9, 4.12, 4.25, 4.24, 4.38, 4.38, 4.4, 3.99, 4.0599, 3.82, 3.76, 3.59, 3.94, 4.19, 4.5199, 4.5599, 4.58, 4.64, 4.48, 4.57, 4.49, 4.63, 4.55, 4.55, 4.82, 4.94, 4.92, 4.96, 5.2699, 5.65, 5.65},
                         new double[] {4.41, 4.32, 4.1, 4.14, 3.87, 3.66, 3.66, 4.1, 3.65, 3.21, 3.07, 2.66, 2.7599, 2.71, 2.57, 3.07, 3.58, 3.14, 3.1, 3.4, 3.12, 2.62, 2.73, 3.16, 2.95, 2.98, 3.22, 3.47, 3.42, 3.71, 3.44, 3.32, 3.58, 3.8, 3.59, 3.42, 3.61, 3.62, 3.51, 3.16, 3.55, 3.48, 3.5, 3.92, 4.8, 4.83, 4.86, 4.49, 4.11, 3.97, 4.0, 4.12, 4.49, 4.0, 3.96, 4.07, 3.55, 3.62, 3.68, 3.66, 3.17, 3.07},
                         new double[] {0.56, 0.56, 0.6, 0.67, 0.5699, 0.58, 0.67, 0.71, 0.82, 0.72, 0.72, 0.8, 0.98, 1.11, 1.08, 0.98, 1.05, 1.09, 1.25, 1.16, 1.1, 0.92, 1.12, 1.11, 1.18, 1.36, 1.6, 1.97, 2.08, 1.75, 1.59, 1.26, 1.1, 1.1299, 1.12, 1.02, 0.87, 0.94, 0.83, 0.94, 0.95, 0.87, 0.99, 1.22, 1.18, 1.3799, 1.54, 1.43, 1.37, 1.28, 1.3799, 1.46, 1.5, 1.66, 1.72, 1.93, 1.93, 2.14, 2.43, 3.02, 2.81, 3.05},
                         new double[] {2.1, 2.13, 2.11, 2.04, 1.84, 1.79, 1.75, 2.0099, 1.96, 1.89, 1.83, 1.68, 1.05, 1.12, 1.18, 1.1, 0.9, 0.83, 0.45, 0.47, 0.44, 0.55, 0.61, 0.75, 0.65, 0.58, 0.7, 0.8, 0.8, 0.93, 0.82, 0.79, 0.91, 1.08, 0.9, 0.44, 0.41, 0.43, 0.44, 0.46, 0.65, 0.72, 0.71, 0.96, 0.96, 0.91, 1.1299, 1.2, 1.4, 1.29, 1.3, 1.43, 1.65, 1.66, 1.64, 1.85, 1.69, 1.68, 1.86, 2.02, 2.12, 2.13}} // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The covariance matrix of the assets historical returns is:" );
      System.Console.WriteLine( " " );
      display2DArray( covariance1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ We next evaluate the expected returns of the asset being considered +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns( double[][] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here    we  evaluate  the  expected  returns  of  the  assets  under" );
      System.Console.WriteLine( "     consideration." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalReturns = {{2.5, 2.6, 2.73, 2.8, 2.82, 2.9, 2.87, 3.05, 3.04, 3.0299, 3.05, 3.13, 3.18, 3.21, 3.25, 3.28, 3.2599, 3.21, 3.27, 3.38, 3.67, 3.62, 3.73, 3.72, 3.64, 3.53, 3.77, 3.76, 3.82, 4.0, 3.82, 3.68, 3.99, 4.12, 4.15, 4.4, 4.35, 4.48, 4.39, 4.48, 4.51, 4.8, 4.5199, 4.5599, 4.5599, 4.29, 4.53, 4.76, 4.87, 4.86, 5.18, 5.46, 4.99, 5.13, 5.0, 5.33, 5.61, 5.69, 5.89, 5.88, 5.99, 6.13}," );
      System.Console.WriteLine( "                         {8.05, 7.54, 6.48, 7.18, 7.42, 7.17, 7.95, 7.34, 7.63, 7.71, 7.8, 8.13, 7.91, 7.87, 8.7899, 8.18, 8.98, 9.39, 8.96, 8.36, 8.03, 7.4, 7.7, 8.66, 9.17, 9.77, 10.3, 9.96, 9.44, 9.44, 9.0, 9.01, 8.6199, 8.56, 9.16, 9.57, 9.72, 8.5399, 8.76, 8.88, 8.66, 8.23, 8.27, 9.51, 10.61, 10.25, 11.38, 10.94, 11.95, 11.07, 12.02, 12.02, 11.25, 11.97, 12.41, 13.05, 13.0, 14.36, 13.85, 15.45, 15.34, 15.2}," );
      System.Console.WriteLine( "                         {1.92, 1.92, 1.92, 1.86, 1.65, 1.56, 1.73, 1.77, 1.76, 1.6299, 1.68, 1.66, 1.79, 1.89, 1.95, 2.0, 2.11, 2.25, 2.49, 2.71, 3.0, 2.54, 2.9, 2.88, 2.92, 3.07, 3.58, 3.52, 3.37, 3.52, 3.42, 3.54, 3.92, 3.89, 3.88, 3.91, 4.05, 4.01, 4.14, 4.39, 4.55, 4.68, 4.83, 5.2699, 5.09, 5.0, 5.25, 4.78, 4.85, 5.2699, 5.86, 6.17, 5.86, 5.79, 6.3099, 7.08, 6.86, 6.89, 6.64, 6.9, 6.87, 6.9}," );
      System.Console.WriteLine( "                         {1.01, 0.99, 0.97, 1.0, 0.91, 0.93, 0.98, 1.16, 1.19, 1.27, 1.22, 1.11, 1.17, 1.1299, 1.26, 1.12, 1.16, 1.37, 1.6299, 1.86, 1.71, 1.59, 1.59, 1.71, 2.06, 2.2599, 2.2599, 2.8, 2.97, 2.92, 2.82, 2.5299, 2.74, 2.64, 2.72, 2.61, 2.45, 2.21, 2.1, 2.4, 2.72, 3.2, 3.04, 3.39, 3.56, 3.2599, 3.62, 3.46, 3.32, 3.41, 3.88, 4.47, 4.85, 4.88, 5.07, 5.29, 5.64, 5.67, 6.01, 6.36, 6.1, 5.87}," );
      System.Console.WriteLine( "                         {1.26, 1.25, 1.22, 1.11, 1.21, 0.95, 0.91, 1.11, 1.03, 0.95, 0.79, 0.72, 0.65, 0.7, 0.64, 0.56, 0.67, 0.64, 0.55, 0.61, 0.61, 0.52, 0.73, 0.91, 1.15, 1.37, 1.68, 1.97, 1.95, 2.02, 2.18, 2.08, 2.1, 2.0099, 1.94, 1.94, 2.04, 1.95, 2.22, 2.27, 2.29, 2.31, 2.18, 2.34, 2.4, 2.54, 2.61, 2.7599, 2.7599, 3.29, 3.0, 3.05, 3.14, 3.24, 3.33, 3.65, 3.88, 4.34, 4.73, 4.54, 4.39, 4.57}," );
      System.Console.WriteLine( "                         {2.37, 2.34, 2.1, 2.2, 2.22, 2.34, 2.8, 2.81, 2.7799, 2.29, 2.38, 2.1, 2.05, 2.21, 2.25, 1.93, 1.96, 1.99, 1.59, 1.32, 1.3799, 0.93, 1.18, 1.4, 1.4, 1.6299, 1.6, 1.94, 1.68, 1.57, 1.5, 1.53, 1.7, 1.61, 1.79, 2.15, 2.09, 2.06, 1.95, 2.2799, 2.72, 2.64, 2.83, 3.37, 3.54, 3.73, 4.26, 4.21, 4.51, 5.57, 6.85, 7.0, 7.79, 8.43, 9.09, 9.01, 9.01, 9.13, 8.93, 9.17, 10.86, 10.67}," );
      System.Console.WriteLine( "                         {1.15, 1.19, 1.05, 1.06, 1.08, 0.96, 1.04, 1.08, 1.1399, 1.07, 1.1, 1.23, 1.15, 1.12, 1.1, 0.93, 0.89, 0.95, 0.94, 0.9, 0.93, 0.96, 0.94, 1.02, 1.02, 1.09, 0.98, 1.1, 1.04, 0.99, 0.92, 0.89, 0.96, 0.91, 0.96, 0.96, 0.96, 0.98, 0.99, 0.97, 0.94, 1.01, 1.21, 1.3799, 1.28, 1.35, 1.32, 1.3, 1.31, 1.24, 1.31, 1.44, 1.48, 1.55, 1.6, 1.83, 1.82, 1.86, 1.89, 2.31, 2.22, 2.25}," );
      System.Console.WriteLine( "                         {1.71, 1.56, 1.71, 2.32, 1.7, 1.75, 2.82, 2.21, 2.21, 2.29, 2.25, 2.07, 2.0299, 2.22, 1.9, 1.61, 2.07, 2.08, 2.11, 2.65, 2.69, 2.34, 3.11, 3.09, 2.89, 3.09, 3.24, 3.43, 3.0099, 2.9, 2.55, 2.2799, 2.32, 2.4, 2.13, 2.07, 2.11, 1.99, 1.91, 2.18, 2.34, 2.81, 3.0099, 2.89, 3.05, 3.4, 3.46, 3.67, 3.81, 3.72, 3.33, 3.12, 3.42, 3.19, 3.19, 3.29, 3.75, 3.98, 4.54, 4.48, 4.73, 4.75}," );
      System.Console.WriteLine( "                         {0.81, 0.78, 0.74, 0.73, 0.72, 0.68, 0.65, 0.72, 0.71, 0.64, 0.65, 0.64, 0.63, 0.6, 0.6, 0.58, 0.53, 0.59, 0.63, 0.73, 0.71, 0.72, 0.72, 0.79, 0.82, 0.87, 0.94, 1.03, 0.98, 0.93, 0.95, 0.91, 1.05, 1.04, 1.09, 1.28, 1.25, 1.3, 1.27, 1.3899, 1.6, 1.77, 2.73, 2.35, 2.33, 2.5, 2.42, 2.19, 2.54, 2.34, 2.72, 2.7, 2.73, 2.56, 2.7, 2.68, 2.85, 3.16, 3.68, 3.6, 3.87, 3.93}," );
      System.Console.WriteLine( "                         {1.6, 1.6299, 1.47, 1.28, 1.19, 1.15, 1.15, 1.15, 1.24, 1.24, 1.22, 1.34, 1.37, 1.35, 1.26, 1.18, 1.15, 1.15, 1.15, 1.2, 1.24, 1.33, 1.36, 1.46, 1.48, 1.58, 2.06, 1.97, 2.31, 2.33, 2.34, 2.25, 2.64, 2.57, 2.7, 2.7599, 2.84, 2.96, 3.17, 3.2, 3.47, 3.52, 3.99, 4.3, 4.47, 4.71, 5.42, 5.45, 6.0599, 5.72, 7.18, 7.22, 7.37, 7.3, 7.55, 8.16, 8.58, 8.24, 8.43, 8.68, 8.26, 8.27}," );
      System.Console.WriteLine( "                         {0.31, 0.29, 0.39, 0.35, 0.38, 0.44, 0.45, 0.43, 0.45, 0.52, 0.55, 0.52, 0.55, 0.59, 0.65, 0.79, 0.93, 0.92, 0.9, 0.84, 0.95, 0.79, 0.86, 0.93, 0.98, 1.07, 1.12, 1.37, 1.47, 1.7, 1.61, 1.52, 1.72, 1.87, 1.85, 1.96, 1.97, 2.13, 2.32, 2.21, 2.38, 2.54, 2.73, 2.86, 2.85, 3.04, 3.08, 2.82, 3.0, 2.72, 2.92, 2.82, 2.98, 3.23, 3.21, 3.32, 3.51, 3.54, 4.13, 4.2, 4.73, 4.71}," );
      System.Console.WriteLine( "                         {2.0099, 1.98, 1.94, 2.1, 1.52, 1.22, 1.1299, 1.17, 1.17, 0.98, 0.94, 0.88, 0.8, 0.74, 0.74, 0.54, 0.6899, 0.63, 0.41, 0.23, 0.28, 0.28, 0.44, 0.5, 0.54, 0.58, 0.56, 0.51, 0.42, 0.36, 0.5699, 0.55, 0.61, 0.54, 0.56, 0.5699, 0.61, 0.62, 0.62, 0.59, 0.68, 0.76, 0.92, 1.1399, 1.52, 1.55, 1.58, 1.47, 1.6299, 1.93, 2.37, 2.59, 2.52, 2.47, 2.82, 3.18, 3.08, 3.12, 3.38, 3.79, 4.21, 4.44}," );
      System.Console.WriteLine( "                         {1.21, 1.34, 1.17, 1.36, 1.34, 1.3899, 1.51, 1.56, 1.51, 1.36, 1.44, 1.42, 1.29, 1.47, 1.52, 1.49, 1.47, 1.43, 1.46, 1.54, 1.49, 1.3, 1.51, 1.6299, 1.72, 1.86, 2.05, 2.14, 2.49, 2.63, 3.04, 3.04, 3.34, 3.51, 3.58, 3.48, 3.53, 3.82, 3.78, 3.9, 3.86, 4.01, 4.5, 5.35, 5.23, 5.73, 6.1, 5.88, 6.57, 7.46, 7.55, 7.86, 7.95, 8.06, 8.73, 9.47, 9.86, 9.78, 10.84, 11.06, 9.3, 9.21}," );
      System.Console.WriteLine( "                         {0.66, 0.6899, 0.71, 0.66, 0.68, 0.68, 0.67, 0.73, 0.76, 0.74, 0.81, 0.85, 0.84, 0.87, 0.84, 0.7, 0.6899, 0.68, 0.71, 0.7, 0.73, 0.61, 0.6899, 0.8, 0.85, 1.12, 1.28, 1.61, 1.78, 1.73, 1.74, 1.69, 1.95, 1.73, 1.6299, 1.75, 1.77, 1.78, 1.61, 1.57, 1.64, 1.78, 1.94, 2.11, 2.25, 2.44, 2.44, 2.48, 2.5, 2.5299, 2.25, 2.15, 2.27, 2.21, 2.12, 2.31, 2.21, 2.05, 2.37, 2.68, 2.66, 2.67}," );
      System.Console.WriteLine( "                         {3.22, 3.21, 3.04, 2.88, 2.7799, 2.61, 2.69, 2.7799, 2.59, 2.56, 2.49, 2.5, 2.49, 2.5, 2.56, 2.48, 2.43, 2.71, 3.09, 3.04, 3.55, 3.48, 3.98, 3.89, 3.76, 3.87, 4.42, 4.49, 3.58, 4.3, 4.21, 3.9, 4.12, 4.25, 4.24, 4.38, 4.38, 4.4, 3.99, 4.0599, 3.82, 3.76, 3.59, 3.94, 4.19, 4.5199, 4.5599, 4.58, 4.64, 4.48, 4.57, 4.49, 4.63, 4.55, 4.55, 4.82, 4.94, 4.92, 4.96, 5.2699, 5.65, 5.65}," );
      System.Console.WriteLine( "                         {4.41, 4.32, 4.1, 4.14, 3.87, 3.66, 3.66, 4.1, 3.65, 3.21, 3.07, 2.66, 2.7599, 2.71, 2.57, 3.07, 3.58, 3.14, 3.1, 3.4, 3.12, 2.62, 2.73, 3.16, 2.95, 2.98, 3.22, 3.47, 3.42, 3.71, 3.44, 3.32, 3.58, 3.8, 3.59, 3.42, 3.61, 3.62, 3.51, 3.16, 3.55, 3.48, 3.5, 3.92, 4.8, 4.83, 4.86, 4.49, 4.11, 3.97, 4.0, 4.12, 4.49, 4.0, 3.96, 4.07, 3.55, 3.62, 3.68, 3.66, 3.17, 3.07}," );
      System.Console.WriteLine( "                         {0.56, 0.56, 0.6, 0.67, 0.5699, 0.58, 0.67, 0.71, 0.82, 0.72, 0.72, 0.8, 0.98, 1.11, 1.08, 0.98, 1.05, 1.09, 1.25, 1.16, 1.1, 0.92, 1.12, 1.11, 1.18, 1.36, 1.6, 1.97, 2.08, 1.75, 1.59, 1.26, 1.1, 1.1299, 1.12, 1.02, 0.87, 0.94, 0.83, 0.94, 0.95, 0.87, 0.99, 1.22, 1.18, 1.3799, 1.54, 1.43, 1.37, 1.28, 1.3799, 1.46, 1.5, 1.66, 1.72, 1.93, 1.93, 2.14, 2.43, 3.02, 2.81, 3.05}," );
      System.Console.WriteLine( "                         {2.1, 2.13, 2.11, 2.04, 1.84, 1.79, 1.75, 2.0099, 1.96, 1.89, 1.83, 1.68, 1.05, 1.12, 1.18, 1.1, 0.9, 0.83, 0.45, 0.47, 0.44, 0.55, 0.61, 0.75, 0.65, 0.58, 0.7, 0.8, 0.8, 0.93, 0.82, 0.79, 0.91, 1.08, 0.9, 0.44, 0.41, 0.43, 0.44, 0.46, 0.65, 0.72, 0.71, 0.96, 0.96, 0.91, 1.1299, 1.2, 1.4, 1.29, 1.3, 1.43, 1.65, 1.66, 1.64, 1.85, 1.69, 1.68, 1.86, 2.02, 2.12, 2.13}}" );
      // Here       we  evaluate  the  expcted  returns  of  the  assets  under
      // consideration.
      double[] expectedReturns1 = instanceOfAssetParameters.ExpectedReturns( 
         new double[][] {new double[] {2.5, 2.6, 2.73, 2.8, 2.82, 2.9, 2.87, 3.05, 3.04, 3.0299, 3.05, 3.13, 3.18, 3.21, 3.25, 3.28, 3.2599, 3.21, 3.27, 3.38, 3.67, 3.62, 3.73, 3.72, 3.64, 3.53, 3.77, 3.76, 3.82, 4.0, 3.82, 3.68, 3.99, 4.12, 4.15, 4.4, 4.35, 4.48, 4.39, 4.48, 4.51, 4.8, 4.5199, 4.5599, 4.5599, 4.29, 4.53, 4.76, 4.87, 4.86, 5.18, 5.46, 4.99, 5.13, 5.0, 5.33, 5.61, 5.69, 5.89, 5.88, 5.99, 6.13},
                         new double[] {8.05, 7.54, 6.48, 7.18, 7.42, 7.17, 7.95, 7.34, 7.63, 7.71, 7.8, 8.13, 7.91, 7.87, 8.7899, 8.18, 8.98, 9.39, 8.96, 8.36, 8.03, 7.4, 7.7, 8.66, 9.17, 9.77, 10.3, 9.96, 9.44, 9.44, 9.0, 9.01, 8.6199, 8.56, 9.16, 9.57, 9.72, 8.5399, 8.76, 8.88, 8.66, 8.23, 8.27, 9.51, 10.61, 10.25, 11.38, 10.94, 11.95, 11.07, 12.02, 12.02, 11.25, 11.97, 12.41, 13.05, 13.0, 14.36, 13.85, 15.45, 15.34, 15.2},
                         new double[] {1.92, 1.92, 1.92, 1.86, 1.65, 1.56, 1.73, 1.77, 1.76, 1.6299, 1.68, 1.66, 1.79, 1.89, 1.95, 2.0, 2.11, 2.25, 2.49, 2.71, 3.0, 2.54, 2.9, 2.88, 2.92, 3.07, 3.58, 3.52, 3.37, 3.52, 3.42, 3.54, 3.92, 3.89, 3.88, 3.91, 4.05, 4.01, 4.14, 4.39, 4.55, 4.68, 4.83, 5.2699, 5.09, 5.0, 5.25, 4.78, 4.85, 5.2699, 5.86, 6.17, 5.86, 5.79, 6.3099, 7.08, 6.86, 6.89, 6.64, 6.9, 6.87, 6.9},
                         new double[] {1.01, 0.99, 0.97, 1.0, 0.91, 0.93, 0.98, 1.16, 1.19, 1.27, 1.22, 1.11, 1.17, 1.1299, 1.26, 1.12, 1.16, 1.37, 1.6299, 1.86, 1.71, 1.59, 1.59, 1.71, 2.06, 2.2599, 2.2599, 2.8, 2.97, 2.92, 2.82, 2.5299, 2.74, 2.64, 2.72, 2.61, 2.45, 2.21, 2.1, 2.4, 2.72, 3.2, 3.04, 3.39, 3.56, 3.2599, 3.62, 3.46, 3.32, 3.41, 3.88, 4.47, 4.85, 4.88, 5.07, 5.29, 5.64, 5.67, 6.01, 6.36, 6.1, 5.87},
                         new double[] {1.26, 1.25, 1.22, 1.11, 1.21, 0.95, 0.91, 1.11, 1.03, 0.95, 0.79, 0.72, 0.65, 0.7, 0.64, 0.56, 0.67, 0.64, 0.55, 0.61, 0.61, 0.52, 0.73, 0.91, 1.15, 1.37, 1.68, 1.97, 1.95, 2.02, 2.18, 2.08, 2.1, 2.0099, 1.94, 1.94, 2.04, 1.95, 2.22, 2.27, 2.29, 2.31, 2.18, 2.34, 2.4, 2.54, 2.61, 2.7599, 2.7599, 3.29, 3.0, 3.05, 3.14, 3.24, 3.33, 3.65, 3.88, 4.34, 4.73, 4.54, 4.39, 4.57},
                         new double[] {2.37, 2.34, 2.1, 2.2, 2.22, 2.34, 2.8, 2.81, 2.7799, 2.29, 2.38, 2.1, 2.05, 2.21, 2.25, 1.93, 1.96, 1.99, 1.59, 1.32, 1.3799, 0.93, 1.18, 1.4, 1.4, 1.6299, 1.6, 1.94, 1.68, 1.57, 1.5, 1.53, 1.7, 1.61, 1.79, 2.15, 2.09, 2.06, 1.95, 2.2799, 2.72, 2.64, 2.83, 3.37, 3.54, 3.73, 4.26, 4.21, 4.51, 5.57, 6.85, 7.0, 7.79, 8.43, 9.09, 9.01, 9.01, 9.13, 8.93, 9.17, 10.86, 10.67},
                         new double[] {1.15, 1.19, 1.05, 1.06, 1.08, 0.96, 1.04, 1.08, 1.1399, 1.07, 1.1, 1.23, 1.15, 1.12, 1.1, 0.93, 0.89, 0.95, 0.94, 0.9, 0.93, 0.96, 0.94, 1.02, 1.02, 1.09, 0.98, 1.1, 1.04, 0.99, 0.92, 0.89, 0.96, 0.91, 0.96, 0.96, 0.96, 0.98, 0.99, 0.97, 0.94, 1.01, 1.21, 1.3799, 1.28, 1.35, 1.32, 1.3, 1.31, 1.24, 1.31, 1.44, 1.48, 1.55, 1.6, 1.83, 1.82, 1.86, 1.89, 2.31, 2.22, 2.25},
                         new double[] {1.71, 1.56, 1.71, 2.32, 1.7, 1.75, 2.82, 2.21, 2.21, 2.29, 2.25, 2.07, 2.0299, 2.22, 1.9, 1.61, 2.07, 2.08, 2.11, 2.65, 2.69, 2.34, 3.11, 3.09, 2.89, 3.09, 3.24, 3.43, 3.0099, 2.9, 2.55, 2.2799, 2.32, 2.4, 2.13, 2.07, 2.11, 1.99, 1.91, 2.18, 2.34, 2.81, 3.0099, 2.89, 3.05, 3.4, 3.46, 3.67, 3.81, 3.72, 3.33, 3.12, 3.42, 3.19, 3.19, 3.29, 3.75, 3.98, 4.54, 4.48, 4.73, 4.75},
                         new double[] {0.81, 0.78, 0.74, 0.73, 0.72, 0.68, 0.65, 0.72, 0.71, 0.64, 0.65, 0.64, 0.63, 0.6, 0.6, 0.58, 0.53, 0.59, 0.63, 0.73, 0.71, 0.72, 0.72, 0.79, 0.82, 0.87, 0.94, 1.03, 0.98, 0.93, 0.95, 0.91, 1.05, 1.04, 1.09, 1.28, 1.25, 1.3, 1.27, 1.3899, 1.6, 1.77, 2.73, 2.35, 2.33, 2.5, 2.42, 2.19, 2.54, 2.34, 2.72, 2.7, 2.73, 2.56, 2.7, 2.68, 2.85, 3.16, 3.68, 3.6, 3.87, 3.93},
                         new double[] {1.6, 1.6299, 1.47, 1.28, 1.19, 1.15, 1.15, 1.15, 1.24, 1.24, 1.22, 1.34, 1.37, 1.35, 1.26, 1.18, 1.15, 1.15, 1.15, 1.2, 1.24, 1.33, 1.36, 1.46, 1.48, 1.58, 2.06, 1.97, 2.31, 2.33, 2.34, 2.25, 2.64, 2.57, 2.7, 2.7599, 2.84, 2.96, 3.17, 3.2, 3.47, 3.52, 3.99, 4.3, 4.47, 4.71, 5.42, 5.45, 6.0599, 5.72, 7.18, 7.22, 7.37, 7.3, 7.55, 8.16, 8.58, 8.24, 8.43, 8.68, 8.26, 8.27},
                         new double[] {0.31, 0.29, 0.39, 0.35, 0.38, 0.44, 0.45, 0.43, 0.45, 0.52, 0.55, 0.52, 0.55, 0.59, 0.65, 0.79, 0.93, 0.92, 0.9, 0.84, 0.95, 0.79, 0.86, 0.93, 0.98, 1.07, 1.12, 1.37, 1.47, 1.7, 1.61, 1.52, 1.72, 1.87, 1.85, 1.96, 1.97, 2.13, 2.32, 2.21, 2.38, 2.54, 2.73, 2.86, 2.85, 3.04, 3.08, 2.82, 3.0, 2.72, 2.92, 2.82, 2.98, 3.23, 3.21, 3.32, 3.51, 3.54, 4.13, 4.2, 4.73, 4.71},
                         new double[] {2.0099, 1.98, 1.94, 2.1, 1.52, 1.22, 1.1299, 1.17, 1.17, 0.98, 0.94, 0.88, 0.8, 0.74, 0.74, 0.54, 0.6899, 0.63, 0.41, 0.23, 0.28, 0.28, 0.44, 0.5, 0.54, 0.58, 0.56, 0.51, 0.42, 0.36, 0.5699, 0.55, 0.61, 0.54, 0.56, 0.5699, 0.61, 0.62, 0.62, 0.59, 0.68, 0.76, 0.92, 1.1399, 1.52, 1.55, 1.58, 1.47, 1.6299, 1.93, 2.37, 2.59, 2.52, 2.47, 2.82, 3.18, 3.08, 3.12, 3.38, 3.79, 4.21, 4.44},
                         new double[] {1.21, 1.34, 1.17, 1.36, 1.34, 1.3899, 1.51, 1.56, 1.51, 1.36, 1.44, 1.42, 1.29, 1.47, 1.52, 1.49, 1.47, 1.43, 1.46, 1.54, 1.49, 1.3, 1.51, 1.6299, 1.72, 1.86, 2.05, 2.14, 2.49, 2.63, 3.04, 3.04, 3.34, 3.51, 3.58, 3.48, 3.53, 3.82, 3.78, 3.9, 3.86, 4.01, 4.5, 5.35, 5.23, 5.73, 6.1, 5.88, 6.57, 7.46, 7.55, 7.86, 7.95, 8.06, 8.73, 9.47, 9.86, 9.78, 10.84, 11.06, 9.3, 9.21},
                         new double[] {0.66, 0.6899, 0.71, 0.66, 0.68, 0.68, 0.67, 0.73, 0.76, 0.74, 0.81, 0.85, 0.84, 0.87, 0.84, 0.7, 0.6899, 0.68, 0.71, 0.7, 0.73, 0.61, 0.6899, 0.8, 0.85, 1.12, 1.28, 1.61, 1.78, 1.73, 1.74, 1.69, 1.95, 1.73, 1.6299, 1.75, 1.77, 1.78, 1.61, 1.57, 1.64, 1.78, 1.94, 2.11, 2.25, 2.44, 2.44, 2.48, 2.5, 2.5299, 2.25, 2.15, 2.27, 2.21, 2.12, 2.31, 2.21, 2.05, 2.37, 2.68, 2.66, 2.67},
                         new double[] {3.22, 3.21, 3.04, 2.88, 2.7799, 2.61, 2.69, 2.7799, 2.59, 2.56, 2.49, 2.5, 2.49, 2.5, 2.56, 2.48, 2.43, 2.71, 3.09, 3.04, 3.55, 3.48, 3.98, 3.89, 3.76, 3.87, 4.42, 4.49, 3.58, 4.3, 4.21, 3.9, 4.12, 4.25, 4.24, 4.38, 4.38, 4.4, 3.99, 4.0599, 3.82, 3.76, 3.59, 3.94, 4.19, 4.5199, 4.5599, 4.58, 4.64, 4.48, 4.57, 4.49, 4.63, 4.55, 4.55, 4.82, 4.94, 4.92, 4.96, 5.2699, 5.65, 5.65},
                         new double[] {4.41, 4.32, 4.1, 4.14, 3.87, 3.66, 3.66, 4.1, 3.65, 3.21, 3.07, 2.66, 2.7599, 2.71, 2.57, 3.07, 3.58, 3.14, 3.1, 3.4, 3.12, 2.62, 2.73, 3.16, 2.95, 2.98, 3.22, 3.47, 3.42, 3.71, 3.44, 3.32, 3.58, 3.8, 3.59, 3.42, 3.61, 3.62, 3.51, 3.16, 3.55, 3.48, 3.5, 3.92, 4.8, 4.83, 4.86, 4.49, 4.11, 3.97, 4.0, 4.12, 4.49, 4.0, 3.96, 4.07, 3.55, 3.62, 3.68, 3.66, 3.17, 3.07},
                         new double[] {0.56, 0.56, 0.6, 0.67, 0.5699, 0.58, 0.67, 0.71, 0.82, 0.72, 0.72, 0.8, 0.98, 1.11, 1.08, 0.98, 1.05, 1.09, 1.25, 1.16, 1.1, 0.92, 1.12, 1.11, 1.18, 1.36, 1.6, 1.97, 2.08, 1.75, 1.59, 1.26, 1.1, 1.1299, 1.12, 1.02, 0.87, 0.94, 0.83, 0.94, 0.95, 0.87, 0.99, 1.22, 1.18, 1.3799, 1.54, 1.43, 1.37, 1.28, 1.3799, 1.46, 1.5, 1.66, 1.72, 1.93, 1.93, 2.14, 2.43, 3.02, 2.81, 3.05},
                         new double[] {2.1, 2.13, 2.11, 2.04, 1.84, 1.79, 1.75, 2.0099, 1.96, 1.89, 1.83, 1.68, 1.05, 1.12, 1.18, 1.1, 0.9, 0.83, 0.45, 0.47, 0.44, 0.55, 0.61, 0.75, 0.65, 0.58, 0.7, 0.8, 0.8, 0.93, 0.82, 0.79, 0.91, 1.08, 0.9, 0.44, 0.41, 0.43, 0.44, 0.46, 0.65, 0.72, 0.71, 0.96, 0.96, 0.91, 1.1299, 1.2, 1.4, 1.29, 1.3, 1.43, 1.65, 1.66, 1.64, 1.85, 1.69, 1.68, 1.86, 2.02, 2.12, 2.13}} // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The expected return of the assets is:" );
      System.Console.WriteLine( " " );
      displayArray( expectedReturns1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Set the Constraints and get the lower constraints on  the  Five  Asset +" );
      System.Console.WriteLine( "+ Weights                                                                +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getLowerConstraints(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we set of the constraints on the weights  on  the  five  assets" );
      System.Console.WriteLine( "     from which the portfolios can be constructed  and  printout  to  the" );
      System.Console.WriteLine( "     console the lower constraints. Note that the market  portfolio  will" );
      System.Console.WriteLine( "     also need to satisfy these constraints." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Here we set the asset weight constraints on the five assets
      instanceOfCapitalMarket.SetConstraints( 
         new double[] {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}, // lowerBounds
         new double[] {0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15} // upperBounds
      );
      // Here we read the lower constraints.
      double[] lowerConstraints1 = instanceOfCapitalMarket.GetLowerConstraints( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The lower constraints on the five assets are:" );
      System.Console.WriteLine( " " );
      displayArray( lowerConstraints1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Set the Constraints and get the upper constraints on  the  Five  Asset +" );
      System.Console.WriteLine( "+ Weights                                                                +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getUpperConstraints(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we set of the constraints on the weights  on  the  five  assets" );
      System.Console.WriteLine( "     from which the portfolios can be constructed  and  printout  to  the" );
      System.Console.WriteLine( "     console the lower constraints. Note that the market  portfolio  will" );
      System.Console.WriteLine( "     also need to satisfy these constraints." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Here we set the asset weight constraints on the five assets
      instanceOfCapitalMarket.SetConstraints( 
         new double[] {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}, // lowerBounds
         new double[] {0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15} // upperBounds
      );
      // Here we read the upper constraints.
      double[] upperConstraints = instanceOfCapitalMarket.GetUpperConstraints( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The upper constraints on the five assets are:" );
      System.Console.WriteLine( " " );
      displayArray( upperConstraints );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Minimum  value  of  the  expected  return  over  which  the  Efficient +" );
      System.Console.WriteLine( "+ Frontier exits                                                         +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minFrontierReturn( double[] expectedReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the minimum  expected  return  of  the  range  over" );
      System.Console.WriteLine( "     which the Efficient Frontier  exits.  As  a  general  remark  it  is" );
      System.Console.WriteLine( "     usually best to evaluate the range of the  expected  return  of  the" );
      System.Console.WriteLine( "     Efficient Frontier in order  to  provide  a  range  over  which  the" );
      System.Console.WriteLine( "     Efficient Frontier can be evaluated and the  range  over  which  the" );
      System.Console.WriteLine( "     optimal portfolio can be sort." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      double min1 = instanceOfCapitalMarket.MinFrontierReturn( 
         expectedReturns1 // expectedReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The minimum  bound  of  the  range  of  the  expected  return  of  the" );
      System.Console.WriteLine( "    Efficient Frontier is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( min1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Maximum  value  of  the  expected  return  over  which  the  Efficient +" );
      System.Console.WriteLine( "+ Frontier exits                                                         +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    maxFrontierReturn( double[] expectedReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the maximum  expected  return  of  the  range  over" );
      System.Console.WriteLine( "     which the Efficient Frontier  exits.  As  a  general  remark  it  is" );
      System.Console.WriteLine( "     usually best to evaluate the range of the  expected  return  of  the" );
      System.Console.WriteLine( "     Efficient Frontier in order  to  provide  a  range  over  which  the" );
      System.Console.WriteLine( "     Efficient Frontier can be evaluated and the  range  over  which  the" );
      System.Console.WriteLine( "     optimal portfolio can be sort." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      double max1 = instanceOfCapitalMarket.MaxFrontierReturn( 
         expectedReturns1 // expectedReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The maximum  bound  of  the  range  of  the  expected  return  of  the" );
      System.Console.WriteLine( "    Efficient Frontier is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( max1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we construct the Efficient Frontier +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    calculateEfficientFrontier( double minimumExpectedReturn," );
      System.Console.WriteLine( "                                double maximumExpectedReturn," );
      System.Console.WriteLine( "                                double[][] covarianceMatrix," );
      System.Console.WriteLine( "                                double[] expectedReturns," );
      System.Console.WriteLine( "                                int numberInterpolationPoints," );
      System.Console.WriteLine( "                                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Construct the Efficient Frontier and  set  the  results  to  private" );
      System.Console.WriteLine( "     fields. Note that we use 100 interpolation points equal  distributed" );
      System.Console.WriteLine( "     over the entire range over which the Efficient Frontier exits." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = min1" );
      System.Console.WriteLine( "    maximumExpectedReturn = max1" );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    numberInterpolationPoints = 100" );
      System.Console.WriteLine( "    precision = 0.000001" );
      // Here we construct the efficient frontier.
      instanceOfCapitalMarket.CalculateEfficientFrontier( 
         min1, // minimumExpectedReturn
         max1, // maximumExpectedReturn
         covariance1, // covarianceMatrix
         expectedReturns1, // expectedReturns
         100, // numberInterpolationPoints
         0.000001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The Efficient Frontier has now been evaluate of persisted  by  setting" );
      System.Console.WriteLine( "    various private fields which record to the points about  which  it  is" );
      System.Console.WriteLine( "    interpolated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Values of  the  Expected  Return  at  the  evaluation  points  of  the +" );
      System.Console.WriteLine( "+ Efficient Frontier                                                     +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we get the values of the Efficient Frontier for the  points  at" );
      System.Console.WriteLine( "     which the Efficient Frontier has been evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Here we get the values of the Expected returns at the points on  which
      // Efficient Frontier has been evaluated.
      double[] returnEF1 = instanceOfCapitalMarket.GetEfficientFrontierExpectedReturns( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the expected return for  which  the  Efficient  Frontier" );
      System.Console.WriteLine( "    has been evaluated are:" );
      System.Console.WriteLine( " " );
      displayArray( returnEF1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Values of the Risk at the evaluation points of the Efficient Frontier +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we get the values of the  Portfolio  Risk  for  the  points  at" );
      System.Console.WriteLine( "     which the Efficient Frontier has been evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      // Here we get the values of the risk at the points  on  which  Efficient
      // Frontier has been evaluated.
      double[] riskEF1 = instanceOfCapitalMarket.GetEfficientFrontierPortfolioRisks( 
         covariance1 // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the risk of the portfolio at the  points  at  which  the" );
      System.Console.WriteLine( "    Efficient Frontier is evaluated is:" );
      System.Console.WriteLine( " " );
      displayArray( riskEF1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Asset Weights of the evaluated Portfolios on the Efficient Frontier +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierAssetWeights(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we get the asset weights of the portfolios  at  the  points  on" );
      System.Console.WriteLine( "     the Efficient Frontier for which it has been evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Here we get the weights of the portfolios on the Efficient Frontier.
      double[][] weightsEF1 = instanceOfCapitalMarket.GetEfficientFrontierAssetWeights( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the portfolios is:" );
      System.Console.WriteLine( " " );
      display2DArray( weightsEF1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Identify the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolio( double minimumExpectedReturn," );
      System.Console.WriteLine( "                     double maximumExpectedReturn," );
      System.Console.WriteLine( "                     double[][] covarianceMatrix," );
      System.Console.WriteLine( "                     double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We will  now  identify  the  market  portfolio  which  lies  of  the" );
      System.Console.WriteLine( "     Efficient Frontier  and  is  distinguished  by  having  the  highest" );
      System.Console.WriteLine( "     return per unit of risk." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = min1" );
      System.Console.WriteLine( "    maximumExpectedReturn = max1" );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      System.Console.WriteLine( "    marketRate = 0.1" );
      // Here we evaluate the  weights  of  the  assets  contained  within  the
      // Market Portfolio
      double[] marketPortfolio1 = instanceOfCapitalMarket.MarketPortfolio( 
         min1, // minimumExpectedReturn
         max1, // maximumExpectedReturn
         covariance1, // covarianceMatrix
         0.1 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the assets within the market portfolio are:" );
      System.Console.WriteLine( " " );
      displayArray( marketPortfolio1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Expected Return of the Market Portfolio +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[] weights," );
      System.Console.WriteLine( "                                   double[][] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the Expected Return of the Market portfolio." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weights = marketPortfolio1" );
      System.Console.WriteLine( "    historicalReturns = {{2.5, 2.6, 2.73, 2.8, 2.82, 2.9, 2.87, 3.05, 3.04, 3.0299, 3.05, 3.13, 3.18, 3.21, 3.25, 3.28, 3.2599, 3.21, 3.27, 3.38, 3.67, 3.62, 3.73, 3.72, 3.64, 3.53, 3.77, 3.76, 3.82, 4.0, 3.82, 3.68, 3.99, 4.12, 4.15, 4.4, 4.35, 4.48, 4.39, 4.48, 4.51, 4.8, 4.5199, 4.5599, 4.5599, 4.29, 4.53, 4.76, 4.87, 4.86, 5.18, 5.46, 4.99, 5.13, 5.0, 5.33, 5.61, 5.69, 5.89, 5.88, 5.99, 6.13}," );
      System.Console.WriteLine( "                         {8.05, 7.54, 6.48, 7.18, 7.42, 7.17, 7.95, 7.34, 7.63, 7.71, 7.8, 8.13, 7.91, 7.87, 8.7899, 8.18, 8.98, 9.39, 8.96, 8.36, 8.03, 7.4, 7.7, 8.66, 9.17, 9.77, 10.3, 9.96, 9.44, 9.44, 9.0, 9.01, 8.6199, 8.56, 9.16, 9.57, 9.72, 8.5399, 8.76, 8.88, 8.66, 8.23, 8.27, 9.51, 10.61, 10.25, 11.38, 10.94, 11.95, 11.07, 12.02, 12.02, 11.25, 11.97, 12.41, 13.05, 13.0, 14.36, 13.85, 15.45, 15.34, 15.2}," );
      System.Console.WriteLine( "                         {1.92, 1.92, 1.92, 1.86, 1.65, 1.56, 1.73, 1.77, 1.76, 1.6299, 1.68, 1.66, 1.79, 1.89, 1.95, 2.0, 2.11, 2.25, 2.49, 2.71, 3.0, 2.54, 2.9, 2.88, 2.92, 3.07, 3.58, 3.52, 3.37, 3.52, 3.42, 3.54, 3.92, 3.89, 3.88, 3.91, 4.05, 4.01, 4.14, 4.39, 4.55, 4.68, 4.83, 5.2699, 5.09, 5.0, 5.25, 4.78, 4.85, 5.2699, 5.86, 6.17, 5.86, 5.79, 6.3099, 7.08, 6.86, 6.89, 6.64, 6.9, 6.87, 6.9}," );
      System.Console.WriteLine( "                         {1.01, 0.99, 0.97, 1.0, 0.91, 0.93, 0.98, 1.16, 1.19, 1.27, 1.22, 1.11, 1.17, 1.1299, 1.26, 1.12, 1.16, 1.37, 1.6299, 1.86, 1.71, 1.59, 1.59, 1.71, 2.06, 2.2599, 2.2599, 2.8, 2.97, 2.92, 2.82, 2.5299, 2.74, 2.64, 2.72, 2.61, 2.45, 2.21, 2.1, 2.4, 2.72, 3.2, 3.04, 3.39, 3.56, 3.2599, 3.62, 3.46, 3.32, 3.41, 3.88, 4.47, 4.85, 4.88, 5.07, 5.29, 5.64, 5.67, 6.01, 6.36, 6.1, 5.87}," );
      System.Console.WriteLine( "                         {1.26, 1.25, 1.22, 1.11, 1.21, 0.95, 0.91, 1.11, 1.03, 0.95, 0.79, 0.72, 0.65, 0.7, 0.64, 0.56, 0.67, 0.64, 0.55, 0.61, 0.61, 0.52, 0.73, 0.91, 1.15, 1.37, 1.68, 1.97, 1.95, 2.02, 2.18, 2.08, 2.1, 2.0099, 1.94, 1.94, 2.04, 1.95, 2.22, 2.27, 2.29, 2.31, 2.18, 2.34, 2.4, 2.54, 2.61, 2.7599, 2.7599, 3.29, 3.0, 3.05, 3.14, 3.24, 3.33, 3.65, 3.88, 4.34, 4.73, 4.54, 4.39, 4.57}," );
      System.Console.WriteLine( "                         {2.37, 2.34, 2.1, 2.2, 2.22, 2.34, 2.8, 2.81, 2.7799, 2.29, 2.38, 2.1, 2.05, 2.21, 2.25, 1.93, 1.96, 1.99, 1.59, 1.32, 1.3799, 0.93, 1.18, 1.4, 1.4, 1.6299, 1.6, 1.94, 1.68, 1.57, 1.5, 1.53, 1.7, 1.61, 1.79, 2.15, 2.09, 2.06, 1.95, 2.2799, 2.72, 2.64, 2.83, 3.37, 3.54, 3.73, 4.26, 4.21, 4.51, 5.57, 6.85, 7.0, 7.79, 8.43, 9.09, 9.01, 9.01, 9.13, 8.93, 9.17, 10.86, 10.67}," );
      System.Console.WriteLine( "                         {1.15, 1.19, 1.05, 1.06, 1.08, 0.96, 1.04, 1.08, 1.1399, 1.07, 1.1, 1.23, 1.15, 1.12, 1.1, 0.93, 0.89, 0.95, 0.94, 0.9, 0.93, 0.96, 0.94, 1.02, 1.02, 1.09, 0.98, 1.1, 1.04, 0.99, 0.92, 0.89, 0.96, 0.91, 0.96, 0.96, 0.96, 0.98, 0.99, 0.97, 0.94, 1.01, 1.21, 1.3799, 1.28, 1.35, 1.32, 1.3, 1.31, 1.24, 1.31, 1.44, 1.48, 1.55, 1.6, 1.83, 1.82, 1.86, 1.89, 2.31, 2.22, 2.25}," );
      System.Console.WriteLine( "                         {1.71, 1.56, 1.71, 2.32, 1.7, 1.75, 2.82, 2.21, 2.21, 2.29, 2.25, 2.07, 2.0299, 2.22, 1.9, 1.61, 2.07, 2.08, 2.11, 2.65, 2.69, 2.34, 3.11, 3.09, 2.89, 3.09, 3.24, 3.43, 3.0099, 2.9, 2.55, 2.2799, 2.32, 2.4, 2.13, 2.07, 2.11, 1.99, 1.91, 2.18, 2.34, 2.81, 3.0099, 2.89, 3.05, 3.4, 3.46, 3.67, 3.81, 3.72, 3.33, 3.12, 3.42, 3.19, 3.19, 3.29, 3.75, 3.98, 4.54, 4.48, 4.73, 4.75}," );
      System.Console.WriteLine( "                         {0.81, 0.78, 0.74, 0.73, 0.72, 0.68, 0.65, 0.72, 0.71, 0.64, 0.65, 0.64, 0.63, 0.6, 0.6, 0.58, 0.53, 0.59, 0.63, 0.73, 0.71, 0.72, 0.72, 0.79, 0.82, 0.87, 0.94, 1.03, 0.98, 0.93, 0.95, 0.91, 1.05, 1.04, 1.09, 1.28, 1.25, 1.3, 1.27, 1.3899, 1.6, 1.77, 2.73, 2.35, 2.33, 2.5, 2.42, 2.19, 2.54, 2.34, 2.72, 2.7, 2.73, 2.56, 2.7, 2.68, 2.85, 3.16, 3.68, 3.6, 3.87, 3.93}," );
      System.Console.WriteLine( "                         {1.6, 1.6299, 1.47, 1.28, 1.19, 1.15, 1.15, 1.15, 1.24, 1.24, 1.22, 1.34, 1.37, 1.35, 1.26, 1.18, 1.15, 1.15, 1.15, 1.2, 1.24, 1.33, 1.36, 1.46, 1.48, 1.58, 2.06, 1.97, 2.31, 2.33, 2.34, 2.25, 2.64, 2.57, 2.7, 2.7599, 2.84, 2.96, 3.17, 3.2, 3.47, 3.52, 3.99, 4.3, 4.47, 4.71, 5.42, 5.45, 6.0599, 5.72, 7.18, 7.22, 7.37, 7.3, 7.55, 8.16, 8.58, 8.24, 8.43, 8.68, 8.26, 8.27}," );
      System.Console.WriteLine( "                         {0.31, 0.29, 0.39, 0.35, 0.38, 0.44, 0.45, 0.43, 0.45, 0.52, 0.55, 0.52, 0.55, 0.59, 0.65, 0.79, 0.93, 0.92, 0.9, 0.84, 0.95, 0.79, 0.86, 0.93, 0.98, 1.07, 1.12, 1.37, 1.47, 1.7, 1.61, 1.52, 1.72, 1.87, 1.85, 1.96, 1.97, 2.13, 2.32, 2.21, 2.38, 2.54, 2.73, 2.86, 2.85, 3.04, 3.08, 2.82, 3.0, 2.72, 2.92, 2.82, 2.98, 3.23, 3.21, 3.32, 3.51, 3.54, 4.13, 4.2, 4.73, 4.71}," );
      System.Console.WriteLine( "                         {2.0099, 1.98, 1.94, 2.1, 1.52, 1.22, 1.1299, 1.17, 1.17, 0.98, 0.94, 0.88, 0.8, 0.74, 0.74, 0.54, 0.6899, 0.63, 0.41, 0.23, 0.28, 0.28, 0.44, 0.5, 0.54, 0.58, 0.56, 0.51, 0.42, 0.36, 0.5699, 0.55, 0.61, 0.54, 0.56, 0.5699, 0.61, 0.62, 0.62, 0.59, 0.68, 0.76, 0.92, 1.1399, 1.52, 1.55, 1.58, 1.47, 1.6299, 1.93, 2.37, 2.59, 2.52, 2.47, 2.82, 3.18, 3.08, 3.12, 3.38, 3.79, 4.21, 4.44}," );
      System.Console.WriteLine( "                         {1.21, 1.34, 1.17, 1.36, 1.34, 1.3899, 1.51, 1.56, 1.51, 1.36, 1.44, 1.42, 1.29, 1.47, 1.52, 1.49, 1.47, 1.43, 1.46, 1.54, 1.49, 1.3, 1.51, 1.6299, 1.72, 1.86, 2.05, 2.14, 2.49, 2.63, 3.04, 3.04, 3.34, 3.51, 3.58, 3.48, 3.53, 3.82, 3.78, 3.9, 3.86, 4.01, 4.5, 5.35, 5.23, 5.73, 6.1, 5.88, 6.57, 7.46, 7.55, 7.86, 7.95, 8.06, 8.73, 9.47, 9.86, 9.78, 10.84, 11.06, 9.3, 9.21}," );
      System.Console.WriteLine( "                         {0.66, 0.6899, 0.71, 0.66, 0.68, 0.68, 0.67, 0.73, 0.76, 0.74, 0.81, 0.85, 0.84, 0.87, 0.84, 0.7, 0.6899, 0.68, 0.71, 0.7, 0.73, 0.61, 0.6899, 0.8, 0.85, 1.12, 1.28, 1.61, 1.78, 1.73, 1.74, 1.69, 1.95, 1.73, 1.6299, 1.75, 1.77, 1.78, 1.61, 1.57, 1.64, 1.78, 1.94, 2.11, 2.25, 2.44, 2.44, 2.48, 2.5, 2.5299, 2.25, 2.15, 2.27, 2.21, 2.12, 2.31, 2.21, 2.05, 2.37, 2.68, 2.66, 2.67}," );
      System.Console.WriteLine( "                         {3.22, 3.21, 3.04, 2.88, 2.7799, 2.61, 2.69, 2.7799, 2.59, 2.56, 2.49, 2.5, 2.49, 2.5, 2.56, 2.48, 2.43, 2.71, 3.09, 3.04, 3.55, 3.48, 3.98, 3.89, 3.76, 3.87, 4.42, 4.49, 3.58, 4.3, 4.21, 3.9, 4.12, 4.25, 4.24, 4.38, 4.38, 4.4, 3.99, 4.0599, 3.82, 3.76, 3.59, 3.94, 4.19, 4.5199, 4.5599, 4.58, 4.64, 4.48, 4.57, 4.49, 4.63, 4.55, 4.55, 4.82, 4.94, 4.92, 4.96, 5.2699, 5.65, 5.65}," );
      System.Console.WriteLine( "                         {4.41, 4.32, 4.1, 4.14, 3.87, 3.66, 3.66, 4.1, 3.65, 3.21, 3.07, 2.66, 2.7599, 2.71, 2.57, 3.07, 3.58, 3.14, 3.1, 3.4, 3.12, 2.62, 2.73, 3.16, 2.95, 2.98, 3.22, 3.47, 3.42, 3.71, 3.44, 3.32, 3.58, 3.8, 3.59, 3.42, 3.61, 3.62, 3.51, 3.16, 3.55, 3.48, 3.5, 3.92, 4.8, 4.83, 4.86, 4.49, 4.11, 3.97, 4.0, 4.12, 4.49, 4.0, 3.96, 4.07, 3.55, 3.62, 3.68, 3.66, 3.17, 3.07}," );
      System.Console.WriteLine( "                         {0.56, 0.56, 0.6, 0.67, 0.5699, 0.58, 0.67, 0.71, 0.82, 0.72, 0.72, 0.8, 0.98, 1.11, 1.08, 0.98, 1.05, 1.09, 1.25, 1.16, 1.1, 0.92, 1.12, 1.11, 1.18, 1.36, 1.6, 1.97, 2.08, 1.75, 1.59, 1.26, 1.1, 1.1299, 1.12, 1.02, 0.87, 0.94, 0.83, 0.94, 0.95, 0.87, 0.99, 1.22, 1.18, 1.3799, 1.54, 1.43, 1.37, 1.28, 1.3799, 1.46, 1.5, 1.66, 1.72, 1.93, 1.93, 2.14, 2.43, 3.02, 2.81, 3.05}," );
      System.Console.WriteLine( "                         {2.1, 2.13, 2.11, 2.04, 1.84, 1.79, 1.75, 2.0099, 1.96, 1.89, 1.83, 1.68, 1.05, 1.12, 1.18, 1.1, 0.9, 0.83, 0.45, 0.47, 0.44, 0.55, 0.61, 0.75, 0.65, 0.58, 0.7, 0.8, 0.8, 0.93, 0.82, 0.79, 0.91, 1.08, 0.9, 0.44, 0.41, 0.43, 0.44, 0.46, 0.65, 0.72, 0.71, 0.96, 0.96, 0.91, 1.1299, 1.2, 1.4, 1.29, 1.3, 1.43, 1.65, 1.66, 1.64, 1.85, 1.69, 1.68, 1.86, 2.02, 2.12, 2.13}}" );
      // Here we evaluate the expected return of the market portfolio.
      double marketPortfolioExpectedReturn1 = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( 
         marketPortfolio1, // weights
         new double[][] {new double[] {2.5, 2.6, 2.73, 2.8, 2.82, 2.9, 2.87, 3.05, 3.04, 3.0299, 3.05, 3.13, 3.18, 3.21, 3.25, 3.28, 3.2599, 3.21, 3.27, 3.38, 3.67, 3.62, 3.73, 3.72, 3.64, 3.53, 3.77, 3.76, 3.82, 4.0, 3.82, 3.68, 3.99, 4.12, 4.15, 4.4, 4.35, 4.48, 4.39, 4.48, 4.51, 4.8, 4.5199, 4.5599, 4.5599, 4.29, 4.53, 4.76, 4.87, 4.86, 5.18, 5.46, 4.99, 5.13, 5.0, 5.33, 5.61, 5.69, 5.89, 5.88, 5.99, 6.13},
                         new double[] {8.05, 7.54, 6.48, 7.18, 7.42, 7.17, 7.95, 7.34, 7.63, 7.71, 7.8, 8.13, 7.91, 7.87, 8.7899, 8.18, 8.98, 9.39, 8.96, 8.36, 8.03, 7.4, 7.7, 8.66, 9.17, 9.77, 10.3, 9.96, 9.44, 9.44, 9.0, 9.01, 8.6199, 8.56, 9.16, 9.57, 9.72, 8.5399, 8.76, 8.88, 8.66, 8.23, 8.27, 9.51, 10.61, 10.25, 11.38, 10.94, 11.95, 11.07, 12.02, 12.02, 11.25, 11.97, 12.41, 13.05, 13.0, 14.36, 13.85, 15.45, 15.34, 15.2},
                         new double[] {1.92, 1.92, 1.92, 1.86, 1.65, 1.56, 1.73, 1.77, 1.76, 1.6299, 1.68, 1.66, 1.79, 1.89, 1.95, 2.0, 2.11, 2.25, 2.49, 2.71, 3.0, 2.54, 2.9, 2.88, 2.92, 3.07, 3.58, 3.52, 3.37, 3.52, 3.42, 3.54, 3.92, 3.89, 3.88, 3.91, 4.05, 4.01, 4.14, 4.39, 4.55, 4.68, 4.83, 5.2699, 5.09, 5.0, 5.25, 4.78, 4.85, 5.2699, 5.86, 6.17, 5.86, 5.79, 6.3099, 7.08, 6.86, 6.89, 6.64, 6.9, 6.87, 6.9},
                         new double[] {1.01, 0.99, 0.97, 1.0, 0.91, 0.93, 0.98, 1.16, 1.19, 1.27, 1.22, 1.11, 1.17, 1.1299, 1.26, 1.12, 1.16, 1.37, 1.6299, 1.86, 1.71, 1.59, 1.59, 1.71, 2.06, 2.2599, 2.2599, 2.8, 2.97, 2.92, 2.82, 2.5299, 2.74, 2.64, 2.72, 2.61, 2.45, 2.21, 2.1, 2.4, 2.72, 3.2, 3.04, 3.39, 3.56, 3.2599, 3.62, 3.46, 3.32, 3.41, 3.88, 4.47, 4.85, 4.88, 5.07, 5.29, 5.64, 5.67, 6.01, 6.36, 6.1, 5.87},
                         new double[] {1.26, 1.25, 1.22, 1.11, 1.21, 0.95, 0.91, 1.11, 1.03, 0.95, 0.79, 0.72, 0.65, 0.7, 0.64, 0.56, 0.67, 0.64, 0.55, 0.61, 0.61, 0.52, 0.73, 0.91, 1.15, 1.37, 1.68, 1.97, 1.95, 2.02, 2.18, 2.08, 2.1, 2.0099, 1.94, 1.94, 2.04, 1.95, 2.22, 2.27, 2.29, 2.31, 2.18, 2.34, 2.4, 2.54, 2.61, 2.7599, 2.7599, 3.29, 3.0, 3.05, 3.14, 3.24, 3.33, 3.65, 3.88, 4.34, 4.73, 4.54, 4.39, 4.57},
                         new double[] {2.37, 2.34, 2.1, 2.2, 2.22, 2.34, 2.8, 2.81, 2.7799, 2.29, 2.38, 2.1, 2.05, 2.21, 2.25, 1.93, 1.96, 1.99, 1.59, 1.32, 1.3799, 0.93, 1.18, 1.4, 1.4, 1.6299, 1.6, 1.94, 1.68, 1.57, 1.5, 1.53, 1.7, 1.61, 1.79, 2.15, 2.09, 2.06, 1.95, 2.2799, 2.72, 2.64, 2.83, 3.37, 3.54, 3.73, 4.26, 4.21, 4.51, 5.57, 6.85, 7.0, 7.79, 8.43, 9.09, 9.01, 9.01, 9.13, 8.93, 9.17, 10.86, 10.67},
                         new double[] {1.15, 1.19, 1.05, 1.06, 1.08, 0.96, 1.04, 1.08, 1.1399, 1.07, 1.1, 1.23, 1.15, 1.12, 1.1, 0.93, 0.89, 0.95, 0.94, 0.9, 0.93, 0.96, 0.94, 1.02, 1.02, 1.09, 0.98, 1.1, 1.04, 0.99, 0.92, 0.89, 0.96, 0.91, 0.96, 0.96, 0.96, 0.98, 0.99, 0.97, 0.94, 1.01, 1.21, 1.3799, 1.28, 1.35, 1.32, 1.3, 1.31, 1.24, 1.31, 1.44, 1.48, 1.55, 1.6, 1.83, 1.82, 1.86, 1.89, 2.31, 2.22, 2.25},
                         new double[] {1.71, 1.56, 1.71, 2.32, 1.7, 1.75, 2.82, 2.21, 2.21, 2.29, 2.25, 2.07, 2.0299, 2.22, 1.9, 1.61, 2.07, 2.08, 2.11, 2.65, 2.69, 2.34, 3.11, 3.09, 2.89, 3.09, 3.24, 3.43, 3.0099, 2.9, 2.55, 2.2799, 2.32, 2.4, 2.13, 2.07, 2.11, 1.99, 1.91, 2.18, 2.34, 2.81, 3.0099, 2.89, 3.05, 3.4, 3.46, 3.67, 3.81, 3.72, 3.33, 3.12, 3.42, 3.19, 3.19, 3.29, 3.75, 3.98, 4.54, 4.48, 4.73, 4.75},
                         new double[] {0.81, 0.78, 0.74, 0.73, 0.72, 0.68, 0.65, 0.72, 0.71, 0.64, 0.65, 0.64, 0.63, 0.6, 0.6, 0.58, 0.53, 0.59, 0.63, 0.73, 0.71, 0.72, 0.72, 0.79, 0.82, 0.87, 0.94, 1.03, 0.98, 0.93, 0.95, 0.91, 1.05, 1.04, 1.09, 1.28, 1.25, 1.3, 1.27, 1.3899, 1.6, 1.77, 2.73, 2.35, 2.33, 2.5, 2.42, 2.19, 2.54, 2.34, 2.72, 2.7, 2.73, 2.56, 2.7, 2.68, 2.85, 3.16, 3.68, 3.6, 3.87, 3.93},
                         new double[] {1.6, 1.6299, 1.47, 1.28, 1.19, 1.15, 1.15, 1.15, 1.24, 1.24, 1.22, 1.34, 1.37, 1.35, 1.26, 1.18, 1.15, 1.15, 1.15, 1.2, 1.24, 1.33, 1.36, 1.46, 1.48, 1.58, 2.06, 1.97, 2.31, 2.33, 2.34, 2.25, 2.64, 2.57, 2.7, 2.7599, 2.84, 2.96, 3.17, 3.2, 3.47, 3.52, 3.99, 4.3, 4.47, 4.71, 5.42, 5.45, 6.0599, 5.72, 7.18, 7.22, 7.37, 7.3, 7.55, 8.16, 8.58, 8.24, 8.43, 8.68, 8.26, 8.27},
                         new double[] {0.31, 0.29, 0.39, 0.35, 0.38, 0.44, 0.45, 0.43, 0.45, 0.52, 0.55, 0.52, 0.55, 0.59, 0.65, 0.79, 0.93, 0.92, 0.9, 0.84, 0.95, 0.79, 0.86, 0.93, 0.98, 1.07, 1.12, 1.37, 1.47, 1.7, 1.61, 1.52, 1.72, 1.87, 1.85, 1.96, 1.97, 2.13, 2.32, 2.21, 2.38, 2.54, 2.73, 2.86, 2.85, 3.04, 3.08, 2.82, 3.0, 2.72, 2.92, 2.82, 2.98, 3.23, 3.21, 3.32, 3.51, 3.54, 4.13, 4.2, 4.73, 4.71},
                         new double[] {2.0099, 1.98, 1.94, 2.1, 1.52, 1.22, 1.1299, 1.17, 1.17, 0.98, 0.94, 0.88, 0.8, 0.74, 0.74, 0.54, 0.6899, 0.63, 0.41, 0.23, 0.28, 0.28, 0.44, 0.5, 0.54, 0.58, 0.56, 0.51, 0.42, 0.36, 0.5699, 0.55, 0.61, 0.54, 0.56, 0.5699, 0.61, 0.62, 0.62, 0.59, 0.68, 0.76, 0.92, 1.1399, 1.52, 1.55, 1.58, 1.47, 1.6299, 1.93, 2.37, 2.59, 2.52, 2.47, 2.82, 3.18, 3.08, 3.12, 3.38, 3.79, 4.21, 4.44},
                         new double[] {1.21, 1.34, 1.17, 1.36, 1.34, 1.3899, 1.51, 1.56, 1.51, 1.36, 1.44, 1.42, 1.29, 1.47, 1.52, 1.49, 1.47, 1.43, 1.46, 1.54, 1.49, 1.3, 1.51, 1.6299, 1.72, 1.86, 2.05, 2.14, 2.49, 2.63, 3.04, 3.04, 3.34, 3.51, 3.58, 3.48, 3.53, 3.82, 3.78, 3.9, 3.86, 4.01, 4.5, 5.35, 5.23, 5.73, 6.1, 5.88, 6.57, 7.46, 7.55, 7.86, 7.95, 8.06, 8.73, 9.47, 9.86, 9.78, 10.84, 11.06, 9.3, 9.21},
                         new double[] {0.66, 0.6899, 0.71, 0.66, 0.68, 0.68, 0.67, 0.73, 0.76, 0.74, 0.81, 0.85, 0.84, 0.87, 0.84, 0.7, 0.6899, 0.68, 0.71, 0.7, 0.73, 0.61, 0.6899, 0.8, 0.85, 1.12, 1.28, 1.61, 1.78, 1.73, 1.74, 1.69, 1.95, 1.73, 1.6299, 1.75, 1.77, 1.78, 1.61, 1.57, 1.64, 1.78, 1.94, 2.11, 2.25, 2.44, 2.44, 2.48, 2.5, 2.5299, 2.25, 2.15, 2.27, 2.21, 2.12, 2.31, 2.21, 2.05, 2.37, 2.68, 2.66, 2.67},
                         new double[] {3.22, 3.21, 3.04, 2.88, 2.7799, 2.61, 2.69, 2.7799, 2.59, 2.56, 2.49, 2.5, 2.49, 2.5, 2.56, 2.48, 2.43, 2.71, 3.09, 3.04, 3.55, 3.48, 3.98, 3.89, 3.76, 3.87, 4.42, 4.49, 3.58, 4.3, 4.21, 3.9, 4.12, 4.25, 4.24, 4.38, 4.38, 4.4, 3.99, 4.0599, 3.82, 3.76, 3.59, 3.94, 4.19, 4.5199, 4.5599, 4.58, 4.64, 4.48, 4.57, 4.49, 4.63, 4.55, 4.55, 4.82, 4.94, 4.92, 4.96, 5.2699, 5.65, 5.65},
                         new double[] {4.41, 4.32, 4.1, 4.14, 3.87, 3.66, 3.66, 4.1, 3.65, 3.21, 3.07, 2.66, 2.7599, 2.71, 2.57, 3.07, 3.58, 3.14, 3.1, 3.4, 3.12, 2.62, 2.73, 3.16, 2.95, 2.98, 3.22, 3.47, 3.42, 3.71, 3.44, 3.32, 3.58, 3.8, 3.59, 3.42, 3.61, 3.62, 3.51, 3.16, 3.55, 3.48, 3.5, 3.92, 4.8, 4.83, 4.86, 4.49, 4.11, 3.97, 4.0, 4.12, 4.49, 4.0, 3.96, 4.07, 3.55, 3.62, 3.68, 3.66, 3.17, 3.07},
                         new double[] {0.56, 0.56, 0.6, 0.67, 0.5699, 0.58, 0.67, 0.71, 0.82, 0.72, 0.72, 0.8, 0.98, 1.11, 1.08, 0.98, 1.05, 1.09, 1.25, 1.16, 1.1, 0.92, 1.12, 1.11, 1.18, 1.36, 1.6, 1.97, 2.08, 1.75, 1.59, 1.26, 1.1, 1.1299, 1.12, 1.02, 0.87, 0.94, 0.83, 0.94, 0.95, 0.87, 0.99, 1.22, 1.18, 1.3799, 1.54, 1.43, 1.37, 1.28, 1.3799, 1.46, 1.5, 1.66, 1.72, 1.93, 1.93, 2.14, 2.43, 3.02, 2.81, 3.05},
                         new double[] {2.1, 2.13, 2.11, 2.04, 1.84, 1.79, 1.75, 2.0099, 1.96, 1.89, 1.83, 1.68, 1.05, 1.12, 1.18, 1.1, 0.9, 0.83, 0.45, 0.47, 0.44, 0.55, 0.61, 0.75, 0.65, 0.58, 0.7, 0.8, 0.8, 0.93, 0.82, 0.79, 0.91, 1.08, 0.9, 0.44, 0.41, 0.43, 0.44, 0.46, 0.65, 0.72, 0.71, 0.96, 0.96, 0.91, 1.1299, 1.2, 1.4, 1.29, 1.3, 1.43, 1.65, 1.66, 1.64, 1.85, 1.69, 1.68, 1.86, 2.02, 2.12, 2.13}} // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The expected return of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioExpectedReturn1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Risk of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioRisk( double[] weights," );
      System.Console.WriteLine( "                         double[][] covariance )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the risk of the  market  portfolio  found.  We  use" );
      System.Console.WriteLine( "     here a method from the Capital  Markets  class  but  we  could  have" );
      System.Console.WriteLine( "     applied the MarketPortfolioRisk method  from  the  Asset  Parameters" );
      System.Console.WriteLine( "     class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weights = marketPortfolio1" );
      System.Console.WriteLine( "    covariance = covariance1" );
      // Here we evaluate the risk of the market portfolio.
      double marketPortfolioRisk1 = instanceOfCapitalMarket.MarketPortfolioRisk( 
         marketPortfolio1, // weights
         covariance1 // covariance
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioRisk1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
