using System;
using WebCab.Lib.Util.Example;

using WebCab.Libraries.Finance.Portfolio;

public class SelectOptimalClient {
   public static void Main( String[] args ) {
      // used components
      AssetParameters instanceOfAssetParameters = new AssetParameters();
      CapitalMarket instanceOfCapitalMarket = new CapitalMarket();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "                     ########################################" );
      System.Console.WriteLine( "                     # SelectOptimal Console Client Example #" );
      System.Console.WriteLine( "                     ########################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    This example  illustrates  the  application  of  the  Capital  Asset" );
      System.Console.WriteLine( "    Pricing Model (CAPM)  functionality  implemented  within  the  class" );
      System.Console.WriteLine( "    CapitalMarket." );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    In particular, we seek the solution to the following question:" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    What is the weighting of  the  Market  Portfolio  and  risk  of  the" );
      System.Console.WriteLine( "    optimal     portfolios  with  respect  to  the  CAPM  which  can  be" );
      System.Console.WriteLine( "    constructed from the assets:" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "       - Fund 1: Historical Returns = 0.03, 0.04, 0.02, 0.05,  0.03  per" );
      System.Console.WriteLine( "       year" );
      System.Console.WriteLine( "       - Fund 2: Historical Returns = 0.04,  0.045,  0.03,  0.03,  0.035" );
      System.Console.WriteLine( "       per year" );
      System.Console.WriteLine( "       - Fund 3: Historical Returns = 0.02, 0.021, 0.021, 0.021,  0.0205" );
      System.Console.WriteLine( "       per year" );
      System.Console.WriteLine( "       - Fund 4: Historical Returns = 0.02, 0.021, 0.021, 0.021,  0.0205" );
      System.Console.WriteLine( "       per year" );
      System.Console.WriteLine( "       - Fund 5: Historical Returns = 0.025, 0.027, 0.03,  0.028,  0.029" );
      System.Console.WriteLine( "       per year" );
      System.Console.WriteLine( "       " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    where the prevailing market rate  at  which  cash  can  be  lent  or" );
      System.Console.WriteLine( "    borrowed is 1.95 percent per year?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ We first evaluate the covariance of the asset being considered +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix( double[][] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We first evaluate the covariance matrix of  the  historical  returns" );
      System.Console.WriteLine( "     from which the portfolios can be constructed." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalReturns = {{0.03, 0.04, 0.02, 0.05, 0.03}," );
      System.Console.WriteLine( "                         {0.03, 0.04, 0.1, 0.03, 0.035}," );
      System.Console.WriteLine( "                         {0.03, 0.032, 0.033, 0.035, 0.036}," );
      System.Console.WriteLine( "                         {0.02, 0.021, 0.021, 0.021, 0.0205}," );
      System.Console.WriteLine( "                         {0.02, 0.01, 0.02, 0.015, 0.028}}" );
      // Here we evaluate the covariance matrix of the  historical  returns  of
      // the assets from which the portfolios can be constructed.
      double[][] covariance1 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {0.03, 0.04, 0.02, 0.05, 0.03},
                         new double[] {0.03, 0.04, 0.1, 0.03, 0.035},
                         new double[] {0.03, 0.032, 0.033, 0.035, 0.036},
                         new double[] {0.02, 0.021, 0.021, 0.021, 0.0205},
                         new double[] {0.02, 0.01, 0.02, 0.015, 0.028}} // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The covariance matrix of the assets historical returns is:" );
      System.Console.WriteLine( " " );
      display2DArray( covariance1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ We next evaluate the expected returns of the asset being considered +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns( double[][] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here    we  evaluate  the  expected  returns  of  the  assets  under" );
      System.Console.WriteLine( "     consideration." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalReturns = {{0.03, 0.04, 0.02, 0.05, 0.03}," );
      System.Console.WriteLine( "                         {0.03, 0.04, 0.1, 0.03, 0.035}," );
      System.Console.WriteLine( "                         {0.03, 0.032, 0.033, 0.035, 0.036}," );
      System.Console.WriteLine( "                         {0.02, 0.021, 0.021, 0.021, 0.0205}," );
      System.Console.WriteLine( "                         {0.02, 0.01, 0.02, 0.015, 0.028}}" );
      // Here       we  evaluate  the  expcted  returns  of  the  assets  under
      // consideration.
      double[] expectedReturns1 = instanceOfAssetParameters.ExpectedReturns( 
         new double[][] {new double[] {0.03, 0.04, 0.02, 0.05, 0.03},
                         new double[] {0.03, 0.04, 0.1, 0.03, 0.035},
                         new double[] {0.03, 0.032, 0.033, 0.035, 0.036},
                         new double[] {0.02, 0.021, 0.021, 0.021, 0.0205},
                         new double[] {0.02, 0.01, 0.02, 0.015, 0.028}} // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The expected return of the assets is:" );
      System.Console.WriteLine( " " );
      displayArray( expectedReturns1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Minimum  value  of  the  expected  return  over  which  the  Efficient +" );
      System.Console.WriteLine( "+ Frontier exits                                                         +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minFrontierReturn( double[] expectedReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the minimum  expected  return  of  the  range  over" );
      System.Console.WriteLine( "     which the Efficient Frontier  exits.  As  a  general  remark  it  is" );
      System.Console.WriteLine( "     usually best to evaluate the range of the  expected  return  of  the" );
      System.Console.WriteLine( "     Efficient Frontier in order  to  provide  a  range  over  which  the" );
      System.Console.WriteLine( "     Efficient Frontier can be evaluated and the  range  over  which  the" );
      System.Console.WriteLine( "     optimal portfolio can be sort." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      double min1 = instanceOfCapitalMarket.MinFrontierReturn( 
         expectedReturns1 // expectedReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The minimum  bound  of  the  range  of  the  expected  return  of  the" );
      System.Console.WriteLine( "    Efficient Frontier is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( min1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Maximum  value  of  the  expected  return  over  which  the  Efficient +" );
      System.Console.WriteLine( "+ Frontier exits                                                         +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    maxFrontierReturn( double[] expectedReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the maximum  expected  return  of  the  range  over" );
      System.Console.WriteLine( "     which the Efficient Frontier  exits.  As  a  general  remark  it  is" );
      System.Console.WriteLine( "     usually best to evaluate the range of the  expected  return  of  the" );
      System.Console.WriteLine( "     Efficient Frontier in order  to  provide  a  range  over  which  the" );
      System.Console.WriteLine( "     Efficient Frontier can be evaluated and the  range  over  which  the" );
      System.Console.WriteLine( "     optimal portfolio can be sort." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      double max1 = instanceOfCapitalMarket.MaxFrontierReturn( 
         expectedReturns1 // expectedReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The maximum  bound  of  the  range  of  the  expected  return  of  the" );
      System.Console.WriteLine( "    Efficient Frontier is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( max1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we construct the Efficient Frontier +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    calculateEfficientFrontier( double minimumExpectedReturn," );
      System.Console.WriteLine( "                                double maximumExpectedReturn," );
      System.Console.WriteLine( "                                double[][] covarianceMatrix," );
      System.Console.WriteLine( "                                double[] expectedReturns," );
      System.Console.WriteLine( "                                int numberInterpolationPoints," );
      System.Console.WriteLine( "                                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Construct the Efficient Frontier and  set  the  results  to  private" );
      System.Console.WriteLine( "     fields. Note that we use 340 interpolation points equal  distributed" );
      System.Console.WriteLine( "     over the entire range over which the Efficient Frontier exits." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = min1" );
      System.Console.WriteLine( "    maximumExpectedReturn = max1" );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    numberInterpolationPoints = 340" );
      System.Console.WriteLine( "    precision = 0.000001" );
      // Here we construct the efficient frontier.
      instanceOfCapitalMarket.CalculateEfficientFrontier( 
         min1, // minimumExpectedReturn
         max1, // maximumExpectedReturn
         covariance1, // covarianceMatrix
         expectedReturns1, // expectedReturns
         340, // numberInterpolationPoints
         0.000001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The Efficient Frontier has now been evaluate of persisted  by  setting" );
      System.Console.WriteLine( "    various private fields which record to the points about  which  it  is" );
      System.Console.WriteLine( "    interpolated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Values of  the  Expected  Return  at  the  evaluation  points  of  the +" );
      System.Console.WriteLine( "+ Efficient Frontier                                                     +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we get the values of the Efficient Frontier for the  points  at" );
      System.Console.WriteLine( "     which the Efficient Frontier has been evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Here we get the values of the Expected returns at the points on  which
      // Efficient Frontier has been evaluated.
      double[] returnEF1 = instanceOfCapitalMarket.GetEfficientFrontierExpectedReturns( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the expected return for  which  the  Efficient  Frontier" );
      System.Console.WriteLine( "    has been evaluated are:" );
      System.Console.WriteLine( " " );
      displayArray( returnEF1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Values of the Risk at the evaluation points of the Efficient Frontier +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we get the values of the  Portfolio  Risk  for  the  points  at" );
      System.Console.WriteLine( "     which the Efficient Frontier has been evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      // Here we get the values of the risk at the points  on  which  Efficient
      // Frontier has been evaluated.
      double[] riskEF1 = instanceOfCapitalMarket.GetEfficientFrontierPortfolioRisks( 
         covariance1 // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the risk of the portfolio at the  points  at  which  the" );
      System.Console.WriteLine( "    Efficient Frontier is evaluated is:" );
      System.Console.WriteLine( " " );
      displayArray( riskEF1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Identify the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolio( double minimumExpectedReturn," );
      System.Console.WriteLine( "                     double maximumExpectedReturn," );
      System.Console.WriteLine( "                     double[][] covarianceMatrix," );
      System.Console.WriteLine( "                     double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We will  now  identify  the  market  portfolio  which  lies  of  the" );
      System.Console.WriteLine( "     Efficient Frontier  and  is  distinguished  by  having  the  highest" );
      System.Console.WriteLine( "     return per unit of risk." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = min1" );
      System.Console.WriteLine( "    maximumExpectedReturn = max1" );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      System.Console.WriteLine( "    marketRate = 0.01" );
      // Here we evaluate the  weights  of  the  assets  contained  within  the
      // Market Portfolio
      double[] marketPortfolio1 = instanceOfCapitalMarket.MarketPortfolio( 
         min1, // minimumExpectedReturn
         max1, // maximumExpectedReturn
         covariance1, // covarianceMatrix
         0.01 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the assets within the market portfolio are:" );
      System.Console.WriteLine( " " );
      displayArray( marketPortfolio1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Risk of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioRisk( double[] weights," );
      System.Console.WriteLine( "                         double[][] covariance )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the risk of the  market  portfolio  found.  We  use" );
      System.Console.WriteLine( "     here a method from the Capital  Markets  class  but  we  could  have" );
      System.Console.WriteLine( "     applied the MarketPortfolioRisk method  from  the  Asset  Parameters" );
      System.Console.WriteLine( "     class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weights = marketPortfolio1" );
      System.Console.WriteLine( "    covariance = covariance1" );
      // Here we evaluate the risk of the market portfolio.
      double marketPortfolioRisk1 = instanceOfCapitalMarket.MarketPortfolioRisk( 
         marketPortfolio1, // weights
         covariance1 // covariance
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioRisk1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Expected return of the Market Portfolio +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[] weights," );
      System.Console.WriteLine( "                                   double[][] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the expected return of the Market  portfolio.  Here" );
      System.Console.WriteLine( "     we    applied  the  method  MarketPortfolioExpectedReturn  from  the" );
      System.Console.WriteLine( "     Capital Market class, but we could have chosen to apply  the  method" );
      System.Console.WriteLine( "     ExpectedReturn from the Asset Parameters class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weights = marketPortfolio1" );
      System.Console.WriteLine( "    historicalReturns = {{0.03, 0.04, 0.02, 0.05, 0.03}," );
      System.Console.WriteLine( "                         {0.03, 0.04, 0.1, 0.03, 0.035}," );
      System.Console.WriteLine( "                         {0.03, 0.032, 0.033, 0.035, 0.036}," );
      System.Console.WriteLine( "                         {0.02, 0.021, 0.021, 0.021, 0.0205}," );
      System.Console.WriteLine( "                         {0.02, 0.01, 0.02, 0.015, 0.028}}" );
      // Here we evaluate the expected return of the market portfolio.
      double marketPortfolioReturn1 = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( 
         marketPortfolio1, // weights
         new double[][] {new double[] {0.03, 0.04, 0.02, 0.05, 0.03},
                         new double[] {0.03, 0.04, 0.1, 0.03, 0.035},
                         new double[] {0.03, 0.032, 0.033, 0.035, 0.036},
                         new double[] {0.02, 0.021, 0.021, 0.021, 0.0205},
                         new double[] {0.02, 0.01, 0.02, 0.015, 0.028}} // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The expected return of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioReturn1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Market Portfolio Weighting when expected return equals 5 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weightCML( double expectedReturn," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the weighting of the market  portfolio  within  the" );
      System.Console.WriteLine( "     optimal portfolio with respect to the CAPM (i.e. a  portfolio  which" );
      System.Console.WriteLine( "     lies on the CML) which has an expected return of 5 percent." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.05" );
      System.Console.WriteLine( "    equityExpected = marketPortfolioReturn1" );
      System.Console.WriteLine( "    marketRate = 0.0195" );
      // Here we evaluate the weighting  of  the  market  portfolio  within  an
      // optimal portfolio with an expected return of 5 percent.
      double weighting1 = instanceOfCapitalMarket.WeightCML( 
         0.05, // expectedReturn
         marketPortfolioReturn1, // equityExpected
         0.0195 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weighting of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weighting1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Risk of the Optimal Portfolio with an expected return equals 5 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    riskCML( double expectedReturn," );
      System.Console.WriteLine( "             double equityExpected," );
      System.Console.WriteLine( "             double marketRate," );
      System.Console.WriteLine( "             double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the risk of the optimal portfolio with  respect  to" );
      System.Console.WriteLine( "     the CAPM (i.e. a portfolio which lies  on  the  CML)  which  has  an" );
      System.Console.WriteLine( "     expected return of 5 percent." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.05" );
      System.Console.WriteLine( "    equityExpected = marketPortfolioReturn1" );
      System.Console.WriteLine( "    marketRate = 0.0195" );
      System.Console.WriteLine( "    equityRisk = marketPortfolioRisk1" );
      // Here we evalaute the risk of the optimal portfolio on the CML with  an
      // expected return of 5 percent.
      double risk1 = instanceOfCapitalMarket.RiskCML( 
         0.05, // expectedReturn
         marketPortfolioReturn1, // equityExpected
         0.0195, // marketRate
         marketPortfolioRisk1 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weighting of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( risk1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Market Portfolio Weighting when expected return equals 3 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weightCML( double expectedReturn," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the weighting of the market  portfolio  within  the" );
      System.Console.WriteLine( "     optimal portfolio with respect to the CAPM (i.e. a  portfolio  which" );
      System.Console.WriteLine( "     lies on the CML) which has an expected return of 3 percent." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.03" );
      System.Console.WriteLine( "    equityExpected = marketPortfolioReturn1" );
      System.Console.WriteLine( "    marketRate = 0.0195" );
      // Here we evaluate the weighting  of  the  market  portfolio  within  an
      // optimal portfolio with an expected return of 3 percent.
      double weighting2 = instanceOfCapitalMarket.WeightCML( 
         0.03, // expectedReturn
         marketPortfolioReturn1, // equityExpected
         0.0195 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weighting of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weighting2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Risk of the Optimal Portfolio with an expected return equals 3 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    riskCML( double expectedReturn," );
      System.Console.WriteLine( "             double equityExpected," );
      System.Console.WriteLine( "             double marketRate," );
      System.Console.WriteLine( "             double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the risk of the optimal portfolio with  respect  to" );
      System.Console.WriteLine( "     the CAPM (i.e. a portfolio which lies  on  the  CML)  which  has  an" );
      System.Console.WriteLine( "     expected return of 3 percent." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.03" );
      System.Console.WriteLine( "    equityExpected = marketPortfolioReturn1" );
      System.Console.WriteLine( "    marketRate = 0.0195" );
      System.Console.WriteLine( "    equityRisk = marketPortfolioRisk1" );
      // Here we evalaute the risk of the optimal portfolio on the CML with  an
      // expected return of 5 percent.
      double risk2 = instanceOfCapitalMarket.RiskCML( 
         0.03, // expectedReturn
         marketPortfolioReturn1, // equityExpected
         0.0195, // marketRate
         marketPortfolioRisk1 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weighting of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( risk2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Market Portfolio Weighting when expected return equals 2.1 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weightCML( double expectedReturn," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the weighting of the market  portfolio  within  the" );
      System.Console.WriteLine( "     optimal portfolio with respect to the CAPM (i.e. a  portfolio  which" );
      System.Console.WriteLine( "     lies on the CML) which has an expected return of 2.1 percent." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.021" );
      System.Console.WriteLine( "    equityExpected = marketPortfolioReturn1" );
      System.Console.WriteLine( "    marketRate = 0.0195" );
      // Here we evaluate the weighting  of  the  market  portfolio  within  an
      // optimal portfolio with an expected return of 2.1 percent.
      double weighting3 = instanceOfCapitalMarket.WeightCML( 
         0.021, // expectedReturn
         marketPortfolioReturn1, // equityExpected
         0.0195 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weighting of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weighting3 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Risk of the Optimal Portfolio with an expected return equals 2.1 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    riskCML( double expectedReturn," );
      System.Console.WriteLine( "             double equityExpected," );
      System.Console.WriteLine( "             double marketRate," );
      System.Console.WriteLine( "             double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the risk of the optimal portfolio with  respect  to" );
      System.Console.WriteLine( "     the CAPM (i.e. a portfolio which lies  on  the  CML)  which  has  an" );
      System.Console.WriteLine( "     expected return of 2.1 percent." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.021" );
      System.Console.WriteLine( "    equityExpected = marketPortfolioReturn1" );
      System.Console.WriteLine( "    marketRate = 0.0195" );
      System.Console.WriteLine( "    equityRisk = marketPortfolioRisk1" );
      // Here we evalaute the risk of the optimal portfolio on the CML with  an
      // expected return of 2.1 percent.
      double risk3 = instanceOfCapitalMarket.RiskCML( 
         0.021, // expectedReturn
         marketPortfolioReturn1, // equityExpected
         0.0195, // marketRate
         marketPortfolioRisk1 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weighting of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( risk3 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
