using System;
using WebCab.Lib.Util.Example;

using WebCab.Libraries.Finance.Portfolio;

public class ThreeAssetPortfolioClient {
   public static void Main( String[] args ) {
      // used components
      AssetParameters instanceOfAssetParameters = new AssetParameters();
      CapitalMarket instanceOfCapitalMarket = new CapitalMarket();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "                  ##############################################" );
      System.Console.WriteLine( "                  # ThreeAssetPortfolio Console Client Example #" );
      System.Console.WriteLine( "                  ##############################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Within this client we provide an example  where  we  illustrate  the" );
      System.Console.WriteLine( "    application of the CAPM to modeling the risk/return  characteristics" );
      System.Console.WriteLine( "    of portfolios which are constructed from three risky  equity  assets" );
      System.Console.WriteLine( "    and/or geared/de-geared at the prevailing risk free  rate  on  cash." );
      System.Console.WriteLine( "    Notes on the Units used for the Source  data  (i.e.  the  Historical" );
      System.Console.WriteLine( "    Values) The units in which the  historical  values  are  given  will" );
      System.Console.WriteLine( "    effect  the  units  used  throughout  the  portfolio  analysis.  For" );
      System.Console.WriteLine( "    example, if the historical values are given in absolute  terms  then" );
      System.Console.WriteLine( "    the following will either be returned  and  should  be  provided  in" );
      System.Console.WriteLine( "    absolute terms:" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "       - Expected Return" );
      System.Console.WriteLine( "       - Utility Function" );
      System.Console.WriteLine( "       " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    Note that the covariance matrix and the asset weights are  invariant" );
      System.Console.WriteLine( "    to the units in which the historical returns are expressed." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluate Points  on  Efficient  Frontier  over  Interval  of  Expected +" );
      System.Console.WriteLine( "+ Returns                                                                +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    calculateEfficientFrontier( double[][] covarianceMatrix," );
      System.Console.WriteLine( "                                double[] expectedReturns," );
      System.Console.WriteLine( "                                int numberInterpolationPoints," );
      System.Console.WriteLine( "                                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We are able to construct  portfolios  from  three  assets  with  the" );
      System.Console.WriteLine( "     following historical returns over the last 18 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - Date Value Date Value Date Value" );
      System.Console.WriteLine( "        -       2005/4/29  16.51425171  2005/4/29  8.481117249  2005/4/29" );
      System.Console.WriteLine( "        1.834869385" );
      System.Console.WriteLine( "        -       2005/5/31  17.02316284  2005/5/31  8.893920898  2005/5/31" );
      System.Console.WriteLine( "        3.440818787" );
      System.Console.WriteLine( "        -       2005/6/30  18.45776367  2005/6/30  8.921325684  2005/6/30" );
      System.Console.WriteLine( "        3.884147644" );
      System.Console.WriteLine( "        -       2005/7/29  18.70734406  2005/7/29  9.067459106  2005/7/29" );
      System.Console.WriteLine( "        3.212486267" );
      System.Console.WriteLine( "        -       2005/8/31  19.45423126  2005/8/31  9.140419006  2005/8/31" );
      System.Console.WriteLine( "        4.062568665" );
      System.Console.WriteLine( "        -       2005/9/30  19.78372192  2005/9/30  9.040168762  2005/9/30" );
      System.Console.WriteLine( "        4.062088013" );
      System.Console.WriteLine( "        -    2005/10/31  18.79850006  2005/10/31  7.945472717  2005/10/31" );
      System.Console.WriteLine( "        3.561515808" );
      System.Console.WriteLine( "        -    2005/11/30  18.96433258  2005/11/30  7.779449463  2005/11/30" );
      System.Console.WriteLine( "        4.741859436" );
      System.Console.WriteLine( "        -     2005/12/30  19.7921524  2005/12/30  8.176712036  2005/12/30" );
      System.Console.WriteLine( "        5.724159241" );
      System.Console.WriteLine( "        -       2006/1/31  19.95635223  2006/1/31  7.900650024  2006/1/31" );
      System.Console.WriteLine( "        5.89730072" );
      System.Console.WriteLine( "        -       2006/2/28  20.20225525  2006/2/28  7.753005981  2006/2/28" );
      System.Console.WriteLine( "        6.588676453" );
      System.Console.WriteLine( "        -       2006/3/31  19.46636963  2006/3/31  6.708740234  2006/3/31" );
      System.Console.WriteLine( "        6.241714478" );
      System.Console.WriteLine( "        -       2006/4/28  19.21924591  2006/4/28  5.849571228  2006/4/28" );
      System.Console.WriteLine( "        6.04788208" );
      System.Console.WriteLine( "        -        2006/5/31  18.5586319  2006/5/31  6.103904724  2006/5/31" );
      System.Console.WriteLine( "        5.91192627" );
      System.Console.WriteLine( "        -        2006/6/30  17.8105011  2006/6/30  5.699882507  2006/6/30" );
      System.Console.WriteLine( "        3.325386047" );
      System.Console.WriteLine( "        -        2006/7/31  18.8155365  2006/7/31  6.228179932  2006/7/31" );
      System.Console.WriteLine( "        5.022331238" );
      System.Console.WriteLine( "        -       2006/8/31  19.72763824  2006/8/31  7.213546753  2006/8/31" );
      System.Console.WriteLine( "        7.044326782" );
      System.Console.WriteLine( "        -       2006/9/29  20.05631256  2006/9/29  7.780410767  2006/9/29" );
      System.Console.WriteLine( "        8.27784729" );
      System.Console.WriteLine( "        -    2006/10/31  20.79341125  2006/10/31  7.808128357  2006/10/31" );
      System.Console.WriteLine( "        9.929779053" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Evaluate 40 points on the (un-constrained) Efficient  Frontier  with" );
      System.Console.WriteLine( "     the precision set to 0.00001" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    numberInterpolationPoints = 40" );
      System.Console.WriteLine( "    precision = 0.00001" );
      // We first use the covariance method from Asset Parameters.
      double[][] covariance1 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {16.5142, 17.0231, 18.4577, 18.7073, 19.4542, 19.7837, 18.7985, 18.9643, 19.7921, 19.9563, 20.2022, 19.4663, 19.2192, 18.5586, 17.8105, 18.8155, 19.7276, 20.0563, 20.7934},
                         new double[] {8.4811, 8.8939, 8.9213, 9.0674, 9.1404, 9.0401, 7.9454, 7.7794, 8.1767, 7.9006, 7.753, 6.7087, 5.8495, 6.1039, 5.6998, 6.2281, 7.2135, 7.7804, 7.8081},
                         new double[] {1.8348, 3.4408, 3.8841, 3.2124, 4.0625, 4.062, 3.5615, 4.7418, 5.7241, 5.8973, 6.5886, 6.2417, 6.0478, 5.9119, 3.3253, 5.0223, 7.0443, 8.2778, 9.9297}} // historicalReturns
      );
      // Next we evaluate the expected returns using  the  historical  approach
      // from Asset Parameters.
      double[] expectedReturns1 = instanceOfAssetParameters.ExpectedReturns( 
         new double[][] {new double[] {16.5142, 17.0231, 18.4577, 18.7073, 19.4542, 19.7837, 18.7985, 18.9643, 19.7921, 19.9563, 20.2022, 19.4663, 19.2192, 18.5586, 17.8105, 18.8155, 19.7276, 20.0563, 20.7934},
                         new double[] {8.4811, 8.8939, 8.9213, 9.0674, 9.1404, 9.0401, 7.9454, 7.7794, 8.1767, 7.9006, 7.753, 6.7087, 5.8495, 6.1039, 5.6998, 6.2281, 7.2135, 7.7804, 7.8081},
                         new double[] {1.8348, 3.4408, 3.8841, 3.2124, 4.0625, 4.062, 3.5615, 4.7418, 5.7241, 5.8973, 6.5886, 6.2417, 6.0478, 5.9119, 3.3253, 5.0223, 7.0443, 8.2778, 9.9297}} // historicalReturns
      );
      // This method sets the coordinates of the Efficient Frontier as  private
      // fields in the class. Note the we evaluate the Efficient Frontier  over
      // the entire region which it exists.
      instanceOfCapitalMarket.CalculateEfficientFrontier( 
         covariance1, // covarianceMatrix
         expectedReturns1, // expectedReturns
         40, // numberInterpolationPoints
         0.00001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Our first aim in virtually all applications and  Portfolio  Theory  is" );
      System.Console.WriteLine( "    to evaluate a collection of points on the  Efficient  Frontier,  which" );
      System.Console.WriteLine( "    we do to applying the  calculate  Efficient  Frontier  method.  Before" );
      System.Console.WriteLine( "    this method  is  applied  we  need  to  evaluate  the  covariance  and" );
      System.Console.WriteLine( "    expected returns using procedures from  the  Asset  Parameters  class." );
      System.Console.WriteLine( "    The points of the Efficient Frontier are stored in  private  field  of" );
      System.Console.WriteLine( "    the class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we return the five values of the expected return for which  the" );
      System.Console.WriteLine( "     efficient was evaluated  within  the  example  ;Evaluate  Points  on" );
      System.Console.WriteLine( "     Efficient Frontier;, and stored within a private field." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Reads the values of the  expected  return  at  the  points  where  the
      // Efficient Frontier was evaluated.
      double[] efficientFrontierExpectedReturns1 = instanceOfCapitalMarket.GetEfficientFrontierExpectedReturns( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the expected return at the  five  points  at  which  the" );
      System.Console.WriteLine( "    efficient was evaluated is:" );
      System.Console.WriteLine( " " );
      displayArray( efficientFrontierExpectedReturns1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierAssetWeights(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Reads the values  of  the  asset  weights  at  the  points  where  the
      // Efficient Frontier was evaluated
      double[][] assetWeightsEfficientFrontier1 = instanceOfCapitalMarket.GetEfficientFrontierAssetWeights( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( " " );
      display2DArray( assetWeightsEfficientFrontier1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Returns the values of the Risks +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Returns the values of the risks at the 20 points  of  the  Efficient" );
      System.Console.WriteLine( "     Frontier evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      double[] risks1 = instanceOfCapitalMarket.GetEfficientFrontierPortfolioRisks( 
         covariance1 // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The 40 values of the risks of the points  on  the  Efficient  Frontier" );
      System.Console.WriteLine( "    are:" );
      System.Console.WriteLine( " " );
      displayArray( risks1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Expected Return of the Market Portfolio +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[][] cov," );
      System.Console.WriteLine( "                                   double[] expectedReturns," );
      System.Console.WriteLine( "                                   double riskFreeRate," );
      System.Console.WriteLine( "                                   double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we get the expected return of the Market Portfolio." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    cov = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    riskFreeRate = 1" );
      System.Console.WriteLine( "    precision = 0.0001" );
      // Here we evaluate the expected return of the market portfolio.
      double marketPortfolioReturn2 = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( 
         covariance1, // cov
         expectedReturns1, // expectedReturns
         1.0, // riskFreeRate
         0.0001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The expected return of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioReturn2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Risk of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioRisk( double[][] cov," );
      System.Console.WriteLine( "                         double[] expectedReturns," );
      System.Console.WriteLine( "                         double riskFreeRate," );
      System.Console.WriteLine( "                         double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the risk of the  market  portfolio  found.  We  use" );
      System.Console.WriteLine( "     here a method from the Capital  Markets  class  but  we  could  have" );
      System.Console.WriteLine( "     applied the MarketPortfolioRisk method  from  the  Asset  Parameters" );
      System.Console.WriteLine( "     class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    cov = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    riskFreeRate = 1" );
      System.Console.WriteLine( "    precision = 0.0001" );
      // Here we evaluate the risk of the market portfolio.
      double marketPortfolioRisk2 = instanceOfCapitalMarket.MarketPortfolioRisk( 
         covariance1, // cov
         expectedReturns1, // expectedReturns
         1.0, // riskFreeRate
         0.0001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioRisk2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Expected Return of the Market Portfolio +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[][] cov," );
      System.Console.WriteLine( "                                   double[] expectedReturns," );
      System.Console.WriteLine( "                                   double riskFreeRate," );
      System.Console.WriteLine( "                                   double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we get the expected return of the Market Portfolio." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    cov = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    riskFreeRate = 4" );
      System.Console.WriteLine( "    precision = 0.0001" );
      // Here we evaluate the expected return of the market portfolio.
      double marketPortfolioReturn3 = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( 
         covariance1, // cov
         expectedReturns1, // expectedReturns
         4.0, // riskFreeRate
         0.0001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The expected return of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioReturn3 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Risk of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioRisk( double[][] cov," );
      System.Console.WriteLine( "                         double[] expectedReturns," );
      System.Console.WriteLine( "                         double riskFreeRate," );
      System.Console.WriteLine( "                         double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the risk of the  market  portfolio  found.  We  use" );
      System.Console.WriteLine( "     here a method from the Capital  Markets  class  but  we  could  have" );
      System.Console.WriteLine( "     applied the MarketPortfolioRisk method  from  the  Asset  Parameters" );
      System.Console.WriteLine( "     class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    cov = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    riskFreeRate = 4" );
      System.Console.WriteLine( "    precision = 0.0001" );
      // Here we evaluate the risk of the market portfolio.
      double marketPortfolioRisk3 = instanceOfCapitalMarket.MarketPortfolioRisk( 
         covariance1, // cov
         expectedReturns1, // expectedReturns
         4.0, // riskFreeRate
         0.0001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioRisk3 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Expected Return of the Market Portfolio +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[][] cov," );
      System.Console.WriteLine( "                                   double[] expectedReturns," );
      System.Console.WriteLine( "                                   double riskFreeRate," );
      System.Console.WriteLine( "                                   double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we get the expected return of the Market Portfolio." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    cov = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    riskFreeRate = 5" );
      System.Console.WriteLine( "    precision = 0.0001" );
      // Here we evaluate the expected return of the market portfolio.
      double marketPortfolioReturn4 = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( 
         covariance1, // cov
         expectedReturns1, // expectedReturns
         5.0, // riskFreeRate
         0.0001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The expected return of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioReturn4 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Risk of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioRisk( double[][] cov," );
      System.Console.WriteLine( "                         double[] expectedReturns," );
      System.Console.WriteLine( "                         double riskFreeRate," );
      System.Console.WriteLine( "                         double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the risk of the  market  portfolio  found.  We  use" );
      System.Console.WriteLine( "     here a method from the Capital  Markets  class  but  we  could  have" );
      System.Console.WriteLine( "     applied the MarketPortfolioRisk method  from  the  Asset  Parameters" );
      System.Console.WriteLine( "     class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    cov = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    riskFreeRate = 5" );
      System.Console.WriteLine( "    precision = 0.0001" );
      // Here we evaluate the risk of the market portfolio.
      double marketPortfolioRisk4 = instanceOfCapitalMarket.MarketPortfolioRisk( 
         covariance1, // cov
         expectedReturns1, // expectedReturns
         5.0, // riskFreeRate
         0.0001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioRisk4 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Expected Return of the Market Portfolio +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[][] cov," );
      System.Console.WriteLine( "                                   double[] expectedReturns," );
      System.Console.WriteLine( "                                   double riskFreeRate," );
      System.Console.WriteLine( "                                   double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we get the expected return of the Market Portfolio." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    cov = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    riskFreeRate = 7" );
      System.Console.WriteLine( "    precision = 0.0001" );
      // Here we evaluate the expected return of the market portfolio.
      double marketPortfolioReturn5 = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( 
         covariance1, // cov
         expectedReturns1, // expectedReturns
         7.0, // riskFreeRate
         0.0001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The expected return of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioReturn5 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Risk of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioRisk( double[][] cov," );
      System.Console.WriteLine( "                         double[] expectedReturns," );
      System.Console.WriteLine( "                         double riskFreeRate," );
      System.Console.WriteLine( "                         double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the risk of the  market  portfolio  found.  We  use" );
      System.Console.WriteLine( "     here a method from the Capital  Markets  class  but  we  could  have" );
      System.Console.WriteLine( "     applied the MarketPortfolioRisk method  from  the  Asset  Parameters" );
      System.Console.WriteLine( "     class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    cov = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    riskFreeRate = 7" );
      System.Console.WriteLine( "    precision = 0.0001" );
      // Here we evaluate the risk of the market portfolio.
      double marketPortfolioRisk5 = instanceOfCapitalMarket.MarketPortfolioRisk( 
         covariance1, // cov
         expectedReturns1, // expectedReturns
         7.0, // riskFreeRate
         0.0001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the market portfolio is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioRisk5 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Find the Risk of the Leveraged Portfolio on the CML +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    riskCML( double expectedReturn," );
      System.Console.WriteLine( "             double equityExpected," );
      System.Console.WriteLine( "             double marketRate," );
      System.Console.WriteLine( "             double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the Risk of  the  Portfolio  on  the  CML  with  an" );
      System.Console.WriteLine( "     expected return of 7 per  period.  Recall,  that  since  the  Market" );
      System.Console.WriteLine( "     Portfolio has a expected return less than 7, we will be  considering" );
      System.Console.WriteLine( "     a leveraged portfolio which has borrowed cash  from  the  market  at" );
      System.Console.WriteLine( "     the prevailing rate of 1 percent per  month  in  order  to  purchase" );
      System.Console.WriteLine( "     more units of the market portfolio." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 7" );
      System.Console.WriteLine( "    equityExpected = marketPortfolioReturn2" );
      System.Console.WriteLine( "    marketRate = 1" );
      System.Console.WriteLine( "    equityRisk = marketPortfolioRisk2" );
      // Here we evaluate the  Risk  of  the  Portfolio  on  the  CML  with  an
      // expected return of 7 per period (i.e. a leveraged portfolio).
      double riskCML1 = instanceOfCapitalMarket.RiskCML( 
         7.0, // expectedReturn
         marketPortfolioReturn2, // equityExpected
         1.0, // marketRate
         marketPortfolioRisk2 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the leveraged  portfolio  on  the  CML  with  an  expected" );
      System.Console.WriteLine( "    return of 7 per period, is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskCML1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Find the Risk of the Unleveraged Portfolio on the CML +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    riskCML( double expectedReturn," );
      System.Console.WriteLine( "             double equityExpected," );
      System.Console.WriteLine( "             double marketRate," );
      System.Console.WriteLine( "             double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the Risk of  the  Portfolio  on  the  CML  with  an" );
      System.Console.WriteLine( "     expected return of 4 per  period.  Recall,  that  since  the  Market" );
      System.Console.WriteLine( "     Portfolio  has  a  expected  return  greater  than  4,  we  will  be" );
      System.Console.WriteLine( "     considering a un-leveraged portfolio which lends cash to the  market" );
      System.Console.WriteLine( "     at the prevailing  rate  of  1  percent  per  month  and  invest  to" );
      System.Console.WriteLine( "     remaining capital within the market portfolio.  That  is,  since  we" );
      System.Console.WriteLine( "     require a portfolio with a return lower than  the  market  portfolio" );
      System.Console.WriteLine( "     we are able to hold a portfolio consisting  of  a  weighting  within" );
      System.Console.WriteLine( "     the market portfolio (i.e. risky asset)  with  the  remaining  being" );
      System.Console.WriteLine( "     lent to the market (i.e. zero risk asset)." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 4" );
      System.Console.WriteLine( "    equityExpected = marketPortfolioReturn2" );
      System.Console.WriteLine( "    marketRate = 1" );
      System.Console.WriteLine( "    equityRisk = marketPortfolioRisk2" );
      // Here we evaluate the  Risk  of  the  Portfolio  on  the  CML  with  an
      // expected return of 4 per period (i.e. a unleveraged portfolio).
      double riskCML2 = instanceOfCapitalMarket.RiskCML( 
         4.0, // expectedReturn
         marketPortfolioReturn2, // equityExpected
         1.0, // marketRate
         marketPortfolioRisk2 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the un-leveraged portfolio on the  CML  with  an  expected" );
      System.Console.WriteLine( "    return of 4 per period, is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskCML2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
