using System;
using WebCab.Lib.Util.Example;

using WebCab.Libraries.Finance.Portfolio;

public class FullEfficientFrontierClient {
   public static void Main( String[] args ) {
      // used components
      Markowitz instanceOfMarkowitz = new Markowitz();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "                 ################################################" );
      System.Console.WriteLine( "                 # FullEfficientFrontier Console Client Example #" );
      System.Console.WriteLine( "                 ################################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    With this example we evaluate the Efficient Frontier on  its  entire" );
      System.Console.WriteLine( "    range at 20 points using each of  the  approaches  available  within" );
      System.Console.WriteLine( "    the Markowitz class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Maximum Expected Return of the Efficient Frontier +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    maxFrontierReturn( double[] expectedReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the maximum expected return of  the  Portfolios  on" );
      System.Console.WriteLine( "     the Efficient Frontier." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns = {0.0453, 0.0618, 0.0696, 0.0767, 0.1024, 0.0855, 0.1229, 0.0908, 0.0738}" );
      // Here we evaluate the maximum expected return  of  the  Portfolio's  on
      // the Efficient Frontier.
      double maxReturn1 = instanceOfMarkowitz.MaxFrontierReturn( 
         new double[] {0.0453, 0.0618, 0.0696, 0.0767, 0.1024, 0.0855, 0.1229, 0.0908, 0.0738} // expectedReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The maximum expected return is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( maxReturn1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Minimum Expected Return of the Efficient Frontier +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minFrontierReturn( double[] expectedReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the minimum expected return of  the  Portfolios  on" );
      System.Console.WriteLine( "     the Efficient Frontier." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns = {0.0453, 0.0618, 0.0696, 0.0767, 0.1024, 0.0855, 0.1229, 0.0908, 0.0738}" );
      // Here we evaluate the minimum expected retrn of the Portfolio's on  the
      // Efficient Frontier.
      double minReturn1 = instanceOfMarkowitz.MinFrontierReturn( 
         new double[] {0.0453, 0.0618, 0.0696, 0.0767, 0.1024, 0.0855, 0.1229, 0.0908, 0.0738} // expectedReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The minimum expected return is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( minReturn1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluation of the Efficient Frontier giving its range +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    calculateEfficientFrontier( double minimumExpectedReturn," );
      System.Console.WriteLine( "                                double maximumExpectedReturn," );
      System.Console.WriteLine( "                                double[][] covarianceMatrix," );
      System.Console.WriteLine( "                                double[] expectedReturns," );
      System.Console.WriteLine( "                                int numberInterpolationPoints," );
      System.Console.WriteLine( "                                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the  Efficient  Frontier  at  20  points,  using  a" );
      System.Console.WriteLine( "     precision of 0.001." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = minReturn1" );
      System.Console.WriteLine( "    maximumExpectedReturn = maxReturn1" );
      System.Console.WriteLine( "    covarianceMatrix = {{8.0E-4, 4.0E-4, 8.0E-4, 6.0E-4, 2.0E-4, 7.0E-4, -5.0E-4, -0.0013, 0.0}," );
      System.Console.WriteLine( "                        {4.0E-4, 0.0016, 0.0026, 0.0042, 0.0037, 0.0021, 0.0033, -8.0E-4, 5.0E-4}," );
      System.Console.WriteLine( "                        {8.0E-4, 0.0026, 0.0044, 0.0066, 0.0069, 0.0043, -9.0E-4, 0.0018, 0.0023}," );
      System.Console.WriteLine( "                        {6.0E-4, 0.0042, 0.0066, 0.012, 0.0103, 0.0038, 0.0020, 0.0029, 0.0038}," );
      System.Console.WriteLine( "                        {2.0E-4, 0.0037, 0.0069, 0.0103, 0.0282, 0.0224, 0.0304, 0.0214, 0.0121}," );
      System.Console.WriteLine( "                        {7.0E-4, 0.0021, 0.0043, 0.0038, 0.0224, 0.0386, 0.024, 0.0223, 0.0193}," );
      System.Console.WriteLine( "                        {-5.0E-4, 0.0033, -9.0E-4, 0.0020, 0.0304, 0.024, 0.0553, 0.0272, 0.0482}," );
      System.Console.WriteLine( "                        {-0.0013, -8.0E-4, 0.0018, 0.0029, 0.0214, 0.0223, 0.0272, 0.0517, 0.0585}," );
      System.Console.WriteLine( "                        {0.0, 5.0E-4, 0.0023, 0.0038, 0.0121, 0.0193, 0.0482, 0.0585, 0.0835}}" );
      System.Console.WriteLine( "    expectedReturns = {0.0453, 0.0618, 0.0696, 0.0767, 0.1024, 0.0855, 0.1229, 0.0908, 0.0738}" );
      System.Console.WriteLine( "    numberInterpolationPoints = 45" );
      System.Console.WriteLine( "    precision = 0.01" );
      // Here we evaluate the Efficient Frontier at 30 points  over  it  entire
      // range.
      instanceOfMarkowitz.CalculateEfficientFrontier( 
         minReturn1, // minimumExpectedReturn
         maxReturn1, // maximumExpectedReturn
         new double[][] {new double[] {8.0E-4, 4.0E-4, 8.0E-4, 6.0E-4, 2.0E-4, 7.0E-4, -5.0E-4, -0.0013, 0.0},
                         new double[] {4.0E-4, 0.0016, 0.0026, 0.0042, 0.0037, 0.0021, 0.0033, -8.0E-4, 5.0E-4},
                         new double[] {8.0E-4, 0.0026, 0.0044, 0.0066, 0.0069, 0.0043, -9.0E-4, 0.0018, 0.0023},
                         new double[] {6.0E-4, 0.0042, 0.0066, 0.012, 0.0103, 0.0038, 0.0020, 0.0029, 0.0038},
                         new double[] {2.0E-4, 0.0037, 0.0069, 0.0103, 0.0282, 0.0224, 0.0304, 0.0214, 0.0121},
                         new double[] {7.0E-4, 0.0021, 0.0043, 0.0038, 0.0224, 0.0386, 0.024, 0.0223, 0.0193},
                         new double[] {-5.0E-4, 0.0033, -9.0E-4, 0.0020, 0.0304, 0.024, 0.0553, 0.0272, 0.0482},
                         new double[] {-0.0013, -8.0E-4, 0.0018, 0.0029, 0.0214, 0.0223, 0.0272, 0.0517, 0.0585},
                         new double[] {0.0, 5.0E-4, 0.0023, 0.0038, 0.0121, 0.0193, 0.0482, 0.0585, 0.0835}}, // covarianceMatrix
         new double[] {0.0453, 0.0618, 0.0696, 0.0767, 0.1024, 0.0855, 0.1229, 0.0908, 0.0738}, // expectedReturns
         45, // numberInterpolationPoints
         0.01 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The Efficient Frontier has now been calculated and set  to  a  private" );
      System.Console.WriteLine( "    field." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Expected Returns at the Efficient Frontier Points +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We read-off the values of the expected returns which have  been  set" );
      System.Console.WriteLine( "     to private fields." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Here we evaluate the values of the expected returns of  the  Efficient
      // Frontier.
      double[] returns1 = instanceOfMarkowitz.GetEfficientFrontierExpectedReturns( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the expected return at  the  evaluation  points  of  the" );
      System.Console.WriteLine( "    Efficient Frontier are:" );
      System.Console.WriteLine( " " );
      displayArray( returns1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Values of the Risk at the Efficient Frontier Points +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the values of the risks of  the  portfolio  at  the" );
      System.Console.WriteLine( "     point at which it has been evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = {{8.0E-4, 4.0E-4, 8.0E-4, 6.0E-4, 2.0E-4, 7.0E-4, -5.0E-4, -0.0013, 0.0}," );
      System.Console.WriteLine( "                        {4.0E-4, 0.0016, 0.0026, 0.0042, 0.0037, 0.0021, 0.0033, -8.0E-4, 5.0E-4}," );
      System.Console.WriteLine( "                        {8.0E-4, 0.0026, 0.0044, 0.0066, 0.0069, 0.0043, -9.0E-4, 0.0018, 0.0023}," );
      System.Console.WriteLine( "                        {6.0E-4, 0.0042, 0.0066, 0.012, 0.0103, 0.0038, 0.0020, 0.0029, 0.0038}," );
      System.Console.WriteLine( "                        {2.0E-4, 0.0037, 0.0069, 0.0103, 0.0282, 0.0224, 0.0304, 0.0214, 0.0121}," );
      System.Console.WriteLine( "                        {7.0E-4, 0.0021, 0.0043, 0.0038, 0.0224, 0.0386, 0.024, 0.0223, 0.0193}," );
      System.Console.WriteLine( "                        {-5.0E-4, 0.0033, -9.0E-4, 0.0020, 0.0304, 0.024, 0.0553, 0.0272, 0.0482}," );
      System.Console.WriteLine( "                        {-0.0013, -8.0E-4, 0.0018, 0.0029, 0.0214, 0.0223, 0.0272, 0.0517, 0.0585}," );
      System.Console.WriteLine( "                        {0.0, 5.0E-4, 0.0023, 0.0038, 0.0121, 0.0193, 0.0482, 0.0585, 0.0835}}" );
      // Here we evaluate the values of the risk of the Efficient  Frontier  at
      // the points at which it is evaluated.
      double[] risk1 = instanceOfMarkowitz.GetEfficientFrontierPortfolioRisks( 
         new double[][] {new double[] {8.0E-4, 4.0E-4, 8.0E-4, 6.0E-4, 2.0E-4, 7.0E-4, -5.0E-4, -0.0013, 0.0},
                         new double[] {4.0E-4, 0.0016, 0.0026, 0.0042, 0.0037, 0.0021, 0.0033, -8.0E-4, 5.0E-4},
                         new double[] {8.0E-4, 0.0026, 0.0044, 0.0066, 0.0069, 0.0043, -9.0E-4, 0.0018, 0.0023},
                         new double[] {6.0E-4, 0.0042, 0.0066, 0.012, 0.0103, 0.0038, 0.0020, 0.0029, 0.0038},
                         new double[] {2.0E-4, 0.0037, 0.0069, 0.0103, 0.0282, 0.0224, 0.0304, 0.0214, 0.0121},
                         new double[] {7.0E-4, 0.0021, 0.0043, 0.0038, 0.0224, 0.0386, 0.024, 0.0223, 0.0193},
                         new double[] {-5.0E-4, 0.0033, -9.0E-4, 0.0020, 0.0304, 0.024, 0.0553, 0.0272, 0.0482},
                         new double[] {-0.0013, -8.0E-4, 0.0018, 0.0029, 0.0214, 0.0223, 0.0272, 0.0517, 0.0585},
                         new double[] {0.0, 5.0E-4, 0.0023, 0.0038, 0.0121, 0.0193, 0.0482, 0.0585, 0.0835}} // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the risks at the  evaluation  points  of  the  Efficient" );
      System.Console.WriteLine( "    Frontier are:" );
      System.Console.WriteLine( " " );
      displayArray( risk1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Values of the Weights of the Portfolios on the Efficient Frontier +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierAssetWeights(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we read off the weights of the  portfolios  at  the  points  at" );
      System.Console.WriteLine( "     which the Efficient Frontier has been evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Weioghts of the portfolio on the Efficient Frontier.
      double[][] weights1 = instanceOfMarkowitz.GetEfficientFrontierAssetWeights( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the portfolios are:" );
      System.Console.WriteLine( " " );
      display2DArray( weights1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Direct Approach to Evaluating the Portfolio with  an  Expected  Return +" );
      System.Console.WriteLine( "+ of 0.07                                                                +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    efficientFrontier( double expectedReturn," );
      System.Console.WriteLine( "                       double[][] covarianceMatrix," );
      System.Console.WriteLine( "                       double[] expectedReturns," );
      System.Console.WriteLine( "                       double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the  weights  of  the  optimal  portfolio  with  an" );
      System.Console.WriteLine( "     expected return of 0.07." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.07" );
      System.Console.WriteLine( "    covarianceMatrix = {{8.0E-4, 4.0E-4, 8.0E-4, 6.0E-4, 2.0E-4, 7.0E-4, -5.0E-4, -0.0013, 0.0}," );
      System.Console.WriteLine( "                        {4.0E-4, 0.0016, 0.0026, 0.0042, 0.0037, 0.0021, 0.0033, -8.0E-4, 5.0E-4}," );
      System.Console.WriteLine( "                        {8.0E-4, 0.0026, 0.0044, 0.0066, 0.0069, 0.0043, -9.0E-4, 0.0018, 0.0023}," );
      System.Console.WriteLine( "                        {6.0E-4, 0.0042, 0.0066, 0.012, 0.0103, 0.0038, 0.0020, 0.0029, 0.0038}," );
      System.Console.WriteLine( "                        {2.0E-4, 0.0037, 0.0069, 0.0103, 0.0282, 0.0224, 0.0304, 0.0214, 0.0121}," );
      System.Console.WriteLine( "                        {7.0E-4, 0.0021, 0.0043, 0.0038, 0.0224, 0.0386, 0.024, 0.0223, 0.0193}," );
      System.Console.WriteLine( "                        {-5.0E-4, 0.0033, -9.0E-4, 0.0020, 0.0304, 0.024, 0.0553, 0.0272, 0.0482}," );
      System.Console.WriteLine( "                        {-0.0013, -8.0E-4, 0.0018, 0.0029, 0.0214, 0.0223, 0.0272, 0.0517, 0.0585}," );
      System.Console.WriteLine( "                        {0.0, 5.0E-4, 0.0023, 0.0038, 0.0121, 0.0193, 0.0482, 0.0585, 0.0835}}" );
      System.Console.WriteLine( "    expectedReturns = {0.0453, 0.0618, 0.0696, 0.0767, 0.1024, 0.0855, 0.1229, 0.0908, 0.0738}" );
      System.Console.WriteLine( "    precision = 0.01" );
      // Here we evaluate the Optimal portfolio  with  an  expected  return  of
      // 0.07.
      double[] efficientDirect1 = instanceOfMarkowitz.EfficientFrontier( 
         0.07, // expectedReturn
         new double[][] {new double[] {8.0E-4, 4.0E-4, 8.0E-4, 6.0E-4, 2.0E-4, 7.0E-4, -5.0E-4, -0.0013, 0.0},
                         new double[] {4.0E-4, 0.0016, 0.0026, 0.0042, 0.0037, 0.0021, 0.0033, -8.0E-4, 5.0E-4},
                         new double[] {8.0E-4, 0.0026, 0.0044, 0.0066, 0.0069, 0.0043, -9.0E-4, 0.0018, 0.0023},
                         new double[] {6.0E-4, 0.0042, 0.0066, 0.012, 0.0103, 0.0038, 0.0020, 0.0029, 0.0038},
                         new double[] {2.0E-4, 0.0037, 0.0069, 0.0103, 0.0282, 0.0224, 0.0304, 0.0214, 0.0121},
                         new double[] {7.0E-4, 0.0021, 0.0043, 0.0038, 0.0224, 0.0386, 0.024, 0.0223, 0.0193},
                         new double[] {-5.0E-4, 0.0033, -9.0E-4, 0.0020, 0.0304, 0.024, 0.0553, 0.0272, 0.0482},
                         new double[] {-0.0013, -8.0E-4, 0.0018, 0.0029, 0.0214, 0.0223, 0.0272, 0.0517, 0.0585},
                         new double[] {0.0, 5.0E-4, 0.0023, 0.0038, 0.0121, 0.0193, 0.0482, 0.0585, 0.0835}}, // covarianceMatrix
         new double[] {0.0453, 0.0618, 0.0696, 0.0767, 0.1024, 0.0855, 0.1229, 0.0908, 0.0738}, // expectedReturns
         0.01 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the optimal portfolio are:" );
      System.Console.WriteLine( " " );
      displayArray( efficientDirect1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
