using System;
using WebCab.Lib.Util.Example;

using WebCab.Libraries.Finance.Portfolio;

public class Optimal2Client {
   public static void Main( String[] args ) {
      // used components
      AssetParameters instanceOfAssetParameters = new AssetParameters();
      Markowitz instanceOfMarkowitz = new Markowitz();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "                       ###################################" );
      System.Console.WriteLine( "                       # Optimal2 Console Client Example #" );
      System.Console.WriteLine( "                       ###################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Example of the Application of Markowitz Theory" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    Within this example we will consider the construction the  efficient" );
      System.Console.WriteLine( "    frontier for the portfolio  which  can  be  constructed  from  three" );
      System.Console.WriteLine( "    managed fund with returns of 12 periods of:" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "       - Fund 1: 1.14, 0.47, 0.84, 0.93, 1.1,  0.82,  0.63,  0.72,  0.8," );
      System.Console.WriteLine( "       1.06, 0.79, 0.78" );
      System.Console.WriteLine( "       - Fund 2: -6.06, -4.15, 1.37,  4.29,  1.37,  -1.21,  4.11,  2.19," );
      System.Console.WriteLine( "       -3.71, 1.18, -1.25, 1.29" );
      System.Console.WriteLine( "       - Fund 3: -1.7, -4.9, 3.64, 4.14, 1.14, 1.66, 4.18, 4.01,  -0.11," );
      System.Console.WriteLine( "       4.24, -2.26, 4.33" );
      System.Console.WriteLine( "       " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    (The above returns are given in percentage form (i.e. 1 = 1  percent" );
      System.Console.WriteLine( "    = 0.01))" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    Within this example we will construct  the  efficient  frontier  and" );
      System.Console.WriteLine( "    then `read off' the optimal portfolio for a given level  of  return." );
      System.Console.WriteLine( "    We will then construct the optimal  portfolio  for  three  different" );
      System.Console.WriteLine( "    investors with relatively a low, medium and high risk tolerance." );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    Each utility function for the  three  investors  is  given  on  five" );
      System.Console.WriteLine( "    points represented a pairs risk and reward  points  on  the  utility" );
      System.Console.WriteLine( "    curve. The three investors correspond to:" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "       - Low Risk Tolerance Investor" );
      System.Console.WriteLine( "       - Medium Risk Investor" );
      System.Console.WriteLine( "       - Higher Risk Investor" );
      System.Console.WriteLine( "       " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    The above returns and risk correspond  to  monthly  expected  return" );
      System.Console.WriteLine( "    and corresponding risk with the  three  investors  are  prepared  to" );
      System.Console.WriteLine( "    accept." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ We first evaluate the Covariance matrix for the assets returns +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix( double[][] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the covariance matrix of the returns of  the  three" );
      System.Console.WriteLine( "     assets under consideration." );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     The returns of the three funds considered over each period is  given" );
      System.Console.WriteLine( "     as a percentage (rather than an absolute value).  That  is,  if  the" );
      System.Console.WriteLine( "     return in the i-th period is 5 percent, then the i-th value  in  the" );
      System.Console.WriteLine( "     corresponding funds array is 5. Please note  that  since  the  asset" );
      System.Console.WriteLine( "     weights and similar quantities are within unit as long of the  units" );
      System.Console.WriteLine( "     used within the application of portfolio theory are  consistent  you" );
      System.Console.WriteLine( "     are free to use your preferred units." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalReturns = {{0.14, 0.47, 0.84, 0.93, 1.1, 0.82, 0.63, 0.72, 0.8, 1.06, 0.79, 0.78}," );
      System.Console.WriteLine( "                         {-2.06, -1.15, 1.37, 2.29, 1.37, -1.21, 3.11, 2.19, -3.71, 1.18, -1.25, 1.29}," );
      System.Console.WriteLine( "                         {-0.7, -0.9, 3.64, 4.14, 1.1399, 1.66, 4.18, 4.01, -0.11, 4.24, -2.2599, 4.33}}" );
      // Here we evaluate the covariance of the returns  of  the  assets  under
      // consideration
      double[][] covariance1 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {0.14, 0.47, 0.84, 0.93, 1.1, 0.82, 0.63, 0.72, 0.8, 1.06, 0.79, 0.78},
                         new double[] {-2.06, -1.15, 1.37, 2.29, 1.37, -1.21, 3.11, 2.19, -3.71, 1.18, -1.25, 1.29},
                         new double[] {-0.7, -0.9, 3.64, 4.14, 1.1399, 1.66, 4.18, 4.01, -0.11, 4.24, -2.2599, 4.33}} // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The covariance matrix of the returns of the three asset is:" );
      System.Console.WriteLine( " " );
      display2DArray( covariance1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ next we evaluate the expected return of each fund +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns( double[][] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we evaluate the expected return of each fund" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalReturns = {{0.14, 0.47, 0.84, 0.93, 1.1, 0.82, 0.63, 0.72, 0.8, 1.06, 0.79, 0.78}," );
      System.Console.WriteLine( "                         {-2.06, -1.15, 1.37, 2.29, 1.37, -1.21, 3.11, 2.19, -3.71, 1.18, -1.25, 1.29}," );
      System.Console.WriteLine( "                         {-0.7, -0.9, 3.64, 4.14, 1.1399, 1.66, 4.18, 4.01, -0.11, 4.24, -2.2599, 4.33}}" );
      // Here we evaluate the expected reurn of each of the funds.
      double[] expectedReturns1 = instanceOfAssetParameters.ExpectedReturns( 
         new double[][] {new double[] {0.14, 0.47, 0.84, 0.93, 1.1, 0.82, 0.63, 0.72, 0.8, 1.06, 0.79, 0.78},
                         new double[] {-2.06, -1.15, 1.37, 2.29, 1.37, -1.21, 3.11, 2.19, -3.71, 1.18, -1.25, 1.29},
                         new double[] {-0.7, -0.9, 3.64, 4.14, 1.1399, 1.66, 4.18, 4.01, -0.11, 4.24, -2.2599, 4.33}} // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The expected return of the funds is:" );
      System.Console.WriteLine( " " );
      displayArray( expectedReturns1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Risk of the first fund +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    portfolioRisk( double[] weight," );
      System.Console.WriteLine( "                   double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we evaluate the risk of the first of the three  funds.  Recall," );
      System.Console.WriteLine( "     that the risk of any portfolio constructed using these  three  funds" );
      System.Console.WriteLine( "     is less than the highest risk of the three funds." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weight = {1.0, 0.0, 0.0}" );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      // Here we evaluate the risk of the first fund
      double riskFund1 = instanceOfAssetParameters.PortfolioRisk( 
         new double[] {1.0, 0.0, 0.0}, // weight
         covariance1 // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the first fund is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskFund1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Risk of the second fund +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    portfolioRisk( double[] weight," );
      System.Console.WriteLine( "                   double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we evaluate the risk of the second of the three funds.  Recall," );
      System.Console.WriteLine( "     that the risk of any portfolio constructed using these  three  funds" );
      System.Console.WriteLine( "     is less than the highest risk of the three funds." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weight = {0.0, 1.0, 0.0}" );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      // Here we evaluate the risk of the second fund
      double riskFund2 = instanceOfAssetParameters.PortfolioRisk( 
         new double[] {0.0, 1.0, 0.0}, // weight
         covariance1 // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the second fund is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskFund2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we evaluate the Risk of the third fund +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    portfolioRisk( double[] weight," );
      System.Console.WriteLine( "                   double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we evaluate the risk of the third of the three  funds.  Recall," );
      System.Console.WriteLine( "     that the risk of any portfolio constructed using these  three  funds" );
      System.Console.WriteLine( "     is less than the highest risk of the three funds." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weight = {0.0, 0.0, 1.0}" );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      // Here we evaluate the risk of the third fund
      double riskFund3 = instanceOfAssetParameters.PortfolioRisk( 
         new double[] {0.0, 0.0, 1.0}, // weight
         covariance1 // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the third fund is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskFund3 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Construction of the Efficient Frontier +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    calculateEfficientFrontier( double[][] covarianceMatrix," );
      System.Console.WriteLine( "                                double[] expectedReturns," );
      System.Console.WriteLine( "                                int numberInterpolationPoints," );
      System.Console.WriteLine( "                                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we construct 30 equally distributed  points  on  the  Efficient" );
      System.Console.WriteLine( "     frontier over its entire range using a tolerance of 0.001." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    numberInterpolationPoints = 20" );
      System.Console.WriteLine( "    precision = 0.000001" );
      // Here we construct the efficient frontier at 30 points.
      instanceOfMarkowitz.CalculateEfficientFrontier( 
         covariance1, // covarianceMatrix
         expectedReturns1, // expectedReturns
         20, // numberInterpolationPoints
         0.000001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Once the point of the efficient frontier have  been  constructed  they" );
      System.Console.WriteLine( "    are set persisted within private fields." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Minimum  value  of  the  expected  return  over  which  the  Efficient +" );
      System.Console.WriteLine( "+ Frontier exits                                                         +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minFrontierReturn( double[] expectedReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we evaluate the minimum  expected  return  of  the  range  over" );
      System.Console.WriteLine( "     which the Efficient Frontier  exits.  As  a  general  remark  it  is" );
      System.Console.WriteLine( "     usually best to evaluate the range of the  expected  return  of  the" );
      System.Console.WriteLine( "     Efficient Frontier in order  to  provide  a  range  over  which  the" );
      System.Console.WriteLine( "     Efficient Frontier can be evaluated and the  range  over  which  the" );
      System.Console.WriteLine( "     optimal portfolio can be sort." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      double min1 = instanceOfMarkowitz.MinFrontierReturn( 
         expectedReturns1 // expectedReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The minimum  bound  of  the  range  of  the  expected  return  of  the" );
      System.Console.WriteLine( "    Efficient Frontier is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( min1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Maximum  value  of  the  expected  return  over  which  the  Efficient +" );
      System.Console.WriteLine( "+ Frontier exits                                                         +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    maxFrontierReturn( double[] expectedReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we evaluate the maximum  expected  return  of  the  range  over" );
      System.Console.WriteLine( "     which the Efficient Frontier  exits.  As  a  general  remark  it  is" );
      System.Console.WriteLine( "     usually best to evaluate the range of the  expected  return  of  the" );
      System.Console.WriteLine( "     Efficient Frontier in order  to  provide  a  range  over  which  the" );
      System.Console.WriteLine( "     Efficient Frontier can be evaluated and the  range  over  which  the" );
      System.Console.WriteLine( "     optimal portfolio can be sort." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      double max1 = instanceOfMarkowitz.MaxFrontierReturn( 
         expectedReturns1 // expectedReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The maximum  bound  of  the  range  of  the  expected  return  of  the" );
      System.Console.WriteLine( "    Efficient Frontier is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( max1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Find the Optimal Portfolios for an investor with a Low Risk Tolerance +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    optimalPortfolio( double minimumExpectedReturn," );
      System.Console.WriteLine( "                      double maximumExpectedReturn," );
      System.Console.WriteLine( "                      double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we evaluate the weights  of  the  optimal  portfolios  for  the" );
      System.Console.WriteLine( "     investor with a low risk tolerance described by  the  given  utility" );
      System.Console.WriteLine( "     function. Note, that we search for the optimal  portfolio  over  the" );
      System.Console.WriteLine( "     entire range of the expected returns (evaluated  above)  over  which" );
      System.Console.WriteLine( "     the Efficient Frontier exits." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = min1" );
      System.Console.WriteLine( "    maximumExpectedReturn = max1" );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      // Here we set the utility function of the investor  with  the  low  risk
      // tolerance
      instanceOfMarkowitz.SetUtilityFunctionInterp( 
         new double[] {0.0, 0.2, 0.4, 0.8, 1.53}, // expectedReturn
         new double[] {0.0, 0.1, 0.2, 0.3, 0.4} // totalRisk
      );
      // Here we find the optimal portfolio(s).
      double[][] optimalLow1 = instanceOfMarkowitz.OptimalPortfolio( 
         min1, // minimumExpectedReturn
         max1, // maximumExpectedReturn
         covariance1 // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the optimal portfolio(s) for the investor  with  a  low" );
      System.Console.WriteLine( "    risk tolerance described by the given utility function are:" );
      System.Console.WriteLine( " " );
      display2DArray( optimalLow1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Find the  Optimal  Portfolio  for  an  investor  with  a  Medium  Risk +" );
      System.Console.WriteLine( "+ Tolerance                                                              +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    optimalPortfolio( double minimumExpectedReturn," );
      System.Console.WriteLine( "                      double maximumExpectedReturn," );
      System.Console.WriteLine( "                      double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we evaluate the weights  of  the  optimal  portfolios  for  the" );
      System.Console.WriteLine( "     investor with  a  medium  risk  tolerance  described  by  the  given" );
      System.Console.WriteLine( "     utility function." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = min1" );
      System.Console.WriteLine( "    maximumExpectedReturn = max1" );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      // Here we set the utility function of the investor with the medium  risk
      // tolerance
      instanceOfMarkowitz.SetUtilityFunctionInterp( 
         new double[] {0.0, 0.4, 0.6, 0.8, 1.2}, // expectedReturn
         new double[] {0.0, 0.1, 0.2, 1.2, 2.3} // totalRisk
      );
      // Here we find the optimal portfolio(s).
      double[][] optimalMedium1 = instanceOfMarkowitz.OptimalPortfolio( 
         min1, // minimumExpectedReturn
         max1, // maximumExpectedReturn
         covariance1 // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the  optimal  portfolio(s)  for  the  investor  with  a" );
      System.Console.WriteLine( "    medium risk tolerance are:" );
      System.Console.WriteLine( " " );
      display2DArray( optimalMedium1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Find the Optimal Portfolio for an investor with a High Risk Tolerance +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    optimalPortfolio( double minimumExpectedReturn," );
      System.Console.WriteLine( "                      double maximumExpectedReturn," );
      System.Console.WriteLine( "                      double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we evaluate the weights of the  optimal  portfolio(s)  for  the" );
      System.Console.WriteLine( "     investor with a high risk tolerance described by the  given  utility" );
      System.Console.WriteLine( "     function." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = min1" );
      System.Console.WriteLine( "    maximumExpectedReturn = max1" );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      // Here we set the utility function of the investor with  the  high  risk
      // tolerance
      instanceOfMarkowitz.SetUtilityFunctionInterp( 
         new double[] {0.0, 0.4, 0.8, 1.2, 1.5}, // expectedReturn
         new double[] {0.0, 0.1, 0.2, 1.5, 2.0} // totalRisk
      );
      // Here we find the optimal portfolio(s).
      double[][] optimalHigh1 = instanceOfMarkowitz.OptimalPortfolio( 
         min1, // minimumExpectedReturn
         max1, // maximumExpectedReturn
         covariance1 // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the optimal portfolio(s) for the investor with  a  high" );
      System.Console.WriteLine( "    risk tolerance are:" );
      System.Console.WriteLine( " " );
      display2DArray( optimalHigh1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
