using System;
using WebCab.Lib.Util.Example;

using WebCab.Libraries.Finance.Portfolio;

public class Optimal4Risk4Return50FundsClient {
   public static void Main( String[] args ) {
      // used components
      AssetParameters instanceOfAssetParameters = new AssetParameters();
      Markowitz instanceOfMarkowitz = new Markowitz();
      SolveFrontier instanceOfSolveFrontier = new SolveFrontier();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "              #####################################################" );
      System.Console.WriteLine( "              # Optimal4Risk4Return50Funds Console Client Example #" );
      System.Console.WriteLine( "              #####################################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Within this client we apply the Markowitz Theory  to  construct  the" );
      System.Console.WriteLine( "    portfolio which has:" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "       - The lowest risk for a  given  expected  return  (i.e.  expected" );
      System.Console.WriteLine( "       performance level)" );
      System.Console.WriteLine( "       - The highest expected return  (i.e.  performance)  for  a  given" );
      System.Console.WriteLine( "       level of risk" );
      System.Console.WriteLine( "       " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    for 50 investment funds where each of the funds  historical  monthly" );
      System.Console.WriteLine( "    returns is known over the past 3 years (approx.)." );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    The historical values of the  funds  is  given  within  a  array  of" );
      System.Console.WriteLine( "    dimension 2 where the first elements consists of  50  members  which" );
      System.Console.WriteLine( "    state the returns in Jan 2001, the second element the return in  Feb" );
      System.Console.WriteLine( "    2001, and so on." );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    Remark: In order to address the first problem it will be  sufficient" );
      System.Console.WriteLine( "    to apply methods from the Markowitz class, in  order  to  solve  for" );
      System.Console.WriteLine( "    risk we  will  need  to  use  in  addition  functionality  from  the" );
      System.Console.WriteLine( "    SolveFrontier class." );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    WARNING: This example will take several minutes to run." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluation of the Expected Returns of the  Covariance  Matrix  of  the +" );
      System.Console.WriteLine( "+ six funds                                                              +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns( double[][] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     We first evaluate the auxiliary results of  the  evaluation  of  the" );
      System.Console.WriteLine( "     expected return and covariance matrix for the six funds." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalReturns = returnsOfFunds" );
      // Here we transform the format which the source  data,  by  transposing'
      // the array of dimension 2.
      double[][] returnsOfFunds = instanceOfAssetParameters.Transpose( 
         new double[][] {new double[] {4.2584, 4.0595, -1.3818, 3.7248, 1.1662, 1.6102, 0.5642, 1.5076, 3.9479, 2.8466, 4.819, 0.4447, -0.8333, 4.01, 4.1951, 12.0877, 1.569, 4.3654, 1.5046, -1.7855, 5.7723, 7.4756, 0.5837, 4.4581, 9.4602, 2.4881, 1.35, -2.2599, 1.3682, 4.1594, 2.2886, 1.5899, 1.2548, 3.1662, 0.9318, 2.9615, 2.4018, -1.5833, 1.2008, 0.5178, 0.5248, 0.5563, 0.5148, 1.5268, 1.0041, -0.0961, 1.568, 5.998, 2.348, -0.502},
                         new double[] {0.2075, -1.3283, 0.1756, 0.0685, 0.4482, 0.1387, 0.5575, 0.1522, -6.3592, 0.3673, -4.6227, -2.9884, 0.5578, 0.03, 0.1669, -2.7902, -3.4544, -0.2373, 0.5306, 1.0339, -0.8961, -4.704, 0.4898, -0.3679, -1.1535, 1.2787, 0.58, 1.0, 0.6749, 0.5321, 0.3754, 0.5583, 0.6497, -1.7007, 0.9324, -4.093, -8.2146, 0.9961, 0.4002, 0.4593, 0.4556, 0.4742, 0.4451, 0.5203, 0.9444, 0.9467, 0.699, -1.071, 0.392, -1.033},
                         new double[] {-5.2432, -7.4548, -0.6309, -3.6981, 0.585, -0.3711, 0.5433, -0.7497, -3.3339, -2.2185, 1.1918, -1.4261, -0.0396, -3.59, -2.4365, -0.7917, -0.2814, -4.7439, 0.6445, -1.6698, -3.5614, -3.6338, 0.4279, -3.3235, 0.2537, -7.978, 0.66, -1.29, 0.5785, -3.8462, -1.0173, -0.1935, 0.5064, 1.5577, 0.5488, 0.392, -5.0408, -1.3573, -1.3066, 0.4593, 0.4852, 0.5014, 0.455, 0.6347, 0.6992, -1.1286, 0.5605, -5.678, -4.8868, 1.1651},
                         new double[] {5.4143, 3.3248, 0.5252, 6.7558, -0.0649, 0.9535, 0.5117, 1.8606, 6.2906, 2.7313, 4.6195, 4.3981, 0.5629, 3.79, 5.0503, 2.2531, 2.9511, 5.7188, -1.0467, 0.674, 4.3864, 4.4505, 0.4636, 3.4377, 1.9343, 4.0858, -0.58, 1.0, -0.8636, 6.3852, 2.5276, 0.8926, -0.4904, 1.5794, -0.8551, 2.5928, 6.6601, 0.5904, 2.2458, 0.4298, 0.4158, 0.4455, 0.4161, -0.4681, -0.89, 2.1275, -0.8051, 5.1499, 2.5648, -1.1347},
                         new double[] {2.0733, 2.0821, 1.6233, 0.6395, -0.2189, 0.5012, 0.465, 0.8626, -0.8569, 0.9813, -1.2665, -1.0532, 1.0724, 2.75, 2.4103, 7.7893, -0.7206, 1.8663, -0.5242, 1.6168, 1.8281, 0.6056, 0.4939, 2.8804, 0.1876, 2.3798, -0.33, 1.45, -0.3598, 2.1974, 0.8911, 0.4381, 0.0604, 0.1157, 0.4771, -0.6232, -0.864, 1.4133, 1.4818, 0.4102, 0.3958, 0.4238, 0.1585, 0.1568, 0.1283, 1.3358, -0.3103, 2.3367, 6.4095, -1.9937},
                         new double[] {4.5148, 1.2904, 2.8109, 4.6969, -0.0897, 0.8886, 0.4633, 1.8278, -2.6966, 2.6744, -4.0698, -3.2773, 3.6433, 4.46, 2.6867, -0.9427, -3.5062, 3.9051, 0.0575, 4.6094, 5.2571, -3.2182, 0.5592, 3.9821, -1.2994, 1.3729, 0.17, 3.61, 0.0855, 4.0819, 1.4058, 0.7046, 0.1809, -2.2463, 0.6118, -3.5087, -2.5799, 4.4993, 1.2004, 0.3806, 0.3952, 0.4515, 0.3289, 0.4548, 0.1872, 3.4602, 0.0118, 3.6632, 8.878, -4.8258},
                         new double[] {-4.4194, -3.0185, 1.8265, -4.015, 0.3434, -0.372, 0.465, -0.8706, -1.0673, -1.5607, -1.8161, -2.9274, 0.8426, -4.49, -3.1718, -4.2824, -1.4296, -4.7662, 0.3403, 1.0993, -4.2768, -2.3736, 0.6221, -3.51, -2.8434, -2.8293, 0.19, 0.54, 0.462, -4.4832, -1.218, -0.0118, 0.767, -0.7531, 1.428, -1.1288, -2.0337, 0.7806, -2.2174, 0.421, 0.4158, 0.4928, 0.4673, -0.2526, 1.8307, 1.4354, 0.4133, -5.1799, -8.3033, -2.1586},
                         new double[] {-0.6765, 0.8308, 4.4965, -0.6308, 1.9365, 0.4007, 0.4517, 0.9004, -8.5827, 0.6302, -7.0, -9.4088, 2.0395, -0.55, 0.4836, -6.5388, -8.4004, -1.1039, 2.8827, 3.5256, -0.2397, -1.221, 0.4119, -0.1485, 0.6879, 2.964, 2.45, 3.73, 2.8028, -0.6364, 0.0942, 0.9279, 1.96, -5.7037, 1.0058, -8.2052, -5.9469, 3.4643, -0.6707, 0.4287, 0.4338, 0.5103, 1.1778, 3.7983, 1.0517, 4.118, 2.5314, -0.9081, 2.8692, -8.2212},
                         new double[] {-8.1724, -9.8035, -0.3957, -6.782, 0.9819, -0.7177, 0.4483, -1.0102, 4.9096, -2.6303, 7.0789, 6.9907, 0.3106, -7.16, -4.3579, -8.8112, -2.6908, -6.1316, 1.0743, -0.4142, -5.0776, -5.9892, 0.4202, -5.8736, -2.4594, -12.4488, 1.4, -0.52, 1.0828, -6.8294, -2.8466, -1.0678, 0.4285, -1.4201, 0.8953, -2.7232, 9.4454, -0.2814, -2.6578, 0.4189, 0.4025, 0.361, 0.5086, -1.0469, 0.8751, -2.0871, 1.1496, -8.441, -10.8917, -2.8687},
                         new double[] {7.5282, 7.4072, 3.1582, 6.803, 1.096, 2.2219, 0.4425, 2.1695, 4.6823, 3.6752, 1.6715, 1.36, 1.8179, 6.7, 5.1093, 6.0475, 0.3386, 6.729, 1.6631, 3.5939, 6.0843, 4.0961, 0.5856, 6.793, 5.7116, 7.0223, 1.6299, 3.09, 1.529, 7.5531, 3.7754, 2.3956, 1.3644, 0.3267, 1.9747, -0.444, 2.3056, 3.6231, 2.6962, 0.3615, 0.366, 0.2264, 0.3556, 1.2081, 1.981, 4.9828, 1.6, 7.08, 12.1, 0.16},
                         new double[] {2.173, 1.9044, 1.1468, 3.5353, -0.6313, 0.4432, 0.4192, 0.5876, 1.9436, 0.7759, 5.588, 3.4605, 0.2105, 2.6, 4.0904, 5.1315, 3.9464, 3.1079, -1.0451, -0.2688, 2.2584, 4.3231, 0.3806, 3.5503, 3.2044, 1.9837, -1.28, -0.44, -0.7438, 3.4173, 2.1782, 1.0236, -0.3014, 3.2327, -0.5919, 2.6508, 6.0016, -0.1467, 0.7947, 0.3506, 0.3449, 0.4026, 0.4429, -0.806, -0.6879, 2.8624, -1.07, 4.11, 1.024, 1.3759},
                         new double[] {2.432, 2.0568, 1.5549, 2.6459, -0.5867, 0.3932, 0.4175, 0.9922, 3.1862, 0.9754, 3.5332, 3.0716, 1.2604, 2.14, 1.7129, 3.7772, 3.1022, 2.5535, -1.1119, 1.9312, 2.2197, 1.8022, 0.5089, 2.1815, 1.0155, 2.4539, -1.06, 2.71, -1.0362, 2.794, 1.3218, 0.3575, -0.5038, 2.539, -0.6766, 2.226, 1.0822, 2.0106, 1.7719, 0.3494, 0.3536, 0.3815, 0.1372, -0.3703, -0.5296, 1.9096, -0.912, 2.107, 3.383, 3.335},
                         new double[] {0.428, 2.2902, -0.7354, 1.1425, -0.1428, 0.0306, 0.415, 0.3781, -2.0881, 0.4743, 3.7416, -2.9801, 1.0805, 2.0299, 3.2732, -0.1661, -2.7639, 1.2795, 0.0188, -0.8088, 1.4162, 0.7159, 0.4551, 2.5641, 2.2244, 2.3493, -0.09, -1.67, 0.0474, 1.2265, 0.1819, 0.2051, 0.327, -1.2381, 0.3384, -2.1133, -1.8279, -1.4327, -0.1421, 0.3523, 0.376, 0.4718, 0.2569, 0.7264, 0.5898, -0.4146, -0.078, 1.78, 3.859, -2.737},
                         new double[] {-2.5753, 2.2097, 0.2623, -2.0644, 0.3526, 0.0599, 0.4108, -0.3432, -1.887, -0.4638, 2.9669, -1.3652, 0.4726, -0.56, 0.9395, -3.4793, -2.8965, -1.1467, 0.5741, 0.6035, -1.266, 2.0689, 0.4066, 0.4861, 2.7582, 1.2818, 0.58, 0.44, 0.5775, -0.7133, -0.1211, 0.145, 0.4991, -1.7019, 0.4473, -2.7071, -0.8246, 0.5881, -1.0244, 0.312, 0.3352, 0.2935, 0.3549, 0.9271, 0.6584, 1.1995, 0.609, 1.157, 1.49, -1.3939},
                         new double[] {0.1675, 2.9455, 1.0995, 1.195, -0.623, 0.0701, 0.41, 0.0078, 4.3077, 0.0566, 3.4756, 2.2145, 0.6246, -0.03, 0.6981, 4.9155, 1.2043, 0.8648, -1.1136, 1.3177, 0.2846, 5.7821, 0.4143, 0.8293, 3.3827, 2.6256, -1.0, 0.66, -0.8755, 0.2657, 0.3864, 0.0085, -0.5068, 0.6956, -0.7634, 1.1374, 4.2108, 1.1053, 0.213, 0.3402, 0.3832, 0.4975, 0.1277, -0.4593, -1.0629, 2.1196, -0.805, 0.208, -2.575, 0.81},
                         new double[] {-2.1836, -2.1723, 1.4368, -2.4582, 1.1671, 0.3285, 0.4108, -0.0509, -3.069, -0.5154, -3.5684, 3.1144, 1.2137, -2.2, -0.211, 0.6, 5.1625, -1.9241, 1.4479, 1.6617, -1.676, -0.4107, 0.4691, 0.1371, 0.2979, -4.7027, 1.68, 1.3, 1.5195, -1.1308, -0.1364, 0.5382, 1.2257, -2.2336, 1.4221, -4.2055, -4.6326, 1.5822, -0.7107, 0.3621, 0.3758, 0.4121, 0.5328, 1.0926, 1.6217, 1.7867, 1.48, -0.62, 2.15, 1.6},
                         new double[] {0.2565, -1.454, 2.9613, 1.5431, -0.328, -0.1255, 0.405, -0.0936, -4.3388, 0.0509, -4.4025, -5.7303, 0.8944, 1.47, 1.7147, -5.842, 6.0129, 0.7914, -0.2892, 2.7138, 0.6707, -0.2896, 0.4654, 1.4374, -0.1432, 1.323, -0.41, 2.43, -0.29, 1.1437, 0.0455, 0.068, 0.1715, -4.5144, 0.519, -5.0213, -0.7826, 2.9511, -0.982, 0.3608, 0.3744, 0.4006, 0.1963, 0.622, 0.6404, 3.1786, -0.22, 1.69, 1.64, 4.87},
                         new double[] {-4.1311, -5.0195, 0.2054, -4.9707, 0.8506, -0.3496, 0.425, -0.4174, 5.2098, -1.2818, -6.6948, -8.0689, 3.384, -4.19, -1.8596, -2.7875, -4.7596, -4.148, 1.1173, -0.2184, -3.2824, -6.3249, 0.4423, -1.7812, -3.9894, -5.0991, 1.49, 0.14, 1.3228, -3.5899, -1.7747, -0.3312, 0.957, -4.621, 1.6034, -5.1958, -7.6839, 0.1062, -2.5515, 0.3887, 0.4018, 0.4574, 0.5554, 1.3031, 1.2524, -0.0788, 1.16, -3.61, -2.42, 3.27},
                         new double[] {-4.993, -4.5326, -1.2458, -4.7326, 0.7594, -0.6835, 0.4333, -0.6026, 6.1987, -1.6584, -7.7679, -6.6015, -0.6257, -4.45, -2.3135, 7.6469, -5.2756, -4.159, 0.8768, -1.8224, -3.652, -5.1121, 0.4385, -3.1825, -3.9283, -5.8319, 1.1399, -2.47, 0.9367, -4.2387, -2.5559, -0.9211, 0.5322, -5.3042, 1.5572, 5.5469, 8.797, -1.7333, -3.2303, 0.411, 0.4151, 0.2999, 0.4173, 0.4561, 1.2529, -2.1056, 0.9201, -3.104, 5.5857, 6.1485},
                         new double[] {2.4279, 3.4448, 1.1973, 1.2188, 0.6433, 0.7053, 0.4333, 0.4999, -0.3328, 1.4331, -1.7673, -2.5305, -0.6366, 2.04, 2.9134, -2.944, 1.4464, 1.4779, 0.8877, 1.2267, 2.2248, -0.4191, 0.4234, 2.7757, 0.0805, 3.5587, 1.01, 1.42, 1.0273, 0.3706, 0.5506, 0.7967, 1.028, -0.2752, 1.7188, -1.0777, 0.2775, 1.3331, 0.1997, 0.3887, 0.4036, 0.3513, 0.3859, -0.1548, 1.405, 1.3757, 1.0314, 3.1683, 2.5676, -0.9661},
                         new double[] {-5.0076, -3.7603, 0.8048, -5.2462, 1.0613, -1.0309, 0.4333, -1.3557, -10.1841, -2.1516, -8.7342, 9.4896, 0.9084, -5.4, -2.8426, 8.0323, -9.9503, -4.2382, 1.4568, 2.06, -3.9753, -8.8497, 0.4365, -4.6198, -8.2333, -2.9559, 1.84, 0.93, 1.61, -3.8758, -3.1056, -1.0086, 1.2749, -11.3043, 1.6898, 9.8997, 11.138, 1.5805, -3.5698, 0.4066, 0.3725, 0.3403, 0.3844, 2.501, 1.5276, 1.5876, 1.7737, -3.1768, -5.1769, 9.7855},
                         new double[] {2.3549, -0.5442, 2.3837, 1.6322, 0.1693, 1.417, 0.4333, 0.6264, 5.7689, 0.8758, 7.0426, 2.8684, 1.1092, 2.05, 1.1934, 0.1302, 6.7633, 2.5558, 0.369, 1.7079, 2.0608, 1.7601, 0.4403, 1.5648, 2.8198, -3.5626, 0.08, 2.5299, 0.1037, 2.8994, 2.5442, 1.3396, 0.0532, 6.2532, -0.1247, 5.1686, 7.5516, 2.2353, 1.2639, 0.3998, 0.3555, 0.0489, 0.3967, 0.2027, -0.0966, 1.6924, 0.196, 1.9, 3.7467, 4.7779},
                         new double[] {1.4987, -2.6256, 1.7544, 1.57, 0.3962, 0.8517, 0.4333, 0.6225, 3.2136, 0.8772, 6.7659, 3.4615, 1.1, 1.72, -0.425, 6.6529, 4.2673, 1.0555, 0.2758, 2.02, -0.3138, 6.7363, 0.4046, 0.4402, 5.8587, -1.5848, 0.16, 2.25, 0.2259, 0.5832, 1.5631, 1.0485, 0.6065, 5.2606, 0.1782, 4.2537, 5.6098, 1.8392, 1.724, 0.3885, 0.4034, 0.7625, 0.3753, 0.4551, 0.3765, 0.9519, 0.0313, -0.4302, -1.3447, 3.776},
                         new double[] {-1.2477, 2.4554, 0.5356, -1.4524, 1.4418, 0.4539, 0.4333, -0.0023, 5.4349, -0.0577, -5.6708, -4.368, 1.0959, -1.21, 0.3246, -2.3163, -5.9258, -1.6066, 2.1934, 1.5518, -2.0719, -5.1872, 0.4368, -0.8273, -3.2972, 0.8505, 1.97, 1.1, 2.1978, -0.6015, -0.3502, 0.6136, 1.7768, -4.5804, 1.9479, 5.2345, 6.3914, 1.5006, -0.6271, 0.4063, 0.441, 0.5627, 0.3837, 1.5605, 1.8969, 1.9453, 2.2452, 0.1807, -3.7508, 4.0198},
                         new double[] {-2.1702, 0.1408, 0.9606, -1.7078, 0.358, -0.3973, 0.4367, -0.1075, 6.7359, -0.3088, -5.9616, -6.9971, 0.297, -1.54, -1.1524, -7.2699, 7.5245, -1.5329, 0.247, 0.2721, -0.9684, -3.2022, 0.4192, -0.9745, -3.3968, -1.3393, 0.4, -0.08, 0.4411, -1.1212, -1.6059, -0.4711, 0.4844, -7.6766, 0.8057, -6.9459, -3.3132, 0.3189, -2.3222, 0.3903, 0.4446, 0.5866, 0.3864, -0.0598, 0.5995, 0.8533, 0.3254, -0.5246, -2.0124, -8.1865},
                         new double[] {-5.0736, -3.5617, -2.4589, -4.8417, 0.7468, -0.6594, 0.4375, -1.1477, 7.5548, -2.0774, -4.0967, 5.1202, -1.2521, -5.29, -3.8164, 4.7779, -5.7249, -4.9666, 0.9127, -2.6294, -5.4348, -4.2369, 0.4025, -4.6933, -3.4432, -5.1211, 0.91, -3.21, 0.8409, -5.0083, -2.8624, -0.9011, 1.0375, -5.6415, 1.1105, -5.088, -1.3442, -2.684, -3.4237, 0.3499, 0.364, 0.4276, 0.3454, 1.7444, 1.2232, -2.1998, 1.0152, -4.0362, -4.2448, -6.2384},
                         new double[] {2.7302, 0.9858, 4.6108, 3.6948, -0.2195, 0.8664, 0.4375, 1.0054, 0.7482, 1.1501, -0.5444, -0.5506, 3.4106, 2.79, 4.0635, 0.3769, 0.4358, 3.7744, -0.4686, 3.5225, 4.7861, 0.5871, 0.4181, 5.0834, -1.6696, 0.7261, -0.33, 6.49, -0.2316, 4.2191, 1.7749, 0.9461, 0.0934, -0.3569, 0.1089, 0.0191, -0.403, 4.3721, 1.6266, 0.3874, 0.4018, 0.3871, 0.3738, -0.6956, 0.1756, 3.7599, -0.5382, 3.0141, 6.4939, -0.9023},
                         new double[] {4.8311, 5.3551, 1.0647, 4.8253, 0.4116, 1.8488, 0.435, 1.637, 3.1443, 2.4927, 5.8311, 5.2048, 0.6133, 4.54, 4.1072, 7.2284, 4.8425, 4.1886, 0.4794, 1.373, 5.0484, 4.6673, 0.4129, 4.1571, 5.4713, 4.6498, 0.5699, 0.5, 0.5828, 4.0147, 3.3696, 2.0419, 0.8262, 5.8771, 0.5615, 5.1802, 8.3622, 1.0057, 2.4223, 0.3801, 0.403, 0.5081, 0.3662, 0.5651, 0.701, 1.3601, 0.4671, 3.2292, 4.6604, 3.9323},
                         new double[] {-0.0214, 0.8743, 3.7122, 0.1589, 0.7844, 1.8234, 0.4283, 0.7248, 1.2717, 0.896, 2.6441, 2.2105, 2.585, 0.38, -0.0942, 6.3909, 1.2815, 0.5796, 1.4497, 5.2489, -0.4169, 6.066, 0.4033, 1.8142, 4.1208, 3.1458, 1.55, 5.36, 1.486, 0.2446, 1.3006, 1.1217, 1.6285, 1.3851, 2.1172, 1.5368, 4.4986, 4.9806, 2.2224, 0.3883, 0.4209, 0.5054, 0.3846, 3.196, 1.6996, 4.2951, 1.5818, 0.2607, 0.4475, 1.7562},
                         new double[] {1.429, 2.3221, -1.4954, 1.4114, 0.2548, 0.1946, 0.4283, 0.7018, -2.687, 0.5618, -0.586, -0.7003, -2.1686, 1.46, 0.8785, -0.2366, -0.9156, 1.1309, 0.1579, -2.5171, 1.0769, 2.4222, 0.4177, 2.8877, 1.0435, 1.2507, 0.08, -2.88, 0.0645, 0.5965, -0.0813, 0.1775, 0.2041, -1.0637, -0.2403, -1.1531, 2.2801, -2.7463, 0.2175, 0.3772, 0.4308, 0.5417, 0.3762, 0.0722, 0.1107, -1.3727, 0.0492, 1.5157, 3.1156, 0.631},
                         new double[] {2.4166, 6.4916, 0.6449, 4.404, -0.2856, 0.4596, 0.4283, 0.9421, 4.0137, 1.1397, 6.362, 5.1362, 0.3529, 3.04, 3.2163, 8.671, 6.6362, 3.1629, -0.9805, -0.197, 1.7403, 6.7933, 0.4581, 1.8349, 7.2781, 9.6611, -0.91, 0.45, -0.7181, 2.9805, 1.9801, 0.738, -1.0969, 4.9708, -1.5804, 5.6051, 3.1511, 0.3718, 2.2729, 0.3753, 0.3931, 0.4504, 0.3582, -2.4596, -1.4796, -0.425, -0.928, 2.7241, 0.1394, 5.3334},
                         new double[] {3.4245, 6.0959, -0.837, 4.4563, -0.0609, 0.5429, 0.4283, 0.4756, 2.1528, 0.7741, 2.4468, 2.991, -0.1417, 3.66, 2.5432, 4.5066, 3.1242, 2.7931, -0.1121, -1.8324, 2.1275, 6.4174, 0.3985, 2.2869, 5.3522, 7.1012, 0.09, -1.74, -0.0577, 2.2474, 1.2557, 0.7634, 0.2207, 1.3285, -0.1427, 2.2664, 2.6709, -1.7776, 1.8006, 0.3884, 0.4111, 0.4663, 0.3569, -0.3022, 0.3513, -0.246, -0.098, 2.4629, -2.7963, 1.8506},
                         new double[] {0.2524, 5.2673, -1.3459, 0.3856, 0.2713, 0.1419, 0.4283, -0.0090, -4.34, 0.7688, -3.3001, 3.485, 1.6194, 0.5699, -0.1396, -2.0699, -4.9597, -0.1969, 0.5197, -2.8508, -0.2105, -1.0271, 0.4395, 0.271, -1.377, 7.4424, 0.52, -1.77, 0.6356, -0.3817, -0.8536, 0.0992, 1.1893, -3.713, 1.8289, -3.908, -1.2665, -2.7147, -1.3229, 0.3772, 0.3997, 0.4448, 0.3754, 1.0975, 1.7391, -1.8497, 0.6546, 0.7806, 1.1966, -4.1642},
                         new double[] {3.464, 1.4006, 1.7841, 4.8124, -0.5521, 0.4274, 0.4275, 0.8484, 0.3267, 0.6421, 1.9114, 1.9057, 0.8023, 3.43, 2.4221, 2.0117, 1.8252, 3.2781, -1.1202, 1.8565, 2.2424, 3.6482, 0.443, 2.5676, 1.4556, 4.5172, -1.21, 1.9, -1.1196, 3.4459, 1.2721, 0.3436, -0.6327, -0.2702, -0.6677, 0.9864, 5.4949, 2.1578, 0.9443, 0.3995, 0.4326, 0.4582, 0.3274, -0.9353, -0.6549, 1.9197, -1.1749, 2.9278, 1.3997, -0.3459},
                         new double[] {-2.1217, -1.1125, 1.7918, -2.8745, -0.2919, -0.2597, 0.4283, -0.5258, -1.6284, -0.9297, -0.5833, 0.0, 1.0911, -2.88, -0.6281, -0.4816, -0.0757, -2.0635, -0.4532, 2.164, -2.1104, -0.4014, 0.4036, -2.4373, 0.1991, -2.5778, -0.17, 1.87, -0.5032, -1.789, -0.5798, -0.1712, -0.0549, -0.2189, 0.5202, -0.411, 0.953, 2.3485, -0.4919, 0.3688, 0.3915, 0.4173, 0.3363, -1.1015, 0.4358, 1.5477, -0.5309, -0.9464, -2.3591, -0.0187},
                         new double[] {3.8704, 4.8937, 2.9532, 3.4706, 1.0246, 1.7173, 0.4475, 1.6602, 2.0337, 2.2132, 1.7702, 2.6575, 1.4081, 3.91, 3.97, 1.0227, 1.2001, 3.804, 1.328, 2.719, 3.3114, 2.9981, 0.4964, 3.5449, 2.2443, 2.1985, 1.57, 2.32, 1.75, 3.97, 2.83, 1.9, 1.45, 2.21, 1.1316, 1.8914, -4.2444, 2.9582, 2.4839, 0.44, 0.4485, 0.47, 0.4633, 2.3123, 1.28, 3.47, 1.7601, 3.4792, 4.6857, 1.9584}} // twoDimArray
      );
      // Here  we  evaluate  the  expected  returns  of  six  funds  using  the
      // historical approach.
      double[] expReturnsOfFunds = instanceOfAssetParameters.ExpectedReturns( 
         returnsOfFunds // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    This expected returns of the funds are:" );
      System.Console.WriteLine( " " );
      displayArray( expReturnsOfFunds );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluation of the Expected Returns of the  Covariance  Matrix  of  the +" );
      System.Console.WriteLine( "+ six funds                                                              +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix( double[][] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalReturns = returnsOfFunds" );
      // Here we evaluate the covariance matrix of the 6 funds considered.
      double[][] covarianceOfFunds = instanceOfAssetParameters.CovarianceMatrix( 
         returnsOfFunds // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The covariance matrix between the funds is:" );
      System.Console.WriteLine( " " );
      display2DArray( covarianceOfFunds );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Upper Bound of the Efficient Frontier +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minFrontierReturn( double[] expectedReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     What is the lowest expected  return  of  the  fund  from  which  the" );
      System.Console.WriteLine( "     portfolio can be constructed. Note, that  this  will  correspond  to" );
      System.Console.WriteLine( "     the minimum of the range over which the Efficient Frontier exists." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns = expReturnsOfFunds" );
      double leastReturn = instanceOfMarkowitz.MinFrontierReturn( 
         expReturnsOfFunds // expectedReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The value of the lowest expected return is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( leastReturn ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Lower Bound of the Efficient Frontier +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    maxFrontierReturn( double[] expectedReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     What is the highest expected return of  the  funds  from  which  the" );
      System.Console.WriteLine( "     portfolio can be constructed. Note, that  this  will  correspond  to" );
      System.Console.WriteLine( "     the maximum of the range over which the Efficient Frontier exists." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturns = expReturnsOfFunds" );
      double greaterReturn = instanceOfMarkowitz.MaxFrontierReturn( 
         expReturnsOfFunds // expectedReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The value of the highest expected return is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( greaterReturn ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluate the Efficient Frontier and read  off  the  Optimal  Portfolio +" );
      System.Console.WriteLine( "+ for an expected return of 0.4                                          +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    efficientFrontier( double expectedReturn," );
      System.Console.WriteLine( "                       int numberOfAssets )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Within this  section  we  evaluate  the  Efficient  Frontier  at  20" );
      System.Console.WriteLine( "     equally distributed points over its range and read off  the  weights" );
      System.Console.WriteLine( "     of the optimal portfolio when the expected return is 0.4." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.4" );
      System.Console.WriteLine( "    numberOfAssets = 50" );
      // Here we evaluate the 20 points on the  Efficient  Frontier  and  store
      // them with a private field.
      instanceOfMarkowitz.CalculateEfficientFrontier( 
         covarianceOfFunds, // covarianceMatrix
         expReturnsOfFunds, // expectedReturns
         20, // numberInterpolationPoints
         0.0001 // precision
      );
      // Here we read off the value  of  the  weights  of  the  assets  of  the
      // optimal portfolio on the Efficient Frontier when the  expected  return
      // is 0.4.
      double[] optimalPortfolio4Return2 = instanceOfMarkowitz.EfficientFrontier( 
         0.4, // expectedReturn
         50 // numberOfAssets
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the assets of the portfolio when the are:" );
      System.Console.WriteLine( " " );
      displayArray( optimalPortfolio4Return2 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Value of  the  Expected  Return  at  the  points  at  which  Efficient +" );
      System.Console.WriteLine( "+ Frontier has been evaluated                                            +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we read off the values of the expected returns  at  the  points" );
      System.Console.WriteLine( "     at which the Efficient Frontier has been evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Read off the values of the expected  return  at  which  the  Efficient
      // Frontier was evaluated above.
      double[] returnAtPoints1 = instanceOfMarkowitz.GetEfficientFrontierExpectedReturns( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the points at which  the  Efficient  Frontier  has  been" );
      System.Console.WriteLine( "    evaluated are:" );
      System.Console.WriteLine( " " );
      displayArray( returnAtPoints1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Asset weights on the evaluated poitns of the Efficient Frontier +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierAssetWeights(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we return the 20 arrays which  represent  the  weights  of  the" );
      System.Console.WriteLine( "     assets within the portfolios  corresponding  to  the  20  points  at" );
      System.Console.WriteLine( "     which the Efficient  Frontier  was  evaluated  using  the  calculate" );
      System.Console.WriteLine( "     Efficient Frontier method. Where each array has the following  form:" );
      System.Console.WriteLine( "     the first term is the weight of  the  first  asset  from  which  the" );
      System.Console.WriteLine( "     portfolio can be selected, the second term is  the  weights  of  the" );
      System.Console.WriteLine( "     second asset and so on." );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Reads the values  of  the  asset  weights  at  the  points  where  the
      // Efficient Frontier was evaluated
      double[][] assetWeightsEfficientFrontier1 = instanceOfMarkowitz.GetEfficientFrontierAssetWeights( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the portfolios at the evaluation points are:" );
      System.Console.WriteLine( " " );
      display2DArray( assetWeightsEfficientFrontier1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Value of  the  Expected  Return  at  the  points  at  which  Efficient +" );
      System.Console.WriteLine( "+ Frontier has been evaluated                                            +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we read off the values of the risk at the points for which  the" );
      System.Console.WriteLine( "     Efficient Frontier has been evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = covarianceOfFunds" );
      // Read off the values of the expected  return  at  which  the  Efficient
      // Frontier was evaluated above.
      double[] riskAtPoints1 = instanceOfMarkowitz.GetEfficientFrontierPortfolioRisks( 
         covarianceOfFunds // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the points at which  the  Efficient  Frontier  has  been" );
      System.Console.WriteLine( "    evaluated are:" );
      System.Console.WriteLine( " " );
      displayArray( riskAtPoints1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Use SolveFrontier to find  the  Optimal  Portfolio  when  the  maximum +" );
      System.Console.WriteLine( "+ level of risk allowed is 0.2.                                          +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    findReturn( double risk," );
      System.Console.WriteLine( "                double[] riskAtPoints," );
      System.Console.WriteLine( "                double[] returnAtPoints )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we find the optimal portfolio when the maximum  level  of  risk" );
      System.Console.WriteLine( "     allowed is 0.2." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = 0.2" );
      System.Console.WriteLine( "    riskAtPoints = riskAtPoints1" );
      System.Console.WriteLine( "    returnAtPoints = returnAtPoints1" );
      // Here we find the expected return of the optimal  portfolio  for  value
      // of the risk.
      double optimalPortfolio4Risk1 = instanceOfSolveFrontier.FindReturn( 
         0.2, // risk
         riskAtPoints1, // riskAtPoints
         returnAtPoints1 // returnAtPoints
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The portfolio with  the  maximum  expected  return  when  the  maximum" );
      System.Console.WriteLine( "    allowable risk is 0.2 is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( optimalPortfolio4Risk1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Use SolveFrontier to find  the  Optimal  Portfolio  when  the  maximum +" );
      System.Console.WriteLine( "+ level of risk allowed is 0.4                                           +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    findReturn( double risk," );
      System.Console.WriteLine( "                double[] riskAtPoints," );
      System.Console.WriteLine( "                double[] returnAtPoints )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we find the optimal portfolio when the maximum  level  of  risk" );
      System.Console.WriteLine( "     allowed is 0.4." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = 0.4" );
      System.Console.WriteLine( "    riskAtPoints = riskAtPoints1" );
      System.Console.WriteLine( "    returnAtPoints = returnAtPoints1" );
      // Here we find the expected return of the optimal  portfolio  for  value
      // of the risk.
      double optimalPortfolio4Risk2 = instanceOfSolveFrontier.FindReturn( 
         0.4, // risk
         riskAtPoints1, // riskAtPoints
         returnAtPoints1 // returnAtPoints
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The portfolio with  the  maximum  expected  return  when  the  maximum" );
      System.Console.WriteLine( "    allowable risk is 0.4 is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( optimalPortfolio4Risk2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Use SolveFrontier to find  the  Optimal  Portfolio  when  the  maximum +" );
      System.Console.WriteLine( "+ level of risk allowed is 0.6                                           +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    findReturn( double risk," );
      System.Console.WriteLine( "                double[] riskAtPoints," );
      System.Console.WriteLine( "                double[] returnAtPoints )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Here we find the optimal portfolio when the maximum  level  of  risk" );
      System.Console.WriteLine( "     allowed is 0.6." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = 0.6" );
      System.Console.WriteLine( "    riskAtPoints = riskAtPoints1" );
      System.Console.WriteLine( "    returnAtPoints = returnAtPoints1" );
      // Here we find the expected return of the optimal  portfolio  for  value
      // of the risk.
      double optimalPortfolio4Risk3 = instanceOfSolveFrontier.FindReturn( 
         0.6, // risk
         riskAtPoints1, // riskAtPoints
         returnAtPoints1 // returnAtPoints
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The portfolio with  the  maximum  expected  return  when  the  maximum" );
      System.Console.WriteLine( "    allowable risk is 0.6 is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( optimalPortfolio4Risk3 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
