using System;
using WebCab.Lib.Util.Example;

using WebCab.Libraries.Finance.Portfolio;

public class ThreeAssetsClient {
   public static void Main( String[] args ) {
      // used components
      Markowitz instanceOfMarkowitz = new Markowitz();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "                      ######################################" );
      System.Console.WriteLine( "                      # ThreeAssets Console Client Example #" );
      System.Console.WriteLine( "                      ######################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Within this example we evaluate  the  Efficient  Frontier  when  the" );
      System.Console.WriteLine( "    portfolio is constructed from three assets." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluates the Efficient Frontier +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    calculateEfficientFrontier( double minimumExpectedReturn," );
      System.Console.WriteLine( "                                double maximumExpectedReturn," );
      System.Console.WriteLine( "                                double[][] covarianceMatrix," );
      System.Console.WriteLine( "                                double[] expectedReturns," );
      System.Console.WriteLine( "                                int numberInterpolationPoints," );
      System.Console.WriteLine( "                                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we evaluate the Efficient  Frontier  at  20  equally  dispersed" );
      System.Console.WriteLine( "     points within the range of the expected return [0.05, 0.15]." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = 0.05" );
      System.Console.WriteLine( "    maximumExpectedReturn = 0.15" );
      System.Console.WriteLine( "    covarianceMatrix = {{1.0, -0.03, 0.02}," );
      System.Console.WriteLine( "                        {-0.03, 1.0, 0.02}," );
      System.Console.WriteLine( "                        {0.02, 0.02, 1.0}}" );
      System.Console.WriteLine( "    expectedReturns = {0.05, 0.15, 0.07}" );
      System.Console.WriteLine( "    numberInterpolationPoints = 20" );
      System.Console.WriteLine( "    precision = 0.00001" );
      instanceOfMarkowitz.CalculateEfficientFrontier( 
         0.05, // minimumExpectedReturn
         0.15, // maximumExpectedReturn
         new double[][] {new double[] {1.0, -0.03, 0.02},
                         new double[] {-0.03, 1.0, 0.02},
                         new double[] {0.02, 0.02, 1.0}}, // covarianceMatrix
         new double[] {0.05, 0.15, 0.07}, // expectedReturns
         20, // numberInterpolationPoints
         0.00001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The Efficient Frontier has been evaluated and set to a private field." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Returns values of the Expected Returns +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Returns the values of the expected returns at the 20 points  of  the" );
      System.Console.WriteLine( "     Efficient Frontier evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      double[] returns1 = instanceOfMarkowitz.GetEfficientFrontierExpectedReturns( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The 20 values of the expected return of the points  on  the  Efficient" );
      System.Console.WriteLine( "    Frontier are:" );
      System.Console.WriteLine( " " );
      displayArray( returns1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Returns the values of the Risks +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Returns the values of the risks at the 20 points  of  the  Efficient" );
      System.Console.WriteLine( "     Frontier evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = {{1.0, -0.03, 0.02}," );
      System.Console.WriteLine( "                        {-0.03, 1.0, 0.02}," );
      System.Console.WriteLine( "                        {0.02, 0.02, 1.0}}" );
      double[] risks1 = instanceOfMarkowitz.GetEfficientFrontierPortfolioRisks( 
         new double[][] {new double[] {1.0, -0.03, 0.02},
                         new double[] {-0.03, 1.0, 0.02},
                         new double[] {0.02, 0.02, 1.0}} // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The 20 values of the risks of the points  on  the  Efficient  Frontier" );
      System.Console.WriteLine( "    are:" );
      System.Console.WriteLine( " " );
      displayArray( risks1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
