using System;
using WebCab.Lib.Util.Example;

using WebCab.Libraries.Finance.Portfolio;

public class VolatilityClient {
   public static void Main( String[] args ) {
      // used components
      Volatility instanceOfVolatility = new Volatility();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "                      #####################################" );
      System.Console.WriteLine( "                      # Volatility Console Client Example #" );
      System.Console.WriteLine( "                      #####################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Within this example we provide one example for each of the  business" );
      System.Console.WriteLine( "    methods within the Volatility Class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ The value of the volatility over a  given  number  of  days  when  the +" );
      System.Console.WriteLine( "+ annual volatility is known                                             +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    yearDaysRescaling( double volPerAnnum," );
      System.Console.WriteLine( "                       double daysRescaledTo," );
      System.Console.WriteLine( "                       int convention )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the volatility per 30 days when the annual  volatility  with" );
      System.Console.WriteLine( "     respect to the 360 day count convention is 25 percent?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    volPerAnnum = 0.25" );
      System.Console.WriteLine( "    daysRescaledTo = 30" );
      System.Console.WriteLine( "    convention = 1" );
      // Calculates the value of the volatility over a  given  number  of  days
      // when the annual volatility is known.  The  annual  volatility  may  be
      // given with respect to the 360 day convention.
      double yearDaysRescaling1 = instanceOfVolatility.YearDaysRescaling( 
         0.25, // volPerAnnum
         30.0, // daysRescaledTo
         1 // convention
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The volatility per 30 days is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( yearDaysRescaling1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ The annual volatility when the volatility of a given  number  of  days +" );
      System.Console.WriteLine( "+ is known                                                               +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    daysYearRescaling( double volPerPeriod," );
      System.Console.WriteLine( "                       double daysInPeriod," );
      System.Console.WriteLine( "                       int convention )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the equivalent annual volatility in accordance with the  365" );
      System.Console.WriteLine( "     day count  convention  when  the  volatility  over  30  days  is  12" );
      System.Console.WriteLine( "     percent?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    volPerPeriod = 0.12" );
      System.Console.WriteLine( "    daysInPeriod = 30" );
      System.Console.WriteLine( "    convention = 2" );
      // Evaluates the annual volatility when the volatility of a given  number
      // of days is known. The annual volatility may be evaluated  with  respec
      // to either the 365 day convention.
      double daysYearRescaling = instanceOfVolatility.DaysYearRescaling( 
         0.12, // volPerPeriod
         30.0, // daysInPeriod
         2 // convention
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The annual volatility in accordance with the 356 day count  convention" );
      System.Console.WriteLine( "    when the volatility over 30 days is 12 percent is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( daysYearRescaling ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Historical estimate of the present volatility +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalEstimate( double[] assetPrices )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the historical estimate of the present volatility  when  the" );
      System.Console.WriteLine( "     assets price at the end of each interval where: 100, 101,  104,  102" );
      System.Console.WriteLine( "     and 99?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    assetPrices = {100.0, 101.0, 104.0, 102.0, 99.0}" );
      // Estimate  (present)  volatility  per  interval  expressed  in  decimal
      // format (i.e. 1 percent = 0.01) using the price  data  at  the  end  of
      // equal time intervals  of  an  investment  asset  which  does  not  pay
      // dividends or interest payments.
      double historicalEstimate1 = instanceOfVolatility.HistoricalEstimate( 
         new double[] {100.0, 101.0, 104.0, 102.0, 99.0} // assetPrices
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The historical estimate of the present volatility is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( historicalEstimate1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Historical estimate of the present volatility taking into account  the +" );
      System.Console.WriteLine( "+ dividends                                                              +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalEstimateWithDividends( double[] assetPrices," );
      System.Console.WriteLine( "                                     double[] dividendsPaid )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the historical estimate of the present volatility  when  the" );
      System.Console.WriteLine( "     assets price at the end of each interval where: 100, 101,  104,  102" );
      System.Console.WriteLine( "     and 99 and the dividends  (or  interest  payments)  from  the  asset" );
      System.Console.WriteLine( "     within each period where: 4, 0, 0, 0, 5?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    assetPrices = {100.0, 101.0, 104.0, 102.0, 99.0}" );
      System.Console.WriteLine( "    dividendsPaid = {4.0, 0.0, 0.0, 0.0, 5.0}" );
      // Calculates the historical estimate of the  present  volatility  taking
      // into account the dividends or  interest  payments  of  the  underlying
      // asset.
      double historicalEstimateWithDividends1 = instanceOfVolatility.HistoricalEstimateWithDividends( 
         new double[] {100.0, 101.0, 104.0, 102.0, 99.0}, // assetPrices
         new double[] {4.0, 0.0, 0.0, 0.0, 5.0} // dividendsPaid
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The historical estimate of the present volatility with dividends is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( historicalEstimateWithDividends1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Returns the estimate of the standard error of the standard  historical +" );
      System.Console.WriteLine( "+ estimation                                                             +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalEstimateStandardError( double numberOfDays," );
      System.Console.WriteLine( "                                     double volatilityEstimate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the  standard  error  of  the  historical  estimate  of  the" );
      System.Console.WriteLine( "     volatility, when the volatility is estimated using 180 days of  data" );
      System.Console.WriteLine( "     and the volatility found was 0.04 (or 4 percent)?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    numberOfDays = 180" );
      System.Console.WriteLine( "    volatilityEstimate = 0.04" );
      // Returns the estimate of the standard error of the standard  historical
      // estimation given by  the  functions  volatilityHistoricalEstimate  and
      // volatilityHistoricalEstimateWithDividends.
      double historicalEstimateStandardError1 = instanceOfVolatility.HistoricalEstimateStandardError( 
         180.0, // numberOfDays
         0.04 // volatilityEstimate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The estimate of the standard error is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( historicalEstimateStandardError1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Volatility according to the ARCH model +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    archVolatilityEstimate( double longTermVolatility," );
      System.Console.WriteLine( "                            double weightOfVolatility," );
      System.Console.WriteLine( "                            double[] observations," );
      System.Console.WriteLine( "                            double[] weights )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the estimate of the volatility  considering  the  long  term" );
      System.Console.WriteLine( "     volatility is 0.04, the weight assigned to the  volatility  is  0.4," );
      System.Console.WriteLine( "     the historical observations where: 78, 67,  89;  and  the  array  of" );
      System.Console.WriteLine( "     weights associated to these historical observations are:  0.2,  0.3," );
      System.Console.WriteLine( "     0.1?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    longTermVolatility = 0.04" );
      System.Console.WriteLine( "    weightOfVolatility = 0.4" );
      System.Console.WriteLine( "    observations = {78.0, 67.0, 89.0}" );
      System.Console.WriteLine( "    weights = {0.2, 0.3, 0.1}" );
      // Returns the estimate of the volatility according to the ARCH model.
      double archVolatilityEstimate1 = instanceOfVolatility.ArchVolatilityEstimate( 
         0.04, // longTermVolatility
         0.4, // weightOfVolatility
         new double[] {78.0, 67.0, 89.0}, // observations
         new double[] {0.2, 0.3, 0.1} // weights
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The estimate of the volatility using this ARCH model is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( archVolatilityEstimate1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Volatility for the ith day made at the end  of  the  previous  (i-1)th +" );
      System.Console.WriteLine( "+ day                                                                    +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    ewmaVolatilityEstimate( double weightRatio," );
      System.Console.WriteLine( "                            double i_1thDayEstimateOfVolatility," );
      System.Console.WriteLine( "                            double endi_1thDay," );
      System.Console.WriteLine( "                            double starti_1thDay )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Estimate the volatility on the i-th  day  using  the  EWMA  approach" );
      System.Console.WriteLine( "     when the estimate of the volatility on the  (i-1)-th  day  is  0.04," );
      System.Console.WriteLine( "     the weight ration used is 0.3, the market price at the close of  the" );
      System.Console.WriteLine( "     (i-1)th day is 120, and  the  market  price  at  the  start  of  the" );
      System.Console.WriteLine( "     (i-1)th day is 123?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weightRatio = 0.3" );
      System.Console.WriteLine( "    i_1thDayEstimateOfVolatility = 0.04" );
      System.Console.WriteLine( "    endi_1thDay = 120" );
      System.Console.WriteLine( "    starti_1thDay = 123" );
      // Returns the estimate of the volatility for the ith  day  made  at  the
      // end of the previous (i-1)th day, using an estimate of  the  volatility
      // on the (i-1)th day  according  to  the  EWMA  model  with  respect  to
      // changes in the market variable.
      double ewmaVolatilityEstimate1 = instanceOfVolatility.EwmaVolatilityEstimate( 
         0.3, // weightRatio
         0.04, // i_1thDayEstimateOfVolatility
         120.0, // endi_1thDay
         123.0 // starti_1thDay
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The estimate of the volatility on the (i-1)-th day is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( ewmaVolatilityEstimate1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ The continuously compounded return of an asset over one day +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    returnDuringithDay( double ithDayAssetPrice," );
      System.Console.WriteLine( "                        double i_1thDayAssetPrice )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the continuously compounded return of an asset over one  day" );
      System.Console.WriteLine( "     if the asset price at the close of business on the ith  day  is  120" );
      System.Console.WriteLine( "     and the asset price at the close of business on  the  i-1th  day  is" );
      System.Console.WriteLine( "     126?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    ithDayAssetPrice = 120" );
      System.Console.WriteLine( "    i_1thDayAssetPrice = 126" );
      // This function returns the continuously compounded return of  an  asset
      // over one day.
      double returnDuringithDay1 = instanceOfVolatility.ReturnDuringithDay( 
         120.0, // ithDayAssetPrice
         126.0 // i_1thDayAssetPrice
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The continuously compounded return is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( returnDuringithDay1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ The variance of the expected returns of an asset +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    variance( double[] probability," );
      System.Console.WriteLine( "              double[] returns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the variance of the expected return of an  asset  given  the" );
      System.Console.WriteLine( "     array of probability: 0.3, 0.5, 0.2; and the returns  of  the  asset" );
      System.Console.WriteLine( "     are 10, 20 and 10?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    probability = {0.3, 0.5, 0.2}" );
      System.Console.WriteLine( "    returns = {10.0, 20.0, 10.0}" );
      // Calculates the variance of the expected returns of an asset given  the
      // assets returns in given market states and  the  probability  of  those
      // market states occurring.
      double variance1 = instanceOfVolatility.Variance( 
         new double[] {0.3, 0.5, 0.2}, // probability
         new double[] {10.0, 20.0, 10.0} // returns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The variance of the expected return is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( variance1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ The variance of the historical returns of an asset +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    variance( double[] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the variance of the historical returns  of  an  asset  given" );
      System.Console.WriteLine( "     the returns of the asset are 11, 12, 10?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    historicalReturns = {11.0, 12.0, 10.0}" );
      // Evaluates the variance of the historical returns of an asset.
      double variance2 = instanceOfVolatility.Variance( 
         new double[] {11.0, 12.0, 10.0} // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The variance of the historical return is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( variance2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
