#using <mscorlib.dll>

using namespace System;
using namespace System::ComponentModel;
using namespace System::Diagnostics;
using namespace System::IO;
using namespace WebCab::Lib::Util::Example;

using namespace WebCab::Libraries::Finance::Portfolio;

template< class T >
T managed2DimArray( T* ca, int dim1, int dim2 ) __gc[,] {
   T a __gc[,] = new T __gc[dim1, dim2];
   for ( int i = 0; i < dim1; i ++ ) {
      for ( int j = 0; j < dim2; j ++ ) {
         a[i, j] = ca[ i * dim2 + j ];
      }
   }
   return a;
}

static double double2Double( double d ) {
   if ( Double::IsNaN( d ) ) return d;
   if ( Double::IsNegativeInfinity( d ) ) return d;
   if ( Double::IsPositiveInfinity( d ) ) return d;
   if ( d == Double::MinValue ) return d;
   if ( d == Double::MaxValue ) return d;
   double frac = d - ( int )d;
   double frac4 = ( int )( frac * 10000 ) / 10000.0;
   return ( int )d + frac4;
}

static void displayArray( double a __gc[], StreamWriter* streamWriter ) {
   for ( int i = 0; i < a->Length; i ++ ) {
      streamWriter->Write( double2Double( a[ i ] ) );
      streamWriter->Write( ", " );
   }
   streamWriter->WriteLine( " " );
}

int main() {
   Process* myProcess;
   myProcess = new Process ();
   myProcess->StartInfo->FileName = "more.com";
   myProcess->StartInfo->UseShellExecute = false;
   myProcess->StartInfo->RedirectStandardInput = true;
   myProcess->Start();
   StreamWriter* streamWriter = myProcess->StandardInput;
   streamWriter->AutoFlush = true;
   // used components
   PerformanceEvaluation* instanceOfPerformanceEvaluation = new PerformanceEvaluation;
   //
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "                 ################################################" );
   streamWriter->WriteLine( "                 # PerformanceEvaluation Console Client Example #" );
   streamWriter->WriteLine( "                 ################################################" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " Overview" );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    Within this client we provide an example of the application of  each" );
   streamWriter->WriteLine( "    of the methods within Performance Evaluation." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Press ENTER or SPACE to scroll down." );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Evaluation of the Expected Return of a Portfolio +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    portfolioReturn( double[] weight," );
   streamWriter->WriteLine( "                     double[] returns )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the expected return of a portfolio  which  consists  of  the" );
   streamWriter->WriteLine( "     following three assets:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - High Return Asset: Weight in Portfolio = 0.1,  Expected  return" );
   streamWriter->WriteLine( "        = 10 percent" );
   streamWriter->WriteLine( "        - Medium Risk Asset  1:  Weight  in  Portfolio  =  0.3,  Expected" );
   streamWriter->WriteLine( "        return = 7 percent" );
   streamWriter->WriteLine( "        - Medium Risk Asset  2:  Weight  in  Portfolio  =  0.4,  Expected" );
   streamWriter->WriteLine( "        return = 4 percent" );
   streamWriter->WriteLine( "        - Low Risk Asset: Weight in Portfolio = 0.2, Expected Return =  5" );
   streamWriter->WriteLine( "        percent" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    weight = {0.1, 0.3, 0.4, 0.2}" );
   streamWriter->WriteLine( "    returns = {0.1, 0.07, 0.04, 0.05}" );
   // Here we evaluate the expected return of the portfolio
   double portfolioReturn1;
   {
   double weight __gc[] = {0.1, 0.3, 0.4, 0.2};
   double returns __gc[] = {0.1, 0.07, 0.04, 0.05};
   portfolioReturn1 = instanceOfPerformanceEvaluation->PortfolioReturn( 
      weight, // weight
      returns // returns
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The Expected Return of the Portfolio is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( portfolioReturn1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ The Total Return of a Portfolio +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    totalReturn( double disbursement," );
   streamWriter->WriteLine( "                 double dividends," );
   streamWriter->WriteLine( "                 double marketPriceStart," );
   streamWriter->WriteLine( "                 double marketPriceEnd )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the total return over one year from a mining  stock  if  the" );
   streamWriter->WriteLine( "     market price of the stock at the start  of  the  year  was  90,  the" );
   streamWriter->WriteLine( "     market price at the end of the year was  80,  during  the  year  the" );
   streamWriter->WriteLine( "     company paid a special dividend of 12.3 USD after  selling  of  some" );
   streamWriter->WriteLine( "     mining asset and the company pay two dividends during the year of  5" );
   streamWriter->WriteLine( "     USD and 3 USD?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    disbursement = 12.3" );
   streamWriter->WriteLine( "    dividends = 8" );
   streamWriter->WriteLine( "    marketPriceStart = 90" );
   streamWriter->WriteLine( "    marketPriceEnd = 80" );
   // Here we evaluate the annual return from the portfolio
   double totalReturn1;
   {
   totalReturn1 = instanceOfPerformanceEvaluation->TotalReturn( 
      12.3, // disbursement
      8.0, // dividends
      90.0, // marketPriceStart
      80.0 // marketPriceEnd
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The total return from the mining stock over the year is" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( totalReturn1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Geometric Mean Return of a Portfolio +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    geometricMeanReturn( double[] returnOverEachPeriod )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     We know that the return of an investment over  the  1st,  2nd,  3rd," );
   streamWriter->WriteLine( "     4th and 5th year will be 10%, 12%, 15%, 13%  and  16%  respectively." );
   streamWriter->WriteLine( "     What is the fixed rate of return over the 5 year period which  would" );
   streamWriter->WriteLine( "     offer the equivalent return?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    returnOverEachPeriod = {0.1, 0.12, 0.15, 0.13, 0.16}" );
   // Here we evaluate the geometric mean return of the investment
   double geometricMeanReturn1;
   {
   double returnOverEachPeriod __gc[] = {0.1, 0.12, 0.15, 0.13, 0.16};
   geometricMeanReturn1 = instanceOfPerformanceEvaluation->GeometricMeanReturn( 
      returnOverEachPeriod // returnOverEachPeriod
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The equivalent rate of equal to the geometric mean return, which is" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( geometricMeanReturn1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Sharpe's Ratio of two Portfolios Performance +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    sharpesRatio( double averageReturn," );
   streamWriter->WriteLine( "                  double standardDeviation," );
   streamWriter->WriteLine( "                  double riskFree )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     Which of the following two portfolios offers  the  best  return  per" );
   streamWriter->WriteLine( "     unit of risk over the annual period considered:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - 1st Portfolio: Annual  Return  =  7  percent,  Annual  Standard" );
   streamWriter->WriteLine( "        Deviation = 2 percent, (Annual)  Risk  free  interest  rate  =  3" );
   streamWriter->WriteLine( "        percent" );
   streamWriter->WriteLine( "        - 2nd Portfolio: Annual  Return  =  8  percent,  Annual  Standard" );
   streamWriter->WriteLine( "        Deviation = 4 percent, (Annual)  Risk  free  interest  rate  =  3" );
   streamWriter->WriteLine( "        percent" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     That is, which portfolio offer the best return  in  accordance  with" );
   streamWriter->WriteLine( "     Sharpe's ratio?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    averageReturn = 0.07" );
   streamWriter->WriteLine( "    standardDeviation = 0.02" );
   streamWriter->WriteLine( "    riskFree = 0.03" );
   // Here we evaluate Sharpe's Ratio of the 1st portfolio with a 7  percent
   // return
   double sharpesRatio1;
   {
   sharpesRatio1 = instanceOfPerformanceEvaluation->SharpesRatio( 
      0.07, // averageReturn
      0.02, // standardDeviation
      0.03 // riskFree
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    Sharpe's Ratio of the 1st Portfolio is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( sharpesRatio1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Sharpe's Ratio of two Portfolios Performance +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    sharpesRatio( double averageReturn," );
   streamWriter->WriteLine( "                  double standardDeviation," );
   streamWriter->WriteLine( "                  double riskFree )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    averageReturn = 0.08" );
   streamWriter->WriteLine( "    standardDeviation = 0.04" );
   streamWriter->WriteLine( "    riskFree = 0.03" );
   // Here we evaluate Sharpe's Ratio of the 2nd portfolio with a 8  percent
   // return
   double sharpesRatio2;
   {
   sharpesRatio2 = instanceOfPerformanceEvaluation->SharpesRatio( 
      0.08, // averageReturn
      0.04, // standardDeviation
      0.03 // riskFree
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    Sharpe's Ratio of the 2nd Portfolio is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( sharpesRatio2 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Treynor's Ratio of Two Portfolios Performance +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    treynorsMeasure( double averageReturn," );
   streamWriter->WriteLine( "                     double beta," );
   streamWriter->WriteLine( "                     double riskFree )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     Which of the following two US Treasury Bond  portfolios  offers  the" );
   streamWriter->WriteLine( "     best risk adjusted return as measured by the Treynor measure over  a" );
   streamWriter->WriteLine( "     1 year period:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - 1st Portfolio: Annual Return = 8 percent, Beta (w.r.t. US  Bond" );
   streamWriter->WriteLine( "        index) = 3, (Annual) Risk free interest rate = 3 percent" );
   streamWriter->WriteLine( "        - 2nd Portfolio: Annual Return = 4 percent, Beta (w.r.t. US  Bond" );
   streamWriter->WriteLine( "        index) = 0.9, (Annual) Risk free interest rate = 3 percent" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     That is, which bond portfolio has the highest Treynor measure?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    averageReturn = 0.08" );
   streamWriter->WriteLine( "    beta = 1.2" );
   streamWriter->WriteLine( "    riskFree = 0.03" );
   // Here we evaluate Treynors Ratio of the 1st portfolio
   double treynorsMeasure1;
   {
   treynorsMeasure1 = instanceOfPerformanceEvaluation->TreynorsMeasure( 
      0.08, // averageReturn
      1.2, // beta
      0.03 // riskFree
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The Treynors measure of the 1st portfolio over the 1 year period is" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( treynorsMeasure1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Treynor's Ratio of Two Portfolios Performance +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    treynorsMeasure( double averageReturn," );
   streamWriter->WriteLine( "                     double beta," );
   streamWriter->WriteLine( "                     double riskFree )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    averageReturn = 0.04" );
   streamWriter->WriteLine( "    beta = 0.9" );
   streamWriter->WriteLine( "    riskFree = 0.03" );
   // Here we evaluate Treynors Ratio of the 2nd portfolio
   double treynorsMeasure2;
   {
   treynorsMeasure2 = instanceOfPerformanceEvaluation->TreynorsMeasure( 
      0.04, // averageReturn
      0.9, // beta
      0.03 // riskFree
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The Treynors measure of the 2nd portfolio over the 1 year period is" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( treynorsMeasure2 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( " " );
   //
   myProcess->StandardInput->Close();
   myProcess->WaitForExit();
}

