#using <mscorlib.dll>

using namespace System;
using namespace System::ComponentModel;
using namespace System::Diagnostics;
using namespace System::IO;
using namespace WebCab::Lib::Util::Example;

using namespace WebCab::Libraries::Finance::Portfolio;

template< class T >
T managed2DimArray( T* ca, int dim1, int dim2 ) __gc[,] {
   T a __gc[,] = new T __gc[dim1, dim2];
   for ( int i = 0; i < dim1; i ++ ) {
      for ( int j = 0; j < dim2; j ++ ) {
         a[i, j] = ca[ i * dim2 + j ];
      }
   }
   return a;
}

static double double2Double( double d ) {
   if ( Double::IsNaN( d ) ) return d;
   if ( Double::IsNegativeInfinity( d ) ) return d;
   if ( Double::IsPositiveInfinity( d ) ) return d;
   if ( d == Double::MinValue ) return d;
   if ( d == Double::MaxValue ) return d;
   double frac = d - ( int )d;
   double frac4 = ( int )( frac * 10000 ) / 10000.0;
   return ( int )d + frac4;
}

static void displayArray( double a __gc[], StreamWriter* streamWriter ) {
   for ( int i = 0; i < a->Length; i ++ ) {
      streamWriter->Write( double2Double( a[ i ] ) );
      streamWriter->Write( ", " );
   }
   streamWriter->WriteLine( " " );
}

int main() {
   Process* myProcess;
   myProcess = new Process ();
   myProcess->StartInfo->FileName = "more.com";
   myProcess->StartInfo->UseShellExecute = false;
   myProcess->StartInfo->RedirectStandardInput = true;
   myProcess->Start();
   StreamWriter* streamWriter = myProcess->StandardInput;
   streamWriter->AutoFlush = true;
   // used components
   SolveFrontier* instanceOfSolveFrontier = new SolveFrontier;
   //
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "                     ########################################" );
   streamWriter->WriteLine( "                     # SolveFrontier Console Client Example #" );
   streamWriter->WriteLine( "                     ########################################" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " Overview" );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    Within this client we provide an example for  each  of  the  methods" );
   streamWriter->WriteLine( "    within Solve Frontier." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Press ENTER or SPACE to scroll down." );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Risk of Portfolios on  the  Efficient  Forntier  with  given  expected +" );
   streamWriter->WriteLine( "+ returns                                                                +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findRisk( double expectedReturn," );
   streamWriter->WriteLine( "              double[] riskPoints," );
   streamWriter->WriteLine( "              double[] returnPoints )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What of the risk of the portfolios on the Efficient  Frontier  which" );
   streamWriter->WriteLine( "     is given by the following points:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - Risk = 0, Expected Return = 3 percent" );
   streamWriter->WriteLine( "        - Risk = 10 percent, Expected Return = 4 percent" );
   streamWriter->WriteLine( "        - Risk = 20 percent, Expected Return = 5 percent" );
   streamWriter->WriteLine( "        - Risk = 40 percent, Expected Return = 10 percent" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     for the following values of the expected return 3.5, 4.5, 5.5,  6.5," );
   streamWriter->WriteLine( "     7.5 percent?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    expectedReturn = 0.035" );
   streamWriter->WriteLine( "    riskPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnPoints = {0.03, 0.04, 0.05, 0.1}" );
   double findRisk1;
   {
   double riskPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findRisk1 = instanceOfSolveFrontier->FindRisk( 
      0.035, // expectedReturn
      riskPoints, // riskPoints
      returnPoints // returnPoints
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the portfolio on the Efficient Frontier with  an  expected" );
   streamWriter->WriteLine( "    return of 3.5 percent is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findRisk1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Risk of Portfolios on  the  Efficient  Forntier  with  given  expected +" );
   streamWriter->WriteLine( "+ returns                                                                +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findRisk( double expectedReturn," );
   streamWriter->WriteLine( "              double[] riskPoints," );
   streamWriter->WriteLine( "              double[] returnPoints )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    expectedReturn = 0.045" );
   streamWriter->WriteLine( "    riskPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnPoints = {0.03, 0.04, 0.05, 0.1}" );
   double findRisk2;
   {
   double riskPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findRisk2 = instanceOfSolveFrontier->FindRisk( 
      0.045, // expectedReturn
      riskPoints, // riskPoints
      returnPoints // returnPoints
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the portfolio on the Efficient Frontier with  an  expected" );
   streamWriter->WriteLine( "    return of 4.5 percent is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findRisk2 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Risk of Portfolios on  the  Efficient  Forntier  with  given  expected +" );
   streamWriter->WriteLine( "+ returns                                                                +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findRisk( double expectedReturn," );
   streamWriter->WriteLine( "              double[] riskPoints," );
   streamWriter->WriteLine( "              double[] returnPoints )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    expectedReturn = 0.055" );
   streamWriter->WriteLine( "    riskPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnPoints = {0.03, 0.04, 0.05, 0.1}" );
   double findRisk3;
   {
   double riskPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findRisk3 = instanceOfSolveFrontier->FindRisk( 
      0.055, // expectedReturn
      riskPoints, // riskPoints
      returnPoints // returnPoints
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the portfolio on the Efficient Frontier with  an  expected" );
   streamWriter->WriteLine( "    return of 5.5 percent is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findRisk3 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Risk of Portfolios on  the  Efficient  Forntier  with  given  expected +" );
   streamWriter->WriteLine( "+ returns                                                                +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findRisk( double expectedReturn," );
   streamWriter->WriteLine( "              double[] riskPoints," );
   streamWriter->WriteLine( "              double[] returnPoints )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    expectedReturn = 0.065" );
   streamWriter->WriteLine( "    riskPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnPoints = {0.03, 0.04, 0.05, 0.1}" );
   double findRisk4;
   {
   double riskPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findRisk4 = instanceOfSolveFrontier->FindRisk( 
      0.065, // expectedReturn
      riskPoints, // riskPoints
      returnPoints // returnPoints
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the portfolio on the Efficient Frontier with  an  expected" );
   streamWriter->WriteLine( "    return of 6.5 percent is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findRisk4 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Risk of Portfolios on  the  Efficient  Forntier  with  given  expected +" );
   streamWriter->WriteLine( "+ returns                                                                +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findRisk( double expectedReturn," );
   streamWriter->WriteLine( "              double[] riskPoints," );
   streamWriter->WriteLine( "              double[] returnPoints )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    expectedReturn = 0.075" );
   streamWriter->WriteLine( "    riskPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnPoints = {0.03, 0.04, 0.05, 0.1}" );
   double findRisk5;
   {
   double riskPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findRisk5 = instanceOfSolveFrontier->FindRisk( 
      0.075, // expectedReturn
      riskPoints, // riskPoints
      returnPoints // returnPoints
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the portfolio on the Efficient Frontier with  an  expected" );
   streamWriter->WriteLine( "    return of 7.5 percent is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findRisk5 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Risk of Optimal Portfolios +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findRisk( double[] riskUtility," );
   streamWriter->WriteLine( "              double[] returnUtility," );
   streamWriter->WriteLine( "              double[] riskPoints," );
   streamWriter->WriteLine( "              double[] returnPoints," );
   streamWriter->WriteLine( "              double precision )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     For a collection of assets  we  know  that  the  Efficient  Frontier" );
   streamWriter->WriteLine( "     passes through the follow four points:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - Risk = 0, Expected Return = 3 percent" );
   streamWriter->WriteLine( "        - Risk = 10 percent, Expected Return = 4 percent" );
   streamWriter->WriteLine( "        - Risk = 20 percent, Expected Return = 5 percent" );
   streamWriter->WriteLine( "        - Risk = 40 percent, Expected Return = 10 percent" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     If we also know that a given investor is prepared  to  accepted  the" );
   streamWriter->WriteLine( "     following levels of risk for the corresponds levels of reward:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - Expected Return = 2 percent, Maximum Risk that will accept  for" );
   streamWriter->WriteLine( "        this level of return = 0" );
   streamWriter->WriteLine( "        - Expected Return = 4 percent, Maximum Risk that will accept  for" );
   streamWriter->WriteLine( "        this level of return = 8 percent" );
   streamWriter->WriteLine( "        - Expected Return = 5 percent, Maximum Risk that will accept  for" );
   streamWriter->WriteLine( "        this level of return = 30 percent" );
   streamWriter->WriteLine( "        - Expected Return = 6 percent, Maximum Risk that will accept  for" );
   streamWriter->WriteLine( "        this level of return = 40 percent" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     assuming that the  investors  increase  in  required  return  varies" );
   streamWriter->WriteLine( "     smoothly with the risk, what is the value of the risk (to 2  decimal" );
   streamWriter->WriteLine( "     places)   of  the  optimal  portfolio  constructed  from  the  given" );
   streamWriter->WriteLine( "     collection of assets?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    riskUtility = {0.0, 0.08, 0.03, 0.04}" );
   streamWriter->WriteLine( "    returnUtility = {0.02, 0.04, 0.05, 0.06}" );
   streamWriter->WriteLine( "    riskPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnPoints = {0.03, 0.04, 0.05, 0.1}" );
   streamWriter->WriteLine( "    precision = 0.001" );
   double findRisk6;
   {
   double riskUtility __gc[] = {0.0, 0.08, 0.03, 0.04};
   double returnUtility __gc[] = {0.02, 0.04, 0.05, 0.06};
   double riskPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findRisk6 = instanceOfSolveFrontier->FindRisk( 
      riskUtility, // riskUtility
      returnUtility, // returnUtility
      riskPoints, // riskPoints
      returnPoints, // returnPoints
      0.001 // precision
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the optimal portfolio is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findRisk6 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Return of the Portfolios  on  the  Efficient  Frontier  with  a  given +" );
   streamWriter->WriteLine( "+ expected return                                                        +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findReturn( double risk," );
   streamWriter->WriteLine( "                double[] riskAtPoints," );
   streamWriter->WriteLine( "                double[] returnAtPoints )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What of the expected return  of  the  portfolios  on  the  Efficient" );
   streamWriter->WriteLine( "     Frontier which is given by the following points:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - Risk = 0, Expected Return = 3 percent" );
   streamWriter->WriteLine( "        - Risk = 10 percent, Expected Return = 4 percent" );
   streamWriter->WriteLine( "        - Risk = 20 percent, Expected Return = 5 percent" );
   streamWriter->WriteLine( "        - Risk = 40 percent, Expected Return = 10 percent" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     for the following values of the risk 5, 15, 25, 35, 45 percent?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    risk = 0.05" );
   streamWriter->WriteLine( "    riskAtPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnAtPoints = {0.03, 0.04, 0.05, 0.1}" );
   double findReturn1;
   {
   double riskAtPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnAtPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findReturn1 = instanceOfSolveFrontier->FindReturn( 
      0.05, // risk
      riskAtPoints, // riskAtPoints
      returnAtPoints // returnAtPoints
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the portfolio on the Efficient Frontier with  an  expected" );
   streamWriter->WriteLine( "    return of 5 percent is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findReturn1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Return of the Portfolios  on  the  Efficient  Frontier  with  a  given +" );
   streamWriter->WriteLine( "+ expected return                                                        +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findReturn( double risk," );
   streamWriter->WriteLine( "                double[] riskAtPoints," );
   streamWriter->WriteLine( "                double[] returnAtPoints )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    risk = 0.15" );
   streamWriter->WriteLine( "    riskAtPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnAtPoints = {0.03, 0.04, 0.05, 0.1}" );
   double findReturn2;
   {
   double riskAtPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnAtPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findReturn2 = instanceOfSolveFrontier->FindReturn( 
      0.15, // risk
      riskAtPoints, // riskAtPoints
      returnAtPoints // returnAtPoints
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the portfolio on the Efficient Frontier with  an  expected" );
   streamWriter->WriteLine( "    return of 15 percent is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findReturn2 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Return of the Portfolios  on  the  Efficient  Frontier  with  a  given +" );
   streamWriter->WriteLine( "+ expected return                                                        +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findReturn( double risk," );
   streamWriter->WriteLine( "                double[] riskAtPoints," );
   streamWriter->WriteLine( "                double[] returnAtPoints )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    risk = 0.25" );
   streamWriter->WriteLine( "    riskAtPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnAtPoints = {0.03, 0.04, 0.05, 0.1}" );
   double findReturn3;
   {
   double riskAtPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnAtPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findReturn3 = instanceOfSolveFrontier->FindReturn( 
      0.25, // risk
      riskAtPoints, // riskAtPoints
      returnAtPoints // returnAtPoints
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the portfolio on the Efficient Frontier with  an  expected" );
   streamWriter->WriteLine( "    return of 25 percent is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findReturn3 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Return of the Portfolios  on  the  Efficient  Frontier  with  a  given +" );
   streamWriter->WriteLine( "+ expected return                                                        +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findReturn( double risk," );
   streamWriter->WriteLine( "                double[] riskAtPoints," );
   streamWriter->WriteLine( "                double[] returnAtPoints )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    risk = 0.35" );
   streamWriter->WriteLine( "    riskAtPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnAtPoints = {0.03, 0.04, 0.05, 0.1}" );
   double findReturn4;
   {
   double riskAtPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnAtPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findReturn4 = instanceOfSolveFrontier->FindReturn( 
      0.35, // risk
      riskAtPoints, // riskAtPoints
      returnAtPoints // returnAtPoints
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the portfolio on the Efficient Frontier with  an  expected" );
   streamWriter->WriteLine( "    return of 35 percent is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findReturn4 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Return of the Portfolios  on  the  Efficient  Frontier  with  a  given +" );
   streamWriter->WriteLine( "+ expected return                                                        +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findReturn( double risk," );
   streamWriter->WriteLine( "                double[] riskAtPoints," );
   streamWriter->WriteLine( "                double[] returnAtPoints )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    risk = 0.45" );
   streamWriter->WriteLine( "    riskAtPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnAtPoints = {0.03, 0.04, 0.05, 0.1}" );
   double findReturn5;
   {
   double riskAtPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnAtPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findReturn5 = instanceOfSolveFrontier->FindReturn( 
      0.45, // risk
      riskAtPoints, // riskAtPoints
      returnAtPoints // returnAtPoints
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the portfolio on the Efficient Frontier with  an  expected" );
   streamWriter->WriteLine( "    return of 45 percent is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findReturn5 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Value of the Expected Return of the Optimal Portfolio +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    findReturn( double[] riskUtility," );
   streamWriter->WriteLine( "                double[] returnUtility," );
   streamWriter->WriteLine( "                double[] riskAtPoints," );
   streamWriter->WriteLine( "                double[] returnAtPoints," );
   streamWriter->WriteLine( "                double precision )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     For a collection of assets  we  know  that  the  Efficient  Frontier" );
   streamWriter->WriteLine( "     passes through the follow four points:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - Risk = 0, Expected Return = 3 percent" );
   streamWriter->WriteLine( "        - Risk = 10 percent, Expected Return = 4 percent" );
   streamWriter->WriteLine( "        - Risk = 20 percent, Expected Return = 5 percent" );
   streamWriter->WriteLine( "        - Risk = 40 percent, Expected Return = 10 percent" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     If we also know that a given investor is prepared  to  accepted  the" );
   streamWriter->WriteLine( "     following levels of risk for the corresponds levels of reward:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - Expected Return = 2 percent, Maximum Risk that will accept  for" );
   streamWriter->WriteLine( "        this level of return = 0" );
   streamWriter->WriteLine( "        - Expected Return = 4 percent, Maximum Risk that will accept  for" );
   streamWriter->WriteLine( "        this level of return = 8 percent" );
   streamWriter->WriteLine( "        - Expected Return = 5 percent, Maximum Risk that will accept  for" );
   streamWriter->WriteLine( "        this level of return = 30 percent" );
   streamWriter->WriteLine( "        - Expected Return = 6 percent, Maximum Risk that will accept  for" );
   streamWriter->WriteLine( "        this level of return = 40 percent" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     also assuming that the investors increase in required return  varies" );
   streamWriter->WriteLine( "     smoothly with the risk, what is the value  of  the  expected  return" );
   streamWriter->WriteLine( "     (to 2 decimal places) of the optimal portfolio constructed from  the" );
   streamWriter->WriteLine( "     given collection of assets?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    riskUtility = {0.01, 0.08, 0.03, 0.04}" );
   streamWriter->WriteLine( "    returnUtility = {0.02, 0.04, 0.05, 0.06}" );
   streamWriter->WriteLine( "    riskAtPoints = {0.0, 0.1, 0.2, 0.4}" );
   streamWriter->WriteLine( "    returnAtPoints = {0.03, 0.04, 0.05, 0.1}" );
   streamWriter->WriteLine( "    precision = 0.001" );
   double findReturn6;
   {
   double riskUtility __gc[] = {0.01, 0.08, 0.03, 0.04};
   double returnUtility __gc[] = {0.02, 0.04, 0.05, 0.06};
   double riskAtPoints __gc[] = {0.0, 0.1, 0.2, 0.4};
   double returnAtPoints __gc[] = {0.03, 0.04, 0.05, 0.1};
   findReturn6 = instanceOfSolveFrontier->FindReturn( 
      riskUtility, // riskUtility
      returnUtility, // returnUtility
      riskAtPoints, // riskAtPoints
      returnAtPoints, // returnAtPoints
      0.001 // precision
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The expected return of the optimal portfolio is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( findReturn6 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( " " );
   //
   myProcess->StandardInput->Close();
   myProcess->WaitForExit();
}

