#using <mscorlib.dll>

using namespace System;
using namespace System::ComponentModel;
using namespace System::Diagnostics;
using namespace System::IO;
using namespace WebCab::Lib::Util::Example;

using namespace WebCab::Libraries::Finance::Portfolio;

template< class T >
T managed2DimArray( T* ca, int dim1, int dim2 ) __gc[,] {
   T a __gc[,] = new T __gc[dim1, dim2];
   for ( int i = 0; i < dim1; i ++ ) {
      for ( int j = 0; j < dim2; j ++ ) {
         a[i, j] = ca[ i * dim2 + j ];
      }
   }
   return a;
}

static double double2Double( double d ) {
   if ( Double::IsNaN( d ) ) return d;
   if ( Double::IsNegativeInfinity( d ) ) return d;
   if ( Double::IsPositiveInfinity( d ) ) return d;
   if ( d == Double::MinValue ) return d;
   if ( d == Double::MaxValue ) return d;
   double frac = d - ( int )d;
   double frac4 = ( int )( frac * 10000 ) / 10000.0;
   return ( int )d + frac4;
}

static void displayArray( double a __gc[], StreamWriter* streamWriter ) {
   for ( int i = 0; i < a->Length; i ++ ) {
      streamWriter->Write( double2Double( a[ i ] ) );
      streamWriter->Write( ", " );
   }
   streamWriter->WriteLine( " " );
}

int main() {
   Process* myProcess;
   myProcess = new Process ();
   myProcess->StartInfo->FileName = "more.com";
   myProcess->StartInfo->UseShellExecute = false;
   myProcess->StartInfo->RedirectStandardInput = true;
   myProcess->Start();
   StreamWriter* streamWriter = myProcess->StandardInput;
   streamWriter->AutoFlush = true;
   // used components
   AssetParameters* instanceOfAssetParameters = new AssetParameters;
   TwoAssetPortfolio* instanceOfTwoAssetPortfolio = new TwoAssetPortfolio;
   //
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "                #################################################" );
   streamWriter->WriteLine( "                # OptimalDiversification Console Client Example #" );
   streamWriter->WriteLine( "                #################################################" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " Overview" );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    With this example we consider the following problem: Say  the  first" );
   streamWriter->WriteLine( "    asset represents a portfolio which is  held  and  the  second  asset" );
   streamWriter->WriteLine( "    represents an asset which the portfolio manager  wishes  to  add  to" );
   streamWriter->WriteLine( "    the portfolio such the optimal effects (i.e. minimize the  risk)  of" );
   streamWriter->WriteLine( "    diversification from the purchase is felt. The question  to  address" );
   streamWriter->WriteLine( "    here now many of shares of Asset 2 should the portfolio manager  buy" );
   streamWriter->WriteLine( "    in order to achieve this? We will consider  two  assets,  for  which" );
   streamWriter->WriteLine( "    you know three possible returns may take place and  you  can  assign" );
   streamWriter->WriteLine( "    corresponding probabilities for each of these returns  taking  place" );
   streamWriter->WriteLine( "    in accordance with the following table: State | Asset1  |  Asset2  |" );
   streamWriter->WriteLine( "    Probability -------------------------------------- 1st state  |  300" );
   streamWriter->WriteLine( "    | 500 | 0.5 2nd state | 310 | 400 | 0.2 3rd state | 330 | 550 | 0.3" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Press ENTER or SPACE to scroll down." );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ First we evaluate the Standard deviation of the  Portfolio  (i.e.  1st +" );
   streamWriter->WriteLine( "+ Asset)                                                                 +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    standardDeviation( double[] historicalReturns )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     Here      we  evaluate  the  standard  deviation  of  the  portfolio" );
   streamWriter->WriteLine( "     corresponding to th 1st asset." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalReturns = {300.0, 310.0, 330.0}" );
   // Here        we  evaluate  the  standard  deviation  of  the  portfolio
   // corresponding to th 1st asset.
   double standard1;
   {
   double historicalReturns __gc[] = {300.0, 310.0, 330.0};
   standard1 = instanceOfTwoAssetPortfolio->StandardDeviation( 
      historicalReturns // historicalReturns
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The standard deviation of the portfolio is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( standard1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Then we evaluate the Standard Deviation of the 2nd Asset +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    standardDeviation( double[] historicalReturns )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     Here we evaluate the standard deviation of the 2nd Asset." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalReturns = {500.0, 400.0, 550.0}" );
   // Here we evaluate the standard deviation of the 2nd Asset.
   double standard2;
   {
   double historicalReturns __gc[] = {500.0, 400.0, 550.0};
   standard2 = instanceOfTwoAssetPortfolio->StandardDeviation( 
      historicalReturns // historicalReturns
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The standard deviation of the 2nd Asset is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( standard2 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Covariance between the two Asset is Evaluted +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    covariance( double[] probability," );
   streamWriter->WriteLine( "                double[] rateOfReturn1," );
   streamWriter->WriteLine( "                double[] rateOfReturn2 )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     Here we evaluate the covariance between the Portfolio  and  the  2nd" );
   streamWriter->WriteLine( "     Asset." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    probability = {0.5, 0.2, 0.3}" );
   streamWriter->WriteLine( "    rateOfReturn1 = {300.0, 310.0, 330.0}" );
   streamWriter->WriteLine( "    rateOfReturn2 = {500.0, 400.0, 550.0}" );
   // Here we evaluate the covariance between  the  Portfolio  and  the  2nd
   // Asset.
   double covariance1;
   {
   double probability __gc[] = {0.5, 0.2, 0.3};
   double rateOfReturn1 __gc[] = {300.0, 310.0, 330.0};
   double rateOfReturn2 __gc[] = {500.0, 400.0, 550.0};
   covariance1 = instanceOfTwoAssetPortfolio->Covariance( 
      probability, // probability
      rateOfReturn1, // rateOfReturn1
      rateOfReturn2 // rateOfReturn2
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The covariance between the two assets is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( covariance1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Now we can evaluate the optimal weight +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    weight2MinimizeRisk( double standardDeviation1," );
   streamWriter->WriteLine( "                         double standardDeviation2," );
   streamWriter->WriteLine( "                         double covariance )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     Here we evaluate the weight of the original  portfolio  (i.e.  Asset" );
   streamWriter->WriteLine( "     1) within the constructed  portfolio  consisting  of  Portfolio  and" );
   streamWriter->WriteLine( "     Asset 2, in which the risk is minimized." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    standardDeviation1 = standard1" );
   streamWriter->WriteLine( "    standardDeviation2 = standard2" );
   streamWriter->WriteLine( "    covariance = covariance1" );
   // Here   is  evaluate  the  weight  of  the  Portfolio  in  the  optimal
   // portfolio.
   double optimalWeight1;
   {
   optimalWeight1 = instanceOfTwoAssetPortfolio->Weight2MinimizeRisk( 
      standard1, // standardDeviation1
      standard2, // standardDeviation2
      covariance1 // covariance
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The optimal weight in terms of minimizing the risk  of  the  portfolio" );
   streamWriter->WriteLine( "    is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( optimalWeight1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Risk of the Optimal Portfolio +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    portfolio2Risk( double alpha," );
   streamWriter->WriteLine( "                    double standardDeviation1," );
   streamWriter->WriteLine( "                    double standardDeviation2," );
   streamWriter->WriteLine( "                    double covariance )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     Here we evaluate the Risk of the Optimal  Portfolio.  That  is,  the" );
   streamWriter->WriteLine( "     risk of the portfolio found which  take  maximum  advantage  of  the" );
   streamWriter->WriteLine( "     effects of diversification." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    alpha = optimalWeight1" );
   streamWriter->WriteLine( "    standardDeviation1 = standard1" );
   streamWriter->WriteLine( "    standardDeviation2 = standard2" );
   streamWriter->WriteLine( "    covariance = covariance1" );
   // Here we evaluate the risk pf the optimal portfolio.
   double riskOptimalPort1;
   {
   riskOptimalPort1 = instanceOfTwoAssetPortfolio->Portfolio2Risk( 
      optimalWeight1, // alpha
      standard1, // standardDeviation1
      standard2, // standardDeviation2
      covariance1 // covariance
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    " );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( riskOptimalPort1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Expected Return of the Optimal Portfolio +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    portfolio2Return( double alpha," );
   streamWriter->WriteLine( "                      double expectedReturn1," );
   streamWriter->WriteLine( "                      double expectedReturn2 )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     Here we evaluate  the  expected  return  of  the  optimal  portfolio" );
   streamWriter->WriteLine( "     found." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    alpha = optimalWeight1" );
   streamWriter->WriteLine( "    expectedReturn1 = expected1" );
   streamWriter->WriteLine( "    expectedReturn2 = expected2" );
   // Here we evaluate the expected return of Asset 1.
   double expected1;
   {
   double probability __gc[] = {0.5, 0.2, 0.3};
   double returns __gc[] = {300.0, 310.0, 330.0};
   expected1 = instanceOfAssetParameters->ExpectedReturn( 
      probability, // probability
      returns // returns
   );
   }
   // Here we evaluate the expected return of Asset 2.
   double expected2;
   {
   double probability __gc[] = {0.5, 0.2, 0.3};
   double returns __gc[] = {500.0, 400.0, 550.0};
   expected2 = instanceOfAssetParameters->ExpectedReturn( 
      probability, // probability
      returns // returns
   );
   }
   // Here we evaluate the expected return of the optimal portfolio found.
   double expectedPort1;
   {
   expectedPort1 = instanceOfTwoAssetPortfolio->Portfolio2Return( 
      optimalWeight1, // alpha
      expected1, // expectedReturn1
      expected2 // expectedReturn2
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The expected return of the optimal portfolio is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( expectedPort1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Risk of the Original Portfolio +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    standardDeviation( double[] historicalReturns )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     Note that the risk is just another terms for the standard  deviation" );
   streamWriter->WriteLine( "     which we evaluate for the original portfolio (i.e. Asset 1) here." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalReturns = {300.0, 310.0, 330.0}" );
   // Here we evaluate the risk (i.e. standard deviation)  of  the  original
   // portfolio (i.e. Asset 1).
   double risk1;
   {
   double historicalReturns __gc[] = {300.0, 310.0, 330.0};
   risk1 = instanceOfTwoAssetPortfolio->StandardDeviation( 
      historicalReturns // historicalReturns
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk (or standard deviation) of the original portfolio is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( risk1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Expected Return of the Original Portfolio +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    expectedReturn( double[] probability," );
   streamWriter->WriteLine( "                    double[] returns )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     Here we evaluate the  expected  return  of  the  original  portfolio" );
   streamWriter->WriteLine( "     (i.e. Asset 1)" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    probability = {0.5, 0.2, 0.3}" );
   streamWriter->WriteLine( "    returns = {300.0, 310.0, 330.0}" );
   // Here we evaluate the expeted return of the original portfolio.
   double originalPortExpectedReturn1;
   {
   double probability __gc[] = {0.5, 0.2, 0.3};
   double returns __gc[] = {300.0, 310.0, 330.0};
   originalPortExpectedReturn1 = instanceOfAssetParameters->ExpectedReturn( 
      probability, // probability
      returns // returns
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The expected return of the original portfolio is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( originalPortExpectedReturn1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( " " );
   //
   myProcess->StandardInput->Close();
   myProcess->WaitForExit();
}

