#using <mscorlib.dll>

using namespace System;
using namespace System::ComponentModel;
using namespace System::Diagnostics;
using namespace System::IO;
using namespace WebCab::Lib::Util::Example;

using namespace WebCab::Libraries::Finance::Portfolio;

template< class T >
T managed2DimArray( T* ca, int dim1, int dim2 ) __gc[,] {
   T a __gc[,] = new T __gc[dim1, dim2];
   for ( int i = 0; i < dim1; i ++ ) {
      for ( int j = 0; j < dim2; j ++ ) {
         a[i, j] = ca[ i * dim2 + j ];
      }
   }
   return a;
}

static double double2Double( double d ) {
   if ( Double::IsNaN( d ) ) return d;
   if ( Double::IsNegativeInfinity( d ) ) return d;
   if ( Double::IsPositiveInfinity( d ) ) return d;
   if ( d == Double::MinValue ) return d;
   if ( d == Double::MaxValue ) return d;
   double frac = d - ( int )d;
   double frac4 = ( int )( frac * 10000 ) / 10000.0;
   return ( int )d + frac4;
}

static void displayArray( double a __gc[], StreamWriter* streamWriter ) {
   for ( int i = 0; i < a->Length; i ++ ) {
      streamWriter->Write( double2Double( a[ i ] ) );
      streamWriter->Write( ", " );
   }
   streamWriter->WriteLine( " " );
}

int main() {
   Process* myProcess;
   myProcess = new Process ();
   myProcess->StartInfo->FileName = "more.com";
   myProcess->StartInfo->UseShellExecute = false;
   myProcess->StartInfo->RedirectStandardInput = true;
   myProcess->Start();
   StreamWriter* streamWriter = myProcess->StandardInput;
   streamWriter->AutoFlush = true;
   // used components
   TwoAssetPortfolio* instanceOfTwoAssetPortfolio = new TwoAssetPortfolio;
   //
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "                   ############################################" );
   streamWriter->WriteLine( "                   # TwoAssetPortfolio Console Client Example #" );
   streamWriter->WriteLine( "                   ############################################" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " Overview" );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    Within this client we provide an example for  each  of  the  methods" );
   streamWriter->WriteLine( "    within Two Asset Portfolio." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Press ENTER or SPACE to scroll down." );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ The Correlation Coefficient between two assets +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    correlationCoef( double covariance," );
   streamWriter->WriteLine( "                     double standardDeviation1," );
   streamWriter->WriteLine( "                     double standardDeviation2 )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the correlation coefficient  when  the  between  two  assets" );
   streamWriter->WriteLine( "     when the standard deviation of the  first  asset  is  0.4,  and  the" );
   streamWriter->WriteLine( "     second asset is 0.6, and the covariance between the assets is 0.2?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    covariance = 0.2" );
   streamWriter->WriteLine( "    standardDeviation1 = 0.4" );
   streamWriter->WriteLine( "    standardDeviation2 = 0.6" );
   double CorrelationCoef1;
   {
   CorrelationCoef1 = instanceOfTwoAssetPortfolio->CorrelationCoef( 
      0.2, // covariance
      0.4, // standardDeviation1
      0.6 // standardDeviation2
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The correlation coefficient is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( CorrelationCoef1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Estimate of the Future Covariance using the Scenario Approach +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    covariance( double[] probability," );
   streamWriter->WriteLine( "                double[] rateOfReturn1," );
   streamWriter->WriteLine( "                double[] rateOfReturn2 )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the expected value of the covariance of the returns  on  two" );
   streamWriter->WriteLine( "     funds when we have  the  following  expectations  of  the  following" );
   streamWriter->WriteLine( "     scenarios:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - The probability of the 1st state occurring is  20  percent,  in" );
   streamWriter->WriteLine( "        this instance:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "           - 1st Fund expected return = -3 percent" );
   streamWriter->WriteLine( "           - 2nd Fund expected return = 6 percent" );
   streamWriter->WriteLine( "           " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - The probability of the 2nd state occurring is  30  percent,  in" );
   streamWriter->WriteLine( "        this instance:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "           - 1st Fund expected return = 3 percent" );
   streamWriter->WriteLine( "           - 2nd Fund expected return = -2 percent" );
   streamWriter->WriteLine( "           " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - The probability of the 3rd state occurring is  30  percent,  in" );
   streamWriter->WriteLine( "        this instance:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "           - 1st Fund expected return = -3 percent" );
   streamWriter->WriteLine( "           - 2nd Fund expected return = 6 percent" );
   streamWriter->WriteLine( "           " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - The probability of the 4th state occurring is  20  percent,  in" );
   streamWriter->WriteLine( "        this instance:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "           - 1st Fund expected return = 3 percent" );
   streamWriter->WriteLine( "           - 2nd Fund expected return = -2 percent" );
   streamWriter->WriteLine( "           " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    probability = {0.2, 0.3, 0.3, 0.2}" );
   streamWriter->WriteLine( "    rateOfReturn1 = {-0.03, 0.03, 0.05, 0.02}" );
   streamWriter->WriteLine( "    rateOfReturn2 = {0.06, -0.02, 0.08, -0.09}" );
   // Here we evaluate the covariance. Note  that  the  probability  of  the
   // occurence sum to 1
   double covariance1;
   {
   double probability __gc[] = {0.2, 0.3, 0.3, 0.2};
   double rateOfReturn1 __gc[] = {-0.03, 0.03, 0.05, 0.02};
   double rateOfReturn2 __gc[] = {0.06, -0.02, 0.08, -0.09};
   covariance1 = instanceOfTwoAssetPortfolio->Covariance( 
      probability, // probability
      rateOfReturn1, // rateOfReturn1
      rateOfReturn2 // rateOfReturn2
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The (expected) covariance is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( covariance1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Evaluation of the covariance using historical values +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    covariance( double[] historicalReturns1," );
   streamWriter->WriteLine( "                double[] historicalReturns2 )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the (historical) covariance between two  assets  which  over" );
   streamWriter->WriteLine( "     the past 6 months have the following monthly returns:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - The 1st Assets percentage monthly returns where: 5, 6,  2,  -1," );
   streamWriter->WriteLine( "        2, -2" );
   streamWriter->WriteLine( "        - The 2nd Assets percentage monthly returns where: 4, 2,  4,  -2," );
   streamWriter->WriteLine( "        4, 1" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalReturns1 = {0.05, 0.06, 0.02, -0.01, 0.02, -0.02}" );
   streamWriter->WriteLine( "    historicalReturns2 = {0.04, 0.02, 0.04, -0.02, 0.04, 0.01}" );
   double covariance2;
   {
   double historicalReturns1 __gc[] = {0.05, 0.06, 0.02, -0.01, 0.02, -0.02};
   double historicalReturns2 __gc[] = {0.04, 0.02, 0.04, -0.02, 0.04, 0.01};
   covariance2 = instanceOfTwoAssetPortfolio->Covariance( 
      historicalReturns1, // historicalReturns1
      historicalReturns2 // historicalReturns2
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The (historical) covariance between the two assets is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( covariance2 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Evaluation of the Standard Deviation using Probability Distribution +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    standardDeviation( double[] probability," );
   streamWriter->WriteLine( "                       double[] rateOfReturn )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What of the expected standard deviation of a fund for which we  know" );
   streamWriter->WriteLine( "     will take  one  of  four  states  with  the  following  returns  and" );
   streamWriter->WriteLine( "     probabilities:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - Probability of 1st State = 20 percent, Return = -3 percent" );
   streamWriter->WriteLine( "        - Probability of 1st State = 30 percent, Return = 2 percent" );
   streamWriter->WriteLine( "        - Probability of 1st State = 30 percent, Return = 1 percent" );
   streamWriter->WriteLine( "        - Probability of 1st State = 20 percent, Return = 0 percent" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    probability = {0.2, 0.3, 0.3, 0.2}" );
   streamWriter->WriteLine( "    rateOfReturn = {-0.03, 0.02, 0.01, 0.0}" );
   // Here we evaluate the standard deviation.
   double standardDeviation1;
   {
   double probability __gc[] = {0.2, 0.3, 0.3, 0.2};
   double rateOfReturn __gc[] = {-0.03, 0.02, 0.01, 0.0};
   standardDeviation1 = instanceOfTwoAssetPortfolio->StandardDeviation( 
      probability, // probability
      rateOfReturn // rateOfReturn
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The (expected) standard deviation is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( standardDeviation1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Evaluation of the Standard Deviation using Probability Distribution +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    standardDeviation( double[] historicalReturns )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the standard deviation  of  the  monthly  returns  when  the" );
   streamWriter->WriteLine( "     historical returns over the past 6 months is: 5, 6,  2,  -1,  2,  -2" );
   streamWriter->WriteLine( "     percent respectively." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalReturns = {0.05, 0.06, 0.02, -0.01, 0.02, -0.02}" );
   // Here we evaluate the standard deviation from the historical values.
   double standardDeviation2;
   {
   double historicalReturns __gc[] = {0.05, 0.06, 0.02, -0.01, 0.02, -0.02};
   standardDeviation2 = instanceOfTwoAssetPortfolio->StandardDeviation( 
      historicalReturns // historicalReturns
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The standard deviation is the returns is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( standardDeviation2 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Risk of a Portfolio with Two Assets +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    portfolio2Variance( double alpha," );
   streamWriter->WriteLine( "                        double standardDeviation1," );
   streamWriter->WriteLine( "                        double standardDeviation2," );
   streamWriter->WriteLine( "                        double correlation )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the risk of a portfolio with two assets where the alpha  (or" );
   streamWriter->WriteLine( "     weight) of the first asset is 0.6 (hence the weight  of  the  second" );
   streamWriter->WriteLine( "     asset is 0.4), the standard deviation of the returns  of  the  first" );
   streamWriter->WriteLine( "     asset is 0.25, and the standard deviation  of  the  returns  of  the" );
   streamWriter->WriteLine( "     second asset is 0.15, and the covariance between the assets is 0.5?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    alpha = 0.6" );
   streamWriter->WriteLine( "    standardDeviation1 = 0.25" );
   streamWriter->WriteLine( "    standardDeviation2 = 0.15" );
   streamWriter->WriteLine( "    correlation = 0.5" );
   double portfolio2Variance1;
   {
   portfolio2Variance1 = instanceOfTwoAssetPortfolio->Portfolio2Variance( 
      0.6, // alpha
      0.25, // standardDeviation1
      0.15, // standardDeviation2
      0.5 // correlation
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The risk of the portfolio with two assets is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( portfolio2Variance1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Expected Return of a Portfolio with Two Assets +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    portfolio2Return( double alpha," );
   streamWriter->WriteLine( "                      double expectedReturn1," );
   streamWriter->WriteLine( "                      double expectedReturn2 )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the expected return of the following two portfolios:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - 1st  Portfolio:  Alpha  (or  weight  of  first  asset)  =  0.5," );
   streamWriter->WriteLine( "        Expected return of the 1st asset = 25  percent,  Expected  return" );
   streamWriter->WriteLine( "        of the 2nd asset = 35 percent" );
   streamWriter->WriteLine( "        - 2nd Portfolio:  Alpha  (or  weight  of  second  asset)  =  0.3," );
   streamWriter->WriteLine( "        Expected return of the 1st asset = 25  percent,  Expected  return" );
   streamWriter->WriteLine( "        of the 2nd asset = 35 percent" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    alpha = 0.5" );
   streamWriter->WriteLine( "    expectedReturn1 = 0.25" );
   streamWriter->WriteLine( "    expectedReturn2 = 0.35" );
   double portfolio2Return1;
   {
   portfolio2Return1 = instanceOfTwoAssetPortfolio->Portfolio2Return( 
      0.5, // alpha
      0.25, // expectedReturn1
      0.35 // expectedReturn2
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The expected return of the 1st Portfolio is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( portfolio2Return1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Expected Return of a Portfolio with Two Assets +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    portfolio2Return( double alpha," );
   streamWriter->WriteLine( "                      double expectedReturn1," );
   streamWriter->WriteLine( "                      double expectedReturn2 )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    alpha = 0.3" );
   streamWriter->WriteLine( "    expectedReturn1 = 0.25" );
   streamWriter->WriteLine( "    expectedReturn2 = 0.35" );
   double portfolio2Return2;
   {
   portfolio2Return2 = instanceOfTwoAssetPortfolio->Portfolio2Return( 
      0.3, // alpha
      0.25, // expectedReturn1
      0.35 // expectedReturn2
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The expected return of the 2nd Portfolio is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( portfolio2Return2 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Optimal Diversification of a Portfolio with Two Assets +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    weight2MinimizeRisk( double standardDeviation1," );
   streamWriter->WriteLine( "                         double standardDeviation2," );
   streamWriter->WriteLine( "                         double covariance )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the weight of the first asset which minimizes the risk of  a" );
   streamWriter->WriteLine( "     portfolio with two assets where the standard deviation of the  first" );
   streamWriter->WriteLine( "     asset is 0.25, the standard deviation of the second asset  is  0.35," );
   streamWriter->WriteLine( "     and the covariance is one of the following:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - Covariance = 1, the assets are perfectly correlated" );
   streamWriter->WriteLine( "        - Covariance = 0.50, the assets are positively correlated" );
   streamWriter->WriteLine( "        - Covariance = 0, the assets are not correlated" );
   streamWriter->WriteLine( "        - Covariance = -0.5, the assets are negatively correlated" );
   streamWriter->WriteLine( "        -     Covariance  =  -1,  the  assets  are  perfectly  negatively" );
   streamWriter->WriteLine( "        correlated" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    standardDeviation1 = 0.25" );
   streamWriter->WriteLine( "    standardDeviation2 = 0.35" );
   streamWriter->WriteLine( "    covariance = 1" );
   double weight2MinimizeRisk1;
   {
   weight2MinimizeRisk1 = instanceOfTwoAssetPortfolio->Weight2MinimizeRisk( 
      0.25, // standardDeviation1
      0.35, // standardDeviation2
      1.0 // covariance
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The weight of  the  first  asset  which  minimizes  the  risk  of  the" );
   streamWriter->WriteLine( "    portfolio where the covariance is 1, is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( weight2MinimizeRisk1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Optimal Diversification of a Portfolio with Two Assets +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    weight2MinimizeRisk( double standardDeviation1," );
   streamWriter->WriteLine( "                         double standardDeviation2," );
   streamWriter->WriteLine( "                         double covariance )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    standardDeviation1 = 0.25" );
   streamWriter->WriteLine( "    standardDeviation2 = 0.35" );
   streamWriter->WriteLine( "    covariance = 0.5" );
   double weight2MinimizeRisk2;
   {
   weight2MinimizeRisk2 = instanceOfTwoAssetPortfolio->Weight2MinimizeRisk( 
      0.25, // standardDeviation1
      0.35, // standardDeviation2
      0.5 // covariance
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The weight of  the  first  asset  which  minimizes  the  risk  of  the" );
   streamWriter->WriteLine( "    portfolio where the covariance is 0.5, is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( weight2MinimizeRisk2 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Optimal Diversification of a Portfolio with Two Assets +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    weight2MinimizeRisk( double standardDeviation1," );
   streamWriter->WriteLine( "                         double standardDeviation2," );
   streamWriter->WriteLine( "                         double covariance )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    standardDeviation1 = 0.25" );
   streamWriter->WriteLine( "    standardDeviation2 = 0.35" );
   streamWriter->WriteLine( "    covariance = 0" );
   double weight2MinimizeRisk3;
   {
   weight2MinimizeRisk3 = instanceOfTwoAssetPortfolio->Weight2MinimizeRisk( 
      0.25, // standardDeviation1
      0.35, // standardDeviation2
      0.0 // covariance
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The weight of  the  first  asset  which  minimizes  the  risk  of  the" );
   streamWriter->WriteLine( "    portfolio where the covariance is 0, is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( weight2MinimizeRisk3 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Optimal Diversification of a Portfolio with Two Assets +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    weight2MinimizeRisk( double standardDeviation1," );
   streamWriter->WriteLine( "                         double standardDeviation2," );
   streamWriter->WriteLine( "                         double covariance )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    standardDeviation1 = 0.25" );
   streamWriter->WriteLine( "    standardDeviation2 = 0.35" );
   streamWriter->WriteLine( "    covariance = -0.5" );
   double weight2MinimizeRisk4;
   {
   weight2MinimizeRisk4 = instanceOfTwoAssetPortfolio->Weight2MinimizeRisk( 
      0.25, // standardDeviation1
      0.35, // standardDeviation2
      -0.5 // covariance
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The weight of  the  first  asset  which  minimizes  the  risk  of  the" );
   streamWriter->WriteLine( "    portfolio where the covariance is -0.5, is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( weight2MinimizeRisk4 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Optimal Diversification of a Portfolio with Two Assets +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    weight2MinimizeRisk( double standardDeviation1," );
   streamWriter->WriteLine( "                         double standardDeviation2," );
   streamWriter->WriteLine( "                         double covariance )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    standardDeviation1 = 0.25" );
   streamWriter->WriteLine( "    standardDeviation2 = 0.35" );
   streamWriter->WriteLine( "    covariance = -1" );
   double weight2MinimizeRisk5;
   {
   weight2MinimizeRisk5 = instanceOfTwoAssetPortfolio->Weight2MinimizeRisk( 
      0.25, // standardDeviation1
      0.35, // standardDeviation2
      -1.0 // covariance
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The weight of  the  first  asset  which  minimizes  the  risk  of  the" );
   streamWriter->WriteLine( "    portfolio where the covariance is -1, is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( weight2MinimizeRisk5 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( " " );
   //
   myProcess->StandardInput->Close();
   myProcess->WaitForExit();
}

