#using <mscorlib.dll>

using namespace System;
using namespace System::ComponentModel;
using namespace System::Diagnostics;
using namespace System::IO;
using namespace WebCab::Lib::Util::Example;

using namespace WebCab::Libraries::Finance::Portfolio;

template< class T >
T managed2DimArray( T* ca, int dim1, int dim2 ) __gc[,] {
   T a __gc[,] = new T __gc[dim1, dim2];
   for ( int i = 0; i < dim1; i ++ ) {
      for ( int j = 0; j < dim2; j ++ ) {
         a[i, j] = ca[ i * dim2 + j ];
      }
   }
   return a;
}

static double double2Double( double d ) {
   if ( Double::IsNaN( d ) ) return d;
   if ( Double::IsNegativeInfinity( d ) ) return d;
   if ( Double::IsPositiveInfinity( d ) ) return d;
   if ( d == Double::MinValue ) return d;
   if ( d == Double::MaxValue ) return d;
   double frac = d - ( int )d;
   double frac4 = ( int )( frac * 10000 ) / 10000.0;
   return ( int )d + frac4;
}

static void displayArray( double a __gc[], StreamWriter* streamWriter ) {
   for ( int i = 0; i < a->Length; i ++ ) {
      streamWriter->Write( double2Double( a[ i ] ) );
      streamWriter->Write( ", " );
   }
   streamWriter->WriteLine( " " );
}

int main() {
   Process* myProcess;
   myProcess = new Process ();
   myProcess->StartInfo->FileName = "more.com";
   myProcess->StartInfo->UseShellExecute = false;
   myProcess->StartInfo->RedirectStandardInput = true;
   myProcess->Start();
   StreamWriter* streamWriter = myProcess->StandardInput;
   streamWriter->AutoFlush = true;
   // used components
   Volatility* instanceOfVolatility = new Volatility;
   //
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "                    ##########################################" );
   streamWriter->WriteLine( "                    # FurtherExamples Console Client Example #" );
   streamWriter->WriteLine( "                    ##########################################" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " Overview" );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    Here we provide more  examples  which  illustrate  the  use  of  the" );
   streamWriter->WriteLine( "    Volatility class." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Press ENTER or SPACE to scroll down." );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Changing the Scale of the Volatility +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    yearDaysRescaling( double volPerAnnum," );
   streamWriter->WriteLine( "                       double daysRescaledTo," );
   streamWriter->WriteLine( "                       int convention )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     If the volatility measured  over  a  year  (i.e.  365  days)  is  12" );
   streamWriter->WriteLine( "     percent, what is same assets volatility wrt 5 day period?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    volPerAnnum = 0.12" );
   streamWriter->WriteLine( "    daysRescaledTo = 5" );
   streamWriter->WriteLine( "    convention = 2" );
   // Here we change the scale to which the volatility is expressed.
   double scale1;
   {
   scale1 = instanceOfVolatility->YearDaysRescaling( 
      0.12, // volPerAnnum
      5.0, // daysRescaledTo
      2 // convention
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The volatility with respect to the 5 days given is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( scale1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Historical Volatility Estimate +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalEstimate( double[] assetPrices )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the historical estimate of the volatility of an asset  which" );
   streamWriter->WriteLine( "     over the past 12 months, which took the following values:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     [ 23.4, 24.1, 24.5, 23.9, 24.5, 24.9, 25.0, 24.8, 25.1, 25.7,  26.0," );
   streamWriter->WriteLine( "     25.8]" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     and during this period paid a dividend of  2  in  the  1st  and  7th" );
   streamWriter->WriteLine( "     month?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    assetPrices = {23.4, 24.1, 24.5, 23.9, 24.5, 24.9, 25.0, 24.8, 25.1, 25.7, 26.0, 25.8}" );
   // He we estimate the volatility from the historical prices.
   double historical1;
   {
   double assetPrices __gc[] = {23.4, 24.1, 24.5, 23.9, 24.5, 24.9, 25.0, 24.8, 25.1, 25.7, 26.0, 25.8};
   historical1 = instanceOfVolatility->HistoricalEstimate( 
      assetPrices // assetPrices
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The historical estimate of the volatility is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( historical1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Here we evaluate the standard Error of the Above Estimate +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalEstimateStandardError( double numberOfDays," );
   streamWriter->WriteLine( "                                     double volatilityEstimate )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     Here we evaluate the standard Error  of  the  historical  volatility" );
   streamWriter->WriteLine( "     estimate above." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    numberOfDays = 12" );
   streamWriter->WriteLine( "    volatilityEstimate = historical1" );
   // Here we evaluate the error of the above estimate.
   double errorEstaimate1;
   {
   errorEstaimate1 = instanceOfVolatility->HistoricalEstimateStandardError( 
      12.0, // numberOfDays
      historical1 // volatilityEstimate
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The standard error is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( errorEstaimate1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Historical Estimate of the Volatility with Dividends +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalEstimateWithDividends( double[] assetPrices," );
   streamWriter->WriteLine( "                                     double[] dividendsPaid )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     What is the historical estimate of the volatility of an asset  which" );
   streamWriter->WriteLine( "     over the past 12 months, which took the following values:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     [ 23.4, 24.1, 24.5, 23.9, 24.5, 24.9, 25.0, 24.8, 25.1, 25.7,  26.0," );
   streamWriter->WriteLine( "     25.8 ]" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     and during this period paid a dividend of  2  in  the  1st  and  7th" );
   streamWriter->WriteLine( "     month?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    assetPrices = {23.4, 24.1, 24.5, 23.9, 24.5, 24.9, 25.0, 24.8, 25.1, 25.7, 26.0, 25.8}" );
   streamWriter->WriteLine( "    dividendsPaid = {2.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0, 0.0, 0.0, 0.0, 0.0, 0.0}" );
   // Here we estimate the volatility  based  in  historical  values  for  a
   // dividend paying stock.
   double historicalDividends1;
   {
   double assetPrices __gc[] = {23.4, 24.1, 24.5, 23.9, 24.5, 24.9, 25.0, 24.8, 25.1, 25.7, 26.0, 25.8};
   double dividendsPaid __gc[] = {2.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.0, 0.0, 0.0, 0.0, 0.0, 0.0};
   historicalDividends1 = instanceOfVolatility->HistoricalEstimateWithDividends( 
      assetPrices, // assetPrices
      dividendsPaid // dividendsPaid
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The historical estimate of the volatility is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( historicalDividends1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Apply the ARCH Volatility Model +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    archVolatilityEstimate( double longTermVolatility," );
   streamWriter->WriteLine( "                            double weightOfVolatility," );
   streamWriter->WriteLine( "                            double[] observations," );
   streamWriter->WriteLine( "                            double[] weights )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     If the long term volatility of the SP500 index is 1.32  percent  per" );
   streamWriter->WriteLine( "     day and the value  of  the  index  over  the  last  week  has  been:" );
   streamWriter->WriteLine( "     1147.39, 1144.58, 1138.49, 1131.87, 1148.7. According  to  the  ARCH" );
   streamWriter->WriteLine( "     Volatility model what will the volatility be?" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     In order to apply the ARCH model we are required to  select  weights" );
   streamWriter->WriteLine( "     for the historical values of the index and the long term average  of" );
   streamWriter->WriteLine( "     the volatility. The weights selected will depend upon  the  personal" );
   streamWriter->WriteLine( "     judgment of the user. If we selected the weights: 0.39, 0.20,  0.15," );
   streamWriter->WriteLine( "     0.10, 0.05 for the last week, and a weight  of  0.11  for  the  long" );
   streamWriter->WriteLine( "     term volatility." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    longTermVolatility = 0.0132" );
   streamWriter->WriteLine( "    weightOfVolatility = 0.11" );
   streamWriter->WriteLine( "    observations = {1147.39, 1144.58, 1138.49, 1131.8699, 1148.7}" );
   streamWriter->WriteLine( "    weights = {0.39, 0.2, 0.15, 0.1, 0.05}" );
   // Apply the ARCH volatility model.
   double arch1;
   {
   double observations __gc[] = {1147.39, 1144.58, 1138.49, 1131.8699, 1148.7};
   double weights __gc[] = {0.39, 0.2, 0.15, 0.1, 0.05};
   arch1 = instanceOfVolatility->ArchVolatilityEstimate( 
      0.0132, // longTermVolatility
      0.11, // weightOfVolatility
      observations, // observations
      weights // weights
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The estimate of the volatility according to the ARCH model is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( arch1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Estimate the Volatility using hte EWMA Model +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    ewmaVolatilityEstimate( double weightRatio," );
   streamWriter->WriteLine( "                            double i_1thDayEstimateOfVolatility," );
   streamWriter->WriteLine( "                            double endi_1thDay," );
   streamWriter->WriteLine( "                            double starti_1thDay )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     What is the estimate of the volatility of the price today  according" );
   streamWriter->WriteLine( "     to the Exponentially Weighted Moving Average model  when  the  price" );
   streamWriter->WriteLine( "     volatility yesterday was recorded to  be  1  percent  per  day,  the" );
   streamWriter->WriteLine( "     asset price at the start of trading yesterday was 20, and closed  at" );
   streamWriter->WriteLine( "     20.404? We take the weight ratio to be 0.90." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    weightRatio = 0.90" );
   streamWriter->WriteLine( "    i_1thDayEstimateOfVolatility = 0.01" );
   streamWriter->WriteLine( "    endi_1thDay = 20.404" );
   streamWriter->WriteLine( "    starti_1thDay = 20" );
   // Here we estimate the volatility using the EWMA model.
   double ewma1;
   {
   ewma1 = instanceOfVolatility->EwmaVolatilityEstimate( 
      0.90, // weightRatio
      0.01, // i_1thDayEstimateOfVolatility
      20.404, // endi_1thDay
      20.0 // starti_1thDay
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The estimate of the volatility is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( ewma1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ EWMA model using Induction +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    ewmaVolatilityEstimateInduction( double weightRatio," );
   streamWriter->WriteLine( "                                     double firstDaysVolatilityEstimate," );
   streamWriter->WriteLine( "                                     double[] endOfDay," );
   streamWriter->WriteLine( "                                     double[] startOfDay )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "     What is the estimate of the price volatility today according to  the" );
   streamWriter->WriteLine( "     Exponentially Weighted Moving Average model when  the  weight  ratio" );
   streamWriter->WriteLine( "     (for the past 5 days) is taken to be 0.90,  the  volatility  5  days" );
   streamWriter->WriteLine( "     ago was 1 percent per day, and the asset price at the start and  end" );
   streamWriter->WriteLine( "     of the last 5 trading sessions was:" );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( "        - start of day = [21, 21.3, 21.2, 21.4, 21.5]" );
   streamWriter->WriteLine( "        - end of day = [21.2, 21.2, 21.35, 21.4, 21.6]" );
   streamWriter->WriteLine( "        " );
   streamWriter->WriteLine( "     " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    weightRatio = 0.9" );
   streamWriter->WriteLine( "    firstDaysVolatilityEstimate = 0.01" );
   streamWriter->WriteLine( "    endOfDay = {21.2, 21.2, 21.35, 21.4, 21.6}" );
   streamWriter->WriteLine( "    startOfDay = {21.0, 21.3, 21.2, 21.4, 21.5}" );
   // Here is use induction in order to estimate the  volatility  using  hte
   // EWMA model.
   double ewmaInduction1;
   {
   double endOfDay __gc[] = {21.2, 21.2, 21.35, 21.4, 21.6};
   double startOfDay __gc[] = {21.0, 21.3, 21.2, 21.4, 21.5};
   ewmaInduction1 = instanceOfVolatility->EwmaVolatilityEstimateInduction( 
      0.9, // weightRatio
      0.01, // firstDaysVolatilityEstimate
      endOfDay, // endOfDay
      startOfDay // startOfDay
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The estimate of the price volatility is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( ewmaInduction1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( " " );
   //
   myProcess->StandardInput->Close();
   myProcess->WaitForExit();
}

