#using <mscorlib.dll>

using namespace System;
using namespace System::ComponentModel;
using namespace System::Diagnostics;
using namespace System::IO;
using namespace WebCab::Lib::Util::Example;

using namespace WebCab::Libraries::Finance::Portfolio;

template< class T >
T managed2DimArray( T* ca, int dim1, int dim2 ) __gc[,] {
   T a __gc[,] = new T __gc[dim1, dim2];
   for ( int i = 0; i < dim1; i ++ ) {
      for ( int j = 0; j < dim2; j ++ ) {
         a[i, j] = ca[ i * dim2 + j ];
      }
   }
   return a;
}

static double double2Double( double d ) {
   if ( Double::IsNaN( d ) ) return d;
   if ( Double::IsNegativeInfinity( d ) ) return d;
   if ( Double::IsPositiveInfinity( d ) ) return d;
   if ( d == Double::MinValue ) return d;
   if ( d == Double::MaxValue ) return d;
   double frac = d - ( int )d;
   double frac4 = ( int )( frac * 10000 ) / 10000.0;
   return ( int )d + frac4;
}

static void displayArray( double a __gc[], StreamWriter* streamWriter ) {
   for ( int i = 0; i < a->Length; i ++ ) {
      streamWriter->Write( double2Double( a[ i ] ) );
      streamWriter->Write( ", " );
   }
   streamWriter->WriteLine( " " );
}

int main() {
   Process* myProcess;
   myProcess = new Process ();
   myProcess->StartInfo->FileName = "more.com";
   myProcess->StartInfo->UseShellExecute = false;
   myProcess->StartInfo->RedirectStandardInput = true;
   myProcess->Start();
   StreamWriter* streamWriter = myProcess->StandardInput;
   streamWriter->AutoFlush = true;
   // used components
   Volatility* instanceOfVolatility = new Volatility;
   //
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "                      #####################################" );
   streamWriter->WriteLine( "                      # Volatility Console Client Example #" );
   streamWriter->WriteLine( "                      #####################################" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " Overview" );
   streamWriter->WriteLine( " --------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    Within this example we provide one example for each of the  business" );
   streamWriter->WriteLine( "    methods within the Volatility Class." );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Press ENTER or SPACE to scroll down." );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ The value of the volatility over a  given  number  of  days  when  the +" );
   streamWriter->WriteLine( "+ annual volatility is known                                             +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    yearDaysRescaling( double volPerAnnum," );
   streamWriter->WriteLine( "                       double daysRescaledTo," );
   streamWriter->WriteLine( "                       int convention )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the volatility per 30 days when the annual  volatility  with" );
   streamWriter->WriteLine( "     respect to the 360 day count convention is 25 percent?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    volPerAnnum = 0.25" );
   streamWriter->WriteLine( "    daysRescaledTo = 30" );
   streamWriter->WriteLine( "    convention = 1" );
   // Calculates the value of the volatility over a  given  number  of  days
   // when the annual volatility is known.  The  annual  volatility  may  be
   // given with respect to the 360 day convention.
   double yearDaysRescaling1;
   {
   yearDaysRescaling1 = instanceOfVolatility->YearDaysRescaling( 
      0.25, // volPerAnnum
      30.0, // daysRescaledTo
      1 // convention
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The volatility per 30 days is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( yearDaysRescaling1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ The annual volatility when the volatility of a given  number  of  days +" );
   streamWriter->WriteLine( "+ is known                                                               +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    daysYearRescaling( double volPerPeriod," );
   streamWriter->WriteLine( "                       double daysInPeriod," );
   streamWriter->WriteLine( "                       int convention )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the equivalent annual volatility in accordance with the  365" );
   streamWriter->WriteLine( "     day count  convention  when  the  volatility  over  30  days  is  12" );
   streamWriter->WriteLine( "     percent?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    volPerPeriod = 0.12" );
   streamWriter->WriteLine( "    daysInPeriod = 30" );
   streamWriter->WriteLine( "    convention = 2" );
   // Evaluates the annual volatility when the volatility of a given  number
   // of days is known. The annual volatility may be evaluated  with  respec
   // to either the 365 day convention.
   double daysYearRescaling;
   {
   daysYearRescaling = instanceOfVolatility->DaysYearRescaling( 
      0.12, // volPerPeriod
      30.0, // daysInPeriod
      2 // convention
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The annual volatility in accordance with the 356 day count  convention" );
   streamWriter->WriteLine( "    when the volatility over 30 days is 12 percent is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( daysYearRescaling ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Historical estimate of the present volatility +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalEstimate( double[] assetPrices )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the historical estimate of the present volatility  when  the" );
   streamWriter->WriteLine( "     assets price at the end of each interval where: 100, 101,  104,  102" );
   streamWriter->WriteLine( "     and 99?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    assetPrices = {100.0, 101.0, 104.0, 102.0, 99.0}" );
   // Estimate  (present)  volatility  per  interval  expressed  in  decimal
   // format (i.e. 1 percent = 0.01) using the price  data  at  the  end  of
   // equal time intervals  of  an  investment  asset  which  does  not  pay
   // dividends or interest payments.
   double historicalEstimate1;
   {
   double assetPrices __gc[] = {100.0, 101.0, 104.0, 102.0, 99.0};
   historicalEstimate1 = instanceOfVolatility->HistoricalEstimate( 
      assetPrices // assetPrices
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The historical estimate of the present volatility is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( historicalEstimate1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Historical estimate of the present volatility taking into account  the +" );
   streamWriter->WriteLine( "+ dividends                                                              +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalEstimateWithDividends( double[] assetPrices," );
   streamWriter->WriteLine( "                                     double[] dividendsPaid )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the historical estimate of the present volatility  when  the" );
   streamWriter->WriteLine( "     assets price at the end of each interval where: 100, 101,  104,  102" );
   streamWriter->WriteLine( "     and 99 and the dividends  (or  interest  payments)  from  the  asset" );
   streamWriter->WriteLine( "     within each period where: 4, 0, 0, 0, 5?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    assetPrices = {100.0, 101.0, 104.0, 102.0, 99.0}" );
   streamWriter->WriteLine( "    dividendsPaid = {4.0, 0.0, 0.0, 0.0, 5.0}" );
   // Calculates the historical estimate of the  present  volatility  taking
   // into account the dividends or  interest  payments  of  the  underlying
   // asset.
   double historicalEstimateWithDividends1;
   {
   double assetPrices __gc[] = {100.0, 101.0, 104.0, 102.0, 99.0};
   double dividendsPaid __gc[] = {4.0, 0.0, 0.0, 0.0, 5.0};
   historicalEstimateWithDividends1 = instanceOfVolatility->HistoricalEstimateWithDividends( 
      assetPrices, // assetPrices
      dividendsPaid // dividendsPaid
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The historical estimate of the present volatility with dividends is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( historicalEstimateWithDividends1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Returns the estimate of the standard error of the standard  historical +" );
   streamWriter->WriteLine( "+ estimation                                                             +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalEstimateStandardError( double numberOfDays," );
   streamWriter->WriteLine( "                                     double volatilityEstimate )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the  standard  error  of  the  historical  estimate  of  the" );
   streamWriter->WriteLine( "     volatility, when the volatility is estimated using 180 days of  data" );
   streamWriter->WriteLine( "     and the volatility found was 0.04 (or 4 percent)?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    numberOfDays = 180" );
   streamWriter->WriteLine( "    volatilityEstimate = 0.04" );
   // Returns the estimate of the standard error of the standard  historical
   // estimation given by  the  functions  volatilityHistoricalEstimate  and
   // volatilityHistoricalEstimateWithDividends.
   double historicalEstimateStandardError1;
   {
   historicalEstimateStandardError1 = instanceOfVolatility->HistoricalEstimateStandardError( 
      180.0, // numberOfDays
      0.04 // volatilityEstimate
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The estimate of the standard error is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( historicalEstimateStandardError1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Volatility according to the ARCH model +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    archVolatilityEstimate( double longTermVolatility," );
   streamWriter->WriteLine( "                            double weightOfVolatility," );
   streamWriter->WriteLine( "                            double[] observations," );
   streamWriter->WriteLine( "                            double[] weights )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the estimate of the volatility  considering  the  long  term" );
   streamWriter->WriteLine( "     volatility is 0.04, the weight assigned to the  volatility  is  0.4," );
   streamWriter->WriteLine( "     the historical observations where: 78, 67,  89;  and  the  array  of" );
   streamWriter->WriteLine( "     weights associated to these historical observations are:  0.2,  0.3," );
   streamWriter->WriteLine( "     0.1?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    longTermVolatility = 0.04" );
   streamWriter->WriteLine( "    weightOfVolatility = 0.4" );
   streamWriter->WriteLine( "    observations = {78.0, 67.0, 89.0}" );
   streamWriter->WriteLine( "    weights = {0.2, 0.3, 0.1}" );
   // Returns the estimate of the volatility according to the ARCH model.
   double archVolatilityEstimate1;
   {
   double observations __gc[] = {78.0, 67.0, 89.0};
   double weights __gc[] = {0.2, 0.3, 0.1};
   archVolatilityEstimate1 = instanceOfVolatility->ArchVolatilityEstimate( 
      0.04, // longTermVolatility
      0.4, // weightOfVolatility
      observations, // observations
      weights // weights
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The estimate of the volatility using this ARCH model is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( archVolatilityEstimate1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ Volatility for the ith day made at the end  of  the  previous  (i-1)th +" );
   streamWriter->WriteLine( "+ day                                                                    +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    ewmaVolatilityEstimate( double weightRatio," );
   streamWriter->WriteLine( "                            double i_1thDayEstimateOfVolatility," );
   streamWriter->WriteLine( "                            double endi_1thDay," );
   streamWriter->WriteLine( "                            double starti_1thDay )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     Estimate the volatility on the i-th  day  using  the  EWMA  approach" );
   streamWriter->WriteLine( "     when the estimate of the volatility on the  (i-1)-th  day  is  0.04," );
   streamWriter->WriteLine( "     the weight ration used is 0.3, the market price at the close of  the" );
   streamWriter->WriteLine( "     (i-1)th day is 120, and  the  market  price  at  the  start  of  the" );
   streamWriter->WriteLine( "     (i-1)th day is 123?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    weightRatio = 0.3" );
   streamWriter->WriteLine( "    i_1thDayEstimateOfVolatility = 0.04" );
   streamWriter->WriteLine( "    endi_1thDay = 120" );
   streamWriter->WriteLine( "    starti_1thDay = 123" );
   // Returns the estimate of the volatility for the ith  day  made  at  the
   // end of the previous (i-1)th day, using an estimate of  the  volatility
   // on the (i-1)th day  according  to  the  EWMA  model  with  respect  to
   // changes in the market variable.
   double ewmaVolatilityEstimate1;
   {
   ewmaVolatilityEstimate1 = instanceOfVolatility->EwmaVolatilityEstimate( 
      0.3, // weightRatio
      0.04, // i_1thDayEstimateOfVolatility
      120.0, // endi_1thDay
      123.0 // starti_1thDay
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The estimate of the volatility on the (i-1)-th day is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( ewmaVolatilityEstimate1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ The continuously compounded return of an asset over one day +" );
   streamWriter->WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    returnDuringithDay( double ithDayAssetPrice," );
   streamWriter->WriteLine( "                        double i_1thDayAssetPrice )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the continuously compounded return of an asset over one  day" );
   streamWriter->WriteLine( "     if the asset price at the close of business on the ith  day  is  120" );
   streamWriter->WriteLine( "     and the asset price at the close of business on  the  i-1th  day  is" );
   streamWriter->WriteLine( "     126?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    ithDayAssetPrice = 120" );
   streamWriter->WriteLine( "    i_1thDayAssetPrice = 126" );
   // This function returns the continuously compounded return of  an  asset
   // over one day.
   double returnDuringithDay1;
   {
   returnDuringithDay1 = instanceOfVolatility->ReturnDuringithDay( 
      120.0, // ithDayAssetPrice
      126.0 // i_1thDayAssetPrice
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The continuously compounded return is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( returnDuringithDay1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ The variance of the expected returns of an asset +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    variance( double[] probability," );
   streamWriter->WriteLine( "              double[] returns )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the variance of the expected return of an  asset  given  the" );
   streamWriter->WriteLine( "     array of probability: 0.3, 0.5, 0.2; and the returns  of  the  asset" );
   streamWriter->WriteLine( "     are 10, 20 and 10?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    probability = {0.3, 0.5, 0.2}" );
   streamWriter->WriteLine( "    returns = {10.0, 20.0, 10.0}" );
   // Calculates the variance of the expected returns of an asset given  the
   // assets returns in given market states and  the  probability  of  those
   // market states occurring.
   double variance1;
   {
   double probability __gc[] = {0.3, 0.5, 0.2};
   double returns __gc[] = {10.0, 20.0, 10.0};
   variance1 = instanceOfVolatility->Variance( 
      probability, // probability
      returns // returns
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The variance of the expected return is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( variance1 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( "+ The variance of the historical returns of an asset +" );
   streamWriter->WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " Within this example we test the following method:" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    variance( double[] historicalReturns )" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " Question:" );
   streamWriter->WriteLine( " ---------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "     What is the variance of the historical returns  of  an  asset  given" );
   streamWriter->WriteLine( "     the returns of the asset are 11, 12, 10?" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " Parameter values required to solve this problem:" );
   streamWriter->WriteLine( " ------------------------------------------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    historicalReturns = {11.0, 12.0, 10.0}" );
   // Evaluates the variance of the historical returns of an asset.
   double variance2;
   {
   double historicalReturns __gc[] = {11.0, 12.0, 10.0};
   variance2 = instanceOfVolatility->Variance( 
      historicalReturns // historicalReturns
   );
   }
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " Answer evaluated:" );
   streamWriter->WriteLine( " -----------------" );
   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "    The variance of the historical return is:" );
   streamWriter->WriteLine( " " );
   streamWriter->Write( "    " );
   streamWriter->WriteLine( double2Double( variance2 ) );
   streamWriter->WriteLine( " " );

   streamWriter->WriteLine( " " );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
   streamWriter->WriteLine( "       -------------------------------------------------------------------" );
   streamWriter->WriteLine( " " );
   //
   myProcess->StandardInput->Close();
   myProcess->WaitForExit();
}

