using System;
using WebCab.Lib.Util.Example;

public class CapitalMarketClient {
   public static void Main( String[] args ) {
      // used components
      AssetParameters instanceOfAssetParameters = new AssetParameters();
      CapitalMarket instanceOfCapitalMarket = new CapitalMarket();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "                     ########################################" );
      System.Console.WriteLine( "                     # CapitalMarket Console Client Example #" );
      System.Console.WriteLine( "                     ########################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Within this client we offer one  example  for  each  of  the  method" );
      System.Console.WriteLine( "    contained within Capital Market." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluate Points on Efficient Frontier +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    calculateEfficientFrontier( double minimumExpectedReturn," );
      System.Console.WriteLine( "                                double maximumExpectedReturn," );
      System.Console.WriteLine( "                                double[][] covarianceMatrix," );
      System.Console.WriteLine( "                                double[] expectedReturns," );
      System.Console.WriteLine( "                                int numberInterpolationPoints," );
      System.Console.WriteLine( "                                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We are able to  construct  portfolios  from  four  assets  with  the" );
      System.Console.WriteLine( "     following historical returns over the last 4 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" );
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" );
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" );
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Evaluate 5 points on the Efficient Frontier with the  precision  set" );
      System.Console.WriteLine( "     to 0.00001" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = min1" );
      System.Console.WriteLine( "    maximumExpectedReturn = max1" );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedreturns1" );
      System.Console.WriteLine( "    numberInterpolationPoints = 5" );
      System.Console.WriteLine( "    precision = 0.0000000001" );
      // We first use the covariance method from Asset Parameters
      double[][] covariance1 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      // Next we evaluate the expected returns using  the  historical  approach
      // from Asset Parameters.
      double[] expectedreturns1 = instanceOfAssetParameters.ExpectedReturns( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      // Here we evaluate the maximum of the expected returns  over  which  the
      // Efficient Frontier exits.
      double max1 = instanceOfCapitalMarket.MaxFrontierReturn( 
         expectedreturns1 // expectedReturns
      );
      // Here we evaluate the minimum of the expected returns  over  which  the
      // Efficient Frontier exits.
      double min1 = instanceOfCapitalMarket.MinFrontierReturn( 
         expectedreturns1 // expectedReturns
      );
      // This method sets the coordinates of the Efficient Frontier as  private
      // fields in the class.
      instanceOfCapitalMarket.CalculateEfficientFrontier( 
         min1, // minimumExpectedReturn
         max1, // maximumExpectedReturn
         covariance1, // covarianceMatrix
         expectedreturns1, // expectedReturns
         5, // numberInterpolationPoints
         0.0000000001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Our first aim in virtually all applications of Portfolio Theory is  to" );
      System.Console.WriteLine( "    evaluate a collection of points on the Efficient  Frontier,  which  we" );
      System.Console.WriteLine( "    do to applying the calculate Efficient Frontier  method.  Before  this" );
      System.Console.WriteLine( "    method is applied we need to  evaluate  the  covariance  and  expected" );
      System.Console.WriteLine( "    returns using procedures from the Asset Parameters class.  The  points" );
      System.Console.WriteLine( "    of the Efficient Frontier are stored in private fields of the class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Get the Efficient Frontier Expected Returns +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we read-off the values of the expected returns  at  the  points" );
      System.Console.WriteLine( "     on which the Efficient Frontier has been evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Here we get the values of the Efficient Frontier which has  just  been
      // evaluated.
      double[] getReturnsEfficient1 = instanceOfCapitalMarket.GetEfficientFrontierExpectedReturns( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The value of the expected returns are:" );
      System.Console.WriteLine( " " );
      displayArray( getReturnsEfficient1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Get the Asset Weights of the Efficient Frontier Portfolio's Evaluated +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierAssetWeights(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we get the Asset weights of the portfolio's which  lie  of  the" );
      System.Console.WriteLine( "     Efficient Frontier which have been  evaluated  and  set  to  private" );
      System.Console.WriteLine( "     fields." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Here we get the Asset weights of the  portfolio's  which  lie  of  the
      // Efficient Frontier which  have  been  evaluated  and  set  to  private
      // fields.
      double[][] getAssetWeights1 = instanceOfCapitalMarket.GetEfficientFrontierAssetWeights( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the asset with the Efficient Frontier Portfolio are:" );
      System.Console.WriteLine( " " );
      display2DArray( getAssetWeights1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Get the values of the risk of the Portfolio on the Efficient Frontier +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we read off the values of the risk of  the  Portfolios  on  the" );
      System.Console.WriteLine( "     Efficient Frontier which have been evaluated and set to the  private" );
      System.Console.WriteLine( "     fields." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = covarianceMatrix2" );
      // We first use the covariance method from Asset Parameters
      double[][] covarianceMatrix2 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      // Here we evaluate the values of the  risk  of  the  Portfolios  on  the
      // Efficient Frontier.
      double[] getRisks1 = instanceOfCapitalMarket.GetEfficientFrontierPortfolioRisks( 
         covarianceMatrix2 // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the  risk  of  the  portfolios  at  the  points  of  the" );
      System.Console.WriteLine( "    Efficient Frontier which have been evaluated are:" );
      System.Console.WriteLine( " " );
      displayArray( getRisks1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Setting constraints on the Weights of the assets +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    setConstraints( double[] lowerBounds," );
      System.Console.WriteLine( "                    double[] upperBounds )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     How can constraints be set on the weights of the  assets  such  that" );
      System.Console.WriteLine( "     all portfolios of the  Efficient  Frontier  have  no  asset  with  a" );
      System.Console.WriteLine( "     weights more than 0.5 (i.e. 50 percent)  and  no  weight  below  0.1" );
      System.Console.WriteLine( "     (i.e. 10 percent)?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    lowerBounds = {0.1, 0.1, 0.1, 0.1}" );
      System.Console.WriteLine( "    upperBounds = {0.5, 0.5, 0.5, 0.5}" );
      instanceOfCapitalMarket.SetConstraints( 
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5} // upperBounds
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The application of this method  will  allow  the  constraints  on  the" );
      System.Console.WriteLine( "    Efficient Frontier portfolios assets weights  to  be  set  before  the" );
      System.Console.WriteLine( "    points on the Efficient Frontier are  evaluated  using  the  calculate" );
      System.Console.WriteLine( "    Efficient Frontier method." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Read off the values of the Lower Constraints +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getLowerConstraints(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Get the value of the lower bounds on the asset  weights  after  they" );
      System.Console.WriteLine( "     have been set." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      instanceOfCapitalMarket.SetConstraints( 
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5} // upperBounds
      );
      // Get the value of the lower bounds of the assets.
      double[] readLowerBounds = instanceOfCapitalMarket.GetLowerConstraints( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The lower bounds on the assets weights are:" );
      System.Console.WriteLine( " " );
      displayArray( readLowerBounds );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Read off the values of the Upper Constraints +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getUpperConstraints(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Get the value of the upper bounds on the asset  weights  after  they" );
      System.Console.WriteLine( "     have been set." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      instanceOfCapitalMarket.SetConstraints( 
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5} // upperBounds
      );
      // Get the value of the upper bounds of the assets.
      double[] readUpperBounds = instanceOfCapitalMarket.GetUpperConstraints( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The upper bounds on the assets weights are:" );
      System.Console.WriteLine( " " );
      displayArray( readUpperBounds );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluation of the Market Portfolio +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolio( double minimumExpectedReturn," );
      System.Console.WriteLine( "                     double maximumExpectedReturn," );
      System.Console.WriteLine( "                     double[][] covarianceMatrix," );
      System.Console.WriteLine( "                     double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the market portfolio (i.e. the portfolio  on  the  Efficient" );
      System.Console.WriteLine( "     Frontier with the highest expected return per  unit  of  risk)  when" );
      System.Console.WriteLine( "     the each portfolio can be  constructed  from  the  following  assets" );
      System.Console.WriteLine( "     with the following historical returns over the last 4 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" );
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" );
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" );
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     where the weights of each asset within  the  market  portfolio  lies" );
      System.Console.WriteLine( "     within the range [0.1, 0.5] (i.e. between 10 and 50 percent)?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = min2" );
      System.Console.WriteLine( "    maximumExpectedReturn = max2" );
      System.Console.WriteLine( "    covarianceMatrix = covarianceMarketPortfolio1" );
      System.Console.WriteLine( "    marketRate = 0.001" );
      // We first use the covariance method from Asset Parameters
      double[][] covarianceMarketPortfolio1 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      // Next we evaluate the expected returns using  the  historical  approach
      // from Asset Parameters.
      double[] expectedreturns2 = instanceOfAssetParameters.ExpectedReturns( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      instanceOfCapitalMarket.SetConstraints( 
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5} // upperBounds
      );
      // Here we evaluate the maximum of the expected returns  over  which  the
      // Efficient Frontier exits.
      double max2 = instanceOfCapitalMarket.MaxFrontierReturn( 
         expectedreturns2 // expectedReturns
      );
      // Here we evaluate the minimum of the expected returns  over  which  the
      // Efficient Frontier exits.
      double min2 = instanceOfCapitalMarket.MinFrontierReturn( 
         expectedreturns2 // expectedReturns
      );
      // Here we evaluate the market  portfolio  which  depends  on  the  prior
      // evaluation of the (constrained) Efficient Frontier
      double[] marketPortWeights = instanceOfCapitalMarket.MarketPortfolio( 
         min2, // minimumExpectedReturn
         max2, // maximumExpectedReturn
         covarianceMarketPortfolio1, // covarianceMatrix
         0.001 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the assets from  which  the  market  portfolio  can  be" );
      System.Console.WriteLine( "    constructed with respect to the constraints is:" );
      System.Console.WriteLine( " " );
      displayArray( marketPortWeights );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the Expected Return of the Market Portfolio +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[] weights," );
      System.Console.WriteLine( "                                   double[][] historicalReturns )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the expected return (based on a  historical  estimate)  when" );
      System.Console.WriteLine( "     the market  portfolio  is  constructed  from  the  assets  with  the" );
      System.Console.WriteLine( "     following historical returns over the last 4 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" );
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" );
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" );
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     where the weights of each asset within  the  market  portfolio  lies" );
      System.Console.WriteLine( "     within the range [0.1, 0.5] (i.e. between 10 and 50 percent)?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weights = marketPortWeights1" );
      System.Console.WriteLine( "    historicalReturns = {{0.02, 0.01, -0.02, 0.02}," );
      System.Console.WriteLine( "                         {0.0, 0.03, -0.01, 0.01}," );
      System.Console.WriteLine( "                         {0.0, -0.01, 0.03, 0.01}," );
      System.Console.WriteLine( "                         {0.01, 0.01, 0.02, 0.01}}" );
      // We first use the covariance method from Asset Parameters.
      double[][] covarianceMarketPortfolio2 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      // Next we evaluate the expected returns using  the  historical  approach
      // from Asset Parameters.
      double[] expectedreturns3 = instanceOfAssetParameters.ExpectedReturns( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      instanceOfCapitalMarket.SetConstraints( 
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5} // upperBounds
      );
      // Here we evaluate the maximum of the expected returns  over  which  the
      // Efficient Frontier exits.
      double max3 = instanceOfCapitalMarket.MaxFrontierReturn( 
         expectedreturns3 // expectedReturns
      );
      // Here we evaluate the minimum of the expected returns  over  which  the
      // Efficient Frontier exits.
      double min3 = instanceOfCapitalMarket.MinFrontierReturn( 
         expectedreturns3 // expectedReturns
      );
      // Here we evaluate the market  portfolio  which  depends  on  the  prior
      // evaluation of the (constrained) Efficient Frontier
      double[] marketPortWeights1 = instanceOfCapitalMarket.MarketPortfolio( 
         min3, // minimumExpectedReturn
         max3, // maximumExpectedReturn
         covarianceMarketPortfolio2, // covarianceMatrix
         0.001 // marketRate
      );
      // We provide the market  portfolios  weights  which  requires  that  the
      // Efficient Frontier is evaluted and the constraints set
      double marketPortfolioExpectedReturn1 = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( 
         marketPortWeights1, // weights
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The expected return of the market portfolio constructed  with  respect" );
      System.Console.WriteLine( "    to the constraints is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioExpectedReturn1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the risk of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolioRisk( double[] weights," );
      System.Console.WriteLine( "                         double[][] covariance )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     What is the risk of the market portfolio which can constructed  from" );
      System.Console.WriteLine( "     the assets with the following historical returns  over  the  last  4" );
      System.Console.WriteLine( "     months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" );
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" );
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" );
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     where the weights of each asset within  the  market  portfolio  lies" );
      System.Console.WriteLine( "     within the range [0.1, 0.5] (i.e. between 10 and 50 percent)?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weights = marketPortWeights2" );
      System.Console.WriteLine( "    covariance = covariance4riskOfMarketPortfolio" );
      // We first evaluate the covariance matrix between  the  asset  from  the
      // market portfolio could be constructed.
      double[][] covariance4riskOfMarketPortfolio = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      // Next we evaluate the expected returns using  the  historical  approach
      // from Asset Parameters.
      double[] expectedreturns4 = instanceOfAssetParameters.ExpectedReturns( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      instanceOfCapitalMarket.SetConstraints( 
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5} // upperBounds
      );
      // Here we evaluate the maximum of the expected returns  over  which  the
      // Efficient Frontier exits.
      double max4 = instanceOfCapitalMarket.MaxFrontierReturn( 
         expectedreturns4 // expectedReturns
      );
      // Here we evaluate the minimum of the expected returns  over  which  the
      // Efficient Frontier exits.
      double min4 = instanceOfCapitalMarket.MinFrontierReturn( 
         expectedreturns4 // expectedReturns
      );
      // Here we evaluate the market  portfolio  which  depends  on  the  prior
      // evaluation of the (constrained) Efficient Frontier
      double[] marketPortWeights2 = instanceOfCapitalMarket.MarketPortfolio( 
         min4, // minimumExpectedReturn
         max4, // maximumExpectedReturn
         covariance4riskOfMarketPortfolio, // covarianceMatrix
         0.001 // marketRate
      );
      // Here we evaluate the risk of the market portfolio which requires  that
      // the weights of the market portfolio are known.
      double marketPortfolioRisk1 = instanceOfCapitalMarket.MarketPortfolioRisk( 
         marketPortWeights2, // weights
         covariance4riskOfMarketPortfolio // covariance
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the market  portfolio  constructed  with  respect  to  the" );
      System.Console.WriteLine( "    constraints is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( marketPortfolioRisk1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weightCML( double expectedReturn," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We  consider  the  situation  where  the  market  portfolio  can  be" );
      System.Console.WriteLine( "     constructed from  the  following  four  assets  with  the  following" );
      System.Console.WriteLine( "     historical returns over the last 4 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" );
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" );
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" );
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Moreover, we require that the weights of these  assets  lies  within" );
      System.Console.WriteLine( "     the interval [0.1, 0.5] (i.e. between 10 and 50  percent).  In  such" );
      System.Console.WriteLine( "     an instance what are the weights of the market  portfolio  when  the" );
      System.Console.WriteLine( "     monthly rate at which cash can be borrowed or lent  is  0.5  percent" );
      System.Console.WriteLine( "     per month, for six portfolios  with  the  following  values  of  the" );
      System.Console.WriteLine( "     expected return: 0.5, 1, 1.5, 2, 3, 4 percent per  month  which  lie" );
      System.Console.WriteLine( "     on the CML (i.e. have the lowest possible risk for a given level  of" );
      System.Console.WriteLine( "     the expected return) ?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.005" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      // We provide the market  portfolios  weights  which  requires  that  the
      // Efficient Frontier is evaluted and the constraints set
      double ExpectedReturnMarketPortfolio = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( 
         marketPortWeights, // weights
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      // Evaluates the weight when the expected return desired is equal to  the
      // return available from cash.
      double weightCML1 = instanceOfCapitalMarket.WeightCML( 
         0.005, // expectedReturn
         ExpectedReturnMarketPortfolio, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weight required when an expected return of 0.5 percent  per  month" );
      System.Console.WriteLine( "    is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weightCML1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weightCML( double expectedReturn," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.01" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      // Evaluates the market portfolio weight when the expected  return  is  1
      // percent per month.
      double weightCML2 = instanceOfCapitalMarket.WeightCML( 
         0.01, // expectedReturn
         ExpectedReturnMarketPortfolio, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weight required when an expected return of  1  percent  per  month" );
      System.Console.WriteLine( "    is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weightCML2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weightCML( double expectedReturn," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.015" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      // Evaluates the market portfolio weight when the expected return is  1.5
      // percent per month.
      double weightCML3 = instanceOfCapitalMarket.WeightCML( 
         0.015, // expectedReturn
         ExpectedReturnMarketPortfolio, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weight required when an expected return of 1.5 percent  per  month" );
      System.Console.WriteLine( "    is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weightCML3 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weightCML( double expectedReturn," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.02" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      // Evaluates the market portfolio weight when the expected  return  is  2
      // percent per month.
      double weightCML4 = instanceOfCapitalMarket.WeightCML( 
         0.02, // expectedReturn
         ExpectedReturnMarketPortfolio, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weight required when an expected return of 2.0 percent  per  month" );
      System.Console.WriteLine( "    is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weightCML4 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weightCML( double expectedReturn," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.03" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      // Evaluates the market portfolio weight when the expected  return  is  3
      // percent per month.
      double weightCML5 = instanceOfCapitalMarket.WeightCML( 
         0.03, // expectedReturn
         ExpectedReturnMarketPortfolio, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weight required when an expected return of 3.0 percent  per  month" );
      System.Console.WriteLine( "    is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weightCML5 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weightCML( double expectedReturn," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.04" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      // Evaluates the market portfolio weight when the expected  return  is  4
      // percent per month.
      double weightCML6 = instanceOfCapitalMarket.WeightCML( 
         0.04, // expectedReturn
         ExpectedReturnMarketPortfolio, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weight required when an expected return of 4.0 percent  per  month" );
      System.Console.WriteLine( "    is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weightCML6 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" );
      System.Console.WriteLine( "+ Risk                                                                   +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    returnCML( double risk," );
      System.Console.WriteLine( "               double equityRisk," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We  consider  the  situation  where  the  market  portfolio  can  be" );
      System.Console.WriteLine( "     constructed from  the  following  four  assets  with  the  following" );
      System.Console.WriteLine( "     historical returns over the last 4 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" );
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" );
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" );
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Moreover, we require that the weights of these  assets  lies  within" );
      System.Console.WriteLine( "     the interval [0.1, 0.5] (i.e. between 10 and 50  percent).  In  such" );
      System.Console.WriteLine( "     an instance what is the expected returns in decimal format  (i.e.  1" );
      System.Console.WriteLine( "     percent = 0.01) when the market portfolio  (as  constructed  in  the" );
      System.Console.WriteLine( "     previous example) when  the  monthly  rate  at  which  cash  can  be" );
      System.Console.WriteLine( "     borrowed or lent is 0.5 percent per month, for six  portfolios  with" );
      System.Console.WriteLine( "     the following values of the total risk: 2, 3, 5, 10, 20, 30  percent" );
      System.Console.WriteLine( "     per month which lie on the  CML  (i.e.  have  the  highest  possible" );
      System.Console.WriteLine( "     expected return for that level of risk) ?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = 0.02" );
      System.Console.WriteLine( "    equityRisk = equityRisk1" );
      System.Console.WriteLine( "    equityExpected = equityExpected1" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      // We first evaluate the covariance matrix between  the  asset  from  the
      // marlet portfolio could be constructed.
      double[][] covariance3 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      double equityRisk1 = instanceOfCapitalMarket.MarketPortfolioRisk( 
         marketPortWeights, // weights
         covariance3 // covariance
      );
      double equityExpected1 = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( 
         marketPortWeights, // weights
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      double returnCML1 = instanceOfCapitalMarket.ReturnCML( 
         0.02, // risk
         equityRisk1, // equityRisk
         equityExpected1, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio  when  the  risk  is  2" );
      System.Console.WriteLine( "    percent per month is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( returnCML1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" );
      System.Console.WriteLine( "+ Risk                                                                   +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    returnCML( double risk," );
      System.Console.WriteLine( "               double equityRisk," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = 0.03" );
      System.Console.WriteLine( "    equityRisk = equityRisk1" );
      System.Console.WriteLine( "    equityExpected = equityExpected1" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      double returnCML2 = instanceOfCapitalMarket.ReturnCML( 
         0.03, // risk
         equityRisk1, // equityRisk
         equityExpected1, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio  when  the  risk  is  3" );
      System.Console.WriteLine( "    percent per month is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( returnCML2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" );
      System.Console.WriteLine( "+ Risk                                                                   +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    returnCML( double risk," );
      System.Console.WriteLine( "               double equityRisk," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = 0.05" );
      System.Console.WriteLine( "    equityRisk = equityRisk1" );
      System.Console.WriteLine( "    equityExpected = equityExpected1" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      double returnCML3 = instanceOfCapitalMarket.ReturnCML( 
         0.05, // risk
         equityRisk1, // equityRisk
         equityExpected1, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio  when  the  risk  is  5" );
      System.Console.WriteLine( "    percent per month is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( returnCML3 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" );
      System.Console.WriteLine( "+ Risk                                                                   +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    returnCML( double risk," );
      System.Console.WriteLine( "               double equityRisk," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = 0.1" );
      System.Console.WriteLine( "    equityRisk = equityRisk1" );
      System.Console.WriteLine( "    equityExpected = equityExpected1" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      double returnCML4 = instanceOfCapitalMarket.ReturnCML( 
         0.1, // risk
         equityRisk1, // equityRisk
         equityExpected1, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio when  the  risk  is  10" );
      System.Console.WriteLine( "    percent per month is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( returnCML4 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" );
      System.Console.WriteLine( "+ Risk                                                                   +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    returnCML( double risk," );
      System.Console.WriteLine( "               double equityRisk," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = 0.2" );
      System.Console.WriteLine( "    equityRisk = equityRisk1" );
      System.Console.WriteLine( "    equityExpected = equityExpected1" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      double returnCML5 = instanceOfCapitalMarket.ReturnCML( 
         0.2, // risk
         equityRisk1, // equityRisk
         equityExpected1, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio when  the  risk  is  20" );
      System.Console.WriteLine( "    percent per month is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( returnCML5 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" );
      System.Console.WriteLine( "+ Risk                                                                   +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    returnCML( double risk," );
      System.Console.WriteLine( "               double equityRisk," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = 0.3" );
      System.Console.WriteLine( "    equityRisk = equityRisk1" );
      System.Console.WriteLine( "    equityExpected = equityExpected1" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      double returnCML6 = instanceOfCapitalMarket.ReturnCML( 
         0.3, // risk
         equityRisk1, // equityRisk
         equityExpected1, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio when  the  risk  is  30" );
      System.Console.WriteLine( "    percent per month is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( returnCML6 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    marketPortfolio( double minimumExpectedReturn," );
      System.Console.WriteLine( "                     double maximumExpectedReturn," );
      System.Console.WriteLine( "                     double[][] covarianceMatrix," );
      System.Console.WriteLine( "                     double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We  consider  the  situation  where  the  market  portfolio  can  be" );
      System.Console.WriteLine( "     constructed from  the  following  four  assets  with  the  following" );
      System.Console.WriteLine( "     historical returns over the last 4 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" );
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" );
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" );
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Moreover, we require that the weights of these  assets  lies  within" );
      System.Console.WriteLine( "     the interval [0.1, 0.5] (i.e. between 10 and 50  percent).  In  such" );
      System.Console.WriteLine( "     an instance what are the values of the risk of the  weights  of  the" );
      System.Console.WriteLine( "     Market Portfolio  when  the  monthly  rate  at  which  cash  can  be" );
      System.Console.WriteLine( "     borrowed or lent is 0.5 percent per month and we wish  to  construct" );
      System.Console.WriteLine( "     six portfolios with the following values  of  the  expected  return:" );
      System.Console.WriteLine( "     1.5, 2, 3, 4, 5, 6 percent per month which have the lowest  possible" );
      System.Console.WriteLine( "     risk?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    minimumExpectedReturn = min5" );
      System.Console.WriteLine( "    maximumExpectedReturn = max5" );
      System.Console.WriteLine( "    covarianceMatrix = covariance4" );
      System.Console.WriteLine( "    marketRate = 0.001" );
      // We provide the market  portfolios  weights  which  requires  that  the
      // Efficient Frontier is evaluted and the constraints set
      double ExpectedReturnMarketPortfolio2 = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( 
         marketPortWeights, // weights
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      double[][] covariance4 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      // Next we evaluate the expected returns using  the  historical  approach
      // from Asset Parameters.
      double[] expectedreturns5 = instanceOfAssetParameters.ExpectedReturns( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      instanceOfCapitalMarket.SetConstraints( 
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5} // upperBounds
      );
      // Here we evaluate the maximum of the expected returns  over  which  the
      // Efficient Frontier exits.
      double max5 = instanceOfCapitalMarket.MaxFrontierReturn( 
         expectedreturns5 // expectedReturns
      );
      // Here we evaluate the minimum of the expected returns  over  which  the
      // Efficient Frontier exits.
      double min5 = instanceOfCapitalMarket.MinFrontierReturn( 
         expectedreturns5 // expectedReturns
      );
      // Here we evaluate the risk of the market portfolio which requires  that
      // the weights of the market portfolio are known.
      double RiskMarketPortfolio = instanceOfCapitalMarket.MarketPortfolioRisk( 
         marketPortWeights, // weights
         covariance4 // covariance
      );
      // Here we evaluate the market  portfolio  which  depends  on  the  prior
      // evaluation of the (constrained) Efficient Frontier
      double[] marketPortWeights4 = instanceOfCapitalMarket.MarketPortfolio( 
         min5, // minimumExpectedReturn
         max5, // maximumExpectedReturn
         covariance4, // covarianceMatrix
         0.001 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the portfolio on the CML with an expected  return  of  1.5" );
      System.Console.WriteLine( "    percent per month (rounded to four decimal places) is:" );
      System.Console.WriteLine( " " );
      displayArray( marketPortWeights4 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    riskCML( double expectedReturn," );
      System.Console.WriteLine( "             double equityExpected," );
      System.Console.WriteLine( "             double marketRate," );
      System.Console.WriteLine( "             double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.015" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" );
      double riskCML1 = instanceOfCapitalMarket.RiskCML( 
         0.015, // expectedReturn
         ExpectedReturnMarketPortfolio2, // equityExpected
         0.005, // marketRate
         RiskMarketPortfolio // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the portfolio on the CML with  an  expected  return  of  2" );
      System.Console.WriteLine( "    percent per month (rounded to four decimal places) is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskCML1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    riskCML( double expectedReturn," );
      System.Console.WriteLine( "             double equityExpected," );
      System.Console.WriteLine( "             double marketRate," );
      System.Console.WriteLine( "             double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.02" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" );
      double riskCML2 = instanceOfCapitalMarket.RiskCML( 
         0.02, // expectedReturn
         ExpectedReturnMarketPortfolio2, // equityExpected
         0.005, // marketRate
         RiskMarketPortfolio // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the portfolio on the CML with  an  expected  return  of  3" );
      System.Console.WriteLine( "    percent per month (rounded to four decimal places)is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskCML2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    riskCML( double expectedReturn," );
      System.Console.WriteLine( "             double equityExpected," );
      System.Console.WriteLine( "             double marketRate," );
      System.Console.WriteLine( "             double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.03" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" );
      double riskCML3 = instanceOfCapitalMarket.RiskCML( 
         0.03, // expectedReturn
         ExpectedReturnMarketPortfolio2, // equityExpected
         0.005, // marketRate
         RiskMarketPortfolio // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the portfolio on the CML with  an  expected  return  of  4" );
      System.Console.WriteLine( "    percent per month (rounded to four decimal places)is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskCML3 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    riskCML( double expectedReturn," );
      System.Console.WriteLine( "             double equityExpected," );
      System.Console.WriteLine( "             double marketRate," );
      System.Console.WriteLine( "             double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.04" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" );
      double riskCML4 = instanceOfCapitalMarket.RiskCML( 
         0.04, // expectedReturn
         ExpectedReturnMarketPortfolio2, // equityExpected
         0.005, // marketRate
         RiskMarketPortfolio // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the portfolio on the CML with  an  expected  return  of  5" );
      System.Console.WriteLine( "    percent per month (rounded to four decimal places)is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskCML4 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    riskCML( double expectedReturn," );
      System.Console.WriteLine( "             double equityExpected," );
      System.Console.WriteLine( "             double marketRate," );
      System.Console.WriteLine( "             double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.05" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" );
      double riskCML5 = instanceOfCapitalMarket.RiskCML( 
         0.05, // expectedReturn
         ExpectedReturnMarketPortfolio2, // equityExpected
         0.005, // marketRate
         RiskMarketPortfolio // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The risk of the portfolio on the CML with  an  expected  return  of  6" );
      System.Console.WriteLine( "    percent per month (rounded to four decimal places)is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskCML5 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" );
      System.Console.WriteLine( "+ return                                                                 +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    riskCML( double expectedReturn," );
      System.Console.WriteLine( "             double equityExpected," );
      System.Console.WriteLine( "             double marketRate," );
      System.Console.WriteLine( "             double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.06" );
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" );
      double riskCML6 = instanceOfCapitalMarket.RiskCML( 
         0.06, // expectedReturn
         ExpectedReturnMarketPortfolio2, // equityExpected
         0.005, // marketRate
         RiskMarketPortfolio // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskCML6 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weight2Risk( double weighting," );
      System.Console.WriteLine( "                 double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We  consider  the  situation  where  the  market  portfolio  can  be" );
      System.Console.WriteLine( "     constructed from  the  following  four  assets  with  the  following" );
      System.Console.WriteLine( "     historical returns over the last 4 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" );
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" );
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" );
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Moreover, we require that the weights of these  assets  lies  within" );
      System.Console.WriteLine( "     the interval [0.1, 0.5] (i.e. between 10 and 50 percent).  In  such," );
      System.Console.WriteLine( "     an instance what is the risk of  the  portfolio  on  the  CML  (i.e." );
      System.Console.WriteLine( "     optimal) such that weighting of the market  portfolio  within  these" );
      System.Console.WriteLine( "     portfolios is: 0.1, 0.5, 0.75, 1, 1.5, 2?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weighting = 0.1" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" );
      double[][] covariance5 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}} // historicalReturns
      );
      // Here we evaluate the risk of the market portfolio which requires  that
      // the weights of the market portfolio are known.
      double RiskMarketPortfolio2 = instanceOfCapitalMarket.MarketPortfolioRisk( 
         marketPortWeights, // weights
         covariance5 // covariance
      );
      double weight2Risk1 = instanceOfCapitalMarket.Weight2Risk( 
         0.1, // weighting
         RiskMarketPortfolio2 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" );
      System.Console.WriteLine( "    Market Portfolio is 0.1 is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weight2Risk1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weight2Risk( double weighting," );
      System.Console.WriteLine( "                 double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weighting = 0.5" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" );
      double weight2Risk2 = instanceOfCapitalMarket.Weight2Risk( 
         0.5, // weighting
         RiskMarketPortfolio2 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" );
      System.Console.WriteLine( "    Market Portfolio is 0.5 is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weight2Risk2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weight2Risk( double weighting," );
      System.Console.WriteLine( "                 double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weighting = 0.75" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" );
      double weight2Risk3 = instanceOfCapitalMarket.Weight2Risk( 
         0.75, // weighting
         RiskMarketPortfolio2 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" );
      System.Console.WriteLine( "    Market Portfolio is 0.75 is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weight2Risk3 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weight2Risk( double weighting," );
      System.Console.WriteLine( "                 double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weighting = 1" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" );
      double weight2Risk4 = instanceOfCapitalMarket.Weight2Risk( 
         1.0, // weighting
         RiskMarketPortfolio2 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" );
      System.Console.WriteLine( "    Market Portfolio is 1 is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weight2Risk4 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weight2Risk( double weighting," );
      System.Console.WriteLine( "                 double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weighting = 1.5" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" );
      double weight2Risk5 = instanceOfCapitalMarket.Weight2Risk( 
         1.5, // weighting
         RiskMarketPortfolio2 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" );
      System.Console.WriteLine( "    Market Portfolio is 1.5 is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weight2Risk5 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weight2Risk( double weighting," );
      System.Console.WriteLine( "                 double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weighting = 2" );
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" );
      double weight2Risk6 = instanceOfCapitalMarket.Weight2Risk( 
         2.0, // weighting
         RiskMarketPortfolio2 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" );
      System.Console.WriteLine( "    Market Portfolio is 2 is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( weight2Risk6 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
