using System;
using WebCab.Lib.Util.Example;

public class CashConstraintsClient {
   public static void Main( String[] args ) {
      // used components
      CapitalMarket instanceOfCapitalMarket = new CapitalMarket();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "                    ##########################################" );
      System.Console.WriteLine( "                    # CashConstraints Console Client Example #" );
      System.Console.WriteLine( "                    ##########################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Within this example we illustrate  how  constraints  may  be  placed" );
      System.Console.WriteLine( "    onto the levels of the cash held or borrowed to  the  market  by  an" );
      System.Console.WriteLine( "    optimal portfolio  constructed  in  accordance  within  the  Capital" );
      System.Console.WriteLine( "    Asset Pricing Model. We show how the resulting continuous  range  of" );
      System.Console.WriteLine( "    the values of the  expected  return  and  total  risk  can  then  be" );
      System.Console.WriteLine( "    deduced which identify which  optimal  portfolios  can  be  selected" );
      System.Console.WriteLine( "    when the cash level constraints are observed." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Placing constraints on the levels of borrowing and lending +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weight2Risk( double weighting," );
      System.Console.WriteLine( "                 double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We know that the market portfolio of a collection of assets  has  an" );
      System.Console.WriteLine( "     expected return of 10 percent per year and a risk of 20 percent  per" );
      System.Console.WriteLine( "     year, we also know that cash can be borrow or lend from  the  market" );
      System.Console.WriteLine( "     at a prevailing rate of 5 percent a year. According to the  CAPM  if" );
      System.Console.WriteLine( "     the  portfolio  manager  can  only  leverage  his  portfolio  by  20" );
      System.Console.WriteLine( "     percent, and must also never hold more than 30 percent of the  funds" );
      System.Console.WriteLine( "     capital in cash then what is the range of total  risk  and  expected" );
      System.Console.WriteLine( "     return of the (optimal) portfolios which the fund manager can hold?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weighting = 0.7" );
      System.Console.WriteLine( "    equityRisk = 0.2" );
      // Here is evaluated the lower bound on the continuos range of the  total
      // risk   of  the  optimal  portfolios  when  the  cash  constraints  are
      // satisfied.
      double riskLower = instanceOfCapitalMarket.Weight2Risk( 
         0.7, // weighting
         0.2 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Since the optimal portfolio will consist of the Market Portfolio  with" );
      System.Console.WriteLine( "    either excess money lent to the market  or  money  borrowed  from  the" );
      System.Console.WriteLine( "    market in order to purchase more baskets of the Market Portfolio.  The" );
      System.Console.WriteLine( "    optimal portfolios will can be constructed will range  from  borrowing" );
      System.Console.WriteLine( "    20 percent of the capital base and purchasing further baskets  of  the" );
      System.Console.WriteLine( "    market portfolio, to lending  30  percent  of  the  capital  base  and" );
      System.Console.WriteLine( "    holding the remaining 70  percent  of  the  capital  base  within  the" );
      System.Console.WriteLine( "    Market Portfolio. Therefore, the Weighting  of  the  Market  Portfolio" );
      System.Console.WriteLine( "    can range from 1.2 (maximum gearing) to  0.7  (maximum  lending).{br/}" );
      System.Console.WriteLine( "    {br/} Now that we know the range  of  the  weighting  of  the  optimal" );
      System.Console.WriteLine( "    portfolios we are able to evaluate the range of the total risk of  the" );
      System.Console.WriteLine( "    optimal       portfolios  by  using  the  weight2risk  method  of  the" );
      System.Console.WriteLine( "    CapitalMarket class. In particular, the lower bound of the  continuous" );
      System.Console.WriteLine( "    range of the total risk is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskLower ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Placing constraints on the levels of borrowing and lending +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weight2Risk( double weighting," );
      System.Console.WriteLine( "                 double equityRisk )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    weighting = 1.2" );
      System.Console.WriteLine( "    equityRisk = 0.2" );
      // Here is evaluated the upper bound on the continuos range of the  total
      // risk   of  the  optimal  portfolios  when  the  cash  constraints  are
      // satisfied.
      double riskUpper = instanceOfCapitalMarket.Weight2Risk( 
         1.2, // weighting
         0.2 // equityRisk
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Similarly the upper bound of the continuous range of  the  total  risk" );
      System.Console.WriteLine( "    is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( riskUpper ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Placing constraints on the levels of borrowing and lending +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    returnCML( double risk," );
      System.Console.WriteLine( "               double equityRisk," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = riskLower" );
      System.Console.WriteLine( "    equityRisk = 0.2" );
      System.Console.WriteLine( "    equityExpected = 0.1" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      // Here is evaluated the lower  bound  on  the  continuos  range  of  the
      // expected return of the optimal portfolios when  the  cash  constraints
      // are satisfied.
      double returnCML1 = instanceOfCapitalMarket.ReturnCML( 
         riskLower, // risk
         0.2, // equityRisk
         0.1, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Once the continuous range of the total risk of the optimal  portfolios" );
      System.Console.WriteLine( "    which satisfy the cash constraints is known we are  able  to  evaluate" );
      System.Console.WriteLine( "    the  continuous  range  of  the  expected  returns  of  these  optimal" );
      System.Console.WriteLine( "    portfolio be using the method returnCML method from the  CapitalMarket" );
      System.Console.WriteLine( "    class. In particular, the lower bound of the expected returns  of  the" );
      System.Console.WriteLine( "    optimal portfolio in accordance within the CAPM (i.e. it lies  on  the" );
      System.Console.WriteLine( "    CML) which corresponds to a weighting of 0.7 is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( returnCML1 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Placing constraints on the levels of borrowing and lending +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    returnCML( double risk," );
      System.Console.WriteLine( "               double equityRisk," );
      System.Console.WriteLine( "               double equityExpected," );
      System.Console.WriteLine( "               double marketRate )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = riskUpper" );
      System.Console.WriteLine( "    equityRisk = 0.2" );
      System.Console.WriteLine( "    equityExpected = 0.1" );
      System.Console.WriteLine( "    marketRate = 0.005" );
      // Here is evaluated the upper  bound  on  the  continuos  range  of  the
      // expected return of the optimal portfolios when  the  cash  constraints
      // are satisfied.
      double returnCML2 = instanceOfCapitalMarket.ReturnCML( 
         riskUpper, // risk
         0.2, // equityRisk
         0.1, // equityExpected
         0.005 // marketRate
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Similarly, the upper bound of  the  expected  return  of  the  optimal" );
      System.Console.WriteLine( "    portfolio in accordance with the CAPM (i.e. it lies on the CML)  which" );
      System.Console.WriteLine( "    corresponds to a weighting of 1.2 is:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       " + double2String( returnCML2 ) );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
