using System;
using WebCab.Lib.Util.Example;

public class Efficient5AssetClient {
   public static void Main( String[] args ) {
      // used components
      AssetParameters instanceOfAssetParameters = new AssetParameters();
      CapitalMarket instanceOfCapitalMarket = new CapitalMarket();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "                    ##########################################" );
      System.Console.WriteLine( "                    # Efficient5Asset Console Client Example #" );
      System.Console.WriteLine( "                    ##########################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Within this  client  we  provide  an  example  which  evaluates  the" );
      System.Console.WriteLine( "    Efficient    Frontier  for  a  class  of  portfolios  which  can  be" );
      System.Console.WriteLine( "    constructed from 5 assets. Notes on the Units used  for  the  Source" );
      System.Console.WriteLine( "    data (i.e. the Historical Values) The units in which the  historical" );
      System.Console.WriteLine( "    values   are  given  will  effect  the  units  used  throughout  the" );
      System.Console.WriteLine( "    portfolio analysis. For example, if the historical values are  given" );
      System.Console.WriteLine( "    in absolute terms then the following will  either  be  returned  and" );
      System.Console.WriteLine( "    should be provided in absolute terms:" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "       - Expected Return" );
      System.Console.WriteLine( "       - Utility Function" );
      System.Console.WriteLine( "       " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    Note that the covariance matrix and the asset weights are  invariant" );
      System.Console.WriteLine( "    to the units in which the historical returns are expressed." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluate Points  on  Efficient  Frontier  over  Interval  of  Expected +" );
      System.Console.WriteLine( "+ Returns                                                                +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    calculateEfficientFrontier( double[][] covarianceMatrix," );
      System.Console.WriteLine( "                                double[] expectedReturns," );
      System.Console.WriteLine( "                                int numberInterpolationPoints," );
      System.Console.WriteLine( "                                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We are able to  construct  portfolios  from  four  assets  with  the" );
      System.Console.WriteLine( "     following historical returns over the last 4 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 100, 105, 106.05, 114.534,  122.55138,  133.5810042," );
      System.Console.WriteLine( "        142.9316745, 154.3662085, 163.628181, 166.9007446, 185.2598265" );
      System.Console.WriteLine( "        - 2nd  Asset:  5,  5.75,  6.7275,  6.9966,  6.436872,  7.4024028," );
      System.Console.WriteLine( "        9.030931416, 9.301859358, 7.999599048, 8.159591029, 9.383529684" );
      System.Console.WriteLine( "        -      3rd  Asset:  2000,  2240,  2531.2,  2759.008,  2952.13856," );
      System.Console.WriteLine( "        3217.83103," );
      System.Console.WriteLine( "        3571.792444,3893.253764,4126.84899,4456.996909,4902.6966" );
      System.Console.WriteLine( "        -   4th  Asset:  0.1,  0.118,  0.13688,  0.1409864,  0.146625856," );
      System.Console.WriteLine( "        0.158355924, 0.174191517, 0.168965771, 0.194310637,  0.233172765," );
      System.Console.WriteLine( "        0.270480407" );
      System.Console.WriteLine( "        -   5th  Asset:  40,  42,  45.36,  49.896,  54.38664,  57.105972," );
      System.Console.WriteLine( "        59.39021088, 61.76581932, 65.47176847, 70.70950995, 77.78046095" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Evaluate 5 points on the (un-constrained)  Efficient  Frontier  with" );
      System.Console.WriteLine( "     the precision set to 0.00001" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedreturns1" );
      System.Console.WriteLine( "    numberInterpolationPoints = 20" );
      System.Console.WriteLine( "    precision = 0.00001" );
      // We first use the covariance method from Asset Parameters.
      double[][] covariance1 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {0.0844, 0.0115, -0.0156, -0.1028, 0.0014},
                         new double[] {0.0115, 1.2329, 0.1996, 0.1633, -0.0539},
                         new double[] {-0.0156, 0.1996, 0.0424, 0.0382, -0.0066},
                         new double[] {-0.1028, 0.1633, 0.0382, 0.5141, 0.0237},
                         new double[] {0.0014, -0.0539, -0.0066, 0.0237, 0.0509}} // historicalReturns
      );
      // Next we evaluate the expected returns using  the  historical  approach
      // from Asset Parameters.
      double[] expectedreturns1 = instanceOfAssetParameters.ExpectedReturns( 
         new double[][] {new double[] {100.0, 105.0, 106.05, 114.534, 122.5513, 133.581, 142.9316, 154.3662, 163.6281, 166.9007, 185.2598},
                         new double[] {5.0, 5.75, 6.7275, 6.9966, 6.4368, 7.4024, 9.0309, 9.3018, 7.9995, 8.1595, 9.3835},
                         new double[] {2000.0, 2240.0, 2531.2, 2759.0079, 2952.1385, 3217.831, 3571.7924, 3893.2537, 4126.8489, 4456.9969, 4902.6966},
                         new double[] {0.1, 0.118, 0.1368, 0.1409, 0.1466, 0.1583, 0.1741, 0.1689, 0.1943, 0.2331, 0.2704},
                         new double[] {40.0, 42.0, 45.36, 49.896, 54.3866, 57.1059, 59.3902, 61.7658, 65.4717, 70.7095, 77.7804}} // historicalReturns
      );
      // This method sets the coordinates of the Efficient Frontier as  private
      // fields in the class. Note the we evaluate the Efficient Frontier  over
      // the entire region which it exists.
      instanceOfCapitalMarket.CalculateEfficientFrontier( 
         covariance1, // covarianceMatrix
         expectedreturns1, // expectedReturns
         20, // numberInterpolationPoints
         0.00001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Our first aim in virtually all applications and  Portfolio  Theory  is" );
      System.Console.WriteLine( "    to evaluate a collection of points on the  Efficient  Frontier,  which" );
      System.Console.WriteLine( "    we do to applying the  calculate  Efficient  Frontier  method.  Before" );
      System.Console.WriteLine( "    this method  is  applied  we  need  to  evaluate  the  covariance  and" );
      System.Console.WriteLine( "    expected returns using procedures from  the  Asset  Parameters  class." );
      System.Console.WriteLine( "    The points of the Efficient Frontier are stored in  private  field  of" );
      System.Console.WriteLine( "    the class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we return the five values of the expected return for which  the" );
      System.Console.WriteLine( "     efficient was evaluated  within  the  example  ;Evaluate  Points  on" );
      System.Console.WriteLine( "     Efficient Frontier;, and stored within a private field." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Reads the values of the  expected  return  at  the  points  where  the
      // Efficient Frontier was evaluated.
      double[] efficientFrontierExpectedReturns1 = instanceOfCapitalMarket.GetEfficientFrontierExpectedReturns( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the expected return at the  five  points  at  which  the" );
      System.Console.WriteLine( "    efficient was evaluated is:" );
      System.Console.WriteLine( " " );
      displayArray( efficientFrontierExpectedReturns1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierAssetWeights(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Reads the values  of  the  asset  weights  at  the  points  where  the
      // Efficient Frontier was evaluated
      double[][] assetWeightsEfficientFrontier1 = instanceOfCapitalMarket.GetEfficientFrontierAssetWeights( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( " " );
      display2DArray( assetWeightsEfficientFrontier1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Returns the values of the Risks +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Returns the values of the risks at the 20 points  of  the  Efficient" );
      System.Console.WriteLine( "     Frontier evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = {{0.0844, 0.0115, -0.0156, -0.1028, 0.0014}," );
      System.Console.WriteLine( "                        {0.0115, 1.2329, 0.1996, 0.1633, -0.0539}," );
      System.Console.WriteLine( "                        {-0.0156, 0.1996, 0.0424, 0.0382, -0.0066}," );
      System.Console.WriteLine( "                        {-0.1028, 0.1633, 0.0382, 0.5141, 0.0237}," );
      System.Console.WriteLine( "                        {0.0014, -0.0539, -0.0066, 0.0237, 0.0509}}" );
      double[] risks1 = instanceOfCapitalMarket.GetEfficientFrontierPortfolioRisks( 
         new double[][] {new double[] {0.0844, 0.0115, -0.0156, -0.1028, 0.0014},
                         new double[] {0.0115, 1.2329, 0.1996, 0.1633, -0.0539},
                         new double[] {-0.0156, 0.1996, 0.0424, 0.0382, -0.0066},
                         new double[] {-0.1028, 0.1633, 0.0382, 0.5141, 0.0237},
                         new double[] {0.0014, -0.0539, -0.0066, 0.0237, 0.0509}} // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The 20 values of the risks of the points  on  the  Efficient  Frontier" );
      System.Console.WriteLine( "    are:" );
      System.Console.WriteLine( " " );
      displayArray( risks1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
