using System;
using WebCab.Lib.Util.Example;

public class PortfolioWithAdditionalConstraintsClient {
   public static void Main( String[] args ) {
      // used components
      AssetParameters instanceOfAssetParameters = new AssetParameters();
      CapitalMarket instanceOfCapitalMarket = new CapitalMarket();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "          #############################################################" );
      System.Console.WriteLine( "          # PortfolioWithAdditionalConstraints Console Client Example #" );
      System.Console.WriteLine( "          #############################################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Within this client we provide an example  where  we  illustrate  the" );
      System.Console.WriteLine( "    application of the CAPM to modeling the risk/return  characteristics" );
      System.Console.WriteLine( "    of portfolios which are constructed from three risky  equity  assets" );
      System.Console.WriteLine( "    and/or geared/de-geared at the prevailing risk free  rate  on  cash." );
      System.Console.WriteLine( "    Notes on the Units used for the Source  data  (i.e.  the  Historical" );
      System.Console.WriteLine( "    Values) The units in which the  historical  values  are  given  will" );
      System.Console.WriteLine( "    effect  the  units  used  throughout  the  portfolio  analysis.  For" );
      System.Console.WriteLine( "    example, if the historical values are given in absolute  terms  then" );
      System.Console.WriteLine( "    the following will either be returned  and  should  be  provided  in" );
      System.Console.WriteLine( "    absolute terms:" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "       - Expected Return" );
      System.Console.WriteLine( "       - Utility Function" );
      System.Console.WriteLine( "       " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    Note that the covariance matrix and the asset weights are  invariant" );
      System.Console.WriteLine( "    to the units in which the historical returns are expressed." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Here we set additional Market Portfolio portfolio constraints +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    setAssetWeightsInequalityConstraints( double[][] inequalityConstraints," );
      System.Console.WriteLine( "                                          double[] constant )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    inequalityConstraints = {{1.0, 1.0, 1.0, 0.0, 0.0, 0.0}," );
      System.Console.WriteLine( "                             {0.0, 0.0, 0.0, 1.0, 1.0, 1.0}}" );
      System.Console.WriteLine( "    constant = {0.3, 0.7}" );
      // Here we set additional asset constraints to the Market Portfolio.
      instanceOfCapitalMarket.SetAssetWeightsInequalityConstraints( 
         new double[][] {new double[] {1.0, 1.0, 1.0, 0.0, 0.0, 0.0},
                         new double[] {0.0, 0.0, 0.0, 1.0, 1.0, 1.0}}, // inequalityConstraints
         new double[] {0.3, 0.7} // constant
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The additional market Market Portfolio constraint had been set." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Evaluate Points  on  Efficient  Frontier  over  Interval  of  Expected +" );
      System.Console.WriteLine( "+ Returns                                                                +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    calculateEfficientFrontier( double[][] covarianceMatrix," );
      System.Console.WriteLine( "                                double[] expectedReturns," );
      System.Console.WriteLine( "                                int numberInterpolationPoints," );
      System.Console.WriteLine( "                                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We are able to construct portfolios from  six  assets.  Evaluate  40" );
      System.Console.WriteLine( "     points     on  the  (un-constrained)  Efficient  Frontier  with  the" );
      System.Console.WriteLine( "     precision set to 0.01" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" );
      System.Console.WriteLine( "    numberInterpolationPoints = 20" );
      System.Console.WriteLine( "    precision = 0.0001" );
      // We first use the covariance method from Asset Parameters.
      double[][] covariance1 = instanceOfAssetParameters.CovarianceMatrix( 
         new double[][] {new double[] {14.1736, 10.7454, 11.4252, 12.6255, 15.4424, 17.9658},
                         new double[] {9.6787, 6.1941, 6.7819, 7.867, 10.6746, 13.0039},
                         new double[] {114.4079, 107.074, 106.4295, 108.5052, 111.9368, 114.3453},
                         new double[] {2.3077, 1.5558, 0.6088, 1.5648, 2.5118, 2.887},
                         new double[] {2.4747, 1.6975, 0.6531, 1.5564, 2.3458, 2.868},
                         new double[] {1.0615, 0.2212, -1.0499, -0.6207, 0.2339, 0.6576}} // historicalReturns
      );
      // Next we evaluate the expected returns using  the  historical  approach
      // from Asset Parameters.
      double[] expectedReturns1 = instanceOfAssetParameters.ExpectedReturns( 
         new double[][] {new double[] {14.1736, 10.7454, 11.4252, 12.6255, 15.4424, 17.9658},
                         new double[] {9.6787, 6.1941, 6.7819, 7.867, 10.6746, 13.0039},
                         new double[] {114.4079, 107.074, 106.4295, 108.5052, 111.9368, 114.3453},
                         new double[] {2.3077, 1.5558, 0.6088, 1.5648, 2.5118, 2.887},
                         new double[] {2.4747, 1.6975, 0.6531, 1.5564, 2.3458, 2.868},
                         new double[] {1.0615, 0.2212, -1.0499, -0.6207, 0.2339, 0.6576}} // historicalReturns
      );
      // This method sets the coordinates of the Efficient Frontier as  private
      // fields in the class. Note the we evaluate the Efficient Frontier  over
      // the entire region which it exists.
      instanceOfCapitalMarket.CalculateEfficientFrontier( 
         covariance1, // covarianceMatrix
         expectedReturns1, // expectedReturns
         20, // numberInterpolationPoints
         0.0001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Our first aim in virtually all applications and  Portfolio  Theory  is" );
      System.Console.WriteLine( "    to evaluate a collection of points on the  Efficient  Frontier,  which" );
      System.Console.WriteLine( "    we do to applying the  calculate  Efficient  Frontier  method.  Before" );
      System.Console.WriteLine( "    this method  is  applied  we  need  to  evaluate  the  covariance  and" );
      System.Console.WriteLine( "    expected returns using procedures from  the  Asset  Parameters  class." );
      System.Console.WriteLine( "    The points of the Efficient Frontier are stored in  private  field  of" );
      System.Console.WriteLine( "    the class." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we return the five values of the expected return for which  the" );
      System.Console.WriteLine( "     efficient was evaluated  within  the  example  ;Evaluate  Points  on" );
      System.Console.WriteLine( "     Efficient Frontier;, and stored within a private field." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Reads the values of the  expected  return  at  the  points  where  the
      // Efficient Frontier was evaluated.
      double[] efficientFrontierExpectedReturns1 = instanceOfCapitalMarket.GetEfficientFrontierExpectedReturns( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The values of the expected return at the  five  points  at  which  the" );
      System.Console.WriteLine( "    efficient was evaluated is:" );
      System.Console.WriteLine( " " );
      displayArray( efficientFrontierExpectedReturns1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierAssetWeights(  )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      // Reads the values  of  the  asset  weights  at  the  points  where  the
      // Efficient Frontier was evaluated
      double[][] assetWeightsEfficientFrontier1 = instanceOfCapitalMarket.GetEfficientFrontierAssetWeights( 
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( " " );
      display2DArray( assetWeightsEfficientFrontier1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Returns the values of the Risks +" );
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Returns the values of the risks at the 100 points of  the  Efficient" );
      System.Console.WriteLine( "     Frontier evaluated." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    covarianceMatrix = covariance1" );
      double[] risks1 = instanceOfCapitalMarket.GetEfficientFrontierPortfolioRisks( 
         covariance1 // covarianceMatrix
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The 40 values of the risks of the points  on  the  Efficient  Frontier" );
      System.Console.WriteLine( "    are:" );
      System.Console.WriteLine( " " );
      displayArray( risks1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
