using System;
using WebCab.Lib.Util.Example;

public class EasyOptimalClient {
   public static void Main( String[] args ) {
      // used components
      EasyOptimal instanceOfEasyOptimal = new EasyOptimal();
      //
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "                      ######################################" );
      System.Console.WriteLine( "                      # EasyOptimal Console Client Example #" );
      System.Console.WriteLine( "                      ######################################" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " Overview" );
      System.Console.WriteLine( " --------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    Here we demonstrate how the optimal portfolios  can  be  constructed" );
      System.Console.WriteLine( "    when   the  historical  returns  are  known  and  the  portfolio  is" );
      System.Console.WriteLine( "    determined by given the expected return and  risk,  in  cases  where" );
      System.Console.WriteLine( "    the portfolio may or may not hold cash. We also select  the  optimal" );
      System.Console.WriteLine( "    portfolio   in  accordance  with  the  investors  utility  function." );
      System.Console.WriteLine( "    Historical Values Used We are  able  to  construct  portfolios  from" );
      System.Console.WriteLine( "    four assets with the following historical returns over  the  last  4" );
      System.Console.WriteLine( "    months:" );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "       - 1st Asset: 102, 101, 99, 101" );
      System.Console.WriteLine( "       - 2nd Asset: 100, 103, 99, 101" );
      System.Console.WriteLine( "       - 3rd Asset: 100, 99, 103, 101" );
      System.Console.WriteLine( "       - 4th Asset: 101, 101, 102, 101" );
      System.Console.WriteLine( "       " );
      System.Console.WriteLine( "    " );
      System.Console.WriteLine( "    In addition to this in some cases the portfolio can lend  or  borrow" );
      System.Console.WriteLine( "    cash to the market at a prevailing rate of 0.5 percent per period." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Construct the Optimal  Portfolio  from  Equity  Assets  with  a  given +" );
      System.Console.WriteLine( "+ expected return                                                        +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    markowitzReturn( double expectedReturn," );
      System.Console.WriteLine( "                     double[] lowerBounds," );
      System.Console.WriteLine( "                     double[] upperBounds," );
      System.Console.WriteLine( "                     double[][] historicalreturns," );
      System.Console.WriteLine( "                     double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we construct the optimal portfolio  which  can  be  constructed" );
      System.Console.WriteLine( "     from Equity Assets only where the weight of each asset  lies  within" );
      System.Console.WriteLine( "     the range [0.1, 0.5], which has an expected return of 101." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 101" );
      System.Console.WriteLine( "    lowerBounds = {0.1, 0.1, 0.1, 0.1}" );
      System.Console.WriteLine( "    upperBounds = {0.5, 0.5, 0.5, 0.5}" );
      System.Console.WriteLine( "    historicalreturns = {{102.0, 101.0, 99.0, 101.0}," );
      System.Console.WriteLine( "                         {100.0, 103.0, 99.0, 101.0}," );
      System.Console.WriteLine( "                         {100.0, 99.0, 103.0, 101.0}," );
      System.Console.WriteLine( "                         {101.0, 101.0, 102.0, 101.0}}" );
      System.Console.WriteLine( "    precision = 0.000000000001" );
      // Here we construct the optimal portfolio  for  a  given  value  of  the
      // expected return
      double[] markowitzReturn1 = instanceOfEasyOptimal.MarkowitzReturn( 
         101.0, // expectedReturn
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5}, // upperBounds
         new double[][] {new double[] {102.0, 101.0, 99.0, 101.0},
                         new double[] {100.0, 103.0, 99.0, 101.0},
                         new double[] {100.0, 99.0, 103.0, 101.0},
                         new double[] {101.0, 101.0, 102.0, 101.0}}, // historicalreturns
         0.000000000001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The equity assets weights of the optimal portfolio are:" );
      System.Console.WriteLine( " " );
      displayArray( markowitzReturn1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Construct the Optimal Portfolio from the Equaity Assets with  a  given +" );
      System.Console.WriteLine( "+ maximum risk                                                           +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    markowitzRisk( double risk," );
      System.Console.WriteLine( "                   double[] lowerBounds," );
      System.Console.WriteLine( "                   double[] upperBounds," );
      System.Console.WriteLine( "                   double[][] historicalReturns," );
      System.Console.WriteLine( "                   int interpolationPoints," );
      System.Console.WriteLine( "                   double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here we construct the optimal portfolio  which  can  be  constructed" );
      System.Console.WriteLine( "     from Equity Assets only where the weights of each asset lies  within" );
      System.Console.WriteLine( "     the range [0.1, 0.5], which has a maximum risk of 1." );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    risk = 1" );
      System.Console.WriteLine( "    lowerBounds = {0.1, 0.1, 0.1, 0.1}" );
      System.Console.WriteLine( "    upperBounds = {0.5, 0.5, 0.5, 0.5}" );
      System.Console.WriteLine( "    historicalReturns = {{102.0, 101.0, 99.0, 101.0}," );
      System.Console.WriteLine( "                         {100.0, 103.0, 99.0, 101.0}," );
      System.Console.WriteLine( "                         {100.0, 99.0, 103.0, 101.0}," );
      System.Console.WriteLine( "                         {101.0, 101.0, 102.0, 101.0}}" );
      System.Console.WriteLine( "    interpolationPoints = 30" );
      System.Console.WriteLine( "    precision = 0.000000000001" );
      // Here we construct the optimal portfolio for a maximum given  portfolio
      // risk
      double[] markowitzRisk1 = instanceOfEasyOptimal.MarkowitzRisk( 
         1.0, // risk
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5}, // upperBounds
         new double[][] {new double[] {102.0, 101.0, 99.0, 101.0},
                         new double[] {100.0, 103.0, 99.0, 101.0},
                         new double[] {100.0, 99.0, 103.0, 101.0},
                         new double[] {101.0, 101.0, 102.0, 101.0}}, // historicalReturns
         30, // interpolationPoints
         0.000000000001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The equity assets weights of the optimal portfolio are:" );
      System.Console.WriteLine( " " );
      displayArray( markowitzRisk1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Construct the Optimal  Portfolio  from  the  available  Equity  Assets +" );
      System.Console.WriteLine( "+ selected in accordance with the investors utility function             +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    markowitzUtility( double[] utilityExpectedReturns," );
      System.Console.WriteLine( "                      double[] utilityRisks," );
      System.Console.WriteLine( "                      double[] lowerBounds," );
      System.Console.WriteLine( "                      double[] upperBounds," );
      System.Console.WriteLine( "                      double[][] historicalReturns," );
      System.Console.WriteLine( "                      int interpolationPoints," );
      System.Console.WriteLine( "                      double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     Here the optimal portfolio (with the  maximum  expected  return)  is" );
      System.Console.WriteLine( "     constructed for an investor who  wishes  to  construct  a  portfolio" );
      System.Console.WriteLine( "     from the four available assets where the  investor  is  prepared  to" );
      System.Console.WriteLine( "     accept the following levels  of  risk  for  the  following  expected" );
      System.Console.WriteLine( "     return:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - Return = 99, Risk = 0.01" );
      System.Console.WriteLine( "        - Return = 100, Risk = 0.1" );
      System.Console.WriteLine( "        - Return = 102, Risk = 0.5" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    utilityExpectedReturns = {100.0, 101.0, 102.0}" );
      System.Console.WriteLine( "    utilityRisks = {0.01, 0.1, 0.5}" );
      System.Console.WriteLine( "    lowerBounds = {0.1, 0.1, 0.1, 0.1}" );
      System.Console.WriteLine( "    upperBounds = {0.5, 0.5, 0.5, 0.5}" );
      System.Console.WriteLine( "    historicalReturns = {{102.0, 101.0, 99.0, 101.0}," );
      System.Console.WriteLine( "                         {100.0, 103.0, 99.0, 101.0}," );
      System.Console.WriteLine( "                         {100.0, 99.0, 103.0, 101.0}," );
      System.Console.WriteLine( "                         {101.0, 101.0, 102.0, 101.0}}" );
      System.Console.WriteLine( "    interpolationPoints = 30" );
      System.Console.WriteLine( "    precision = 0.000000000001" );
      // Here    we  evaluate  the  optimal  portfolio  which  is  selected  in
      // accordance with the in vestores utility function.
      double[] markowitzUtility1 = instanceOfEasyOptimal.MarkowitzUtility( 
         new double[] {100.0, 101.0, 102.0}, // utilityExpectedReturns
         new double[] {0.01, 0.1, 0.5}, // utilityRisks
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5}, // upperBounds
         new double[][] {new double[] {102.0, 101.0, 99.0, 101.0},
                         new double[] {100.0, 103.0, 99.0, 101.0},
                         new double[] {100.0, 99.0, 103.0, 101.0},
                         new double[] {101.0, 101.0, 102.0, 101.0}}, // historicalReturns
         30, // interpolationPoints
         0.000000000001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The equity asset weights of the optimal portfolio are:" );
      System.Console.WriteLine( " " );
      displayArray( markowitzUtility1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Construct the Optimal Portfolio from Cash and Equity Assets  when  the +" );
      System.Console.WriteLine( "+ expected return is 1.5 percent                                         +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    capmReturn( double expectedReturn," );
      System.Console.WriteLine( "                double[][] historicalReturns," );
      System.Console.WriteLine( "                double[] lowerBounds," );
      System.Console.WriteLine( "                double[] upperBounds," );
      System.Console.WriteLine( "                double marketRate," );
      System.Console.WriteLine( "                int interpolationPoints," );
      System.Console.WriteLine( "                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We consider the situation where the we can construct  the  portfolio" );
      System.Console.WriteLine( "     from the following four equity assets with the following  historical" );
      System.Console.WriteLine( "     returns over the last 4 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" );
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" );
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" );
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Moreover, we require that the weights of these  assets  lies  within" );
      System.Console.WriteLine( "     the interval [0.1, 0.5] (i.e. between  10  and  50  percent  of  the" );
      System.Console.WriteLine( "     total EQUITY portfolio). Moreover, the portfolio can  also  lend  or" );
      System.Console.WriteLine( "     borrow cash to the market at a prevailing rate of  0.5  percent  per" );
      System.Console.WriteLine( "     month. What are the weights of the cash  (i.e.  positive  (lend)  or" );
      System.Console.WriteLine( "     negative (borrow)) and the equity assets of  the  optimal  portfolio" );
      System.Console.WriteLine( "     (i.e. the portfolio with the lowest risk  when  expected  return  of" );
      System.Console.WriteLine( "     the portfolio is: 1.5 percent per month?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.015" );
      System.Console.WriteLine( "    historicalReturns = {{0.02, 0.01, -0.02, 0.02}," );
      System.Console.WriteLine( "                         {0.0, 0.03, -0.01, 0.01}," );
      System.Console.WriteLine( "                         {0.0, -0.01, 0.03, 0.01}," );
      System.Console.WriteLine( "                         {0.01, 0.01, 0.02, 0.01}}" );
      System.Console.WriteLine( "    lowerBounds = {0.1, 0.1, 0.1, 0.1}" );
      System.Console.WriteLine( "    upperBounds = {0.5, 0.5, 0.5, 0.5}" );
      System.Console.WriteLine( "    marketRate = 0.0005" );
      System.Console.WriteLine( "    interpolationPoints = 20" );
      System.Console.WriteLine( "    precision = 0.000000000001" );
      // Here we evaluate the Optimal portfolio which can be  constructed  from
      // equity assets and cash with an expected return of 1.5 percent.
      double[] capitalReturn1 = instanceOfEasyOptimal.CapmReturn( 
         0.015, // expectedReturn
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}}, // historicalReturns
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5}, // upperBounds
         0.0005, // marketRate
         20, // interpolationPoints
         0.000000000001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the cash component (i.e  the  first  element)  and  the" );
      System.Console.WriteLine( "    equity assets (i.e. the remaining elements) of the  optimal  portfolio" );
      System.Console.WriteLine( "    with an expected return of 1.5 percent per month is:" );
      System.Console.WriteLine( " " );
      displayArray( capitalReturn1 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Construct the Optimal Portfolio from Cash and Equity Assets  when  the +" );
      System.Console.WriteLine( "+ expected return is 3 percent                                           +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    capmReturn( double expectedReturn," );
      System.Console.WriteLine( "                double[][] historicalReturns," );
      System.Console.WriteLine( "                double[] lowerBounds," );
      System.Console.WriteLine( "                double[] upperBounds," );
      System.Console.WriteLine( "                double marketRate," );
      System.Console.WriteLine( "                int interpolationPoints," );
      System.Console.WriteLine( "                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We consider the situation where the we can construct  the  portfolio" );
      System.Console.WriteLine( "     from the following four equity assets with the following  historical" );
      System.Console.WriteLine( "     returns over the last 4 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" );
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" );
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" );
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Moreover, we require that the weights of these  assets  lies  within" );
      System.Console.WriteLine( "     the interval [0.1, 0.5] (i.e. between  10  and  50  percent  of  the" );
      System.Console.WriteLine( "     total EQUITY portfolio). Moreover, the portfolio can  also  lend  or" );
      System.Console.WriteLine( "     borrow cash to the market at a prevailing rate of  0.5  percent  per" );
      System.Console.WriteLine( "     month. What are the weights of the cash  (i.e.  positive  (lend)  or" );
      System.Console.WriteLine( "     negative (borrow)) and the equity assets of  the  optimal  portfolio" );
      System.Console.WriteLine( "     (i.e. the portfolio with the lowest risk  when  expected  return  of" );
      System.Console.WriteLine( "     the portfolio is: 3 percent per month?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.001" );
      System.Console.WriteLine( "    historicalReturns = {{0.02, 0.01, -0.02, 0.02}," );
      System.Console.WriteLine( "                         {0.0, 0.03, -0.01, 0.01}," );
      System.Console.WriteLine( "                         {0.0, -0.01, 0.03, 0.01}," );
      System.Console.WriteLine( "                         {0.01, 0.01, 0.02, 0.01}}" );
      System.Console.WriteLine( "    lowerBounds = {0.1, 0.1, 0.1, 0.1}" );
      System.Console.WriteLine( "    upperBounds = {0.5, 0.5, 0.5, 0.5}" );
      System.Console.WriteLine( "    marketRate = 0.0005" );
      System.Console.WriteLine( "    interpolationPoints = 20" );
      System.Console.WriteLine( "    precision = 0.000000000001" );
      // Here we evaluate the Optimal portfolio which can be  constructed  from
      // equity assets and cash with an expected return of 3 percent.
      double[] capitalReturn2 = instanceOfEasyOptimal.CapmReturn( 
         0.001, // expectedReturn
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}}, // historicalReturns
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5}, // upperBounds
         0.0005, // marketRate
         20, // interpolationPoints
         0.000000000001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the cash component (i.e  the  first  element)  and  the" );
      System.Console.WriteLine( "    equity assets (i.e. the remaining elements) of the  optimal  portfolio" );
      System.Console.WriteLine( "    with an expected return of 3 percent per month is:" );
      System.Console.WriteLine( " " );
      displayArray( capitalReturn2 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( "+ Construct the Optimal Portfolio from Cash and Equity Assets  when  the +" );
      System.Console.WriteLine( "+ expected return is 5 percent                                           +" );
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " Within this example we test the following method:" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    capmReturn( double expectedReturn," );
      System.Console.WriteLine( "                double[][] historicalReturns," );
      System.Console.WriteLine( "                double[] lowerBounds," );
      System.Console.WriteLine( "                double[] upperBounds," );
      System.Console.WriteLine( "                double marketRate," );
      System.Console.WriteLine( "                int interpolationPoints," );
      System.Console.WriteLine( "                double precision )" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " Question:" );
      System.Console.WriteLine( " ---------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "     We consider the situation where the we can construct  the  portfolio" );
      System.Console.WriteLine( "     from the following four equity assets with the following  historical" );
      System.Console.WriteLine( "     returns over the last 4 months:" );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" );
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" );
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" );
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" );
      System.Console.WriteLine( "        " );
      System.Console.WriteLine( "     " );
      System.Console.WriteLine( "     Moreover, we require that the weights of these  assets  lies  within" );
      System.Console.WriteLine( "     the interval [0.1, 0.5] (i.e. between  10  and  50  percent  of  the" );
      System.Console.WriteLine( "     total EQUITY portfolio). Moreover, the portfolio can  also  lend  or" );
      System.Console.WriteLine( "     borrow cash to the market at a prevailing rate of  0.5  percent  per" );
      System.Console.WriteLine( "     month. What are the weights of the cash  (i.e.  positive  (lend)  or" );
      System.Console.WriteLine( "     negative (borrow)) and the equity assets of  the  optimal  portfolio" );
      System.Console.WriteLine( "     (i.e. the portfolio with the lowest risk  when  expected  return  of" );
      System.Console.WriteLine( "     the portfolio is: 5 percent per month?" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " Parameter values required to solve this problem:" );
      System.Console.WriteLine( " ------------------------------------------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    expectedReturn = 0.005" );
      System.Console.WriteLine( "    historicalReturns = {{0.02, 0.01, -0.02, 0.02}," );
      System.Console.WriteLine( "                         {0.0, 0.03, -0.01, 0.01}," );
      System.Console.WriteLine( "                         {0.0, -0.01, 0.03, 0.01}," );
      System.Console.WriteLine( "                         {0.01, 0.01, 0.02, 0.01}}" );
      System.Console.WriteLine( "    lowerBounds = {0.1, 0.1, 0.1, 0.1}" );
      System.Console.WriteLine( "    upperBounds = {0.5, 0.5, 0.5, 0.5}" );
      System.Console.WriteLine( "    marketRate = 0.0005" );
      System.Console.WriteLine( "    interpolationPoints = 20" );
      System.Console.WriteLine( "    precision = 0.000000000001" );
      // Here we evaluate the Optimal portfolio which can be  constructed  from
      // equity assets and cash with an expected return of 5 percent.
      double[] capitalReturn3 = instanceOfEasyOptimal.CapmReturn( 
         0.005, // expectedReturn
         new double[][] {new double[] {0.02, 0.01, -0.02, 0.02},
                         new double[] {0.0, 0.03, -0.01, 0.01},
                         new double[] {0.0, -0.01, 0.03, 0.01},
                         new double[] {0.01, 0.01, 0.02, 0.01}}, // historicalReturns
         new double[] {0.1, 0.1, 0.1, 0.1}, // lowerBounds
         new double[] {0.5, 0.5, 0.5, 0.5}, // upperBounds
         0.0005, // marketRate
         20, // interpolationPoints
         0.000000000001 // precision
      );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " Answer evaluated:" );
      System.Console.WriteLine( " -----------------" );
      System.Console.WriteLine( " " );
      System.Console.WriteLine( "    The weights of the cash component (i.e  the  first  element)  and  the" );
      System.Console.WriteLine( "    equity assets (i.e. the remaining elements) of the  optimal  portfolio" );
      System.Console.WriteLine( "    with an expected return of 5 percent per month is:" );
      System.Console.WriteLine( " " );
      displayArray( capitalReturn3 );
      System.Console.WriteLine( " " );

      System.Console.WriteLine( " " );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" );
      System.Console.WriteLine( "       -------------------------------------------------------------------" );
      System.Console.WriteLine( " " );
   }

   public static String double2String( double d ) {
      if ( System.Double.IsNaN( d ) ) return d.ToString();
      if ( System.Double.IsNegativeInfinity( d ) ) return d.ToString();
      if ( System.Double.IsPositiveInfinity( d ) ) return d.ToString();
      if ( d == Double.MinValue ) return d.ToString();
      if ( d == Double.MaxValue ) return d.ToString();
      double frac = d - ( int )d;
      double frac4 = ( int )( frac * 10000 ) / 10000.0;
      return "" + ( ( int )d + frac4 );
   }

   public static void displayArray( double[] a ) {
      System.Console.Write( "     {" );
      for ( int i = 0; i < a.Length; i ++ ) {
         System.Console.Write( double2String( a[ i ] ) );
         if ( i < -1 + a.Length ) {
            System.Console.Write( ", " );
         }
      }
      System.Console.Write( "}" );
   }

   public static void display2DArray( double[][] a ) {
      for ( int i = 0; i < a.Length; i ++ ) {
         if ( i == 0 ) {
            System.Console.Write( "     {{" );
         } else {
            System.Console.Write( "     {" );
         }
         for ( int j = 0; j < a[ i ].Length; j ++ ) {
            System.Console.Write( double2String( a[ i ][ j ] ) );
            if ( j < -1 + a[ i ].Length ) {
               System.Console.Write( ", " );
            } else {
               System.Console.Write( "}" );
            }
         }
         if ( i < -1 + a.Length ) {
            System.Console.Write( ",\n " );
         } else {
            System.Console.Write( "}" );
         }
      }
   }
}
