Imports System
Imports Microsoft.VisualBasic
Imports WebCab.Lib.Util.Example
Imports WebCab.Libraries.Finance.Portfolio

Module CapitalMarketClient
   Sub Main()
      ' used components
      Dim instanceOfAssetParameters As AssetParameters = New AssetParameters
      Dim instanceOfCapitalMarket As WebCab.Libraries.Finance.Portfolio.CapitalMarket = New WebCab.Libraries.Finance.Portfolio.CapitalMarket
      '
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "                     ########################################" )
      System.Console.WriteLine( "                     # CapitalMarket Console Client Example #" )
      System.Console.WriteLine( "                     ########################################" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " --------" )
      System.Console.WriteLine( " Overview" )
      System.Console.WriteLine( " --------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Within this client we offer one  example  for  each  of  the  method" )
      System.Console.WriteLine( "    contained within Capital Market." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluate Points on Efficient Frontier +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    calculateEfficientFrontier( double minimumExpectedReturn," )
      System.Console.WriteLine( "                                double maximumExpectedReturn," )
      System.Console.WriteLine( "                                double[][] covarianceMatrix," )
      System.Console.WriteLine( "                                double[] expectedReturns," )
      System.Console.WriteLine( "                                int numberInterpolationPoints," )
      System.Console.WriteLine( "                                double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     We are able to  construct  portfolios  from  four  assets  with  the" )
      System.Console.WriteLine( "     following historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" )
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" )
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" )
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     Evaluate 5 points on the Efficient Frontier with the  precision  set" )
      System.Console.WriteLine( "     to 0.00001" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    minimumExpectedReturn = min1" )
      System.Console.WriteLine( "    maximumExpectedReturn = max1" )
      System.Console.WriteLine( "    covarianceMatrix = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedreturns1" )
      System.Console.WriteLine( "    numberInterpolationPoints = 5" )
      System.Console.WriteLine( "    precision = 0.0000000001" )
      ' We first use the covariance method from Asset Parameters
      Dim covariance1 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedreturns1 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      ' Here we evaluate the maximum of the expected returns  over  which  the
      ' Efficient Frontier exits.
      Dim max1 As Double = instanceOfCapitalMarket.MaxFrontierReturn( _
         expectedreturns1 _
      )
      ' Here we evaluate the minimum of the expected returns  over  which  the
      ' Efficient Frontier exits.
      Dim min1 As Double = instanceOfCapitalMarket.MinFrontierReturn( _
         expectedreturns1 _
      )
      ' This method sets the coordinates of the Efficient Frontier as  private
      ' fields in the class.
      instanceOfCapitalMarket.CalculateEfficientFrontier( _
         min1, _
         max1, _
         covariance1, _
         expectedreturns1, _
         5, _
         0.0000000001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Our first aim in virtually all applications of Portfolio Theory is  to" )
      System.Console.WriteLine( "    evaluate a collection of points on the Efficient  Frontier,  which  we" )
      System.Console.WriteLine( "    do to applying the calculate Efficient Frontier  method.  Before  this" )
      System.Console.WriteLine( "    method is applied we need to  evaluate  the  covariance  and  expected" )
      System.Console.WriteLine( "    returns using procedures from the Asset Parameters class.  The  points" )
      System.Console.WriteLine( "    of the Efficient Frontier are stored in private fields of the class." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Get the Efficient Frontier Expected Returns +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we read-off the values of the expected returns  at  the  points" )
      System.Console.WriteLine( "     on which the Efficient Frontier has been evaluated." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      ' Here we get the values of the Efficient Frontier which has  just  been
      ' evaluated.
      Dim getReturnsEfficient1 As Double() = instanceOfCapitalMarket.GetEfficientFrontierExpectedReturns( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The value of the expected returns are:" )
      System.Console.WriteLine( " " )
      displayArray( getReturnsEfficient1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Get the Asset Weights of the Efficient Frontier Portfolio's Evaluated +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierAssetWeights(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we get the Asset weights of the portfolio's which  lie  of  the" )
      System.Console.WriteLine( "     Efficient Frontier which have been  evaluated  and  set  to  private" )
      System.Console.WriteLine( "     fields." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      ' Here we get the Asset weights of the  portfolio's  which  lie  of  the
      ' Efficient Frontier which  have  been  evaluated  and  set  to  private
      ' fields.
      Dim getAssetWeights1 As Double()() = instanceOfCapitalMarket.GetEfficientFrontierAssetWeights( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weights of the asset with the Efficient Frontier Portfolio are:" )
      System.Console.WriteLine( " " )
      display2DArray( getAssetWeights1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Get the values of the risk of the Portfolio on the Efficient Frontier +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we read off the values of the risk of  the  Portfolios  on  the" )
      System.Console.WriteLine( "     Efficient Frontier which have been evaluated and set to the  private" )
      System.Console.WriteLine( "     fields." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    covarianceMatrix = covarianceMatrix2" )
      ' We first use the covariance method from Asset Parameters
      Dim covarianceMatrix2 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      ' Here we evaluate the values of the  risk  of  the  Portfolios  on  the
      ' Efficient Frontier.
      Dim getRisks1 As Double() = instanceOfCapitalMarket.GetEfficientFrontierPortfolioRisks( _
         covarianceMatrix2 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The values of the  risk  of  the  portfolios  at  the  points  of  the" )
      System.Console.WriteLine( "    Efficient Frontier which have been evaluated are:" )
      System.Console.WriteLine( " " )
      displayArray( getRisks1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Setting constraints on the Weights of the assets +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    setConstraints( double[] lowerBounds," )
      System.Console.WriteLine( "                    double[] upperBounds )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     How can constraints be set on the weights of the  assets  such  that" )
      System.Console.WriteLine( "     all portfolios of the  Efficient  Frontier  have  no  asset  with  a" )
      System.Console.WriteLine( "     weights more than 0.5 (i.e. 50 percent)  and  no  weight  below  0.1" )
      System.Console.WriteLine( "     (i.e. 10 percent)?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    lowerBounds = {0.1, 0.1, 0.1, 0.1}" )
      System.Console.WriteLine( "    upperBounds = {0.5, 0.5, 0.5, 0.5}" )
      instanceOfCapitalMarket.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The application of this method  will  allow  the  constraints  on  the" )
      System.Console.WriteLine( "    Efficient Frontier portfolios assets weights  to  be  set  before  the" )
      System.Console.WriteLine( "    points on the Efficient Frontier are  evaluated  using  the  calculate" )
      System.Console.WriteLine( "    Efficient Frontier method." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Read off the values of the Lower Constraints +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getLowerConstraints(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Get the value of the lower bounds on the asset  weights  after  they" )
      System.Console.WriteLine( "     have been set." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      instanceOfCapitalMarket.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      ' Get the value of the lower bounds of the assets.
      Dim readLowerBounds As Double() = instanceOfCapitalMarket.GetLowerConstraints( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The lower bounds on the assets weights are:" )
      System.Console.WriteLine( " " )
      displayArray( readLowerBounds )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Read off the values of the Upper Constraints +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getUpperConstraints(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Get the value of the upper bounds on the asset  weights  after  they" )
      System.Console.WriteLine( "     have been set." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      instanceOfCapitalMarket.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      ' Get the value of the upper bounds of the assets.
      Dim readUpperBounds As Double() = instanceOfCapitalMarket.GetUpperConstraints( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The upper bounds on the assets weights are:" )
      System.Console.WriteLine( " " )
      displayArray( readUpperBounds )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluation of the Market Portfolio +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolio( double minimumExpectedReturn," )
      System.Console.WriteLine( "                     double maximumExpectedReturn," )
      System.Console.WriteLine( "                     double[][] covarianceMatrix," )
      System.Console.WriteLine( "                     double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     What is the market portfolio (i.e. the portfolio  on  the  Efficient" )
      System.Console.WriteLine( "     Frontier with the highest expected return per  unit  of  risk)  when" )
      System.Console.WriteLine( "     the each portfolio can be  constructed  from  the  following  assets" )
      System.Console.WriteLine( "     with the following historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" )
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" )
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" )
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     where the weights of each asset within  the  market  portfolio  lies" )
      System.Console.WriteLine( "     within the range [0.1, 0.5] (i.e. between 10 and 50 percent)?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    minimumExpectedReturn = min2" )
      System.Console.WriteLine( "    maximumExpectedReturn = max2" )
      System.Console.WriteLine( "    covarianceMatrix = covarianceMarketPortfolio1" )
      System.Console.WriteLine( "    marketRate = 0.001" )
      ' We first use the covariance method from Asset Parameters
      Dim covarianceMarketPortfolio1 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedreturns2 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      instanceOfCapitalMarket.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      ' Here we evaluate the maximum of the expected returns  over  which  the
      ' Efficient Frontier exits.
      Dim max2 As Double = instanceOfCapitalMarket.MaxFrontierReturn( _
         expectedreturns2 _
      )
      ' Here we evaluate the minimum of the expected returns  over  which  the
      ' Efficient Frontier exits.
      Dim min2 As Double = instanceOfCapitalMarket.MinFrontierReturn( _
         expectedreturns2 _
      )
      ' Here we evaluate the market  portfolio  which  depends  on  the  prior
      ' evaluation of the (constrained) Efficient Frontier
      Dim marketPortWeights As Double() = instanceOfCapitalMarket.MarketPortfolio( _
         min2, _
         max2, _
         covarianceMarketPortfolio1, _
         0.001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weights of the assets from  which  the  market  portfolio  can  be" )
      System.Console.WriteLine( "    constructed with respect to the constraints is:" )
      System.Console.WriteLine( " " )
      displayArray( marketPortWeights )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the Expected Return of the Market Portfolio +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[] weights," )
      System.Console.WriteLine( "                                   double[][] historicalReturns )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     What is the expected return (based on a  historical  estimate)  when" )
      System.Console.WriteLine( "     the market  portfolio  is  constructed  from  the  assets  with  the" )
      System.Console.WriteLine( "     following historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" )
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" )
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" )
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     where the weights of each asset within  the  market  portfolio  lies" )
      System.Console.WriteLine( "     within the range [0.1, 0.5] (i.e. between 10 and 50 percent)?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weights = marketPortWeights1" )
      System.Console.WriteLine( "    historicalReturns = {{0.02, 0.01, -0.02, 0.02}," )
      System.Console.WriteLine( "                        {0.0, 0.03, -0.01, 0.01}," )
      System.Console.WriteLine( "                        {0.0, -0.01, 0.03, 0.01}," )
      System.Console.WriteLine( "                        {0.01, 0.01, 0.02, 0.01}}" )
      ' We first use the covariance method from Asset Parameters.
      Dim covarianceMarketPortfolio2 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedreturns3 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      instanceOfCapitalMarket.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      ' Here we evaluate the maximum of the expected returns  over  which  the
      ' Efficient Frontier exits.
      Dim max3 As Double = instanceOfCapitalMarket.MaxFrontierReturn( _
         expectedreturns3 _
      )
      ' Here we evaluate the minimum of the expected returns  over  which  the
      ' Efficient Frontier exits.
      Dim min3 As Double = instanceOfCapitalMarket.MinFrontierReturn( _
         expectedreturns3 _
      )
      ' Here we evaluate the market  portfolio  which  depends  on  the  prior
      ' evaluation of the (constrained) Efficient Frontier
      Dim marketPortWeights1 As Double() = instanceOfCapitalMarket.MarketPortfolio( _
         min3, _
         max3, _
         covarianceMarketPortfolio2, _
         0.001 _
      )
      ' We provide the market  portfolios  weights  which  requires  that  the
      ' Efficient Frontier is evaluted and the constraints set
      Dim marketPortfolioExpectedReturn1 As Double = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( _
         marketPortWeights1, _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The expected return of the market portfolio constructed  with  respect" )
      System.Console.WriteLine( "    to the constraints is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( marketPortfolioExpectedReturn1 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the risk of the Market Portfolio +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolioRisk( double[] weights," )
      System.Console.WriteLine( "                         double[][] covariance )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     What is the risk of the market portfolio which can constructed  from" )
      System.Console.WriteLine( "     the assets with the following historical returns  over  the  last  4" )
      System.Console.WriteLine( "     months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" )
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" )
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" )
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     where the weights of each asset within  the  market  portfolio  lies" )
      System.Console.WriteLine( "     within the range [0.1, 0.5] (i.e. between 10 and 50 percent)?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weights = marketPortWeights2" )
      System.Console.WriteLine( "    covariance = covariance4riskOfMarketPortfolio" )
      ' We first evaluate the covariance matrix between  the  asset  from  the
      ' market portfolio could be constructed.
      Dim covariance4riskOfMarketPortfolio As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedreturns4 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      instanceOfCapitalMarket.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      ' Here we evaluate the maximum of the expected returns  over  which  the
      ' Efficient Frontier exits.
      Dim max4 As Double = instanceOfCapitalMarket.MaxFrontierReturn( _
         expectedreturns4 _
      )
      ' Here we evaluate the minimum of the expected returns  over  which  the
      ' Efficient Frontier exits.
      Dim min4 As Double = instanceOfCapitalMarket.MinFrontierReturn( _
         expectedreturns4 _
      )
      ' Here we evaluate the market  portfolio  which  depends  on  the  prior
      ' evaluation of the (constrained) Efficient Frontier
      Dim marketPortWeights2 As Double() = instanceOfCapitalMarket.MarketPortfolio( _
         min4, _
         max4, _
         covariance4riskOfMarketPortfolio, _
         0.001 _
      )
      ' Here we evaluate the risk of the market portfolio which requires  that
      ' the weights of the market portfolio are known.
      Dim marketPortfolioRisk1 As Double = instanceOfCapitalMarket.MarketPortfolioRisk( _
         marketPortWeights2, _
         covariance4riskOfMarketPortfolio _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the market  portfolio  constructed  with  respect  to  the" )
      System.Console.WriteLine( "    constraints is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( marketPortfolioRisk1 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weightCML( double expectedReturn," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     We  consider  the  situation  where  the  market  portfolio  can  be" )
      System.Console.WriteLine( "     constructed from  the  following  four  assets  with  the  following" )
      System.Console.WriteLine( "     historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" )
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" )
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" )
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     Moreover, we require that the weights of these  assets  lies  within" )
      System.Console.WriteLine( "     the interval [0.1, 0.5] (i.e. between 10 and 50  percent).  In  such" )
      System.Console.WriteLine( "     an instance what are the weights of the market  portfolio  when  the" )
      System.Console.WriteLine( "     monthly rate at which cash can be borrowed or lent  is  0.5  percent" )
      System.Console.WriteLine( "     per month, for six portfolios  with  the  following  values  of  the" )
      System.Console.WriteLine( "     expected return: 0.5, 1, 1.5, 2, 3, 4 percent per  month  which  lie" )
      System.Console.WriteLine( "     on the CML (i.e. have the lowest possible risk for a given level  of" )
      System.Console.WriteLine( "     the expected return) ?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.005" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      ' We provide the market  portfolios  weights  which  requires  that  the
      ' Efficient Frontier is evaluted and the constraints set
      Dim ExpectedReturnMarketPortfolio As Double = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( _
         marketPortWeights, _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      ' Evaluates the weight when the expected return desired is equal to  the
      ' return available from cash.
      Dim weightCML1 As Double = instanceOfCapitalMarket.WeightCML( _
         0.005, _
         ExpectedReturnMarketPortfolio, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weight required when an expected return of 0.5 percent  per  month" )
      System.Console.WriteLine( "    is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weightCML1 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weightCML( double expectedReturn," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.01" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      ' Evaluates the market portfolio weight when the expected  return  is  1
      ' percent per month.
      Dim weightCML2 As Double = instanceOfCapitalMarket.WeightCML( _
         0.01, _
         ExpectedReturnMarketPortfolio, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weight required when an expected return of  1  percent  per  month" )
      System.Console.WriteLine( "    is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weightCML2 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weightCML( double expectedReturn," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.015" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      ' Evaluates the market portfolio weight when the expected return is  1.5
      ' percent per month.
      Dim weightCML3 As Double = instanceOfCapitalMarket.WeightCML( _
         0.015, _
         ExpectedReturnMarketPortfolio, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weight required when an expected return of 1.5 percent  per  month" )
      System.Console.WriteLine( "    is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weightCML3 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weightCML( double expectedReturn," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.02" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      ' Evaluates the market portfolio weight when the expected  return  is  2
      ' percent per month.
      Dim weightCML4 As Double = instanceOfCapitalMarket.WeightCML( _
         0.02, _
         ExpectedReturnMarketPortfolio, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weight required when an expected return of 2.0 percent  per  month" )
      System.Console.WriteLine( "    is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weightCML4 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weightCML( double expectedReturn," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.03" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      ' Evaluates the market portfolio weight when the expected  return  is  3
      ' percent per month.
      Dim weightCML5 As Double = instanceOfCapitalMarket.WeightCML( _
         0.03, _
         ExpectedReturnMarketPortfolio, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weight required when an expected return of 3.0 percent  per  month" )
      System.Console.WriteLine( "    is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weightCML5 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Finding the Market Portfolio Weight  required  for  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weightCML( double expectedReturn," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.04" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      ' Evaluates the market portfolio weight when the expected  return  is  4
      ' percent per month.
      Dim weightCML6 As Double = instanceOfCapitalMarket.WeightCML( _
         0.04, _
         ExpectedReturnMarketPortfolio, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weight required when an expected return of 4.0 percent  per  month" )
      System.Console.WriteLine( "    is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weightCML6 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" )
      System.Console.WriteLine( "+ Risk                                                                   +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    returnCML( double risk," )
      System.Console.WriteLine( "               double equityRisk," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     We  consider  the  situation  where  the  market  portfolio  can  be" )
      System.Console.WriteLine( "     constructed from  the  following  four  assets  with  the  following" )
      System.Console.WriteLine( "     historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" )
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" )
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" )
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     Moreover, we require that the weights of these  assets  lies  within" )
      System.Console.WriteLine( "     the interval [0.1, 0.5] (i.e. between 10 and 50  percent).  In  such" )
      System.Console.WriteLine( "     an instance what is the expected returns in decimal format  (i.e.  1" )
      System.Console.WriteLine( "     percent = 0.01) when the market portfolio  (as  constructed  in  the" )
      System.Console.WriteLine( "     previous example) when  the  monthly  rate  at  which  cash  can  be" )
      System.Console.WriteLine( "     borrowed or lent is 0.5 percent per month, for six  portfolios  with" )
      System.Console.WriteLine( "     the following values of the total risk: 2, 3, 5, 10, 20, 30  percent" )
      System.Console.WriteLine( "     per month which lie on the  CML  (i.e.  have  the  highest  possible" )
      System.Console.WriteLine( "     expected return for that level of risk) ?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    risk = 0.02" )
      System.Console.WriteLine( "    equityRisk = equityRisk1" )
      System.Console.WriteLine( "    equityExpected = equityExpected1" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      ' We first evaluate the covariance matrix between  the  asset  from  the
      ' marlet portfolio could be constructed.
      Dim covariance3 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      Dim equityRisk1 As Double = instanceOfCapitalMarket.MarketPortfolioRisk( _
         marketPortWeights, _
         covariance3 _
      )
      Dim equityExpected1 As Double = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( _
         marketPortWeights, _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      Dim returnCML1 As Double = instanceOfCapitalMarket.ReturnCML( _
         0.02, _
         equityRisk1, _
         equityExpected1, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio  when  the  risk  is  2" )
      System.Console.WriteLine( "    percent per month is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( returnCML1 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" )
      System.Console.WriteLine( "+ Risk                                                                   +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    returnCML( double risk," )
      System.Console.WriteLine( "               double equityRisk," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    risk = 0.03" )
      System.Console.WriteLine( "    equityRisk = equityRisk1" )
      System.Console.WriteLine( "    equityExpected = equityExpected1" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      Dim returnCML2 As Double = instanceOfCapitalMarket.ReturnCML( _
         0.03, _
         equityRisk1, _
         equityExpected1, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio  when  the  risk  is  3" )
      System.Console.WriteLine( "    percent per month is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( returnCML2 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" )
      System.Console.WriteLine( "+ Risk                                                                   +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    returnCML( double risk," )
      System.Console.WriteLine( "               double equityRisk," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    risk = 0.05" )
      System.Console.WriteLine( "    equityRisk = equityRisk1" )
      System.Console.WriteLine( "    equityExpected = equityExpected1" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      Dim returnCML3 As Double = instanceOfCapitalMarket.ReturnCML( _
         0.05, _
         equityRisk1, _
         equityExpected1, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio  when  the  risk  is  5" )
      System.Console.WriteLine( "    percent per month is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( returnCML3 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" )
      System.Console.WriteLine( "+ Risk                                                                   +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    returnCML( double risk," )
      System.Console.WriteLine( "               double equityRisk," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    risk = 0.1" )
      System.Console.WriteLine( "    equityRisk = equityRisk1" )
      System.Console.WriteLine( "    equityExpected = equityExpected1" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      Dim returnCML4 As Double = instanceOfCapitalMarket.ReturnCML( _
         0.1, _
         equityRisk1, _
         equityExpected1, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio when  the  risk  is  10" )
      System.Console.WriteLine( "    percent per month is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( returnCML4 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" )
      System.Console.WriteLine( "+ Risk                                                                   +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    returnCML( double risk," )
      System.Console.WriteLine( "               double equityRisk," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    risk = 0.2" )
      System.Console.WriteLine( "    equityRisk = equityRisk1" )
      System.Console.WriteLine( "    equityExpected = equityExpected1" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      Dim returnCML5 As Double = instanceOfCapitalMarket.ReturnCML( _
         0.2, _
         equityRisk1, _
         equityExpected1, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio when  the  risk  is  20" )
      System.Console.WriteLine( "    percent per month is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( returnCML5 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the return of a Portfolio on the CML  with  a  given  Total +" )
      System.Console.WriteLine( "+ Risk                                                                   +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    returnCML( double risk," )
      System.Console.WriteLine( "               double equityRisk," )
      System.Console.WriteLine( "               double equityExpected," )
      System.Console.WriteLine( "               double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    risk = 0.3" )
      System.Console.WriteLine( "    equityRisk = equityRisk1" )
      System.Console.WriteLine( "    equityExpected = equityExpected1" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      Dim returnCML6 As Double = instanceOfCapitalMarket.ReturnCML( _
         0.3, _
         equityRisk1, _
         equityExpected1, _
         0.005 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The (maximum) expected return of the portfolio when  the  risk  is  30" )
      System.Console.WriteLine( "    percent per month is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( returnCML6 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolio( double minimumExpectedReturn," )
      System.Console.WriteLine( "                     double maximumExpectedReturn," )
      System.Console.WriteLine( "                     double[][] covarianceMatrix," )
      System.Console.WriteLine( "                     double marketRate )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     We  consider  the  situation  where  the  market  portfolio  can  be" )
      System.Console.WriteLine( "     constructed from  the  following  four  assets  with  the  following" )
      System.Console.WriteLine( "     historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" )
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" )
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" )
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     Moreover, we require that the weights of these  assets  lies  within" )
      System.Console.WriteLine( "     the interval [0.1, 0.5] (i.e. between 10 and 50  percent).  In  such" )
      System.Console.WriteLine( "     an instance what are the values of the risk of the  weights  of  the" )
      System.Console.WriteLine( "     Market Portfolio  when  the  monthly  rate  at  which  cash  can  be" )
      System.Console.WriteLine( "     borrowed or lent is 0.5 percent per month and we wish  to  construct" )
      System.Console.WriteLine( "     six portfolios with the following values  of  the  expected  return:" )
      System.Console.WriteLine( "     1.5, 2, 3, 4, 5, 6 percent per month which have the lowest  possible" )
      System.Console.WriteLine( "     risk?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    minimumExpectedReturn = min5" )
      System.Console.WriteLine( "    maximumExpectedReturn = max5" )
      System.Console.WriteLine( "    covarianceMatrix = covariance4" )
      System.Console.WriteLine( "    marketRate = 0.001" )
      ' We provide the market  portfolios  weights  which  requires  that  the
      ' Efficient Frontier is evaluted and the constraints set
      Dim ExpectedReturnMarketPortfolio2 As Double = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( _
         marketPortWeights, _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      Dim covariance4 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedreturns5 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      instanceOfCapitalMarket.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      ' Here we evaluate the maximum of the expected returns  over  which  the
      ' Efficient Frontier exits.
      Dim max5 As Double = instanceOfCapitalMarket.MaxFrontierReturn( _
         expectedreturns5 _
      )
      ' Here we evaluate the minimum of the expected returns  over  which  the
      ' Efficient Frontier exits.
      Dim min5 As Double = instanceOfCapitalMarket.MinFrontierReturn( _
         expectedreturns5 _
      )
      ' Here we evaluate the risk of the market portfolio which requires  that
      ' the weights of the market portfolio are known.
      Dim RiskMarketPortfolio As Double = instanceOfCapitalMarket.MarketPortfolioRisk( _
         marketPortWeights, _
         covariance4 _
      )
      ' Here we evaluate the market  portfolio  which  depends  on  the  prior
      ' evaluation of the (constrained) Efficient Frontier
      Dim marketPortWeights4 As Double() = instanceOfCapitalMarket.MarketPortfolio( _
         min5, _
         max5, _
         covariance4, _
         0.001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the portfolio on the CML with an expected  return  of  1.5" )
      System.Console.WriteLine( "    percent per month (rounded to four decimal places) is:" )
      System.Console.WriteLine( " " )
      displayArray( marketPortWeights4 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    riskCML( double expectedReturn," )
      System.Console.WriteLine( "             double equityExpected," )
      System.Console.WriteLine( "             double marketRate," )
      System.Console.WriteLine( "             double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.015" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" )
      Dim riskCML1 As Double = instanceOfCapitalMarket.RiskCML( _
         0.015, _
         ExpectedReturnMarketPortfolio2, _
         0.005, _
         RiskMarketPortfolio _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the portfolio on the CML with  an  expected  return  of  2" )
      System.Console.WriteLine( "    percent per month (rounded to four decimal places) is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( riskCML1 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    riskCML( double expectedReturn," )
      System.Console.WriteLine( "             double equityExpected," )
      System.Console.WriteLine( "             double marketRate," )
      System.Console.WriteLine( "             double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.02" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" )
      Dim riskCML2 As Double = instanceOfCapitalMarket.RiskCML( _
         0.02, _
         ExpectedReturnMarketPortfolio2, _
         0.005, _
         RiskMarketPortfolio _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the portfolio on the CML with  an  expected  return  of  3" )
      System.Console.WriteLine( "    percent per month (rounded to four decimal places)is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( riskCML2 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    riskCML( double expectedReturn," )
      System.Console.WriteLine( "             double equityExpected," )
      System.Console.WriteLine( "             double marketRate," )
      System.Console.WriteLine( "             double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.03" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" )
      Dim riskCML3 As Double = instanceOfCapitalMarket.RiskCML( _
         0.03, _
         ExpectedReturnMarketPortfolio2, _
         0.005, _
         RiskMarketPortfolio _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the portfolio on the CML with  an  expected  return  of  4" )
      System.Console.WriteLine( "    percent per month (rounded to four decimal places)is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( riskCML3 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    riskCML( double expectedReturn," )
      System.Console.WriteLine( "             double equityExpected," )
      System.Console.WriteLine( "             double marketRate," )
      System.Console.WriteLine( "             double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.04" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" )
      Dim riskCML4 As Double = instanceOfCapitalMarket.RiskCML( _
         0.04, _
         ExpectedReturnMarketPortfolio2, _
         0.005, _
         RiskMarketPortfolio _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the portfolio on the CML with  an  expected  return  of  5" )
      System.Console.WriteLine( "    percent per month (rounded to four decimal places)is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( riskCML4 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    riskCML( double expectedReturn," )
      System.Console.WriteLine( "             double equityExpected," )
      System.Console.WriteLine( "             double marketRate," )
      System.Console.WriteLine( "             double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.05" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" )
      Dim riskCML5 As Double = instanceOfCapitalMarket.RiskCML( _
         0.05, _
         ExpectedReturnMarketPortfolio2, _
         0.005, _
         RiskMarketPortfolio _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the portfolio on the CML with  an  expected  return  of  6" )
      System.Console.WriteLine( "    percent per month (rounded to four decimal places)is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( riskCML5 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluating the risk of a Portfolio on the CML with  a  given  expected +" )
      System.Console.WriteLine( "+ return                                                                 +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    riskCML( double expectedReturn," )
      System.Console.WriteLine( "             double equityExpected," )
      System.Console.WriteLine( "             double marketRate," )
      System.Console.WriteLine( "             double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 0.06" )
      System.Console.WriteLine( "    equityExpected = ExpectedReturnMarketPortfolio2" )
      System.Console.WriteLine( "    marketRate = 0.005" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio" )
      Dim riskCML6 As Double = instanceOfCapitalMarket.RiskCML( _
         0.06, _
         ExpectedReturnMarketPortfolio2, _
         0.005, _
         RiskMarketPortfolio _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( riskCML6 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weight2Risk( double weighting," )
      System.Console.WriteLine( "                 double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     We  consider  the  situation  where  the  market  portfolio  can  be" )
      System.Console.WriteLine( "     constructed from  the  following  four  assets  with  the  following" )
      System.Console.WriteLine( "     historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 2, 1, -2, 2 percent" )
      System.Console.WriteLine( "        - 2nd Asset: 0, 3, -1, 1 percent" )
      System.Console.WriteLine( "        - 3rd Asset: 0, -1, 3, 1 percent" )
      System.Console.WriteLine( "        - 4th Asset: 1, 1, 2, 1 percent" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     Moreover, we require that the weights of these  assets  lies  within" )
      System.Console.WriteLine( "     the interval [0.1, 0.5] (i.e. between 10 and 50 percent).  In  such," )
      System.Console.WriteLine( "     an instance what is the risk of  the  portfolio  on  the  CML  (i.e." )
      System.Console.WriteLine( "     optimal) such that weighting of the market  portfolio  within  these" )
      System.Console.WriteLine( "     portfolios is: 0.1, 0.5, 0.75, 1, 1.5, 2?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weighting = 0.1" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" )
      Dim covariance5 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {0.02, 0.01, -0.02, 0.02}, _
                         New Double() {0.0, 0.03, -0.01, 0.01}, _
                         New Double() {0.0, -0.01, 0.03, 0.01}, _
                         New Double() {0.01, 0.01, 0.02, 0.01}} _
      )
      ' Here we evaluate the risk of the market portfolio which requires  that
      ' the weights of the market portfolio are known.
      Dim RiskMarketPortfolio2 As Double = instanceOfCapitalMarket.MarketPortfolioRisk( _
         marketPortWeights, _
         covariance5 _
      )
      Dim weight2Risk1 As Double = instanceOfCapitalMarket.Weight2Risk( _
         0.1, _
         RiskMarketPortfolio2 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" )
      System.Console.WriteLine( "    Market Portfolio is 0.1 is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weight2Risk1 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weight2Risk( double weighting," )
      System.Console.WriteLine( "                 double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weighting = 0.5" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" )
      Dim weight2Risk2 As Double = instanceOfCapitalMarket.Weight2Risk( _
         0.5, _
         RiskMarketPortfolio2 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" )
      System.Console.WriteLine( "    Market Portfolio is 0.5 is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weight2Risk2 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weight2Risk( double weighting," )
      System.Console.WriteLine( "                 double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weighting = 0.75" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" )
      Dim weight2Risk3 As Double = instanceOfCapitalMarket.Weight2Risk( _
         0.75, _
         RiskMarketPortfolio2 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" )
      System.Console.WriteLine( "    Market Portfolio is 0.75 is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weight2Risk3 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weight2Risk( double weighting," )
      System.Console.WriteLine( "                 double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weighting = 1" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" )
      Dim weight2Risk4 As Double = instanceOfCapitalMarket.Weight2Risk( _
         1.0, _
         RiskMarketPortfolio2 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" )
      System.Console.WriteLine( "    Market Portfolio is 1 is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weight2Risk4 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weight2Risk( double weighting," )
      System.Console.WriteLine( "                 double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weighting = 1.5" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" )
      Dim weight2Risk5 As Double = instanceOfCapitalMarket.Weight2Risk( _
         1.5, _
         RiskMarketPortfolio2 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" )
      System.Console.WriteLine( "    Market Portfolio is 1.5 is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weight2Risk5 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluation of the required weight of the Market Portfolio +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weight2Risk( double weighting," )
      System.Console.WriteLine( "                 double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    weighting = 2" )
      System.Console.WriteLine( "    equityRisk = RiskMarketPortfolio2" )
      Dim weight2Risk6 As Double = instanceOfCapitalMarket.Weight2Risk( _
         2.0, _
         RiskMarketPortfolio2 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The Total Risk of the portfolio on the CML where the weighting of  the" )
      System.Console.WriteLine( "    Market Portfolio is 2 is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( weight2Risk6 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       -------------------------------------------------------------------" )
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" )
      System.Console.WriteLine( "       -------------------------------------------------------------------" )
      System.Console.WriteLine( " " )
   End Sub

   Function Double2String(ByVal d As Double) As String
      If (System.Double.IsNaN(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (System.Double.IsNegativeInfinity(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (System.Double.IsPositiveInfinity(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (d = Double.MinValue) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (d = Double.MaxValue) Then
          Double2String = d.ToString()
          Exit Function
      End If
      Dim frac As Double = d - Int(d)
      Dim frac4 As Double = Int(frac * 10000) / 10000.0
      Double2String = (Int(d) + frac4).ToString()
      Exit Function
   End Function

   Sub displayArray(ByVal a As Double())
      System.Console.Write("     {")
      For I As Integer = 0 To -1 + a.Length
         System.Console.Write(Double2String(a(I)))
         If (I < -1 + a.Length) Then System.Console.Write(", ")
      Next I
      System.Console.Write("}")
   End Sub

   Sub display2DArray(ByVal a As Double()())
      For I As Integer = 0 To -1 + a.Length
         If (I = 0) Then
            System.Console.Write("     {{")
         Else
            System.Console.Write("      {")
         End If
         For J As Integer = 0 To -1 + a(I).Length
            System.Console.Write(Double2String(a(I)(J)))
            If (J < -1 + a(I).Length) Then
               System.Console.Write(", ")
            Else
               System.Console.Write("}")
               End If
         Next J
         If (I < -1 + a.Length) Then
            System.Console.Write(",")
            System.Console.WriteLine()
            Else
               System.Console.Write("}")
            End If
      Next I
   End Sub
End Module

