Imports System
Imports Microsoft.VisualBasic
Imports WebCab.Lib.Util.Example
Imports WebCab.Libraries.Finance.Portfolio

Module TwelveAssetPortfolioClient
   Sub Main()
      ' used components
      Dim instanceOfAssetParameters As AssetParameters = New AssetParameters
      Dim instanceOfCapitalMarket As WebCab.Libraries.Finance.Portfolio.CapitalMarket = New WebCab.Libraries.Finance.Portfolio.CapitalMarket
      '
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "                 ###############################################" )
      System.Console.WriteLine( "                 # TwelveAssetPortfolio Console Client Example #" )
      System.Console.WriteLine( "                 ###############################################" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " --------" )
      System.Console.WriteLine( " Overview" )
      System.Console.WriteLine( " --------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Within this client we provide an example  where  we  illustrate  the" )
      System.Console.WriteLine( "    application of the CAPM to modeling the risk/return  characteristics" )
      System.Console.WriteLine( "    of portfolios which are constructed from three risky  equity  assets" )
      System.Console.WriteLine( "    and/or geared/de-geared at the prevailing risk free  rate  on  cash." )
      System.Console.WriteLine( "    Notes on the Units used for the Source  data  (i.e.  the  Historical" )
      System.Console.WriteLine( "    Values) The units in which the  historical  values  are  given  will" )
      System.Console.WriteLine( "    effect  the  units  used  throughout  the  portfolio  analysis.  For" )
      System.Console.WriteLine( "    example, if the historical values are given in absolute  terms  then" )
      System.Console.WriteLine( "    the following will either be returned  and  should  be  provided  in" )
      System.Console.WriteLine( "    absolute terms:" )
      System.Console.WriteLine( "    " )
      System.Console.WriteLine( "       - Expected Return" )
      System.Console.WriteLine( "       - Utility Function" )
      System.Console.WriteLine( "       " )
      System.Console.WriteLine( "    " )
      System.Console.WriteLine( "    Note that the covariance matrix and the asset weights are  invariant" )
      System.Console.WriteLine( "    to the units in which the historical returns are expressed." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluate Points  on  Efficient  Frontier  over  Interval  of  Expected +" )
      System.Console.WriteLine( "+ Returns                                                                +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    calculateEfficientFrontier( double[][] covarianceMatrix," )
      System.Console.WriteLine( "                                double[] expectedReturns," )
      System.Console.WriteLine( "                                int numberInterpolationPoints," )
      System.Console.WriteLine( "                                double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Evaluate 40 points on the (un-constrained) Efficient  Frontier  with" )
      System.Console.WriteLine( "     the precision set to 0.00001" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    covarianceMatrix = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" )
      System.Console.WriteLine( "    numberInterpolationPoints = 40" )
      System.Console.WriteLine( "    precision = 0.001" )
      ' We first use the covariance method from Asset Parameters.
      Dim covariance1 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {0.366, -3.4281, 0.6797, 1.2003, 2.8169, 2.5234, 1.8987}, _
                         New Double() {0.2437, -3.4846, 0.5877, 1.0851, 2.8076, 2.3293, 1.8053}, _
                         New Double() {4.3369, -7.3339, -0.6444, 2.0757, 3.4316, 2.4084, 4.0556}, _
                         New Double() {2.3077, -0.7518, -0.9469, 0.956, 0.9469, 0.3752, 1.8691}, _
                         New Double() {2.4747, -0.7771, -1.0443, 0.9032, 0.7894, 0.5221, 1.7299}, _
                         New Double() {1.0615, -0.8403, -1.2711, 0.4291, 0.8546, 0.4237, 1.6877}, _
                         New Double() {2.3882, -1.0389, -1.0498, 0.6494, 0.7936, 0.5249, 1.5665}, _
                         New Double() {4.2674, -7.4215, -0.7368, 1.9893, 3.3478, 2.338, 4.0187}, _
                         New Double() {0.1297, -10.8423, 1.8895, 0.4279, 1.089, 4.4964, 2.4652}, _
                         New Double() {2.4142, -5.0454, 0.4355, -0.6938, 2.7074, 0.7653, 4.8523}, _
                         New Double() {2.3047, -5.1494, 0.3877, -0.7725, 2.6277, 0.6638, 4.7574}, _
                         New Double() {-0.6503, -7.7294, -1.5706, -0.5319, 3.4759, 0.7751, -1.282}} _
      )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedReturns1 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {0.366, -3.4281, 0.6797, 1.2003, 2.8169, 2.5234, 1.8987}, _
                         New Double() {0.2437, -3.4846, 0.5877, 1.0851, 2.8076, 2.3293, 1.8053}, _
                         New Double() {4.3369, -7.3339, -0.6444, 2.0757, 3.4316, 2.4084, 4.0556}, _
                         New Double() {2.3077, -0.7518, -0.9469, 0.956, 0.9469, 0.3752, 1.8691}, _
                         New Double() {2.4747, -0.7771, -1.0443, 0.9032, 0.7894, 0.5221, 1.7299}, _
                         New Double() {1.0615, -0.8403, -1.2711, 0.4291, 0.8546, 0.4237, 1.6877}, _
                         New Double() {2.3882, -1.0389, -1.0498, 0.6494, 0.7936, 0.5249, 1.5665}, _
                         New Double() {4.2674, -7.4215, -0.7368, 1.9893, 3.3478, 2.338, 4.0187}, _
                         New Double() {0.1297, -10.8423, 1.8895, 0.4279, 1.089, 4.4964, 2.4652}, _
                         New Double() {2.4142, -5.0454, 0.4355, -0.6938, 2.7074, 0.7653, 4.8523}, _
                         New Double() {2.3047, -5.1494, 0.3877, -0.7725, 2.6277, 0.6638, 4.7574}, _
                         New Double() {-0.6503, -7.7294, -1.5706, -0.5319, 3.4759, 0.7751, -1.282}} _
      )
      ' This method sets the coordinates of the Efficient Frontier as  private
      ' fields in the class. Note the we evaluate the Efficient Frontier  over
      ' the entire region which it exists.
      instanceOfCapitalMarket.CalculateEfficientFrontier( _
         covariance1, _
         expectedReturns1, _
         40, _
         0.001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Our first aim in virtually all applications and  Portfolio  Theory  is" )
      System.Console.WriteLine( "    to evaluate a collection of points on the  Efficient  Frontier,  which" )
      System.Console.WriteLine( "    we do to applying the  calculate  Efficient  Frontier  method.  Before" )
      System.Console.WriteLine( "    this method  is  applied  we  need  to  evaluate  the  covariance  and" )
      System.Console.WriteLine( "    expected returns using procedures from  the  Asset  Parameters  class." )
      System.Console.WriteLine( "    The points of the Efficient Frontier are stored in  private  field  of" )
      System.Console.WriteLine( "    the class." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we return the five values of the expected return for which  the" )
      System.Console.WriteLine( "     efficient was evaluated  within  the  example  ;Evaluate  Points  on" )
      System.Console.WriteLine( "     Efficient Frontier;, and stored within a private field." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      ' Reads the values of the  expected  return  at  the  points  where  the
      ' Efficient Frontier was evaluated.
      Dim efficientFrontierExpectedReturns1 As Double() = instanceOfCapitalMarket.GetEfficientFrontierExpectedReturns( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The values of the expected return at the  five  points  at  which  the" )
      System.Console.WriteLine( "    efficient was evaluated is:" )
      System.Console.WriteLine( " " )
      displayArray( efficientFrontierExpectedReturns1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierAssetWeights(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      ' Reads the values  of  the  asset  weights  at  the  points  where  the
      ' Efficient Frontier was evaluated
      Dim assetWeightsEfficientFrontier1 As Double()() = instanceOfCapitalMarket.GetEfficientFrontierAssetWeights( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    " )
      System.Console.WriteLine( " " )
      display2DArray( assetWeightsEfficientFrontier1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Returns the values of the Risks +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Returns the values of the risks at the 20 points  of  the  Efficient" )
      System.Console.WriteLine( "     Frontier evaluated." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    covarianceMatrix = covariance1" )
      Dim risks1 As Double() = instanceOfCapitalMarket.GetEfficientFrontierPortfolioRisks( _
         covariance1 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The 40 values of the risks of the points  on  the  Efficient  Frontier" )
      System.Console.WriteLine( "    are:" )
      System.Console.WriteLine( " " )
      displayArray( risks1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Here we evaluate the Expected Return of the Market Portfolio +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[][] cov," )
      System.Console.WriteLine( "                                   double[] expectedReturns," )
      System.Console.WriteLine( "                                   double riskFreeRate," )
      System.Console.WriteLine( "                                   double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we get the expected return of the Market Portfolio." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    cov = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" )
      System.Console.WriteLine( "    riskFreeRate = 1" )
      System.Console.WriteLine( "    precision = 0.0001" )
      ' Here we evaluate the expected return of the market portfolio.
      Dim marketPortfolioReturn2 As Double = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( _
         covariance1, _
         expectedReturns1, _
         1.0, _
         0.0001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The expected return of the market portfolio is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( marketPortfolioReturn2 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Here we evaluate the Risk of the Market Portfolio +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolioRisk( double[][] cov," )
      System.Console.WriteLine( "                         double[] expectedReturns," )
      System.Console.WriteLine( "                         double riskFreeRate," )
      System.Console.WriteLine( "                         double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we evaluate the risk of the  market  portfolio  found.  We  use" )
      System.Console.WriteLine( "     here a method from the Capital  Markets  class  but  we  could  have" )
      System.Console.WriteLine( "     applied the MarketPortfolioRisk method  from  the  Asset  Parameters" )
      System.Console.WriteLine( "     class." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    cov = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" )
      System.Console.WriteLine( "    riskFreeRate = 1" )
      System.Console.WriteLine( "    precision = 0.0001" )
      ' Here we evaluate the risk of the market portfolio.
      Dim marketPortfolioRisk2 As Double = instanceOfCapitalMarket.MarketPortfolioRisk( _
         covariance1, _
         expectedReturns1, _
         1.0, _
         0.0001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the market portfolio is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( marketPortfolioRisk2 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Here we evaluate the Expected Return of the Market Portfolio +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[][] cov," )
      System.Console.WriteLine( "                                   double[] expectedReturns," )
      System.Console.WriteLine( "                                   double riskFreeRate," )
      System.Console.WriteLine( "                                   double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we get the expected return of the Market Portfolio." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    cov = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" )
      System.Console.WriteLine( "    riskFreeRate = 4" )
      System.Console.WriteLine( "    precision = 0.0001" )
      ' Here we evaluate the expected return of the market portfolio.
      Dim marketPortfolioReturn3 As Double = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( _
         covariance1, _
         expectedReturns1, _
         4.0, _
         0.0001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The expected return of the market portfolio is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( marketPortfolioReturn3 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Here we evaluate the Risk of the Market Portfolio +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolioRisk( double[][] cov," )
      System.Console.WriteLine( "                         double[] expectedReturns," )
      System.Console.WriteLine( "                         double riskFreeRate," )
      System.Console.WriteLine( "                         double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we evaluate the risk of the  market  portfolio  found.  We  use" )
      System.Console.WriteLine( "     here a method from the Capital  Markets  class  but  we  could  have" )
      System.Console.WriteLine( "     applied the MarketPortfolioRisk method  from  the  Asset  Parameters" )
      System.Console.WriteLine( "     class." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    cov = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" )
      System.Console.WriteLine( "    riskFreeRate = 4" )
      System.Console.WriteLine( "    precision = 0.0001" )
      ' Here we evaluate the risk of the market portfolio.
      Dim marketPortfolioRisk3 As Double = instanceOfCapitalMarket.MarketPortfolioRisk( _
         covariance1, _
         expectedReturns1, _
         4.0, _
         0.0001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the market portfolio is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( marketPortfolioRisk3 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Here we evaluate the Expected Return of the Market Portfolio +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[][] cov," )
      System.Console.WriteLine( "                                   double[] expectedReturns," )
      System.Console.WriteLine( "                                   double riskFreeRate," )
      System.Console.WriteLine( "                                   double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we get the expected return of the Market Portfolio." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    cov = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" )
      System.Console.WriteLine( "    riskFreeRate = 5" )
      System.Console.WriteLine( "    precision = 0.0001" )
      ' Here we evaluate the expected return of the market portfolio.
      Dim marketPortfolioReturn4 As Double = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( _
         covariance1, _
         expectedReturns1, _
         5.0, _
         0.0001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The expected return of the market portfolio is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( marketPortfolioReturn4 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Here we evaluate the Risk of the Market Portfolio +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolioRisk( double[][] cov," )
      System.Console.WriteLine( "                         double[] expectedReturns," )
      System.Console.WriteLine( "                         double riskFreeRate," )
      System.Console.WriteLine( "                         double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we evaluate the risk of the  market  portfolio  found.  We  use" )
      System.Console.WriteLine( "     here a method from the Capital  Markets  class  but  we  could  have" )
      System.Console.WriteLine( "     applied the MarketPortfolioRisk method  from  the  Asset  Parameters" )
      System.Console.WriteLine( "     class." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    cov = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" )
      System.Console.WriteLine( "    riskFreeRate = 5" )
      System.Console.WriteLine( "    precision = 0.0001" )
      ' Here we evaluate the risk of the market portfolio.
      Dim marketPortfolioRisk4 As Double = instanceOfCapitalMarket.MarketPortfolioRisk( _
         covariance1, _
         expectedReturns1, _
         5.0, _
         0.0001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the market portfolio is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( marketPortfolioRisk4 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Here we evaluate the Expected Return of the Market Portfolio +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolioExpectedReturn( double[][] cov," )
      System.Console.WriteLine( "                                   double[] expectedReturns," )
      System.Console.WriteLine( "                                   double riskFreeRate," )
      System.Console.WriteLine( "                                   double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we get the expected return of the Market Portfolio." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    cov = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" )
      System.Console.WriteLine( "    riskFreeRate = 7" )
      System.Console.WriteLine( "    precision = 0.0001" )
      ' Here we evaluate the expected return of the market portfolio.
      Dim marketPortfolioReturn5 As Double = instanceOfCapitalMarket.MarketPortfolioExpectedReturn( _
         covariance1, _
         expectedReturns1, _
         7.0, _
         0.0001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The expected return of the market portfolio is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( marketPortfolioReturn5 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Here we evaluate the Risk of the Market Portfolio +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    marketPortfolioRisk( double[][] cov," )
      System.Console.WriteLine( "                         double[] expectedReturns," )
      System.Console.WriteLine( "                         double riskFreeRate," )
      System.Console.WriteLine( "                         double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we evaluate the risk of the  market  portfolio  found.  We  use" )
      System.Console.WriteLine( "     here a method from the Capital  Markets  class  but  we  could  have" )
      System.Console.WriteLine( "     applied the MarketPortfolioRisk method  from  the  Asset  Parameters" )
      System.Console.WriteLine( "     class." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    cov = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedReturns1" )
      System.Console.WriteLine( "    riskFreeRate = 7" )
      System.Console.WriteLine( "    precision = 0.0001" )
      ' Here we evaluate the risk of the market portfolio.
      Dim marketPortfolioRisk5 As Double = instanceOfCapitalMarket.MarketPortfolioRisk( _
         covariance1, _
         expectedReturns1, _
         7.0, _
         0.0001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the market portfolio is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( marketPortfolioRisk5 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Find the Risk of the Leveraged Portfolio on the CML +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    riskCML( double expectedReturn," )
      System.Console.WriteLine( "             double equityExpected," )
      System.Console.WriteLine( "             double marketRate," )
      System.Console.WriteLine( "             double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we evaluate the Risk of  the  Portfolio  on  the  CML  with  an" )
      System.Console.WriteLine( "     expected return of 7 per  period.  Recall,  that  since  the  Market" )
      System.Console.WriteLine( "     Portfolio has a expected return less than 7, we will be  considering" )
      System.Console.WriteLine( "     a leveraged portfolio which has borrowed cash  from  the  market  at" )
      System.Console.WriteLine( "     the prevailing rate of 1 percent per  month  in  order  to  purchase" )
      System.Console.WriteLine( "     more units of the market portfolio." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 7" )
      System.Console.WriteLine( "    equityExpected = marketPortfolioReturn2" )
      System.Console.WriteLine( "    marketRate = 1" )
      System.Console.WriteLine( "    equityRisk = marketPortfolioRisk2" )
      ' Here we evaluate the  Risk  of  the  Portfolio  on  the  CML  with  an
      ' expected return of 7 per period (i.e. a leveraged portfolio).
      Dim riskCML1 As Double = instanceOfCapitalMarket.RiskCML( _
         7.0, _
         marketPortfolioReturn2, _
         1.0, _
         marketPortfolioRisk2 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the leveraged  portfolio  on  the  CML  with  an  expected" )
      System.Console.WriteLine( "    return of 7 per period, is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( riskCML1 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Find the Risk of the Unleveraged Portfolio on the CML +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    riskCML( double expectedReturn," )
      System.Console.WriteLine( "             double equityExpected," )
      System.Console.WriteLine( "             double marketRate," )
      System.Console.WriteLine( "             double equityRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we evaluate the Risk of  the  Portfolio  on  the  CML  with  an" )
      System.Console.WriteLine( "     expected return of 4 per  period.  Recall,  that  since  the  Market" )
      System.Console.WriteLine( "     Portfolio  has  a  expected  return  greater  than  4,  we  will  be" )
      System.Console.WriteLine( "     considering a unleveraged portfolio which lends cash to  the  market" )
      System.Console.WriteLine( "     at the prevailing  rate  of  1  percent  per  month  and  invest  to" )
      System.Console.WriteLine( "     remaining capital within the market portfolio.  That  is,  since  we" )
      System.Console.WriteLine( "     require a portfolio with a return lower than  the  market  portfolio" )
      System.Console.WriteLine( "     we are able to hold a portfolio consisting  of  a  weighting  within" )
      System.Console.WriteLine( "     the market portfolio (i.e. risky asset)  with  the  remaining  being" )
      System.Console.WriteLine( "     lent to the market (i.e. zero risk asset)." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 4" )
      System.Console.WriteLine( "    equityExpected = marketPortfolioReturn2" )
      System.Console.WriteLine( "    marketRate = 1" )
      System.Console.WriteLine( "    equityRisk = marketPortfolioRisk2" )
      ' Here we evaluate the  Risk  of  the  Portfolio  on  the  CML  with  an
      ' expected return of 4 per period (i.e. a unleveraged portfolio).
      Dim riskCML2 As Double = instanceOfCapitalMarket.RiskCML( _
         4.0, _
         marketPortfolioReturn2, _
         1.0, _
         marketPortfolioRisk2 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The risk of the unleveraged portfolio on  the  CML  with  an  expected" )
      System.Console.WriteLine( "    return of 4 per period, is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( riskCML2 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       -------------------------------------------------------------------" )
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" )
      System.Console.WriteLine( "       -------------------------------------------------------------------" )
      System.Console.WriteLine( " " )
   End Sub

   Function Double2String(ByVal d As Double) As String
      If (System.Double.IsNaN(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (System.Double.IsNegativeInfinity(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (System.Double.IsPositiveInfinity(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (d = Double.MinValue) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (d = Double.MaxValue) Then
          Double2String = d.ToString()
          Exit Function
      End If
      Dim frac As Double = d - Int(d)
      Dim frac4 As Double = Int(frac * 10000) / 10000.0
      Double2String = (Int(d) + frac4).ToString()
      Exit Function
   End Function

   Sub displayArray(ByVal a As Double())
      System.Console.Write("     {")
      For I As Integer = 0 To -1 + a.Length
         System.Console.Write(Double2String(a(I)))
         If (I < -1 + a.Length) Then System.Console.Write(", ")
      Next I
      System.Console.Write("}")
   End Sub

   Sub display2DArray(ByVal a As Double()())
      For I As Integer = 0 To -1 + a.Length
         If (I = 0) Then
            System.Console.Write("     {{")
         Else
            System.Console.Write("      {")
         End If
         For J As Integer = 0 To -1 + a(I).Length
            System.Console.Write(Double2String(a(I)(J)))
            If (J < -1 + a(I).Length) Then
               System.Console.Write(", ")
            Else
               System.Console.Write("}")
               End If
         Next J
         If (I < -1 + a.Length) Then
            System.Console.Write(",")
            System.Console.WriteLine()
            Else
               System.Console.Write("}")
            End If
      Next I
   End Sub
End Module

