Imports System
Imports Microsoft.VisualBasic
Imports WebCab.Lib.Util.Example
Imports WebCab.Libraries.Finance.Portfolio

Module MarkowitzClient
   Sub Main()
      ' used components
      Dim instanceOfAssetParameters As AssetParameters = New AssetParameters
      Dim instanceOfMarkowitz As WebCab.Libraries.Finance.Portfolio.Markowitz = New WebCab.Libraries.Finance.Portfolio.Markowitz
      '
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "                       ####################################" )
      System.Console.WriteLine( "                       # Markowitz Console Client Example #" )
      System.Console.WriteLine( "                       ####################################" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " --------" )
      System.Console.WriteLine( " Overview" )
      System.Console.WriteLine( " --------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Within this client we provide one examples for each of  the  methods" )
      System.Console.WriteLine( "    contained within the Markowitz class. Notes on the  Units  used  for" )
      System.Console.WriteLine( "    the Source data (i.e. the Historical Values) The units in which  the" )
      System.Console.WriteLine( "    historical values are given will effect the  units  used  throughout" )
      System.Console.WriteLine( "    the portfolio analysis. For example, if the  historical  values  are" )
      System.Console.WriteLine( "    given in absolute terms then the following will either  be  returned" )
      System.Console.WriteLine( "    and should be provided in absolute terms:" )
      System.Console.WriteLine( "    " )
      System.Console.WriteLine( "       - Expected Return" )
      System.Console.WriteLine( "       - Utility Function" )
      System.Console.WriteLine( "       " )
      System.Console.WriteLine( "    " )
      System.Console.WriteLine( "    Note that the covariance matrix and the asset weights are  invariant" )
      System.Console.WriteLine( "    to the units in which the historical returns are expressed." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluate Points  on  Efficient  Frontier  over  Interval  of  Expected +" )
      System.Console.WriteLine( "+ Returns                                                                +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    calculateEfficientFrontier( double[][] covarianceMatrix," )
      System.Console.WriteLine( "                                double[] expectedReturns," )
      System.Console.WriteLine( "                                int numberInterpolationPoints," )
      System.Console.WriteLine( "                                double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     We are able to  construct  portfolios  from  four  assets  with  the" )
      System.Console.WriteLine( "     following historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 102, 101, 99, 101" )
      System.Console.WriteLine( "        - 2nd Asset: 100, 103, 99, 101" )
      System.Console.WriteLine( "        - 3rd Asset: 100, 99, 103, 101" )
      System.Console.WriteLine( "        - 4th Asset: 101, 101, 102, 101" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     Evaluate 5 points on the (un-constrained)  Efficient  Frontier  with" )
      System.Console.WriteLine( "     the precision set to 0.00001" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    covarianceMatrix = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedreturns1" )
      System.Console.WriteLine( "    numberInterpolationPoints = 5" )
      System.Console.WriteLine( "    precision = 0.00001" )
      ' We first use the covariance method from Asset Parameters.
      Dim covariance1 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedreturns1 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' This method sets the coordinates of the Efficient Frontier as  private
      ' fields in the class. Note the we evaluate the Efficient Frontier  over
      ' the entire region which it exists.
      instanceOfMarkowitz.CalculateEfficientFrontier( _
         covariance1, _
         expectedreturns1, _
         5, _
         0.00001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Our first aim in virtually all applications and  Portfolio  Theory  is" )
      System.Console.WriteLine( "    to evaluate a collection of points on the  Efficient  Frontier,  which" )
      System.Console.WriteLine( "    we do to applying the  calculate  Efficient  Frontier  method.  Before" )
      System.Console.WriteLine( "    this method  is  applied  we  need  to  evaluate  the  covariance  and" )
      System.Console.WriteLine( "    expected returns using procedures from  the  Asset  Parameters  class." )
      System.Console.WriteLine( "    The points of the Efficient Frontier are stored in  private  field  of" )
      System.Console.WriteLine( "    the class." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluate Points on Efficient Frontier over entire  range  of  Expected +" )
      System.Console.WriteLine( "+ Returns                                                                +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    calculateEfficientFrontier( double[][] covarianceMatrix," )
      System.Console.WriteLine( "                                double[] expectedReturns," )
      System.Console.WriteLine( "                                int numberInterpolationPoints," )
      System.Console.WriteLine( "                                double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     We are able to  construct  portfolios  from  four  assets  with  the" )
      System.Console.WriteLine( "     following historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 102, 101, 99, 101" )
      System.Console.WriteLine( "        - 2nd Asset: 100, 103, 99, 101" )
      System.Console.WriteLine( "        - 3rd Asset: 100, 99, 103, 101" )
      System.Console.WriteLine( "        - 4th Asset: 101, 101, 102, 101" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     Evaluate 5 points on the (un-constrained)  Efficient  Frontier  with" )
      System.Console.WriteLine( "     the precision set to 0.00001" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    covarianceMatrix = covarianceEntire" )
      System.Console.WriteLine( "    expectedReturns = expectedreturnsEntire" )
      System.Console.WriteLine( "    numberInterpolationPoints = 5" )
      System.Console.WriteLine( "    precision = 0.00001" )
      ' We first use the covariance method from Asset Parameters.
      Dim covarianceEntire As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedreturnsEntire As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' This method sets the coordinates of the Efficient Frontier as  private
      ' fields in the class.
      instanceOfMarkowitz.CalculateEfficientFrontier( _
         covarianceEntire, _
         expectedreturnsEntire, _
         5, _
         0.00001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Our first aim in virtually all applications and  Portfolio  Theory  is" )
      System.Console.WriteLine( "    to evaluate a collection of points on the  Efficient  Frontier,  which" )
      System.Console.WriteLine( "    we do to applying the  calculate  Efficient  Frontier  method.  Before" )
      System.Console.WriteLine( "    this method  is  applied  we  need  to  evaluate  the  covariance  and" )
      System.Console.WriteLine( "    expected returns using procedures from  the  Asset  Parameters  class." )
      System.Console.WriteLine( "    The points of the Efficient Frontier are stored in  private  field  of" )
      System.Console.WriteLine( "    the class." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we return the five values of the expected return for which  the" )
      System.Console.WriteLine( "     efficient was evaluated  within  the  example  ;Evaluate  Points  on" )
      System.Console.WriteLine( "     Efficient Frontier;, and stored within a private field." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      ' Reads the values of the  expected  return  at  the  points  where  the
      ' Efficient Frontier was evaluated.
      Dim efficientFrontierExpectedReturns1 As Double() = instanceOfMarkowitz.GetEfficientFrontierExpectedReturns( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The values of the expected return at the  five  points  at  which  the" )
      System.Console.WriteLine( "    efficient was evaluated is:" )
      System.Console.WriteLine( " " )
      displayArray( efficientFrontierExpectedReturns1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierAssetWeights(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      ' Reads the values  of  the  asset  weights  at  the  points  where  the
      ' Efficient Frontier was evaluated
      Dim assetWeightsEfficientFrontier1 As Double()() = instanceOfMarkowitz.GetEfficientFrontierAssetWeights( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    " )
      System.Console.WriteLine( " " )
      display2DArray( assetWeightsEfficientFrontier1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Weight of Assets of an Arbitary Portfolio on the Efficient Frontier +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    efficientFrontier( double expectedReturn," )
      System.Console.WriteLine( "                       int numberOfAssets )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     What are the  weights  of  the  assets  of  the  portfolios  on  the" )
      System.Console.WriteLine( "     Efficient     Frontier  which  was  evaluated  using  the  calculate" )
      System.Console.WriteLine( "     Efficient Frontier method for  the  four  assets  with  the  monthly" )
      System.Console.WriteLine( "     returns:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 102, 101, 99, 101" )
      System.Console.WriteLine( "        - 2nd Asset: 100, 103, 99, 101" )
      System.Console.WriteLine( "        - 3rd Asset: 100, 99, 103, 101" )
      System.Console.WriteLine( "        - 4th Asset: 101, 101, 102, 101" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     where the expected returns of the portfolio is 101? Note that  since" )
      System.Console.WriteLine( "     the  historical  returns  are  given  in  absolute  terms  then  the" )
      System.Console.WriteLine( "     expected returns we need to be provided in absolute terms." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 101" )
      System.Console.WriteLine( "    numberOfAssets = 4" )
      ' Evaluate the weights of the assets of the portfolio of  the  Efficient
      ' Frontier with an expected return of 101.
      Dim efficientFrontier1 As Double() = instanceOfMarkowitz.EfficientFrontier( _
         101.0, _
         4 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weights of the  assets  within  the  portfolio  on  the  Efficient" )
      System.Console.WriteLine( "    Frontier with an expected return of 101 is:" )
      System.Console.WriteLine( " " )
      displayArray( efficientFrontier1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Setting constraints on the Weights of the assets +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    setConstraints( double[] lowerBounds," )
      System.Console.WriteLine( "                    double[] upperBounds )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     How can constraints be set of the weights of the  assets  such  that" )
      System.Console.WriteLine( "     all portfolios of the  Efficient  Frontier  have  no  asset  with  a" )
      System.Console.WriteLine( "     weights more than 0.5 (i.e. 50 percent)  and  no  weight  below  0.1" )
      System.Console.WriteLine( "     (i.e. 10 percent)?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    lowerBounds = {0.1, 0.1, 0.1, 0.1}" )
      System.Console.WriteLine( "    upperBounds = {0.5, 0.5, 0.5, 0.5}" )
      instanceOfMarkowitz.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The application of this method  will  allow  the  constraints  on  the" )
      System.Console.WriteLine( "    Efficient Frontier portfolios assets weights  to  be  set  before  the" )
      System.Console.WriteLine( "    points on the Efficient Frontier are  evaluated  using  the  calculate" )
      System.Console.WriteLine( "    Efficient Frontier method." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Read off the values of the Lower Constraints +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getLowerConstraints(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Get the value of the lower bounds on the asset  weights  after  they" )
      System.Console.WriteLine( "     have been set." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      instanceOfMarkowitz.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      ' Get the value of the lower bounds of the assets.
      Dim readLowerBounds As Double() = instanceOfMarkowitz.GetLowerConstraints( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The lower bounds on the assets weights are:" )
      System.Console.WriteLine( " " )
      displayArray( readLowerBounds )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Read off the values of the Upper Constraints +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getUpperConstraints(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Get the value of the upper bounds on the asset  weights  after  they" )
      System.Console.WriteLine( "     have been set." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      instanceOfMarkowitz.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      ' Get the value of the upper bounds of the assets.
      Dim readUpperBounds As Double() = instanceOfMarkowitz.GetUpperConstraints( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The upper bounds on the assets weights are:" )
      System.Console.WriteLine( " " )
      displayArray( readUpperBounds )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Minimum value of the Expected Return of the Efficient Frontier +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    minFrontierReturn( double[] expectedReturns )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     What is the expected  return  of  the  Portfolio  on  the  Efficient" )
      System.Console.WriteLine( "     Frontier where the portfolios  can  be  constructed  from  the  four" )
      System.Console.WriteLine( "     assets with monthly returns:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 102, 101, 99, 101" )
      System.Console.WriteLine( "        - 2nd Asset: 100, 103, 99, 101" )
      System.Console.WriteLine( "        - 3rd Asset: 100, 99, 103, 101" )
      System.Console.WriteLine( "        - 4th Asset: 101, 101, 102, 101" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     and the weights of  each  of  there  assets  which  the  constructed" )
      System.Console.WriteLine( "     portfolio must lie within the interval [0.1, 0.5]?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturns = expectedreturnsMinExpReturn" )
      instanceOfMarkowitz.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedreturnsMinExpReturn As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' Evaluates the minimum value of the expected return of  the  portfolios
      ' on the (constrained) Efficient Frontier.
      Dim minexpReturn As Double = instanceOfMarkowitz.MinFrontierReturn( _
         expectedreturnsMinExpReturn _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The value of the expected return of the portfolio within  the  minimum" )
      System.Console.WriteLine( "    value of the expected return on the Efficient Frontier is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( minexpReturn ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Weights of the assets of the Portfolio on  the  Constrained  Efficient +" )
      System.Console.WriteLine( "+ Frontier with the minimum Expected Return                              +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    minFrontierReturnWeights( double[] expectedReturns )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     What are the weights of the portfolio which  lie  of  the  Efficient" )
      System.Console.WriteLine( "     Frontier where the portfolios  can  be  constructed  from  the  four" )
      System.Console.WriteLine( "     assets with monthly returns:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 102, 101, 99, 101" )
      System.Console.WriteLine( "        - 2nd Asset: 100, 103, 99, 101" )
      System.Console.WriteLine( "        - 3rd Asset: 100, 99, 103, 101" )
      System.Console.WriteLine( "        - 4th Asset: 101, 101, 102, 101" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     and the weights of  each  of  there  assets  which  the  constructed" )
      System.Console.WriteLine( "     portfolio must lie within the interval [0.1, 0.5]?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturns = expectedreturnsMinExpReturn1" )
      instanceOfMarkowitz.SetConstraints( _
         New Double() {0.1, 0.1, 0.1, 0.1}, _
         New Double() {0.5, 0.5, 0.5, 0.5} _
      )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedreturnsMinExpReturn1 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' Evaluates the minimum value of the expected return of  the  portfolios
      ' on the (constrained) Efficient Frontier.
      Dim minexpReturn1 As Double() = instanceOfMarkowitz.MinFrontierReturnWeights( _
         expectedreturnsMinExpReturn1 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The value of the expected return of the portfolio within  the  minimum" )
      System.Console.WriteLine( "    value of the expected return on the Efficient Frontier is:" )
      System.Console.WriteLine( " " )
      displayArray( minexpReturn1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Construction of Portfolios on the Efficient Frontier +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    efficientFrontier( double expectedReturn," )
      System.Console.WriteLine( "                       double[][] covarianceMatrix," )
      System.Console.WriteLine( "                       double[] expectedReturns," )
      System.Console.WriteLine( "                       double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     We are able to  construct  portfolios  from  four  assets  with  the" )
      System.Console.WriteLine( "     following historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 102, 101, 99, 101" )
      System.Console.WriteLine( "        - 2nd Asset: 100, 103, 99, 101" )
      System.Console.WriteLine( "        - 3rd Asset: 100, 99, 103, 101" )
      System.Console.WriteLine( "        - 4th Asset: 101, 101, 102, 101" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     What are the weights of the assets within the portfolios  which  can" )
      System.Console.WriteLine( "     be constructed from the above four  assets  which  have  the  lowest" )
      System.Console.WriteLine( "     risk when the expected return is 101?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = 101" )
      System.Console.WriteLine( "    covarianceMatrix = covariance2" )
      System.Console.WriteLine( "    expectedReturns = expectedreturns2" )
      System.Console.WriteLine( "    precision = 0.00001" )
      ' We first use the covariance method from Asset Parameters.
      Dim covariance2 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' Next to evaluate the expected return  using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedreturns2 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' Evaluates the weights of the portfolio on the Efficient Frontier  will
      ' a expected return of 101.
      Dim efficientFrontierStateless1 As Double() = instanceOfMarkowitz.EfficientFrontier( _
         101.0, _
         covariance2, _
         expectedreturns2, _
         0.00001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weights of the portfolio with the expected return of 101 is:" )
      System.Console.WriteLine( " " )
      displayArray( efficientFrontierStateless1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Set investors Utility function by Interpolation +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    setUtilityFunctionInterp( double[] expectedReturn," )
      System.Console.WriteLine( "                              double[] totalRisk )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     How do we set the investors utility function when we know  that  the" )
      System.Console.WriteLine( "     investor for the given levels of the expected return is prepared  to" )
      System.Console.WriteLine( "     accept the following levels of risk:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - Expected return = 99, Risk = 0.01" )
      System.Console.WriteLine( "        - Expected return = 100, Risk = 0.1" )
      System.Console.WriteLine( "        - Expected return = 102, Risk = 0.50" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturn = {99.0, 100.0, 102.0}" )
      System.Console.WriteLine( "    totalRisk = {0.01, 0.1, 0.5}" )
      ' Sets the investors utility function by providing a set of  risk/return
      ' interpolation points.
      instanceOfMarkowitz.SetUtilityFunctionInterp( _
         New Double() {99.0, 100.0, 102.0}, _
         New Double() {0.01, 0.1, 0.5} _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Once this method has been called the utility function  is  set  within" )
      System.Console.WriteLine( "    the classes private fields." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Set investors Utility function by Polynomial +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    setUtilityFunctionPoly( double[] coefficient )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     This  is  an  alternative  means  by  which  the  investors  utility" )
      System.Console.WriteLine( "     function  can  be  set.  That  is,  consider  the  function  of  the" )
      System.Console.WriteLine( "     following form:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     p(x) = 0.1 + (0.2 * x)" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     where p(x) of the value of the expected return  which  the  investor" )
      System.Console.WriteLine( "     expects when the level of risk take is x. This  methods  allows  the" )
      System.Console.WriteLine( "     utility  function  to  be  given  in  this  form  by  providing  the" )
      System.Console.WriteLine( "     coefficient of the polynomial." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    coefficient = {0.1, 0.2}" )
      ' Set the investors utility function.
      instanceOfMarkowitz.SetUtilityFunctionPoly( _
         New Double() {0.1, 0.2} _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Once the method is  called  the  utility  function  is  stored  within" )
      System.Console.WriteLine( "    private fields of the class." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Select the optimal Portfolio in accordance with an  investors  utility +" )
      System.Console.WriteLine( "+ function                                                               +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    optimalPortfolio( double[][] covarianceMatrix," )
      System.Console.WriteLine( "                      double[] expectedReturns )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     We are able to  construct  portfolios  from  four  assets  with  the" )
      System.Console.WriteLine( "     following historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 102, 101, 99, 101" )
      System.Console.WriteLine( "        - 2nd Asset: 100, 103, 99, 101" )
      System.Console.WriteLine( "        - 3rd Asset: 100, 99, 103, 101" )
      System.Console.WriteLine( "        - 4th Asset: 101, 101, 102, 101" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     Moreover, an investor who  wishes  to  construct  a  portfolio  from" )
      System.Console.WriteLine( "     these assets is prepared to accept the following levels of risk  for" )
      System.Console.WriteLine( "     the following expected returns:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - Return = 99, Risk = 0.01" )
      System.Console.WriteLine( "        - Return = 100, Risk = 0.1" )
      System.Console.WriteLine( "        - Return = 102, Risk = 0.5" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     What are the weights of the assets  within  the  optimal  portfolios" )
      System.Console.WriteLine( "     which are selected from the  Efficient  Frontier  by  the  investors" )
      System.Console.WriteLine( "     utility function given above?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    covarianceMatrix = covariance4" )
      System.Console.WriteLine( "    expectedReturns = expectedReturns4optimal1" )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedReturns4optimal1 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' We first use the covariance method from Asset Parameters.
      Dim covariance4 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' We next set the investors utility function.
      instanceOfMarkowitz.SetUtilityFunctionInterp( _
         New Double() {100.0, 101.0, 102.0}, _
         New Double() {0.01, 0.1, 0.5} _
      )
      ' Here we find the optimal portfolio in accordance with the set  utility
      ' function.
      Dim optimalPortfolio1 As Double()() = instanceOfMarkowitz.OptimalPortfolio( _
         covariance4, _
         expectedReturns4optimal1 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weights of the assets of the optimal portfolios are:" )
      System.Console.WriteLine( " " )
      display2DArray( optimalPortfolio1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Select the optimal Portfolio with the Maximum Expected Return +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    optimalPortfolioMaxExpected( double minimumExpectedReturn," )
      System.Console.WriteLine( "                                 double maximumExpectedReturn," )
      System.Console.WriteLine( "                                 double[][] covarianceMatrix )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     We are able to  construct  portfolios  from  four  assets  with  the" )
      System.Console.WriteLine( "     following historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 102, 101, 99, 101" )
      System.Console.WriteLine( "        - 2nd Asset: 100, 103, 99, 101" )
      System.Console.WriteLine( "        - 3rd Asset: 100, 99, 103, 101" )
      System.Console.WriteLine( "        - 4th Asset: 101, 101, 102, 101" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     Moreover, an investor who  wishes  to  construct  a  portfolio  from" )
      System.Console.WriteLine( "     these assets is prepared to accept the following levels of risk  for" )
      System.Console.WriteLine( "     the following expected returns:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - Return = 100, Risk = 0.01" )
      System.Console.WriteLine( "        - Return = 100.5, Risk = 0.1" )
      System.Console.WriteLine( "        - Return = 102, Risk = 0.5" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     What are the weights of the assets within the optimal  portfolio  in" )
      System.Console.WriteLine( "     accordance with the investors utility  function  given  above  which" )
      System.Console.WriteLine( "     has the highest expected return?" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    minimumExpectedReturn = minfrontier4optimal1" )
      System.Console.WriteLine( "    maximumExpectedReturn = maxfrontier4optimal1" )
      System.Console.WriteLine( "    covarianceMatrix = covariance5" )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedReturns4optimal2 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' Here we evaluate the maximum value  of  the  expected  return  on  the
      ' efficient frontier.
      Dim maxfrontier4optimal1 As Double = instanceOfMarkowitz.MaxFrontierReturn( _
         expectedReturns4optimal2 _
      )
      ' Here we evaluate the minimum value  of  the  expected  return  on  the
      ' efficient frontier.
      Dim minfrontier4optimal1 As Double = instanceOfMarkowitz.MinFrontierReturn( _
         expectedReturns4optimal2 _
      )
      ' We first use the covariance method from Asset Parameters.
      Dim covariance5 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {102.0, 101.0, 99.0, 101.0}, _
                         New Double() {100.0, 103.0, 99.0, 101.0}, _
                         New Double() {100.0, 99.0, 103.0, 101.0}, _
                         New Double() {101.0, 101.0, 102.0, 101.0}} _
      )
      ' We next set the investors utility function.
      instanceOfMarkowitz.SetUtilityFunctionInterp( _
         New Double() {100.0, 101.0, 102.0}, _
         New Double() {0.01, 0.1, 0.5} _
      )
      ' Here we find the optimal portfolio in accordance with the set  utility
      ' function.
      Dim optimalPortfolioMaxExpected1 As Double() = instanceOfMarkowitz.OptimalPortfolioMaxExpected( _
         minfrontier4optimal1, _
         maxfrontier4optimal1, _
         covariance5 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The weights of the assets of the optimal portfolio  with  the  highest" )
      System.Console.WriteLine( "    expected return is:" )
      System.Console.WriteLine( " " )
      displayArray( optimalPortfolioMaxExpected1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       -------------------------------------------------------------------" )
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" )
      System.Console.WriteLine( "       -------------------------------------------------------------------" )
      System.Console.WriteLine( " " )
   End Sub

   Function Double2String(ByVal d As Double) As String
      If (System.Double.IsNaN(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (System.Double.IsNegativeInfinity(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (System.Double.IsPositiveInfinity(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (d = Double.MinValue) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (d = Double.MaxValue) Then
          Double2String = d.ToString()
          Exit Function
      End If
      Dim frac As Double = d - Int(d)
      Dim frac4 As Double = Int(frac * 10000) / 10000.0
      Double2String = (Int(d) + frac4).ToString()
      Exit Function
   End Function

   Sub displayArray(ByVal a As Double())
      System.Console.Write("     {")
      For I As Integer = 0 To -1 + a.Length
         System.Console.Write(Double2String(a(I)))
         If (I < -1 + a.Length) Then System.Console.Write(", ")
      Next I
      System.Console.Write("}")
   End Sub

   Sub display2DArray(ByVal a As Double()())
      For I As Integer = 0 To -1 + a.Length
         If (I = 0) Then
            System.Console.Write("     {{")
         Else
            System.Console.Write("      {")
         End If
         For J As Integer = 0 To -1 + a(I).Length
            System.Console.Write(Double2String(a(I)(J)))
            If (J < -1 + a(I).Length) Then
               System.Console.Write(", ")
            Else
               System.Console.Write("}")
               End If
         Next J
         If (I < -1 + a.Length) Then
            System.Console.Write(",")
            System.Console.WriteLine()
            Else
               System.Console.Write("}")
            End If
      Next I
   End Sub
End Module

