Imports System
Imports Microsoft.VisualBasic
Imports WebCab.Lib.Util.Example
Imports WebCab.Libraries.Finance.Portfolio

Module TurningPointClient
   Sub Main()
      ' used components
      Dim instanceOfAssetParameters As AssetParameters = New AssetParameters
      Dim instanceOfMarkowitz As WebCab.Libraries.Finance.Portfolio.Markowitz = New WebCab.Libraries.Finance.Portfolio.Markowitz
      '
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "                     #######################################" )
      System.Console.WriteLine( "                     # TurningPoint Console Client Example #" )
      System.Console.WriteLine( "                     #######################################" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " --------" )
      System.Console.WriteLine( " Overview" )
      System.Console.WriteLine( " --------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Within this  client  we  provide  an  example  which  evaluates  the" )
      System.Console.WriteLine( "    Efficient    Frontier  for  a  class  of  portfolios  which  can  be" )
      System.Console.WriteLine( "    constructed from 5 assets, and then evaluates  the  point  at  which" )
      System.Console.WriteLine( "    the Efficient Frontier Turns back on itself. That is,  we  find  the" )
      System.Console.WriteLine( "    point   on  the  Efficient  Frontier  where  portfolios  with  lower" )
      System.Console.WriteLine( "    expected return have higher levels of risk. Notes on the Units  used" )
      System.Console.WriteLine( "    for the Source data (i.e. the Historical Values) The units in  which" )
      System.Console.WriteLine( "    the   historical  values  are  given  will  effect  the  units  used" )
      System.Console.WriteLine( "    throughout the portfolio analysis. For example,  if  the  historical" )
      System.Console.WriteLine( "    values are given in absolute terms then the  following  will  either" )
      System.Console.WriteLine( "    be returned and should be provided in absolute terms:" )
      System.Console.WriteLine( "    " )
      System.Console.WriteLine( "       - Expected Return" )
      System.Console.WriteLine( "       - Utility Function" )
      System.Console.WriteLine( "       " )
      System.Console.WriteLine( "    " )
      System.Console.WriteLine( "    Note that the covariance matrix and the asset weights are  invariant" )
      System.Console.WriteLine( "    to the units in which the historical returns are expressed." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluate Points  on  Efficient  Frontier  over  Interval  of  Expected +" )
      System.Console.WriteLine( "+ Returns                                                                +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    calculateEfficientFrontier( double[][] covarianceMatrix," )
      System.Console.WriteLine( "                                double[] expectedReturns," )
      System.Console.WriteLine( "                                int numberInterpolationPoints," )
      System.Console.WriteLine( "                                double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     We are able to  construct  portfolios  from  four  assets  with  the" )
      System.Console.WriteLine( "     following historical returns over the last 4 months:" )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "        - 1st Asset: 100, 105, 106.05, 114.534,  122.55138,  133.5810042," )
      System.Console.WriteLine( "        142.9316745, 154.3662085, 163.628181, 166.9007446, 185.2598265" )
      System.Console.WriteLine( "        - 2nd  Asset:  5,  5.75,  6.7275,  6.9966,  6.436872,  7.4024028," )
      System.Console.WriteLine( "        9.030931416, 9.301859358, 7.999599048, 8.159591029, 9.383529684" )
      System.Console.WriteLine( "        -      3rd  Asset:  2000,  2240,  2531.2,  2759.008,  2952.13856," )
      System.Console.WriteLine( "        3217.83103," )
      System.Console.WriteLine( "        3571.792444,3893.253764,4126.84899,4456.996909,4902.6966" )
      System.Console.WriteLine( "        -   4th  Asset:  0.1,  0.118,  0.13688,  0.1409864,  0.146625856," )
      System.Console.WriteLine( "        0.158355924, 0.174191517, 0.168965771, 0.194310637,  0.233172765," )
      System.Console.WriteLine( "        0.270480407" )
      System.Console.WriteLine( "        -   5th  Asset:  40,  42,  45.36,  49.896,  54.38664,  57.105972," )
      System.Console.WriteLine( "        59.39021088, 61.76581932, 65.47176847, 70.70950995, 77.78046095" )
      System.Console.WriteLine( "        " )
      System.Console.WriteLine( "     " )
      System.Console.WriteLine( "     Evaluate 5 points on the (un-constrained)  Efficient  Frontier  with" )
      System.Console.WriteLine( "     the precision set to 0.00001" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    covarianceMatrix = covariance1" )
      System.Console.WriteLine( "    expectedReturns = expectedreturns1" )
      System.Console.WriteLine( "    numberInterpolationPoints = 20" )
      System.Console.WriteLine( "    precision = 0.00001" )
      ' We first use the covariance method from Asset Parameters.
      Dim covariance1 As Double()() = instanceOfAssetParameters.CovarianceMatrix( _
         New Double()() {New Double() {0.0844, 0.0115, -0.0156, -0.1028, 0.0014}, _
                         New Double() {0.0115, 1.2329, 0.1996, 0.1633, -0.0539}, _
                         New Double() {-0.0156, 0.1996, 0.0424, 0.0382, -0.0066}, _
                         New Double() {-0.1028, 0.1633, 0.0382, 0.5141, 0.0237}, _
                         New Double() {0.0014, -0.0539, -0.0066, 0.0237, 0.0509}} _
      )
      ' Next we evaluate the expected returns using  the  historical  approach
      ' from Asset Parameters.
      Dim expectedreturns1 As Double() = instanceOfAssetParameters.ExpectedReturns( _
         New Double()() {New Double() {100.0, 105.0, 106.05, 114.534, 122.5513, 133.581, 142.9316, 154.3662, 163.6281, 166.9007, 185.2598}, _
                         New Double() {5.0, 5.75, 6.7275, 6.9966, 6.4368, 7.4024, 9.0309, 9.3018, 7.9995, 8.1595, 9.3835}, _
                         New Double() {2000.0, 2240.0, 2531.2, 2759.0079, 2952.1385, 3217.831, 3571.7924, 3893.2537, 4126.8489, 4456.9969, 4902.6966}, _
                         New Double() {0.1, 0.118, 0.1368, 0.1409, 0.1466, 0.1583, 0.1741, 0.1689, 0.1943, 0.2331, 0.2704}, _
                         New Double() {40.0, 42.0, 45.36, 49.896, 54.3866, 57.1059, 59.3902, 61.7658, 65.4717, 70.7095, 77.7804}} _
      )
      ' This method sets the coordinates of the Efficient Frontier as  private
      ' fields in the class. Note the we evaluate the Efficient Frontier  over
      ' the entire region which it exists.
      instanceOfMarkowitz.CalculateEfficientFrontier( _
         covariance1, _
         expectedreturns1, _
         20, _
         0.00001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Our first aim in virtually all applications and  Portfolio  Theory  is" )
      System.Console.WriteLine( "    to evaluate a collection of points on the  Efficient  Frontier,  which" )
      System.Console.WriteLine( "    we do to applying the  calculate  Efficient  Frontier  method.  Before" )
      System.Console.WriteLine( "    this method  is  applied  we  need  to  evaluate  the  covariance  and" )
      System.Console.WriteLine( "    expected returns using procedures from  the  Asset  Parameters  class." )
      System.Console.WriteLine( "    The points of the Efficient Frontier are stored in  private  field  of" )
      System.Console.WriteLine( "    the class." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we return the five values of the expected return for which  the" )
      System.Console.WriteLine( "     efficient was evaluated  within  the  example  ;Evaluate  Points  on" )
      System.Console.WriteLine( "     Efficient Frontier;, and stored within a private field." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      ' Reads the values of the  expected  return  at  the  points  where  the
      ' Efficient Frontier was evaluated.
      Dim efficientFrontierExpectedReturns1 As Double() = instanceOfMarkowitz.GetEfficientFrontierExpectedReturns( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The values of the expected return at the  five  points  at  which  the" )
      System.Console.WriteLine( "    efficient was evaluated is:" )
      System.Console.WriteLine( " " )
      displayArray( efficientFrontierExpectedReturns1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Expected Returns on the evaluated points of the Efficient Frontier +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Returns the values of the risks at the 20 points  of  the  Efficient" )
      System.Console.WriteLine( "     Frontier evaluated." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    covarianceMatrix = {{0.0844, 0.0115, -0.0156, -0.1028, 0.0014}," )
      System.Console.WriteLine( "                       {0.0115, 1.2329, 0.1996, 0.1633, -0.0539}," )
      System.Console.WriteLine( "                       {-0.0156, 0.1996, 0.0424, 0.0382, -0.0066}," )
      System.Console.WriteLine( "                       {-0.1028, 0.1633, 0.0382, 0.5141, 0.0237}," )
      System.Console.WriteLine( "                       {0.0014, -0.0539, -0.0066, 0.0237, 0.0509}}" )
      Dim risks1 As Double() = instanceOfMarkowitz.GetEfficientFrontierPortfolioRisks( _
         New Double()() {New Double() {0.0844, 0.0115, -0.0156, -0.1028, 0.0014}, _
                         New Double() {0.0115, 1.2329, 0.1996, 0.1633, -0.0539}, _
                         New Double() {-0.0156, 0.1996, 0.0424, 0.0382, -0.0066}, _
                         New Double() {-0.1028, 0.1633, 0.0382, 0.5141, 0.0237}, _
                         New Double() {0.0014, -0.0539, -0.0066, 0.0237, 0.0509}} _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The 20 values of the risks of the points  on  the  Efficient  Frontier" )
      System.Console.WriteLine( "    are:" )
      System.Console.WriteLine( " " )
      displayArray( risks1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Returns the value of the Expected Return at  the  Turning  Point  when +" )
      System.Console.WriteLine( "+ iterations = 1                                                         +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    turningPointExpectedReturn( double[] expectedReturns," )
      System.Console.WriteLine( "                                double[][] covarianceMatrix," )
      System.Console.WriteLine( "                                double precision," )
      System.Console.WriteLine( "                                int iterations )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Evaluates the value of the expected return (using  1  iteration)  at" )
      System.Console.WriteLine( "     which the Efficient Frontier turns." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturns = expectedreturns1" )
      System.Console.WriteLine( "    covarianceMatrix = covariance1" )
      System.Console.WriteLine( "    precision = 0.0000001" )
      System.Console.WriteLine( "    iterations = 1" )
      ' Evaluates the
      Dim turningPointExpectedReturn1 As Double = instanceOfMarkowitz.TurningPointExpectedReturn( _
         expectedreturns1, _
         covariance1, _
         0.0000001, _
         1 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The value of the  expected  return  at  the  turning  point  (using  1" )
      System.Console.WriteLine( "    iterations) is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( turningPointExpectedReturn1 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Returns the value of the Expected Return at  the  Turning  Point  when +" )
      System.Console.WriteLine( "+ iterations = 3                                                         +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    turningPointExpectedReturn( double[] expectedReturns," )
      System.Console.WriteLine( "                                double[][] covarianceMatrix," )
      System.Console.WriteLine( "                                double precision," )
      System.Console.WriteLine( "                                int iterations )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Evaluates the value of the expected return (using 3  iterations)  at" )
      System.Console.WriteLine( "     which the Efficient Frontier turns." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturns = expectedreturns1" )
      System.Console.WriteLine( "    covarianceMatrix = covariance1" )
      System.Console.WriteLine( "    precision = 0.0000001" )
      System.Console.WriteLine( "    iterations = 3" )
      ' Evaluates the
      Dim turningPointExpectedReturn2 As Double = instanceOfMarkowitz.TurningPointExpectedReturn( _
         expectedreturns1, _
         covariance1, _
         0.0000001, _
         3 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The value of the  expected  return  at  the  turning  point  (using  3" )
      System.Console.WriteLine( "    iterations) is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( turningPointExpectedReturn2 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Returns the value of the Expected Return at  the  Turning  Point  when +" )
      System.Console.WriteLine( "+ iterations = 5                                                         +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    turningPointExpectedReturn( double[] expectedReturns," )
      System.Console.WriteLine( "                                double[][] covarianceMatrix," )
      System.Console.WriteLine( "                                double precision," )
      System.Console.WriteLine( "                                int iterations )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Evaluates the value of the expected return (using 5  iterations)  at" )
      System.Console.WriteLine( "     which the Efficient Frontier turns." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturns = expectedreturns1" )
      System.Console.WriteLine( "    covarianceMatrix = covariance1" )
      System.Console.WriteLine( "    precision = 0.0000001" )
      System.Console.WriteLine( "    iterations = 5" )
      ' Evaluates the
      Dim turningPointExpectedReturn3 As Double = instanceOfMarkowitz.TurningPointExpectedReturn( _
         expectedreturns1, _
         covariance1, _
         0.0000001, _
         5 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The value of the  expected  return  at  the  turning  point  (using  5" )
      System.Console.WriteLine( "    iterations) is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( turningPointExpectedReturn3 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       -------------------------------------------------------------------" )
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" )
      System.Console.WriteLine( "       -------------------------------------------------------------------" )
      System.Console.WriteLine( " " )
   End Sub

   Function Double2String(ByVal d As Double) As String
      If (System.Double.IsNaN(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (System.Double.IsNegativeInfinity(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (System.Double.IsPositiveInfinity(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (d = Double.MinValue) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (d = Double.MaxValue) Then
          Double2String = d.ToString()
          Exit Function
      End If
      Dim frac As Double = d - Int(d)
      Dim frac4 As Double = Int(frac * 10000) / 10000.0
      Double2String = (Int(d) + frac4).ToString()
      Exit Function
   End Function

   Sub displayArray(ByVal a As Double())
      System.Console.Write("     {")
      For I As Integer = 0 To -1 + a.Length
         System.Console.Write(Double2String(a(I)))
         If (I < -1 + a.Length) Then System.Console.Write(", ")
      Next I
      System.Console.Write("}")
   End Sub

   Sub display2DArray(ByVal a As Double()())
      For I As Integer = 0 To -1 + a.Length
         If (I = 0) Then
            System.Console.Write("     {{")
         Else
            System.Console.Write("      {")
         End If
         For J As Integer = 0 To -1 + a(I).Length
            System.Console.Write(Double2String(a(I)(J)))
            If (J < -1 + a(I).Length) Then
               System.Console.Write(", ")
            Else
               System.Console.Write("}")
               End If
         Next J
         If (I < -1 + a.Length) Then
            System.Console.Write(",")
            System.Console.WriteLine()
            Else
               System.Console.Write("}")
            End If
      Next I
   End Sub
End Module

