Imports System
Imports Microsoft.VisualBasic
Imports WebCab.Lib.Util.Example
Imports WebCab.Libraries.Finance.Portfolio

Module TurningPointThreeAssetsClient
   Sub Main()
      ' used components
      Dim instanceOfMarkowitz As WebCab.Libraries.Finance.Portfolio.Markowitz = New WebCab.Libraries.Finance.Portfolio.Markowitz
      '
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "                ##################################################" )
      System.Console.WriteLine( "                # TurningPointThreeAssets Console Client Example #" )
      System.Console.WriteLine( "                ##################################################" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " --------" )
      System.Console.WriteLine( " Overview" )
      System.Console.WriteLine( " --------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    Within this example we first evaluate the  Efficient  Frontier  when" )
      System.Console.WriteLine( "    the portfolio is constructed from three assets,  and  then  evaluate" )
      System.Console.WriteLine( "    the point at which the Efficient Frontier 'turns back on itself'." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Press ENTER or SPACE to scroll down." )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Evaluates the Efficient Frontier +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    calculateEfficientFrontier( double minimumExpectedReturn," )
      System.Console.WriteLine( "                                double maximumExpectedReturn," )
      System.Console.WriteLine( "                                double[][] covarianceMatrix," )
      System.Console.WriteLine( "                                double[] expectedReturns," )
      System.Console.WriteLine( "                                int numberInterpolationPoints," )
      System.Console.WriteLine( "                                double precision )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Here we evaluate the  Efficient  Frontier  at  3  equally  dispersed" )
      System.Console.WriteLine( "     points within the range of the expected return [0.05, 0.15]." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    minimumExpectedReturn = 0.05" )
      System.Console.WriteLine( "    maximumExpectedReturn = 0.15" )
      System.Console.WriteLine( "    covarianceMatrix = {{1.0, -0.03, 0.02}," )
      System.Console.WriteLine( "                       {-0.03, 1.0, 0.02}," )
      System.Console.WriteLine( "                       {0.02, 0.02, 1.0}}" )
      System.Console.WriteLine( "    expectedReturns = {0.05, 0.15, 0.07}" )
      System.Console.WriteLine( "    numberInterpolationPoints = 3" )
      System.Console.WriteLine( "    precision = 0.00001" )
      instanceOfMarkowitz.CalculateEfficientFrontier( _
         0.05, _
         0.15, _
         New Double()() {New Double() {1.0, -0.03, 0.02}, _
                         New Double() {-0.03, 1.0, 0.02}, _
                         New Double() {0.02, 0.02, 1.0}}, _
         New Double() {0.05, 0.15, 0.07}, _
         3, _
         0.00001 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The Efficient Frontier has been evaluated and set to a private field." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Returns values of the Expected Returns +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierExpectedReturns(  )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Returns the values of the expected returns at the 3  points  of  the" )
      System.Console.WriteLine( "     Efficient Frontier evaluated." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      Dim returns1 As Double() = instanceOfMarkowitz.GetEfficientFrontierExpectedReturns( _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The 20 values of the expected return of the points  on  the  Efficient" )
      System.Console.WriteLine( "    Frontier are:" )
      System.Console.WriteLine( " " )
      displayArray( returns1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Returns the values of the Risks +" )
      System.Console.WriteLine( "+++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    getEfficientFrontierPortfolioRisks( double[][] covarianceMatrix )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Returns the values of the risks at the 3  points  of  the  Efficient" )
      System.Console.WriteLine( "     Frontier evaluated." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    covarianceMatrix = {{1.0, -0.03, 0.02}," )
      System.Console.WriteLine( "                       {-0.03, 1.0, 0.02}," )
      System.Console.WriteLine( "                       {0.02, 0.02, 1.0}}" )
      Dim risks1 As Double() = instanceOfMarkowitz.GetEfficientFrontierPortfolioRisks( _
         New Double()() {New Double() {1.0, -0.03, 0.02}, _
                         New Double() {-0.03, 1.0, 0.02}, _
                         New Double() {0.02, 0.02, 1.0}} _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The 20 values of the risks of the points  on  the  Efficient  Frontier" )
      System.Console.WriteLine( "    are:" )
      System.Console.WriteLine( " " )
      displayArray( risks1 )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Returns the value of the Expected Return at  the  Turning  Point  when +" )
      System.Console.WriteLine( "+ iterations = 1                                                         +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    turningPointExpectedReturn( double[] expectedReturns," )
      System.Console.WriteLine( "                                double[][] covarianceMatrix," )
      System.Console.WriteLine( "                                double precision," )
      System.Console.WriteLine( "                                int iterations )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Evaluates the value of the expected return (using  1  iteration)  at" )
      System.Console.WriteLine( "     the point at which the Efficient Frontier turn back on itself." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturns = returns1" )
      System.Console.WriteLine( "    covarianceMatrix = {{1.0, -0.03, 0.02}," )
      System.Console.WriteLine( "                       {-0.03, 1.0, 0.02}," )
      System.Console.WriteLine( "                       {0.02, 0.02, 1.0}}" )
      System.Console.WriteLine( "    precision = 0.0000001" )
      System.Console.WriteLine( "    iterations = 1" )
      ' Evaluates the
      Dim turningPointExpectedReturn1 As Double = instanceOfMarkowitz.TurningPointExpectedReturn( _
         returns1, _
         New Double()() {New Double() {1.0, -0.03, 0.02}, _
                         New Double() {-0.03, 1.0, 0.02}, _
                         New Double() {0.02, 0.02, 1.0}}, _
         0.0000001, _
         1 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The value of the  expected  return  at  the  turning  point  (using  1" )
      System.Console.WriteLine( "    iteration) is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( turningPointExpectedReturn1 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( "+ Returns the value of the Expected Return at  the  Turning  Point  when +" )
      System.Console.WriteLine( "+ iterations = 3                                                         +" )
      System.Console.WriteLine( "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " Within this example we test the following method:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    turningPointExpectedReturn( double[] expectedReturns," )
      System.Console.WriteLine( "                                double[][] covarianceMatrix," )
      System.Console.WriteLine( "                                double precision," )
      System.Console.WriteLine( "                                int iterations )" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " Question:" )
      System.Console.WriteLine( " ---------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "     Evaluates the value of the expected return (using  3  iteration)  at" )
      System.Console.WriteLine( "     the point at which the Efficient Frontier turn back on itself." )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " Parameter values required to solve this problem:" )
      System.Console.WriteLine( " ------------------------------------------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    expectedReturns = returns1" )
      System.Console.WriteLine( "    covarianceMatrix = {{1.0, -0.03, 0.02}," )
      System.Console.WriteLine( "                       {-0.03, 1.0, 0.02}," )
      System.Console.WriteLine( "                       {0.02, 0.02, 1.0}}" )
      System.Console.WriteLine( "    precision = 0.0000001" )
      System.Console.WriteLine( "    iterations = 3" )
      ' Evaluates the
      Dim turningPointExpectedReturn2 As Double = instanceOfMarkowitz.TurningPointExpectedReturn( _
         returns1, _
         New Double()() {New Double() {1.0, -0.03, 0.02}, _
                         New Double() {-0.03, 1.0, 0.02}, _
                         New Double() {0.02, 0.02, 1.0}}, _
         0.0000001, _
         3 _
      )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " Answer evaluated:" )
      System.Console.WriteLine( " -----------------" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "    The value of the  expected  return  at  the  turning  point  (using  3" )
      System.Console.WriteLine( "    iterations) is:" )
      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       "  )
      System.Console.Write( double2String( turningPointExpectedReturn2 ) )
      System.Console.WriteLine( " " )

      System.Console.WriteLine( " " )
      System.Console.WriteLine( "       -------------------------------------------------------------------" )
      System.Console.WriteLine( "       - Please scroll up to the top of the console to view the examples -" )
      System.Console.WriteLine( "       -------------------------------------------------------------------" )
      System.Console.WriteLine( " " )
   End Sub

   Function Double2String(ByVal d As Double) As String
      If (System.Double.IsNaN(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (System.Double.IsNegativeInfinity(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (System.Double.IsPositiveInfinity(d)) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (d = Double.MinValue) Then
          Double2String = d.ToString()
          Exit Function
      End If
      If (d = Double.MaxValue) Then
          Double2String = d.ToString()
          Exit Function
      End If
      Dim frac As Double = d - Int(d)
      Dim frac4 As Double = Int(frac * 10000) / 10000.0
      Double2String = (Int(d) + frac4).ToString()
      Exit Function
   End Function

   Sub displayArray(ByVal a As Double())
      System.Console.Write("     {")
      For I As Integer = 0 To -1 + a.Length
         System.Console.Write(Double2String(a(I)))
         If (I < -1 + a.Length) Then System.Console.Write(", ")
      Next I
      System.Console.Write("}")
   End Sub

   Sub display2DArray(ByVal a As Double()())
      For I As Integer = 0 To -1 + a.Length
         If (I = 0) Then
            System.Console.Write("     {{")
         Else
            System.Console.Write("      {")
         End If
         For J As Integer = 0 To -1 + a(I).Length
            System.Console.Write(Double2String(a(I)(J)))
            If (J < -1 + a(I).Length) Then
               System.Console.Write(", ")
            Else
               System.Console.Write("}")
               End If
         Next J
         If (I < -1 + a.Length) Then
            System.Console.Write(",")
            System.Console.WriteLine()
            Else
               System.Console.Write("}")
            End If
      Next I
   End Sub
End Module

